/******************************************************************************************
    loongson3_ddr2_config.S
        used to set up ddr controllers MC0 and MC1
        and set up the memory space on L2 Xbar
    input: s1--MC1 & MC0 DIMM info and Node ID
    note: s1 is damaged by the end of this file
    original: whd
    rewrite by cxk on 11/11/2010
    1: reorder the program
    2: DIMM info and memory size is set according to s1[MC1&0_MEMSIZE]
    note: config L2 Xbar still need to be finished,currently only support limited MEMSIZE.
    v1.0    raw
    v1.2    add support for 4G memsize per MC, modify the L2-Xbar config manner of MC1
            to reduce code size.
    v1.4    Modify L2 Xbar config reg code at Interleave mode to reduce code size
            new code:
            1. according to Memsize config open space
            2. config interleave bits
*******************************************************************************************/
    //###Set DDR frequency ### 
    /* software ddr clk_sel can take effect when hardware ddrclk is in bypass mode */
    TTYDBG  ("\r\nSet DDR frequency by software\r\n")
    dli t2, 0x900000001fe00180

    /* clear ddr_clksel */
    lw  a1, 0x0(t2)
    li  a0, 0xe0ffffff
    and a1, a1,a0

    li  a0, 0x06000000 // ddr_freq X 0x12

    or  a1, a1, a0
    sw  a1, 0x0(t2)

    /* enable software ddr_clksel */
    li  a0, 0x00000008
    or  a1, a1, a0
    sw  a1, 0x0(t2)

no_software_ddrclksel:

    //make sure s1[3:2] is correctly set.
    GET_MC_SEL_BITS
    dli     t5, 3
    bne     a1, t5, 1f
    nop
    
    //s1[3:2]=0b'11, clear to 0b'00
    dli     t5, 0xc
    not     t5, t5
    and     s1, s1, t5
1:
/**************************************************************************************
1. 1. check NODE memory size.
*  2. set MC0/1_ONLY if the following 2 conditions are satisfied:
*     (1). s1[3:2]=0b'00
*     (2). MC0 or MC1 MEMSIZE > 0.
* when use AUTO_DDR_CONFIG, one MC may have no DIMM while the other has, in this case,
* the code should set MC0_ONLY or MC1_ONLY in s1 automatically, because the code of 
* configuring L2-Xbar will use this message.
***************************************************************************************/
    GET_MC0_ONLY
    bnez    a1, 1f
    nop
    GET_MC1_ONLY
    bnez    a1, 2f
    nop
    //s1[3:2]=0b'00
    //check memory size in this case
    GET_MC0_MEMSIZE
    move    t5, a1
    GET_MC1_MEMSIZE
    daddu   a1, a1, t5
    beqz    a1, 70f
    nop
    dli     t5, 0x10
    bgt     a1, t5, 70f
    nop
    GET_MC0_MEMSIZE
    bnez    a1, 3f
    nop
    //MC0_MEMSIZE=0, MC1_MEMSIZE must !=0, set MC1_ONLY
    dli     t5, 0x8
    or      s1, t5
    b       4f
    nop
3:  //MC0_MEMSIZE!=0
    GET_MC1_MEMSIZE
    bnez    a1, 4f
    nop
    //MC1_MEMSIZE=0 set use MC0_ONLY
    dli     t5, 0x4
    or      s1, t5
    b       4f
    nop
1:  //MC0_ONLY
    GET_MC0_MEMSIZE
    b       5f
    nop
2:  //MC1_ONLY
    GET_MC1_MEMSIZE
5:
    beqz    a1, 70f
    nop
    dli     t5, 0x8
    bgt     a1, t5, 70f
    nop
4:

/**************************
2. set up Memory Controller.
***************************/

/**************************************************
for single chip or multi-chip:
t0: X-bar config base
t2: chip configuration register location
t0,t2 shouldn't be changed to the end of this file.
***************************************************/
    GET_NODE_ID_a0
    dli     t2, 0x900000001fe00180
    dli     t0, 0x900000003ff00000
    or      t2, t2, a0
    or      t0, t0, a0

    //config MC0 if not define MC1_ONLY
10:
    GET_MC1_ONLY
    bnez    a1, 11f
    nop

    TTYDBG	("\r\nEnable register space of MEMORY\r\n")
    lw      a1, 0x0(t2)
    li      a0, 0xfffffeff
    and     a1, a1,a0
    sw      a1, 0x0(t2)

    GET_NODE_ID_a0;
    XBAR_CONFIG_NODE_a0(0x10, \
                        0x0000000000000000, \
                        0xfffffffff0000000, \
                        0x00000000000000f0)
    sync

    dli     a1, 0x900000000ff00000
    or      t8, a0, a1
    dli     t3, 0x0
    bal	    ddr2_config
    nop
    PRINTSTR("\r\nMC0 Config DONE\r\n")
#ifdef  DEBUG_DDR_PARAM   //print registers
    PRINTSTR("The MC0 configuration is:\r\n")
    #ifdef  loongson3A3
    dli     t1, 180
    #else
    dli     t1, 152
    #endif
    GET_NODE_ID_a0
    dli     t5, 0x900000000ff00000
    or      t5, t5, a0
1:
    ld      t6, 0x0(t5)
    dsrl    a0, t6, 32
    bal	    hexserial
    nop
    PRINTSTR("  ")
    move    a0, t6
    bal	    hexserial
    nop
    PRINTSTR("\r\n")

    daddiu  t1, t1, -1
    daddiu  t5, t5, 16
    bnez    t1, 1b
    nop
#endif

    TTYDBG("Disable register space of MEMORY\r\n")
    lw      a1,0x0(t2)
    or      a1,a1,0x100
    sw      a1,0x0(t2)



1:
    //config MC1 if not define MC0_ONLY
11:
    GET_MC0_ONLY
    bnez    a1, 12f
    nop

    TTYDBG	("\r\nEnable register space of MEMORY\r\n")
    lw      a1, 0x0(t2)
    li      a0, 0xfffffeff
    and     a1, a1,a0
    sw      a1, 0x0(t2)

    //shift MC1 DIMM info to low 32bit of s1
    dsrl    t5, s1, 32
    dli     a1, 0xffff0000
    and     t5, t5, a1
    dli     a1, 0xffffffff0000ffff
    and     s1, s1, a1
    or      s1, s1, t5

    GET_NODE_ID_a0;
    XBAR_CONFIG_NODE_a0(0x10, \
                        0x0000000000000000, \
                        0xfffffffff0000000, \
                        0x00000000000000f1)
    sync

    dli     a1, 0x900000000ff00000
    or      t8, a0, a1
    dli     t3, 0x1
    bal	    ddr2_config
    nop
    PRINTSTR("\r\nMC1 Config DONE\r\n")
#ifdef  DEBUG_DDR_PARAM   //print registers
    PRINTSTR("The MC1 configuration is:\r\n")
    #ifdef  loongson3A3
    dli     t1, 180
    #else
    dli     t1, 152
    #endif
    GET_NODE_ID_a0
    dli     t5, 0x900000000ff00000
    or      t5, t5, a0
1:
    ld      t6, 0x0(t5)
    dsrl    a0, t6, 32
    bal	    hexserial
    nop
    PRINTSTR("  ")
    move    a0, t6
    bal	    hexserial
    nop
    PRINTSTR("\r\n")

    daddiu  t1, t1, -1
    daddiu  t5, t5, 16
    bnez    t1, 1b
    nop
#endif

    TTYDBG("Disable register space of MEMORY\r\n")
    lw      a1,0x0(t2)
    or      a1,a1,0x100
    sw      a1,0x0(t2)


12:
    // AdonWang disable ddr3 readbuff
    /*      May Affect the Performance     */
    //This seems better for the spec2000
    TTYDBG("Disable read buffer\r\n")
    lw      t1, 0x4(t2)
    li      a0, 0x18
    or      t1, t1, a0
    sw      t1, 0x4(t2)

    // AdonWang disable cpu buffered read
    /* !!!!!!!!!!!!! IMPORTANT !!!!!!!!!!!! */
    TTYDBG("Disable cpu buffered read\r\n")
    lw      t1, 0x0(t2)
    li      a0, 0xfffffdff
    and     t1, t1, a0
    sw      t1, 0x0(t2)

/*****************************
 3. set msize for this NODE
******************************/
    //only NODE 0 uses msize
    GET_NODE_ID_a0;
    bnez    a0, 3f
    nop
    GET_MC0_ONLY
    beqz    a1, 1f
    nop
    //use MC0 only
    GET_MC0_MEMSIZE
    move    t5, a1
    dsll    msize, t5, 29   //t5*512M
    b       2f
    nop
1:
    GET_MC1_ONLY
    beqz    a1, 1f
    nop
    //use MC1 only
    GET_MC1_MEMSIZE
    move    t5, a1
    dsll    msize, t5, 29   //t5*512M
    b       2f
    nop
1:
    //use MC0&MC1
    GET_MC0_MEMSIZE
    move    t5, a1
    GET_MC1_MEMSIZE
    daddu   t5, t5, a1
    dsll    msize, t5, 29   //t5*512M
2:
    dli     t5, 0x1000000   //16M
    dsubu   msize, msize, t5//msize-=16M
    PRINTSTR("\r\nmsize = 0x")
    move    a0, msize
    bal     hexserial
    nop
    PRINTSTR("\r\n")
3:

/*******************************
 4. config L2 X-bar
    code procedure: first, MC*_ONLY bits in s1 decides whether this MC is
    used,then according to MC*_MEMSIZE bits in s1 decide memory size and how
    the L2 X-bar windows will be configured.
    note: currently,when use only 1 MC,support memory size: 512M, 1G, 2G, 3G, 4G;
          when use MC0&MC1 both, only support 1G, 2G or 4G Memory size of each Controller.
*******************************/
    //disable default pci window
    //L2XBAR_DISABLE_WINDOW(0x100);
    GET_MC_SEL_BITS
    beqz    a1, 1f
    nop
    GET_MC0_ONLY
    beqz  a1,5f
    /*MC0_ONLY */
    GET_NODE_ID_a0;
    XBAR_CONFIG_NODE_a0(0x10, \
                        0x0000000000000000, \
                        0xFFFFFFFFF0000000, \
                        0x00000000000000F0)
    PRINTSTR("MC0 space open : 0x00000000 - 0x0FFFFFFF\r\n")
    b      2f
    nop
    /* MC1_ONLY */
5:
    GET_NODE_ID_a0;
    XBAR_CONFIG_NODE_a0(0x10, \
                        0x0000000000000000, \
                        0xfffffffff0000000, \
                        0x00000000000000f1) 
    PRINTSTR("MC1 space open : 0x00000000 - 0x0FFFFFFF\r\n")
        
    b       2f
    nop
1:
#ifdef NO_INTERLEAVE
    GET_NODE_ID_a0;
    XBAR_CONFIG_NODE_a0(0x10, \
                        0x0000000000000000, \
                        0xFFFFFFFFF0000000, \
                        0x00000000000000F0)
    PRINTSTR("DDR space open : 0x00000000 - 0x0FFFFFFF\r\n")
#else
    #ifdef INTERLEAVE_27
    PRINTSTR("DDR Interleave using Bit 27\r\n")
    GET_NODE_ID_a0;
    XBAR_CONFIG_NODE_a0(0x10, \
                        0x0000000000000000, \
                        0xFFFFFFFFF8000000, \
                        0x00000000000000F0)
    XBAR_CONFIG_NODE_a0(0x18, \
                        0x0000000008000000, \
                        0xFFFFFFFFF8000000, \
                        0x00000000000000F1)
    #else
        #ifdef INTERLEAVE_13
	    PRINTSTR("DDR Interleave using Bit 13\r\n")
        GET_NODE_ID_a0;
        XBAR_CONFIG_NODE_a0(0x10, \
                            0x0000000000000000, \
                            0xFFFFFFFFF0002000, \
                            0x00000000000000F0)
        XBAR_CONFIG_NODE_a0(0x18, \
                            0x0000000000002000, \
                            0xFFFFFFFFF0002000, \
                            0x00000000000000F1)
        #else
            #ifdef INTERLEAVE_12
	        PRINTSTR("DDR Interleave using Bit 12\r\n")
            GET_NODE_ID_a0;
            XBAR_CONFIG_NODE_a0(0x10, \
                            0x0000000000000000, \
                            0xFFFFFFFFF0001000, \
                            0x00000000000000F0)
            XBAR_CONFIG_NODE_a0(0x18, \
                            0x0000000000001000, \
                            0xFFFFFFFFF0001000, \
                            0x00000000000000F1)
            #else
                #ifdef INTERLEAVE_11
	            PRINTSTR("DDR Interleave using Bit 11\r\n")
                GET_NODE_ID_a0;
                XBAR_CONFIG_NODE_a0(0x10, \
                            0x0000000000000000, \
                            0xFFFFFFFFF0000800, \
                            0x00000000000000F0)
                XBAR_CONFIG_NODE_a0(0x18, \
                            0x0000000000000800, \
                            0xFFFFFFFFF0000800, \
                            0x00000000000000F1)
                #else   //Interleave_10
	            PRINTSTR("DDR Interleave space open : 0x00000000 - 0x0FFFFFFF\r\n")
                GET_NODE_ID_a0;
                XBAR_CONFIG_NODE_a0(0x10, \
                            0x0000000000000000, \
                            0xFFFFFFFFF0000400, \
                            0x00000000000000F0)
                XBAR_CONFIG_NODE_a0(0x18, \
                            0x0000000000000400, \
                            0xFFFFFFFFF0000400, \
                            0x00000000000000F1)

                #endif
            #endif
        #endif
    #endif
#endif
2:
    //MC0&MC1
    GET_MC_SEL_BITS
    beqz    a1, 4f
    nop
    GET_MC0_MEMSIZE
    move    t5, a1

    //MC0_ONLY
    GET_MC0_ONLY
    bnez    a1, 1f
    nop
        
    //MC1_ONLY
    GET_MC1_MEMSIZE        
    move    t5, a1
    b  6f
    nop

1:     
    //use MC0 only, currently only support 512M, 1G, 2G, 3G, 4G
    move    a1, t5
    dli     t5, 0x1
    beq     a1, t5, 10f
    nop
    dli     t5, 0x2
    beq     a1, t5, 20f
    nop
    dli     t5, 0x4
    beq     a1, t5, 40f
    nop
    dli     t5, 0x8
    //temp code, MEM size >= 4G, use 4G only
    bgeu    a1, t5, 80f
    nop
    b       70f  //error condition
    nop
10:     
    //ddr_512MB_MC0:    
    PRINTSTR("MC0 pace open : 0x9000_0000 - 0x9FFF_FFFF\r\n")

    GET_NODE_ID_a0;
    XBAR_CONFIG_NODE_a0(0x20, \
                        0x0000000080000000, \
                        0xffffffffe0000000, \
                        0x00000000000000f0)
    b       2f
    nop
20:     
    //ddr_1GB_MC0:
    PRINTSTR("MC0 space open : 0x9000_0000 - 0xBFFF_FFFF\r\n")
    GET_NODE_ID_a0;
    XBAR_CONFIG_NODE_a0(0x20, \
                        0x0000000080000000, \
                        0xffffffffc0000000, \
                        0x00000000000000f0)        
    b       2f
    nop
40:     
    //ddr_2GB_MC0:        
    PRINTSTR("MC0 space open : 0x9000_0000 - 0xFFFF_FFFF\r\n")
    GET_NODE_ID_a0;
    XBAR_CONFIG_NODE_a0(0x20, \
                        0x0000000080000000, \
                        0xffffffff80000000, \
                        0x00000000000000f0)
    b       2f
    nop
80:
    //ddr_4GB_MC0:
    GET_NODE_ID_a0;
    PRINTSTR("MC0 space open : 0x9000_0000 - 0x1_7FFF_FFFF\r\n")
    XBAR_CONFIG_NODE_a0(0x20, \
                        0x0000000080000000, \
                        0xFFFFFFFF80000000, \
                        0x00000000000000F0)
    XBAR_CONFIG_NODE_a0(0x28, \
                        0x0000000100000000, \
                        0xFFFFFFFF80000000, \
                        0x00000000000000F0)

    b       2f
    nop
6:
    //use MC1 only, currently only support 512M, 1G, 2G, 3G, 4G
    move    a1, t5
    dli     t5, 0x1
    beq     a1, t5, 10f
    nop
    dli     t5, 0x2
    beq     a1, t5, 20f
    nop
    dli     t5, 0x4
    beq     a1, t5, 40f
    nop
    dli     t5, 0x8
    //temp code, MEM size >= 4G, use 4G only
    bgeu    a1, t5, 80f
    nop
    b       70f  //error condition
    nop
10:     
    //ddr_512MB_MC1:        
    PRINTSTR("MC1 pace open : 0x9000_0000 - 0x9FFF_FFFF\r\n")
    GET_NODE_ID_a0;
    XBAR_CONFIG_NODE_a0(0x20, \
                        0x0000000080000000, \
                        0xffffffffe0000000, \
                        0x00000000000000f1)
    b       3f
    nop
20:    
    //ddr_1GB_MC1:    
    PRINTSTR("MC1 space open : 0x9000_0000 - 0xBFFF_FFFF\r\n")
    GET_NODE_ID_a0;
    XBAR_CONFIG_NODE_a0(0x20, \
                        0x0000000080000000, \
                        0xffffffffc0000000, \
                        0x00000000000000f1)        
    b       3f
    nop
40:     
    //ddr_2GB_MC1:        
    PRINTSTR("MC1 space open : 0x9000_0000 - 0xFFFF_FFFF\r\n")
    GET_NODE_ID_a0;
    XBAR_CONFIG_NODE_a0(0x20, \
                        0x0000000080000000, \
                        0xffffffff80000000, \
                        0x00000000000000f1)
    b       3f
    nop
80:     
    //ddr_4GB_MC1:
    GET_NODE_ID_a0;
    PRINTSTR("MC1 space open : 0x9000_0000 - 0x1_7FFF_FFFF\r\n")
    XBAR_CONFIG_NODE_a0(0x20, \
                        0x0000000080000000, \
                        0xFFFFFFFF80000000, \
                        0x00000000000000F1)
    XBAR_CONFIG_NODE_a0(0x28, \
                        0x0000000100000000, \
                        0xFFFFFFFF80000000, \
                        0x00000000000000F1)

3:
    PRINTSTR("PCI DMA SPACE reassigned to MC1\r\n")
    dli     t1, 0x00000000000000f1
    sd      t1, 0x180(t0)

    b       2f
    nop
        
4:  
    //use MC0&MC1
    /* ONLY 1GB, 2GB and 4G has INTERLEAVE mode now */
    GET_MC0_MEMSIZE
    move    t5, a1
    GET_MC1_MEMSIZE
    bne     a1, t5, 3f
    nop

    //universal set
    GET_NODE_ID_a0;
    //config mask  
    dli     a2, 0xffffffffffffffff
    dsll    a2, a2, 28
1:
    dsll    a2, a2, 1
    dsrl    a1, a1, 1
    bnez    a1, 1b
    nop
    sd      a2, 0x60(t0)
    sd      a2, 0x68(t0)
    sd      a2, 0x70(t0)
    sd      a2, 0x78(t0)
    //config base
    GET_MC0_MEMSIZE
    dsll    a2, a1, 30  //a2=a1*512M*2
    or      a2, a2, a0
    sd      a2, 0x20(t0)
    sd      a2, 0x28(t0)
    dsll    a1, a1, 29  //a1=a1*512M
    or      a2, a2, a1  //a2=a1+a2
    or      a2, a2, a0
    sd      a2, 0x30(t0)
    sd      a2, 0x38(t0)
    PRINTSTR("DDR all space open\r\n")


#define PER_CONTROLLER_512M 0x1
#define PER_CONTROLLER_1G	0x2
#define PER_CONTROLLER_2G	0x4

    GET_MC0_MEMSIZE
    beq     a1, PER_CONTROLLER_1G, interleave_1g
    nop
    beq     a1, PER_CONTROLLER_2G, interleave_2g
    nop
interleave_1g:
#ifdef NO_INTERLEAVE
    PRINTSTR("DDR space open : 0x80000000 - 0xFFFFFFFF\r\n")
    GET_NODE_ID_a0;
    XBAR_CONFIG_NODE_a0(0x20, \
                        0x0000000080000000, \
                        0xffffffffc0000000, \
                        0x00000000000000f0)
    XBAR_CONFIG_NODE_a0(0x38, \
                        0x00000000c0000000, \
                        0xffffffffc0000000, \
                        0x00000000000000f1)
    PRINTSTR("PCI DDR space open : 0x80000000 - 0xFFFFFFFF\r\n")
    // Disable the default Windows
    dli     t1, 0x0
    sd      t1, 0x180(t0)

    // Enable others
    GET_NODE_ID_a0;
    XBAR_CONFIG_NODE_a0(0x120, \
                        0x0000000080000000, \
                        0xffffffffc0000000, \
                        0x00000000000000f0)
    XBAR_CONFIG_NODE_a0(0x138, \
                        0x00000000c0000000, \
                        0xffffffffc0000000, \
                        0x00000000000000f1)
#else
    #ifdef INTERLEAVE_27
    PRINTSTR("DDR Interleave space open : 0x80000000 - 0xFFFFFFFF\r\n")
    PRINTSTR("DDR Interleave using Bit 27\r\n")
    GET_NODE_ID_a0;
    XBAR_CONFIG_NODE_a0(0x20, \
                        0x0000000080000000, \
                        0xffffffffc8000000, \
                        0x00000000000000f0)
    XBAR_CONFIG_NODE_a0(0x28, \
                        0x0000000088000000, \
                        0xffffffffc8000000, \
                        0x00000000000000f1)
    XBAR_CONFIG_NODE_a0(0x30, \
                        0x00000000c0000000, \
                        0xffffffffc8000000, \
                        0x00000000080000f0)
    XBAR_CONFIG_NODE_a0(0x38, \
                        0x00000000c8000000, \
                        0xffffffffc8000000, \
                        0x00000000080000f1)

    #else
        #ifdef INTERLEAVE_13
        GET_MC0_MEMSIZE
        //dli     t5, 0x2
        beq     a1, PER_CONTROLLER_2G, INTERLEAVE_2GB
        nop
        beq     a1, PER_CONTROLLER_512M, INTERLEAVE_512MB
        nop

INTERLEAVE_1GB:  
        // 1Gx2
	    PRINTSTR("DDR Interleave space open : 0x9000_0000 - 0xFFFF_FFFF\r\n")
        PRINTSTR("DDR Interleave using Bit 13\r\n")
        GET_NODE_ID_a0;
        XBAR_CONFIG_NODE_a0(0x20, \
                            0x0000000080000000, \
                            0xffffffffc0002000, \
                            0x00000000000000f0)
        XBAR_CONFIG_NODE_a0(0x28, \
                            0x0000000080002000, \
                            0xffffffffc0002000, \
                            0x00000000000000f1)
        XBAR_CONFIG_NODE_a0(0x30, \
                            0x00000000c0000000, \
                            0xffffffffc0002000, \
                            0x00000000000020f0)
        XBAR_CONFIG_NODE_a0(0x38, \
                            0x00000000c0002000, \
                            0xffffffffc0002000, \
                            0x00000000000020f1)
	    b 1f
INTERLEAVE_512MB:  
        // 512MBx2
	    PRINTSTR("DDR Interleave space open : 0x9000_0000 - 0xBFFF_FFFF\r\n")
	    PRINTSTR("DDR Interleave using Bit 13\r\n")
        GET_NODE_ID_a0;
        XBAR_CONFIG_NODE_a0(0x20, \
                            0x0000000080000000, \
                            0xffffffffe0002000, \
                            0x00000000000000f0)
        XBAR_CONFIG_NODE_a0(0x28, \
                            0x0000000080002000, \
                            0xffffffffe0002000, \
                            0x00000000000000f1)
        XBAR_CONFIG_NODE_a0(0x30, \
                            0x00000000a0000000, \
                            0xffffffffe0002000, \
                            0x00000000000020f0)
        XBAR_CONFIG_NODE_a0(0x38, \
                            0x00000000a0002000, \
                            0xffffffffe0002000, \
                            0x00000000000020f1)
	    b 1f

INTERLEAVE_2GB: // 2Gx2
	    PRINTSTR("DDR Interleave space open : 0x9000_0000 - 0x1_7FFF_FFFF\r\n")
	    PRINTSTR("DDR Interleave using Bit 13\r\n")
	    GET_NODE_ID_a0;
        XBAR_CONFIG_NODE_a0(0x20, \
                            0x0000000080000000, \
                            0xffffffff80002000, \
                            0x00000000000000f0)
        XBAR_CONFIG_NODE_a0(0x28, \
                            0x0000000080002000, \
                            0xffffffff80002000, \
                            0x00000000000000f1)
        XBAR_CONFIG_NODE_a0(0x30, \
                            0x0000000100000000, \
                            0xffffffff80002000, \
                            0x00000000000020f0)
        XBAR_CONFIG_NODE_a0(0x38, \
                            0x0000000100002000, \
                            0xffffffff80002000, \
                            0x00000000000020f1)
	    b 1f

1:
        #else
            #ifdef INTERLEAVE_12
	        PRINTSTR("DDR Interleave space open : 0x00000000 - 0x0FFFFFFF\r\n")
	        PRINTSTR("DDR Interleave using Bit 12\r\n")
            GET_NODE_ID_a0;
            XBAR_CONFIG_NODE_a0(0x10, \
                                0x0000000000000000, \
                                0xfffffffff0001000, \
                                0x00000000000000f0)
            XBAR_CONFIG_NODE_a0(0x18, \
                                0x0000000000001000, \
                                0xfffffffff0001000, \
                                0x00000000000000f1)

            #else
                #ifdef INTERLEAVE_11
	            PRINTSTR("DDR Interleave space open : 0x80000000 - 0xFFFFFFFF\r\n")
	            PRINTSTR("DDR Interleave using Bit 11\r\n")
                GET_NODE_ID_a0;
                XBAR_CONFIG_NODE_a0(0x20, \
                                    0x0000000080000000, \
                                    0xffffffffc0000800, \
                                    0x00000000000000f0)
                XBAR_CONFIG_NODE_a0(0x28, \
                                    0x0000000080000800, \
                                    0xffffffffc0000800, \
                                    0x00000000000000f1)
                XBAR_CONFIG_NODE_a0(0x30, \
                                    0x00000000c0000000, \
                                    0xffffffffc0000800, \
                                    0x00000000000008f0)
                XBAR_CONFIG_NODE_a0(0x38, \
                                    0x00000000c0000800, \
                                    0xffffffffc0000800, \
                                    0x00000000000008f1)
	            PRINTSTR("PCI DDR Interleave space open : 0x80000000 - 0xFFFFFFFF\r\n")
	            PRINTSTR("PCI DDR Interleave using Bit 11\r\n")

                // Disable the default Windows
                dli     t1, 0x0
                sd      t1, 0x180(t0)

                // Enable others
                GET_NODE_ID_a0;
                XBAR_CONFIG_NODE_a0(0x120, \
                                    0x0000000080000000, \
                                    0xffffffffc0000800, \
                                    0x00000000000000f0)
                XBAR_CONFIG_NODE_a0(0x128, \
                                    0x0000000080000800, \
                                    0xffffffffc0000800, \
                                    0x00000000000000f1)
                XBAR_CONFIG_NODE_a0(0x130, \
                                    0x00000000c0000000, \
                                    0xffffffffc0000800, \
                                    0x00000000000008f0)
                XBAR_CONFIG_NODE_a0(0x138, \
                                    0x00000000c0000800, \
                                    0xffffffffc0000800, \
                                    0x00000000000008f1)

                #else
	            PRINTSTR("DDR Interleave space open : 0x80000000 - 0xFFFFFFFF\r\n")
                GET_NODE_ID_a0;
                XBAR_CONFIG_NODE_a0(0x20, \
                                    0x0000000080000000, \
                                    0xffffffffc0000400, \
                                    0x00000000000000f0)
                XBAR_CONFIG_NODE_a0(0x28, \
                                    0x0000000080000400, \
                                    0xffffffffc0000400, \
                                    0x00000000000000f1)
                XBAR_CONFIG_NODE_a0(0x30, \
                                    0x00000000c0000000, \
                                    0xffffffffc0000400, \
                                    0x00000000000004f0)
                XBAR_CONFIG_NODE_a0(0x38, \
                                    0x00000000c0000400, \
                                    0xffffffffc0000400, \
                                    0x00000000000004f1)

                #endif
            #endif
        #endif
    #endif
#endif
    b  2f;
    nop

interleave_2g:
#ifdef NO_INTERLEAVE
    PRINTSTR("DDR space open : 0x100000000 - 0x1FFFFFFFF\r\n")
    GET_NODE_ID_a0;
    XBAR_CONFIG_NODE_a0(0x20, \
                        0x0000000100000000, \
                        0xffffffff80000000, \
                        0x00000000000000f0)
    XBAR_CONFIG_NODE_a0(0x38, \
                        0x0000000180000000, \
                        0xffffffff80000000, \
                        0x00000000000000f1)
    PRINTSTR("PCI DDR space open : 0x100000000 - 0x1FFFFFFFF\r\n")

    // Disable the default Windows
    dli     t1, 0x0
    sd      t1, 0x180(t0)

    // Enable others
    GET_NODE_ID_a0;
    XBAR_CONFIG_NODE_a0(0x120, \
                        0x0000000100000000, \
                        0xffffffff80000000, \
                        0x00000000000000f0)
    XBAR_CONFIG_NODE_a0(0x138, \
                        0x0000000180000000, \
                        0xffffffff80000000, \
                        0x00000000000000f1)
#else
    #ifdef INTERLEAVE_27
    PRINTSTR("DDR Interleave space open : 0x100000000 - 0x1FFFFFFFF\r\n")
    PRINTSTR("DDR Interleave using Bit 27\r\n")
    GET_NODE_ID_a0;
    XBAR_CONFIG_NODE_a0(0x20, \
                        0x0000000100000000, \
                        0xffffffff88000000, \
                        0x00000000000000f0)
    XBAR_CONFIG_NODE_a0(0x28, \
                        0x0000000108000000, \
                        0xffffffff88000000, \
                        0x00000000000000f1)
    XBAR_CONFIG_NODE_a0(0x30, \
                        0x0000000180000000, \
                        0xffffffff88000000, \
                        0x00000000080000f0)
    XBAR_CONFIG_NODE_a0(0x38, \
                        0x0000000188000000, \
                        0xffffffff88000000, \
                        0x00000000080000f1)
    #else
        #ifdef INTERLEAVE_13
	    PRINTSTR("DDR Interleave space open : 0x100000000 - 0x1FFFFFFFF\r\n")
	    PRINTSTR("DDR Interleave using Bit 13\r\n")

        GET_NODE_ID_a0;
        XBAR_CONFIG_NODE_a0(0x20, \
                            0x0000000100000000, \
                            0xffffffff80002000, \
                            0x00000000000000f0)
        XBAR_CONFIG_NODE_a0(0x28, \
                            0x0000000100002000, \
                            0xffffffff80002000, \
                            0x00000000000000f1)
        XBAR_CONFIG_NODE_a0(0x30, \
                            0x0000000180000000, \
                            0xffffffff80002000, \
                            0x00000000000020f0)
        XBAR_CONFIG_NODE_a0(0x38, \
                            0x0000000180002000, \
                            0xffffffff80002000, \
                            0x00000000000020f1)
        #else
            #ifdef INTERLEAVE_12
	        PRINTSTR("DDR Interleave space open : 0x100000000 - 0x1FFFFFFFF\r\n")
	        PRINTSTR("DDR Interleave using Bit 12\r\n")

            GET_NODE_ID_a0;
            XBAR_CONFIG_NODE_a0(0x20, \
                                0x0000000100000000, \
                                0xffffffff80001000, \
                                0x00000000000000f0)
            XBAR_CONFIG_NODE_a0(0x28, \
                                0x0000000100001000, \
                                0xffffffff80001000, \
                                0x00000000000000f1)
            XBAR_CONFIG_NODE_a0(0x30, \
                                0x0000000180000000, \
                                0xffffffff80001000, \
                                0x00000000000010f0)
            XBAR_CONFIG_NODE_a0(0x38, \
                                0x0000000180001000, \
                                0xffffffff80001000, \
                                0x00000000000010f1)
            #else
                #ifdef INTERLEAVE_11
	            PRINTSTR("DDR Interleave space open : 0x100000000 - 0x1FFFFFFFF\r\n")
	            PRINTSTR("DDR Interleave using Bit 11\r\n")

                GET_NODE_ID_a0;
                XBAR_CONFIG_NODE_a0(0x20, \
                                    0x0000000100000000, \
                                    0xffffffff80000800, \
                                    0x00000000000000f0)            
                XBAR_CONFIG_NODE_a0(0x28, \
                                    0x0000000100000800, \
                                    0xffffffff80000800, \
                                    0x00000000000000f1)
                XBAR_CONFIG_NODE_a0(0x30, \
                                    0x0000000180000000, \
                                    0xffffffff80000800, \
                                    0x00000000000008f0)
                XBAR_CONFIG_NODE_a0(0x38, \
                                    0x0000000180000800, \
                                    0xffffffff80000800, \
                                    0x00000000000008f1)
	            PRINTSTR("PCI DDR Interleave space open : 0x80000000 - 0xFFFFFFFF\r\n")
	            PRINTSTR("PCI DDR Interleave using Bit 11\r\n")

                // Disable the default Windows
                dli     t1, 0x0
                sd      t1, 0x180(t0)

                // Enable others
                GET_NODE_ID_a0;
                XBAR_CONFIG_NODE_a0(0x120, \
                                    0x0000000100000000, \
                                    0xffffffff80000800, \
                                    0x00000000000000f0)
                XBAR_CONFIG_NODE_a0(0x128, \
                                    0x0000000100000800, \
                                    0xffffffff80000800, \
                                    0x00000000000000f1)
                XBAR_CONFIG_NODE_a0(0x130, \
                                    0x0000000180000000, \
                                    0xffffffff80000800, \
                                    0x00000000000008f0)
                XBAR_CONFIG_NODE_a0(0x138, \
                                    0x0000000180000800, \
                                    0xffffffff80000800, \
                                    0x00000000000008f1)
                #else
	            PRINTSTR("DDR Interleave space open : 0x100000000 - 0x1FFFFFFFF\r\n")

                GET_NODE_ID_a0;
                XBAR_CONFIG_NODE_a0(0x20, \
                                    0x0000000100000000, \
                                    0xffffffff80000400, \
                                    0x00000000000000f0)
                XBAR_CONFIG_NODE_a0(0x28, \
                                    0x0000000100000400, \
                                    0xffffffff80000400, \
                                    0x00000000000000f1)
                XBAR_CONFIG_NODE_a0(0x30, \
                                    0x0000000180000000, \
                                    0xffffffff80000400, \
                                    0x00000000000004f0)
                XBAR_CONFIG_NODE_a0(0x38, \
                                    0x0000000180000400, \
                                    0xffffffff80000400, \
                                    0x00000000000004f1)
                #endif
            #endif
        #endif
    #endif
#endif 
2:
    b       71f
    nop
3:      
    //error: MC0_MEMSIZE != MC1_MEMSIZE
    PRINTSTR("When use MC0&MC1 interleave mode,the two MC_MEMSIZE must be equal,the L2-Xbar will not be configured!!!\r\n")
    PRINTSTR("-------------------------------------------\r\n")
    b       71f
    nop
70:     
    //error: memory size not in support range
    PRINTSTR("The MEMSIZE is not supported, the L2-Xbar will not be configured!!!\r\n")
    PRINTSTR("-------------------------------------------\r\n")
71:
