#include <stdlib.h>
#include "bpfilter.h"

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/mbuf.h>
#include <sys/malloc.h>
#include <sys/kernel.h>
#include <sys/socket.h>
#include <sys/syslog.h>

#include <net/if.h>
#include <net/if_dl.h>
#include <net/if_media.h>
#include <net/if_types.h>

#ifdef INET
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/in_var.h>
#include <netinet/ip.h>
#endif

#if defined(__OpenBSD__)
#include <sys/ioctl.h>
#include <sys/errno.h>
#include <sys/device.h>
#include <netinet/if_ether.h>
#endif

#include <dev/mii/miivar.h>
#include <dev/mii/mii.h>

#include <dev/pci/pcivar.h>
#include <dev/pci/pcireg.h>

#include "rtl8168.h"

#define udelay(m)	delay(m)

/* Maximum events (Rx packets, etc.) to handle at each interrupt. */
static const int max_interrupt_work = 20;

/* Maximum number of multicast addresses to filter (vs. Rx-all-multicast).
 *    The RTL chips use a 64 element hash table based on the Ethernet CRC. */
static const int multicast_filter_limit = 32;

#define _R(NAME,MAC,RCR,MASK, JumFrameSz) \
	{ .name = NAME, .mcfg = MAC, .RCR_Cfg = RCR, .RxConfigMask = MASK, .jumbo_frame_sz = JumFrameSz }

static const struct {
	const char *name;
	u8 mcfg;
	u32 RCR_Cfg;
	u32 RxConfigMask;	/* Clears the bits supported by this chip */
	u32 jumbo_frame_sz;
} rtl_chip_info[] = {
	_R("RTL8168B/8111B",
	   CFG_METHOD_1,
	   (Reserved2_data << Reserved2_shift) | (RX_DMA_BURST << RxCfgDMAShift),
	   0xff7e1880,
	   Jumbo_Frame_4k),

	_R("RTL8168B/8111B",
	   CFG_METHOD_2,
	   (Reserved2_data << Reserved2_shift) | (RX_DMA_BURST << RxCfgDMAShift),
	   0xff7e1880,
	   Jumbo_Frame_4k),

	_R("RTL8168B/8111B",
	   CFG_METHOD_3,
	   (Reserved2_data << Reserved2_shift) | (RX_DMA_BURST << RxCfgDMAShift),
	   0xff7e1880,
	   Jumbo_Frame_4k),

	_R("RTL8168C/8111C",
	   CFG_METHOD_4, RxCfg_128_int_en | RxCfg_fet_multi_en | (RX_DMA_BURST << RxCfgDMAShift),
	   0xff7e1880,
	   Jumbo_Frame_6k),	

	_R("RTL8168C/8111C",
	   CFG_METHOD_5,
	   RxCfg_128_int_en | RxCfg_fet_multi_en | (RX_DMA_BURST << RxCfgDMAShift),
	   0xff7e1880,
	   Jumbo_Frame_6k),	

	_R("RTL8168C/8111C",
	   CFG_METHOD_6,
	   RxCfg_128_int_en | RxCfg_fet_multi_en | (RX_DMA_BURST << RxCfgDMAShift),
	   0xff7e1880,
	   Jumbo_Frame_6k),

	_R("RTL8168CP/8111CP",
	   CFG_METHOD_7,
	   RxCfg_128_int_en | RxCfg_fet_multi_en | (RX_DMA_BURST << RxCfgDMAShift),
	   0xff7e1880,
	   Jumbo_Frame_6k),

	_R("RTL8168CP/8111CP",
	   CFG_METHOD_8,
	   RxCfg_128_int_en | RxCfg_fet_multi_en | (RX_DMA_BURST << RxCfgDMAShift),
	   0xff7e1880,
	   Jumbo_Frame_6k),

	_R("RTL8168D/8111D",
	   CFG_METHOD_9,
	   RxCfg_128_int_en | (RX_DMA_BURST << RxCfgDMAShift),
	   0xff7e1880,
	   Jumbo_Frame_9k),

	_R("RTL8168D/8111D",
	   CFG_METHOD_10,
	   RxCfg_128_int_en | (RX_DMA_BURST << RxCfgDMAShift),
	   0xff7e1880,
	   Jumbo_Frame_9k),

	_R("RTL8168DP/8111DP",
	   CFG_METHOD_11,
	   RxCfg_128_int_en | (RX_DMA_BURST << RxCfgDMAShift),
	   0xff7e1880,
	   Jumbo_Frame_9k),
};
#undef _R

/* media options */
#define MAX_UNITS 8
static int speed[MAX_UNITS] = { -1, -1, -1, -1, -1, -1, -1, -1 };
static int duplex[MAX_UNITS] = { -1, -1, -1, -1, -1, -1, -1, -1 };
static int autoneg[MAX_UNITS] = { -1, -1, -1, -1, -1, -1, -1, -1 };

typedef struct rtl8168_private r8168;
static int r8168_match( struct device *, void *, void *);
static void r8168_attach(struct device *, struct device *, void *);
static void rtl8168_get_mac_version(struct rtl8168_private *, void *);
static void rtl8168_print_mac_version(struct rtl8168_private *);
static int rtl8168_set_speed_xmii(struct rtl8168_private *, u8, u16, u8);
static void rtl8168_phy_power_up (struct rtl8168_private *);
static void rtl8168_phy_power_down (struct rtl8168_private *);
static void mdio_write(struct rtl8168_private *, int , int );
static int mdio_read(struct rtl8168_private *, int);
static void rtl8168_gset_xmii(struct rtl8168_private * ,struct ethtool_cmd *);
static void rtl8168_xmii_reset_enable(struct rtl8168_private * );
static unsigned int rtl8168_xmii_reset_pending(struct rtl8168_private *);
static unsigned int rtl8168_xmii_link_ok(struct rtl8168_private * );
static unsigned rtl8168_try_msi(struct rtl8168_private *);
static void rtl8168_hw_phy_config(struct rtl8168_private *);
static u8 rtl8168_efuse_read(struct rtl8168_private *, u16);
static void rtl8168_link_option(int , u8 *, u16 *, u8 *);
static int rtl8168_set_speed(struct rtl8168_private *, u8 , u16 , u8);
static int rtl8168_ether_ioctl(struct ifnet *, unsigned long , caddr_t);
static void rtl8168_start_xmit(struct ifnet *);
static int rtl8168_xmit_frags(struct rtl8168_private *, struct ifnet *);
static int rtl8168_open(struct rtl8168_private *);
static void rtl8168_init_ring_indexes(struct rtl8168_private *tp);
static int rtl8168_init_ring(struct rtl8168_private *tp);
static u32 rtl8168_rx_fill(struct rtl8168_private *, u32, u32, int );
static u32 rtl8168_tx_fill(struct rtl8168_private *,u32, u32, u32);
static void rtl8168_rx_clear(struct rtl8168_private *);
static void rtl8168_tx_clear(struct rtl8168_private *);
static void rtl8168_free_tx(struct rtl8168_private *, unsigned char **, struct TxDesc * );
static void rtl8168_free_rx(struct rtl8168_private *,unsigned char *, struct RxDesc *);
static inline void rtl8168_make_unusable_by_asic(struct RxDesc *);
static inline void rtl8168_map_to_asic(struct RxDesc *, unsigned long, u32 , int);
static inline void rtl8168_mark_to_asic(struct RxDesc *, u32, int);
static inline void rtl8168_mark_as_last_descriptor(struct RxDesc *);
static void rtl8168_hw_reset(struct  rtl8168_private *);
static void rtl8168_hw_start(struct rtl8168_private *);
static void rtl8168_check_link_status(struct rtl8168_private *tp);
static void rtl8168dp_10mbps_gphy_para(struct rtl8168_private *tp);
static void rtl8168_set_rx_mode(struct rtl8168_private *tp);
static int rtl8168_csi_read(struct rtl8168_private * tp,int addr);
static void rtl8168_csi_write(struct rtl8168_private * tp,int addr,int value);
static void rtl8168_ephy_write(struct rtl8168_private *tp, int RegAddr, int value);
static u16 rtl8168_ephy_read(struct rtl8168_private * tp, int RegAddr);
int rtl8168_eri_read(struct rtl8168_private * tp, int addr, int len, int type);
int rtl8168_eri_write(struct rtl8168_private * tp, int addr, int len, int value, int type);
static int rtl8168_alloc_tx(unsigned char **tx_buffer, struct TxDesc *desc, int tx_buf_sz);
static int rtl8168_alloc_rx(unsigned char **rx_buffer, struct RxDesc *desc, int rx_buf_sz, int caller);
static void rtl8168_nic_reset(struct rtl8168_private *tp);
static void rtl8168_irq_mask_and_ack(struct rtl8168_private *tp);
static void rtl8168_dsm(struct rtl8168_private *tp, int dev_state);
static int rtl8168_interrupt(void *arg);
static void rtl8168_pcierr_interrupt(struct rtl8168_private *tp);
static int rtl8168_rx_interrupt(struct rtl8168_private *tp);
static void rtl8168_tx_interrupt(struct rtl8168_private *tp);
static int rtl8168_rx_csum(struct rtl8168_private *tp,struct RxDesc *desc);
static struct mbuf * getmbuf(struct rtl8168_private *tp);






static u16 rtl8168_intr_mask =
    SYSErr | LinkChg | RxDescUnavail | TxErr | TxOK | RxErr | RxOK;
static const u16 rtl8168_napi_event =
    RxOK | RxDescUnavail | RxFIFOOver | TxOK | TxErr;

struct cfattach rte_ca = {
	sizeof(r8168), r8168_match, r8168_attach, 
};

struct cfdriver rte_cd = {
	NULL, "rte", DV_IFNET,
};


#define PCI_VENDOR_ID_REALTEK       0x10ec

static int r8168_match(struct device *parent,void *match,void * aux)
{
	struct pci_attach_args *pa = aux;

	if(PCI_VENDOR(pa->pa_id) == PCI_VENDOR_ID_REALTEK) {
		if(PCI_PRODUCT(pa->pa_id) == 0x8168)
			return 1;
	}
	return 0;
}

static int
rtl8168_init_board(struct rtl8168_private *tp, struct pci_attach_args *pa)
{
	void *ioaddr = NULL;
	int offset, value;
	int rc = 0, i, acpi_idle_state = 0, pm_cap;
	pci_chipset_tag_t pc = pa->pa_pc;
	int tmp;
    int iobase, iosize;

	/* save power state before pci_enable_device overwrites it */
	pm_cap = pci_get_capability(pc, pa->pa_tag, PCI_CAP_PWRMGMT, &offset, &value);
	if (pm_cap) {
		u16 pwr_command;

		pwr_command = _pci_conf_read(pa->pa_tag, offset + PCI_PM_CTRL);
		acpi_idle_state = pwr_command & PCI_PM_CTRL_STATE_MASK;
	} else {
		printf("PowerManagement capability not found.\n");
	}
    //tp->cp_cmd = PCIMulRW | RxChkSum;  // by xiangy

	//pci_set_master(tp);
	tmp = _pci_conf_read(pa->pa_tag, PCI_COMMAND_STATUS_REG);

	if(!(tmp & PCI_COMMAND_MASTER_ENABLE)) {
		tmp |= PCI_COMMAND_MASTER_ENABLE;
		_pci_conf_write(pa->pa_tag, PCI_COMMAND_STATUS_REG, tmp);
	}

    //disable SERREN
	tmp = _pci_conf_read(pa->pa_tag, PCI_COMMAND_STATUS_REG);
	tmp &= ~PCI_COMMAND_SERR_ENABLE;
	_pci_conf_write(pa->pa_tag, PCI_COMMAND_STATUS_REG, tmp);

    if (pci_io_find(pc, pa->pa_tag, 0x10, &iobase, &iosize)) {
		printf(": can't find i/o space\n");
		return -1;
	}

	if (bus_space_map(pa->pa_iot, iobase, iosize, 0, &tp->sc_sh)) {
		printf(": can't map i/o space\n");
		return -1;
	}

	tp->sc_st = pa->pa_iot;
	tp->sc_pc = pc;
//	tp->cp_cmd = PCIMulRW | RxChkSum; //by xiangy

	tp->ioaddr = (unsigned long)tp->sc_sh;
#if 0
    //by liuqi
    status = RTL_R16(tp, IntrStatus);
    printf("before softreset -> %x ", status);
	/* Soft reset the chip. */
	RTL_W8(tp, ChipCmd, CmdReset);
 
    //by liuqi
    status = RTL_R16(tp, IntrStatus);
    printf("After softreset -> %x ", status);
	/* Check that the chip has finished the reset. */
	for (i = 1000; i > 0; i--) {
		if ((RTL_R8(tp, ChipCmd) & CmdReset) == 0)
			break;
		delay(100);
	}

	if (RTL_R8(tp, Config2) & 0x7) {
		RTL_W32(tp, 0x7c, 0x00ffffff);
		printf("PCI clock is 66M\n");
	}
	printf("r8168: Reset device %s \n", i ? "success": "timeout");
#endif

	/* Identify chip attached to board */
	rtl8168_get_mac_version(tp, ioaddr);

	rtl8168_print_mac_version(tp);
	for (i = sizeof(rtl_chip_info)/sizeof(rtl_chip_info[0]) - 1; i >= 0; i--) {
		if (tp->mcfg == rtl_chip_info[i].mcfg)
			break;
	}
	if (i < 0) {
		/* Unknown chip: assume array element #0, original RTL-8168 */
		printf( "unknown chip version, assuming %s\n", rtl_chip_info[0].name);
		i++;
	}
	tp->chipset = i;
#if 0
	RTL_W8(tp, Cfg9346, Cfg9346_Unlock);
    //RTL_W8( tp, Config0, RTL_R8(tp, Config0) | 0x15);
    RTL_W8(tp, Config1, RTL_R8( tp, Config1) | PMEnable);
    //RTL_W8( tp, Config3, RTL_R8(tp, Config3) | 0xA1);
    //RTL_W8( tp, Config4, RTL_R8(tp, Config4) | 0x80);
	RTL_W8(tp, Config5, (RTL_R8(tp, Config5) & PMEStatus)); 
	//RTL_W8(tp, Cfg9346, Cfg9346_Lock);
#endif
#if 0	
	rtl8168_get_phy_version(tp, ioaddr);
	rtl8168_print_phy_version(tp);
#endif

	return rc;

}

static struct rtl8168_private* myRTL[2] = {0, 0};
static int pn = 0;
static int cnt = 0;

void r8168_asic_down()
{
    int i;
    for (i = 0;i < 2; i++) {
        if (myRTL[i] != 0) {
	    RTL_W8(myRTL[i], ChipCmd, 0x00);	
    	    rtl8168_irq_mask_and_ack(myRTL[i]);
    	    RTL_R16(myRTL[i], CPlusCmd);
    	}
   }
}

static void
r8168_attach(struct device * parent, struct device * self, void *aux)
{
	struct rtl8168_private *tp = (struct rtl8168_private *)self;
	static int board_idx = -1;
	u8 autoneg, duplex;
	u16 speed;
	int i, rc;
	struct  ifnet *ifp;
	pci_intr_handle_t ih;
	const char *intrstr = NULL;
    u32 status;
	struct pci_attach_args *pa = aux;

	if (cnt >= 2)
		cnt = 0;
	pn += cnt;
	myRTL[pn] = tp;
	cnt++;

	/*
	rtl8168_config_reg();
	rtl8168_io_reg();
	*/

	tp->pa = (struct pci_attach_args *)aux;
	myRTL[pn] = tp;

	board_idx++;
    if_in_attach = 1;
    if_frequency = 0x13;
	rc = rtl8168_init_board(tp, aux);
	if (rc < 0)
		return;

    tp->set_speed = rtl8168_set_speed_xmii;
    tp->get_settings = rtl8168_gset_xmii;
    tp->phy_reset_enable = rtl8168_xmii_reset_enable;
    tp->phy_reset_pending = rtl8168_xmii_reset_pending;
    tp->link_ok = rtl8168_xmii_link_ok;

    RTL_W8(tp, Cfg9346, Cfg9346_Unlock);
    tp->features |= rtl8168_try_msi(tp);
    RTL_W8(tp, Cfg9346, Cfg9346_Lock);

	/* Read eeprom content */
	{
		int data;
		for (i = 0; i < 64; i++){
			data = read_eeprom(tp, i);

			printf("%x ", data);
			if ((i+1)>0 && (i+1)%4==0)
				printf("\n");
		}
		printf("\n");
	}
#ifdef SET_MAC_MANUAL
	if(pn == 0)
	{
		/* Set MAC ADDRESS */
		tp->dev_addr[0] = 0x00;
		tp->dev_addr[1] = 0x00;
		tp->dev_addr[2] = 0x22;
		tp->dev_addr[3] = 0x33;
		tp->dev_addr[4] = 0x44;
		tp->dev_addr[5] = 0x1f;
	}
	else
	{
		/* Set MAC ADDRESS */
		tp->dev_addr[0] = 0x00;
		tp->dev_addr[1] = 0x00;
		tp->dev_addr[2] = 0x22;
		tp->dev_addr[3] = 0x33;
		tp->dev_addr[4] = 0x44;
		tp->dev_addr[5] = 0x2f;
	}
#else
    /* Get MAC address by reading EEPROM */
	{
		u16 data = 0;

		data = read_eeprom(tp, 7);
		tp->dev_addr[0] = 0xff & data;
		tp->dev_addr[1] = 0xff & (data >> 8);
		data = read_eeprom(tp, 8);
		tp->dev_addr[2] = 0xff & data;
		tp->dev_addr[3] = 0xff & (data >> 8);
		data = read_eeprom(tp, 9);
		tp->dev_addr[4] = 0xff & data;
		tp->dev_addr[5] = 0xff & (data >> 8);

	}
#endif
		printf("MAC ADDRESS: ");
		printf("%02x:%02x:%02x:%02x:%02x:%02x\n", tp->dev_addr[0],tp->dev_addr[1],
				tp->dev_addr[2],tp->dev_addr[3],tp->dev_addr[4],tp->dev_addr[5]);

#ifdef CONFIG_R8168_NAPI
    RTL_NAPI_CONFIG(dev, tp, rtl8168_poll, R8168_NAPI_WEIGHT);
#endif

#ifdef CONFIG_R8168_VLAN
    dev->features |= NETIF_F_HW_VLAN_TX | NETIF_F_HW_VLAN_RX;
    dev->vlan_rx_register = rtl8168_vlan_rx_register;
    dev->vlan_rx_kill_vid = rtl8168_vlan_rx_kill_vid;
#endif

#ifdef CONFIG_NET_POLL_CONTROLLER
    dev->poll_controller = rtl8168_netpoll;
#endif

    tp->cp_cmd |= RxChkSum;
    tp->cp_cmd |= RTL_R16(tp, CPlusCmd);
    tp->intr_mask = rtl8168_intr_mask;
    tp->mmio_addr = tp->ioaddr;
    tp->max_jumbo_frame_size = rtl_chip_info[tp->chipset].jumbo_frame_sz;
	rtl8168_phy_power_up (tp);

    rtl8168_hw_phy_config(tp);

#define PCI_LATENCY_TIMER 0x0d
	_pci_conf_writen(pa->pa_tag, PCI_LATENCY_TIMER, 0x40, 1);
    rtl8168_link_option(board_idx, &autoneg, &speed, &duplex);
    rtl8168_set_speed(tp, autoneg, speed, duplex);
	ifp = &tp->arpcom.ac_if;
	bcopy(tp->dev_addr, tp->arpcom.ac_enaddr, sizeof(tp->arpcom.ac_enaddr));
	bcopy(tp->sc_dev.dv_xname, ifp->if_xname, IFNAMSIZ);

	ifp->if_softc = tp;
	ifp->if_ioctl = rtl8168_ether_ioctl;
	ifp->if_start = rtl8168_start_xmit;
	ifp->if_snd.ifq_maxlen = NUM_TX_DESC - 1;

	if_attach(ifp);
	ether_ifattach(ifp);
    #if 0
    rc = rtl8168_open(tp);
	if(rc) {
		printf("rtl8168_open: error code %d \n", rc);
		return;
	}
    #endif
    status = RTL_R16(tp, IntrStatus);
	if (pci_intr_map(pc, pa->pa_intrtag, pa->pa_intrpin, pa->pa_intrline, &ih)) {
		printf(": couldn't map interrupt\n");
		return;
	}

	intrstr = pci_intr_string(pc, ih);
    status = RTL_R16(tp, IntrStatus);
    tp->sc_ih = pci_intr_establish(pc, ih, IPL_NET, rtl8168_interrupt, tp, self->dv_xname);
#if 0
	if(tp->sc_ih == NULL){
             printf("error: could not establish interrupt !");
	      return;
	}else{
             printf("interrupt established!");
	}
#endif
    if_in_attach = 1;

	return ;
}

static void 
rtl8168_get_mac_version(struct rtl8168_private *tp,
			void *ioaddr)
{
	u32 reg,val32;
	u32 ICVerID;

	val32 = RTL_R32(tp, TxConfig);
	reg = val32 & 0x7c800000;
	ICVerID = val32 & 0x00700000;

	switch(reg) {
		case 0x30000000:
			tp->mcfg = CFG_METHOD_1;
			tp->efuse = EFUSE_NOT_SUPPORT;
			break;
		case 0x38000000:
			if(ICVerID == 0x00000000) {
				tp->mcfg = CFG_METHOD_2;
			} else if(ICVerID == 0x00500000) {
				tp->mcfg = CFG_METHOD_3;
			} else {
				tp->mcfg = CFG_METHOD_3;
			}
			tp->efuse = EFUSE_NOT_SUPPORT;
			break;
		case 0x3C000000:
			if(ICVerID == 0x00000000) {
				tp->mcfg = CFG_METHOD_4;
			} else if(ICVerID == 0x00200000) {
				tp->mcfg = CFG_METHOD_5;
			} else if(ICVerID == 0x00400000) {
				tp->mcfg = CFG_METHOD_6;
			} else {
				tp->mcfg = CFG_METHOD_6;
			}
			tp->efuse = EFUSE_NOT_SUPPORT;
			break;
		case 0x3C800000:
			if (ICVerID == 0x00100000){
				tp->mcfg = CFG_METHOD_7;
			} else if (ICVerID == 0x00300000){
				tp->mcfg = CFG_METHOD_8;
			} else {
				tp->mcfg = CFG_METHOD_8;
			}
			tp->efuse = EFUSE_NOT_SUPPORT;
			break;
		case 0x28000000:
			if(ICVerID == 0x00100000) {
				tp->mcfg = CFG_METHOD_9;
			} else if(ICVerID == 0x00300000) {
				tp->mcfg = CFG_METHOD_10;
			} else {
				tp->mcfg = CFG_METHOD_10;
			}
			tp->efuse = EFUSE_SUPPORT;
			break;
		case 0x28800000:
			tp->mcfg = CFG_METHOD_11;
			tp->efuse = EFUSE_SUPPORT;
			break;
		default:
			tp->mcfg = 0xFFFFFFFF;
			dprintk("unknown chip version (%x)\n",reg);
			break;
	}
	printf("mcfg is %x\n", tp->mcfg);
}


static void
rtl8168_print_mac_version(struct rtl8168_private *tp)
{
    int i;
    for (i = sizeof(rtl_chip_info)/sizeof(rtl_chip_info[0]) - 1; i >= 0; i--) {
        if (tp->mcfg == rtl_chip_info[i].mcfg){
            dprintk("mcfg == %s (%04d)\n", rtl_chip_info[i].name, rtl_chip_info[i].mcfg);
            return;
        }
    }

    dprintk("mac_version == Unknown\n");
}

static int 
rtl8168_set_speed_xmii(struct rtl8168_private *tp,
		       u8 autoneg, 
		       u16 speed, 
		       u8 duplex)
{
	int auto_nego = 0;
	int giga_ctrl = 0;
	int bmcr_true_force = 0;

	if ((speed != SPEED_1000) && 
	    (speed != SPEED_100) && 
	    (speed != SPEED_10)) {
		speed = SPEED_1000;
		duplex = DUPLEX_FULL;
	}
	if ((autoneg == AUTONEG_ENABLE) || (speed == SPEED_1000)) {
		/*n-way force*/
		if ((speed == SPEED_10) && (duplex == DUPLEX_HALF)) {
			auto_nego |= ADVERTISE_10HALF;
		} else if ((speed == SPEED_10) && (duplex == DUPLEX_FULL)) {
			auto_nego |= ADVERTISE_10HALF |
				     ADVERTISE_10FULL;
		} else if ((speed == SPEED_100) && (duplex == DUPLEX_HALF)) {
			auto_nego |= ADVERTISE_100HALF | 
				     ADVERTISE_10HALF | 
				     ADVERTISE_10FULL;
		} else if ((speed == SPEED_100) && (duplex == DUPLEX_FULL)) {
			auto_nego |= ADVERTISE_100HALF | 
				     ADVERTISE_100FULL |
				     ADVERTISE_10HALF | 
				     ADVERTISE_10FULL;
		} else if (speed == SPEED_1000) {
			giga_ctrl |= ADVERTISE_1000HALF | 
				     ADVERTISE_1000FULL;

			auto_nego |= ADVERTISE_100HALF | 
				     ADVERTISE_100FULL |
				     ADVERTISE_10HALF | 
				     ADVERTISE_10FULL;
		}

		//disable flow contorol
		auto_nego &= ~ADVERTISE_PAUSE_CAP;
		auto_nego &= ~ADVERTISE_PAUSE_ASYM;

		tp->phy_auto_nego_reg = auto_nego;
		tp->phy_1000_ctrl_reg = giga_ctrl;

		tp->autoneg = autoneg;
		tp->speed = speed;
		tp->duplex = duplex;

		//rtl8168_phy_power_up (tp);

		//delay(100);

		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, MII_ADVERTISE, auto_nego);
		mdio_write(tp, MII_CTRL1000, giga_ctrl);
		mdio_write(tp, MII_BMCR, BMCR_RESET | BMCR_ANENABLE | BMCR_ANRESTART);

		mdelay(100);
	} else {
		/*true force*/
#ifndef BMCR_SPEED100
#define BMCR_SPEED100	0x0040
#endif

#ifndef BMCR_SPEED10
#define BMCR_SPEED10	0x0000
#endif
		if ((speed == SPEED_10) && (duplex == DUPLEX_HALF)) {
			bmcr_true_force = BMCR_SPEED10;
		} else if ((speed == SPEED_10) && (duplex == DUPLEX_FULL)) {
			bmcr_true_force = BMCR_SPEED10 | BMCR_FULLDPLX;
		} else if ((speed == SPEED_100) && (duplex == DUPLEX_HALF)) {
			bmcr_true_force = BMCR_SPEED100;
		} else if ((speed == SPEED_100) && (duplex == DUPLEX_FULL)) {
			bmcr_true_force = BMCR_SPEED100 | BMCR_FULLDPLX;
		}

		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, MII_BMCR, bmcr_true_force);
	}

	if (tp->mcfg == CFG_METHOD_11) {
		if (speed == SPEED_10) {
			mdio_write(tp, 0x1F, 0x0000);
			mdio_write(tp, 0x10, 0x04EE);
		} else {
			mdio_write(tp, 0x1F, 0x0000);
			mdio_write(tp, 0x10, 0x01EE);
		}
	}
	return 0;
}

static void rtl8168_phy_power_up (struct rtl8168_private *tp)
{
	mdio_write(tp, 0x1F, 0x0000);
	switch (tp->mcfg)
	{
		case CFG_METHOD_14:
		case CFG_METHOD_15:
		case CFG_METHOD_16:
		case CFG_METHOD_17:
		case CFG_METHOD_18:
		case CFG_METHOD_19:
		case CFG_METHOD_20:
			break;
		default:
			mdio_write(tp, 0x0E, 0x0000);
			break;
	}
	mdio_write(tp, MII_BMCR, BMCR_ANENABLE);
}

static void rtl8168_phy_power_down (struct rtl8168_private *tp)
{
	mdio_write(tp, 0x1F, 0x0000);
	mdio_write(tp, 0x0E, 0x0200);
	mdio_write(tp, MII_BMCR, BMCR_PDOWN);
}

static void mdio_write(struct rtl8168_private *tp, int RegAddr, int value)
{
	int i;
	RTL_W32(tp, PHYAR, PHYAR_Write | (RegAddr & PHYAR_Reg_Mask) << PHYAR_Reg_shift | (value & PHYAR_Data_Mask));

	for (i = 0; i < 10; i++) {
		/* Check if the RTL8168 has completed writing to the specified MII register */
		if (!(RTL_R32(tp, PHYAR) & PHYAR_Flag)) 
			break;
		delay(100);
	}
}

static int mdio_read(struct rtl8168_private * tp, int RegAddr)
{
	int i, value = -1;

	RTL_W32(tp, PHYAR, 
		PHYAR_Read | (RegAddr & PHYAR_Reg_Mask) << PHYAR_Reg_shift);

	for (i = 0; i < 10; i++) {
		/* Check if the RTL8168 has completed retrieving data from the specified MII register */
		if (RTL_R32(tp, PHYAR) & PHYAR_Flag) {
			value = (int) (RTL_R32(tp, PHYAR) & PHYAR_Data_Mask);
			break;
		}
		delay(100);
	}

	return value;
}
u32 OCP_read(struct rtl8168_private *tp, u8 mask, u16 Reg)
{
	//void __iomem *ioaddr = tp->mmio_addr;
	int	i;

	RTL_W32(tp,OCPAR, ((u32)mask&0xF)<<12 | (Reg&0xFFF));
	for (i=0;i<20;i++)
	{
		delay(100);
		if (RTL_R32(tp,OCPAR) & OCPAR_Flag)
			break;
	}
	return RTL_R32(tp,OCPDR);
}

void OCP_write(struct rtl8168_private *tp, u8 mask, u16 Reg, u32 data)
{
	//void __iomem *ioaddr = tp->mmio_addr;
	int	i;

	RTL_W32(tp,OCPDR, data);
	RTL_W32(tp,OCPAR, OCPAR_Flag | ((u32)mask&0xF)<<12 | (Reg&0xFFF));
	for (i=0;i<20;i++)
	{
		delay(100);
		if ( (RTL_R32(tp,OCPAR)&OCPAR_Flag) == 0)
			break;
	}
}

static void OOB_mutex_lock(struct rtl8168_private *tp)
{
	u32 reg;

	if (tp->mcfg == CFG_METHOD_13)
		reg = 0x04;
	else
		reg = 0x14;

	OCP_write(tp, 0x8, reg, 0x01000000);

	while (OCP_read(tp, 0xF, reg) & 0x00FF0000) {
		if (OCP_read(tp, 0xF, 0x09C) & 0x000000FF) {
			OCP_write(tp, 0x8, reg, 0x00000000);

			while (OCP_read(tp, 0xF, 0x09C) & 0x000000FF);

			OCP_write(tp, 0x8, reg, 0x01000000);
		}
	}
}

static void OOB_mutex_unlock(struct rtl8168_private *tp)
{
	u32 reg;

	if (tp->mcfg == CFG_METHOD_13)
		reg = 0x04;
	else
		reg = 0x14;

	OCP_write(tp, 0x1, 0x9C, 0x00000001);
	OCP_write(tp, 0x8, reg, 0x00000000);
}

void OOB_notify(struct rtl8168_private *tp, u8 cmd)
{
	//void __iomem *ioaddr = tp->mmio_addr;
	int	i;

	RTL_W8(tp,ERIDR, cmd);
	RTL_W32(tp,ERIAR, 0x800010E8);
	mdelay(2);
	for (i = 0; i < 5; i++) {
		delay(100);
		if ( !(RTL_R32(tp,ERIAR) & ERIAR_Flag))
			break;
	}

	OCP_write(tp, 0x1, 0x30, 0x00000001);
}

static int rtl8168_check_dash(struct rtl8168_private *tp)
{
	u32 reg;

	if (tp->mcfg == CFG_METHOD_13)
		reg = 0xb8;
	else
		reg = 0x10;

	if (OCP_read(tp, 0xF, reg) & 0x00008000)
		return 1;
	else
		return 0;
}

static void rtl8168_gset_xmii(struct rtl8168_private * tp,struct ethtool_cmd *cmd)
{
	u8 status;
	cmd->supported = SUPPORTED_10baseT_Half |
			 SUPPORTED_10baseT_Full |
			 SUPPORTED_100baseT_Half |
			 SUPPORTED_100baseT_Full |
			 SUPPORTED_1000baseT_Full |
			 SUPPORTED_Autoneg |
		         SUPPORTED_TP;

	cmd->autoneg = (mdio_read(tp, MII_BMCR) & BMCR_ANENABLE) ? 1 : 0;
	cmd->advertising = ADVERTISED_TP | ADVERTISED_Autoneg;

	if (tp->phy_auto_nego_reg & ADVERTISE_10HALF)
		cmd->advertising |= ADVERTISED_10baseT_Half;
	if (tp->phy_auto_nego_reg & ADVERTISE_10FULL)
		cmd->advertising |= ADVERTISED_10baseT_Full;
	if (tp->phy_auto_nego_reg & ADVERTISE_100HALF)
		cmd->advertising |= ADVERTISED_100baseT_Half;
	if (tp->phy_auto_nego_reg & ADVERTISE_100FULL)
		cmd->advertising |= ADVERTISED_100baseT_Full;
	if (tp->phy_1000_ctrl_reg & ADVERTISE_1000FULL)
		cmd->advertising |= ADVERTISED_1000baseT_Full;

	status = RTL_R8(tp, PHYstatus);

	if (status & _1000bpsF)
		cmd->speed = SPEED_1000;
	else if (status & _100bps)
		cmd->speed = SPEED_100;
	else if (status & _10bps)
		cmd->speed = SPEED_10;

	if (status & TxFlowCtrl)
		cmd->advertising |= ADVERTISED_Asym_Pause;

	if (status & RxFlowCtrl)
		cmd->advertising |= ADVERTISED_Pause;

	cmd->duplex = ((status & _1000bpsF) || (status & FullDup)) ?
		      DUPLEX_FULL : DUPLEX_HALF;

	tp->autoneg = cmd->autoneg; 
	tp->speed = cmd->speed; 
	tp->duplex = cmd->duplex; 
}

static void 
rtl8168_xmii_reset_enable(struct rtl8168_private * tp)
{
	int i;

	mdio_write(tp, 0x1f, 0x0000);
	mdio_write(tp, MII_BMCR, mdio_read(tp, MII_BMCR) | BMCR_RESET);

	for(i = 0; i < 2500; i++) {
		if(!(mdio_read(tp, MII_BMSR) & BMCR_RESET))
			return;

		delay(100);
	}
}

static unsigned int 
rtl8168_xmii_reset_pending(struct rtl8168_private *tp)
{
	unsigned int retval;

	mdio_write(tp, 0x1f, 0x0000);
	retval = mdio_read(tp, MII_BMCR) & BMCR_RESET;

	return retval;
}

static unsigned int 
rtl8168_xmii_link_ok(struct rtl8168_private * tp)
{
	mdio_write(tp, 0x1f, 0x0000);

	return RTL_R8(tp, PHYstatus) & LinkStatus;
}

/* Cfg9346_Unlock assumed. */
static unsigned rtl8168_try_msi(struct rtl8168_private *tp)
{
	return 0;	
}

static void rtl8168_hw_phy_config(struct rtl8168_private *tp)
{
	unsigned int gphy_val,i;
    unsigned short subsystem_vendor;
    unsigned short subsystem_device;
    unsigned int subsystem_id;

    subsystem_id = pci_conf_read(0,tp->pa->pa_tag,PCI_SUBSYS_ID_REG);
    subsystem_vendor = subsystem_id&0xffff;
	subsystem_device = subsystem_id>>16;


	if (tp->mcfg == CFG_METHOD_1) {
		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x0B, 0x94B0);

		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x12, 0x6096);
		mdio_write(tp, 0x1F, 0x0000);

		mdio_write(tp, 0x0D, 0xF8A0);
	} else if (tp->mcfg == CFG_METHOD_2) {
		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x0B, 0x94B0);

		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x12, 0x6096);

		mdio_write(tp, 0x1F, 0x0000);
	} else if (tp->mcfg == CFG_METHOD_3) {
		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x0B, 0x94B0);

		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x12, 0x6096);

		mdio_write(tp, 0x1F, 0x0000);
	} else if (tp->mcfg == CFG_METHOD_4) {
		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x12, 0x2300);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x16, 0x000A);
		mdio_write(tp, 0x1F, 0x0000);

		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x12, 0xC096);
		mdio_write(tp, 0x1F, 0x0000);

		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x00, 0x88DE);
		mdio_write(tp, 0x01, 0x82B1);
		mdio_write(tp, 0x1F, 0x0000);
	
		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x08, 0x9E30);
		mdio_write(tp, 0x09, 0x01F0);
		mdio_write(tp, 0x1F, 0x0000);

		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x0A, 0x5500);
		mdio_write(tp, 0x1F, 0x0000);

		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x03, 0x7002);
		mdio_write(tp, 0x1F, 0x0000);

		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x0C, 0x00C8);
		mdio_write(tp, 0x1F, 0x0000);

		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x14, mdio_read(tp, 0x14) | (1 << 5));
		mdio_write(tp, 0x0D, mdio_read(tp, 0x0D) & ~(1 << 5));
	} else if (tp->mcfg == CFG_METHOD_5) {
		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x12, 0x2300);
		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x16, 0x0F0A);
		mdio_write(tp, 0x1F, 0x0000);

		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x00, 0x88DE);
		mdio_write(tp, 0x01, 0x82B1);
		mdio_write(tp, 0x1F, 0x0000);

		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x0C, 0x7EB8);
		mdio_write(tp, 0x1F, 0x0000);

		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x06, 0x0761);
		mdio_write(tp, 0x1F, 0x0000);

		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x03, 0x802F);
		mdio_write(tp, 0x02, 0x4F02);
		mdio_write(tp, 0x01, 0x0409);
		mdio_write(tp, 0x00, 0xF099);
		mdio_write(tp, 0x04, 0x9800);
		mdio_write(tp, 0x04, 0x9000);
		mdio_write(tp, 0x1F, 0x0000);

		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x16, mdio_read(tp, 0x16) | (1 << 0));

		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x14, mdio_read(tp, 0x14) | (1 << 5));
		mdio_write(tp, 0x0D, mdio_read(tp, 0x0D) & ~(1 << 5));

		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x1D, 0x3D98);
		mdio_write(tp, 0x1F, 0x0000);

		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x17, 0x0CC0);
		mdio_write(tp, 0x1F, 0x0000);
	} else if (tp->mcfg == CFG_METHOD_6) {
		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x12, 0x2300);
		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x16, 0x0F0A);
		mdio_write(tp, 0x1F, 0x0000);

		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x00, 0x88DE);
		mdio_write(tp, 0x01, 0x82B1);
		mdio_write(tp, 0x1F, 0x0000);

		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x0C, 0x7EB8);
		mdio_write(tp, 0x1F, 0x0000);

		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x06, 0x5461);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x06, 0x5461);
		mdio_write(tp, 0x1F, 0x0000);

		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x16, mdio_read(tp, 0x16) | (1 << 0));

		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x14, mdio_read(tp, 0x14) | (1 << 5));
		mdio_write(tp, 0x0D, mdio_read(tp, 0x0D) & ~(1 << 5));

		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x1D, 0x3D98);
		mdio_write(tp, 0x1F, 0x0000);

		mdio_write(tp, 0x1f, 0x0001);
		mdio_write(tp, 0x17, 0x0CC0);
		mdio_write(tp, 0x1F, 0x0000);
	} else if (tp->mcfg == CFG_METHOD_7) {
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x14, mdio_read(tp, 0x14) | (1 << 5));
		mdio_write(tp, 0x0D, mdio_read(tp, 0x0D) & ~(1 << 5));

		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x1D, 0x3D98);

		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x14, 0xCAA3);
		mdio_write(tp, 0x1C, 0x000A);
		mdio_write(tp, 0x18, 0x65D0);
		
		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x17, 0xB580);
		mdio_write(tp, 0x18, 0xFF54);
		mdio_write(tp, 0x19, 0x3954);

		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x0D, 0x310C);
		mdio_write(tp, 0x0E, 0x310C);
		mdio_write(tp, 0x0F, 0x311C);
		mdio_write(tp, 0x06, 0x0761);

		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x18, 0xFF55);
		mdio_write(tp, 0x19, 0x3955);
		mdio_write(tp, 0x18, 0xFF54);
		mdio_write(tp, 0x19, 0x3954);

		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x17, 0x0CC0);

		mdio_write(tp, 0x1F, 0x0000);
	} else if (tp->mcfg == CFG_METHOD_8) {
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x14, mdio_read(tp, 0x14) | (1 << 5));
		mdio_write(tp, 0x0D, mdio_read(tp, 0x0D) & ~(1 << 5));

		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x14, 0xCAA3);
		mdio_write(tp, 0x1C, 0x000A);
		mdio_write(tp, 0x18, 0x65D0);
		
		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x17, 0xB580);
		mdio_write(tp, 0x18, 0xFF54);
		mdio_write(tp, 0x19, 0x3954);

		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x0D, 0x310C);
		mdio_write(tp, 0x0E, 0x310C);
		mdio_write(tp, 0x0F, 0x311C);
		mdio_write(tp, 0x06, 0x0761);

		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x18, 0xFF55);
		mdio_write(tp, 0x19, 0x3955);
		mdio_write(tp, 0x18, 0xFF54);
		mdio_write(tp, 0x19, 0x3954);

		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x17, 0x0CC0);

		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x16, mdio_read(tp, 0x16) | (1 << 0));

		mdio_write(tp, 0x1F, 0x0000);
	} else if (tp->mcfg == CFG_METHOD_9) {
		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x06, 0x4064);
		mdio_write(tp, 0x07, 0x2863);
		mdio_write(tp, 0x08, 0x059C);
		mdio_write(tp, 0x09, 0x26B4);
		mdio_write(tp, 0x0A, 0x6A19);
		mdio_write(tp, 0x0B, 0xDCC8);
		mdio_write(tp, 0x10, 0xF06D);
		mdio_write(tp, 0x14, 0x7F68);
		mdio_write(tp, 0x18, 0x7FD9);
		mdio_write(tp, 0x1C, 0xF0FF);
		mdio_write(tp, 0x1D, 0x3D9C);
		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x12, 0xF49F);
		mdio_write(tp, 0x13, 0x070B);
		mdio_write(tp, 0x1A, 0x05AD);
		mdio_write(tp, 0x14, 0x94C0);

		mdio_write(tp, 0x1F, 0x0002);
		gphy_val = mdio_read(tp, 0x0B) & 0xFF00;
		gphy_val |= 0x10;
		mdio_write(tp, 0x0B, gphy_val);
		gphy_val = mdio_read(tp, 0x0C) & 0x00FF;
		gphy_val |= 0xA200;
		mdio_write(tp, 0x0C, gphy_val);

		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x06, 0x5561);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8332);
		mdio_write(tp, 0x06, 0x5561);
		
		if (rtl8168_efuse_read(tp, 0x01) == 0xb1) {
			mdio_write(tp, 0x1F, 0x0002);
			mdio_write(tp, 0x05, 0x669A);
			mdio_write(tp, 0x1F, 0x0005);
			mdio_write(tp, 0x05, 0x8330);
			mdio_write(tp, 0x06, 0x669A);

			mdio_write(tp, 0x1F, 0x0002);
			gphy_val = mdio_read(tp, 0x0D);
			if ((gphy_val & 0x00FF) != 0x006C) {
				gphy_val &= 0xFF00;
				mdio_write(tp, 0x1F, 0x0002);
				mdio_write(tp, 0x0D, gphy_val | 0x0065);
				mdio_write(tp, 0x0D, gphy_val | 0x0066);
				mdio_write(tp, 0x0D, gphy_val | 0x0067);
				mdio_write(tp, 0x0D, gphy_val | 0x0068);
				mdio_write(tp, 0x0D, gphy_val | 0x0069);
				mdio_write(tp, 0x0D, gphy_val | 0x006A);
				mdio_write(tp, 0x0D, gphy_val | 0x006B);
				mdio_write(tp, 0x0D, gphy_val | 0x006C);
			}
		} else {
			mdio_write(tp, 0x1F, 0x0002);
			mdio_write(tp, 0x05, 0x6662);
			mdio_write(tp, 0x1F, 0x0005);
			mdio_write(tp, 0x05, 0x8330);
			mdio_write(tp, 0x06, 0x6662);
		}

		mdio_write(tp, 0x1F, 0x0002);
		gphy_val = mdio_read(tp, 0x0D);
		gphy_val |= BIT_9;
		gphy_val |= BIT_8;
		mdio_write(tp, 0x0D, gphy_val);
		gphy_val = mdio_read(tp, 0x0F);
		gphy_val |= BIT_4;
		mdio_write(tp, 0x0F, gphy_val);

		mdio_write(tp, 0x1F, 0x0002);
		gphy_val = mdio_read(tp, 0x02);
		gphy_val &= ~BIT_10;
		gphy_val &= ~BIT_9;
		gphy_val |= BIT_8;
		mdio_write(tp, 0x02, gphy_val);
		gphy_val = mdio_read(tp, 0x03);
		gphy_val &= ~BIT_15;
		gphy_val &= ~BIT_14;
		gphy_val &= ~BIT_13;
		mdio_write(tp, 0x03, gphy_val);

		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x17, 0x0CC0);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x001B);
		if (mdio_read(tp, 0x06) == 0xBF00) {
		mdio_write(tp, 0x1F, 0x0005);
			mdio_write(tp, 0x05, 0xfff6);
			mdio_write(tp, 0x06, 0x0080);
		mdio_write(tp, 0x05, 0x8000);
		mdio_write(tp, 0x06, 0xF8F9);
		mdio_write(tp, 0x06, 0xFAEF);
		mdio_write(tp, 0x06, 0x59EE);
		mdio_write(tp, 0x06, 0xF8EA);
		mdio_write(tp, 0x06, 0x00EE);
		mdio_write(tp, 0x06, 0xF8EB);
		mdio_write(tp, 0x06, 0x00E0);
		mdio_write(tp, 0x06, 0xF87C);
		mdio_write(tp, 0x06, 0xE1F8);
		mdio_write(tp, 0x06, 0x7D59);
		mdio_write(tp, 0x06, 0x0FEF);
		mdio_write(tp, 0x06, 0x0139);
		mdio_write(tp, 0x06, 0x029E);
		mdio_write(tp, 0x06, 0x06EF);
		mdio_write(tp, 0x06, 0x1039);
		mdio_write(tp, 0x06, 0x089F);
		mdio_write(tp, 0x06, 0x2AEE);
		mdio_write(tp, 0x06, 0xF8EA);
		mdio_write(tp, 0x06, 0x00EE);
		mdio_write(tp, 0x06, 0xF8EB);
		mdio_write(tp, 0x06, 0x01E0);
		mdio_write(tp, 0x06, 0xF87C);
		mdio_write(tp, 0x06, 0xE1F8);
		mdio_write(tp, 0x06, 0x7D58);
		mdio_write(tp, 0x06, 0x409E);
		mdio_write(tp, 0x06, 0x0F39);
		mdio_write(tp, 0x06, 0x46AA);
		mdio_write(tp, 0x06, 0x0BBF);
		mdio_write(tp, 0x06, 0x8290);
		mdio_write(tp, 0x06, 0xD682);
		mdio_write(tp, 0x06, 0x9802);
		mdio_write(tp, 0x06, 0x014F);
		mdio_write(tp, 0x06, 0xAE09);
		mdio_write(tp, 0x06, 0xBF82);
		mdio_write(tp, 0x06, 0x98D6);
		mdio_write(tp, 0x06, 0x82A0);
		mdio_write(tp, 0x06, 0x0201);
		mdio_write(tp, 0x06, 0x4FEF);
		mdio_write(tp, 0x06, 0x95FE);
		mdio_write(tp, 0x06, 0xFDFC);
		mdio_write(tp, 0x06, 0x05F8);
		mdio_write(tp, 0x06, 0xF9FA);
		mdio_write(tp, 0x06, 0xEEF8);
		mdio_write(tp, 0x06, 0xEA00);
		mdio_write(tp, 0x06, 0xEEF8);
		mdio_write(tp, 0x06, 0xEB00);
		mdio_write(tp, 0x06, 0xE2F8);
		mdio_write(tp, 0x06, 0x7CE3);
		mdio_write(tp, 0x06, 0xF87D);
		mdio_write(tp, 0x06, 0xA511);
		mdio_write(tp, 0x06, 0x1112);
		mdio_write(tp, 0x06, 0xD240);
		mdio_write(tp, 0x06, 0xD644);
		mdio_write(tp, 0x06, 0x4402);
		mdio_write(tp, 0x06, 0x8217);
		mdio_write(tp, 0x06, 0xD2A0);
		mdio_write(tp, 0x06, 0xD6AA);
		mdio_write(tp, 0x06, 0xAA02);
		mdio_write(tp, 0x06, 0x8217);
		mdio_write(tp, 0x06, 0xAE0F);
		mdio_write(tp, 0x06, 0xA544);
		mdio_write(tp, 0x06, 0x4402);
		mdio_write(tp, 0x06, 0xAE4D);
		mdio_write(tp, 0x06, 0xA5AA);
		mdio_write(tp, 0x06, 0xAA02);
		mdio_write(tp, 0x06, 0xAE47);
		mdio_write(tp, 0x06, 0xAF82);
		mdio_write(tp, 0x06, 0x13EE);
		mdio_write(tp, 0x06, 0x834E);
		mdio_write(tp, 0x06, 0x00EE);
		mdio_write(tp, 0x06, 0x834D);
		mdio_write(tp, 0x06, 0x0FEE);
		mdio_write(tp, 0x06, 0x834C);
		mdio_write(tp, 0x06, 0x0FEE);
		mdio_write(tp, 0x06, 0x834F);
		mdio_write(tp, 0x06, 0x00EE);
		mdio_write(tp, 0x06, 0x8351);
		mdio_write(tp, 0x06, 0x00EE);
		mdio_write(tp, 0x06, 0x834A);
		mdio_write(tp, 0x06, 0xFFEE);
		mdio_write(tp, 0x06, 0x834B);
		mdio_write(tp, 0x06, 0xFFE0);
		mdio_write(tp, 0x06, 0x8330);
		mdio_write(tp, 0x06, 0xE183);
		mdio_write(tp, 0x06, 0x3158);
		mdio_write(tp, 0x06, 0xFEE4);
		mdio_write(tp, 0x06, 0xF88A);
		mdio_write(tp, 0x06, 0xE5F8);
		mdio_write(tp, 0x06, 0x8BE0);
		mdio_write(tp, 0x06, 0x8332);
		mdio_write(tp, 0x06, 0xE183);
		mdio_write(tp, 0x06, 0x3359);
		mdio_write(tp, 0x06, 0x0FE2);
		mdio_write(tp, 0x06, 0x834D);
		mdio_write(tp, 0x06, 0x0C24);
		mdio_write(tp, 0x06, 0x5AF0);
		mdio_write(tp, 0x06, 0x1E12);
		mdio_write(tp, 0x06, 0xE4F8);
		mdio_write(tp, 0x06, 0x8CE5);
		mdio_write(tp, 0x06, 0xF88D);
		mdio_write(tp, 0x06, 0xAF82);
		mdio_write(tp, 0x06, 0x13E0);
		mdio_write(tp, 0x06, 0x834F);
		mdio_write(tp, 0x06, 0x10E4);
		mdio_write(tp, 0x06, 0x834F);
		mdio_write(tp, 0x06, 0xE083);
		mdio_write(tp, 0x06, 0x4E78);
		mdio_write(tp, 0x06, 0x009F);
		mdio_write(tp, 0x06, 0x0AE0);
		mdio_write(tp, 0x06, 0x834F);
		mdio_write(tp, 0x06, 0xA010);
		mdio_write(tp, 0x06, 0xA5EE);
		mdio_write(tp, 0x06, 0x834E);
		mdio_write(tp, 0x06, 0x01E0);
		mdio_write(tp, 0x06, 0x834E);
		mdio_write(tp, 0x06, 0x7805);
		mdio_write(tp, 0x06, 0x9E9A);
		mdio_write(tp, 0x06, 0xE083);
		mdio_write(tp, 0x06, 0x4E78);
		mdio_write(tp, 0x06, 0x049E);
		mdio_write(tp, 0x06, 0x10E0);
		mdio_write(tp, 0x06, 0x834E);
		mdio_write(tp, 0x06, 0x7803);
		mdio_write(tp, 0x06, 0x9E0F);
		mdio_write(tp, 0x06, 0xE083);
		mdio_write(tp, 0x06, 0x4E78);
		mdio_write(tp, 0x06, 0x019E);
		mdio_write(tp, 0x06, 0x05AE);
		mdio_write(tp, 0x06, 0x0CAF);
		mdio_write(tp, 0x06, 0x81F8);
		mdio_write(tp, 0x06, 0xAF81);
		mdio_write(tp, 0x06, 0xA3AF);
		mdio_write(tp, 0x06, 0x81DC);
		mdio_write(tp, 0x06, 0xAF82);
		mdio_write(tp, 0x06, 0x13EE);
		mdio_write(tp, 0x06, 0x8348);
		mdio_write(tp, 0x06, 0x00EE);
		mdio_write(tp, 0x06, 0x8349);
		mdio_write(tp, 0x06, 0x00E0);
		mdio_write(tp, 0x06, 0x8351);
		mdio_write(tp, 0x06, 0x10E4);
		mdio_write(tp, 0x06, 0x8351);
		mdio_write(tp, 0x06, 0x5801);
		mdio_write(tp, 0x06, 0x9FEA);
		mdio_write(tp, 0x06, 0xD000);
		mdio_write(tp, 0x06, 0xD180);
		mdio_write(tp, 0x06, 0x1F66);
		mdio_write(tp, 0x06, 0xE2F8);
		mdio_write(tp, 0x06, 0xEAE3);
		mdio_write(tp, 0x06, 0xF8EB);
		mdio_write(tp, 0x06, 0x5AF8);
		mdio_write(tp, 0x06, 0x1E20);
		mdio_write(tp, 0x06, 0xE6F8);
		mdio_write(tp, 0x06, 0xEAE5);
		mdio_write(tp, 0x06, 0xF8EB);
		mdio_write(tp, 0x06, 0xD302);
		mdio_write(tp, 0x06, 0xB3FE);
		mdio_write(tp, 0x06, 0xE2F8);
		mdio_write(tp, 0x06, 0x7CEF);
		mdio_write(tp, 0x06, 0x325B);
		mdio_write(tp, 0x06, 0x80E3);
		mdio_write(tp, 0x06, 0xF87D);
		mdio_write(tp, 0x06, 0x9E03);
		mdio_write(tp, 0x06, 0x7DFF);
		mdio_write(tp, 0x06, 0xFF0D);
		mdio_write(tp, 0x06, 0x581C);
		mdio_write(tp, 0x06, 0x551A);
		mdio_write(tp, 0x06, 0x6511);
		mdio_write(tp, 0x06, 0xA190);
		mdio_write(tp, 0x06, 0xD3E2);
		mdio_write(tp, 0x06, 0x8348);
		mdio_write(tp, 0x06, 0xE383);
		mdio_write(tp, 0x06, 0x491B);
		mdio_write(tp, 0x06, 0x56AB);
		mdio_write(tp, 0x06, 0x08EF);
		mdio_write(tp, 0x06, 0x56E6);
		mdio_write(tp, 0x06, 0x8348);
		mdio_write(tp, 0x06, 0xE783);
		mdio_write(tp, 0x06, 0x4910);
		mdio_write(tp, 0x06, 0xD180);
		mdio_write(tp, 0x06, 0x1F66);
		mdio_write(tp, 0x06, 0xA004);
		mdio_write(tp, 0x06, 0xB9E2);
		mdio_write(tp, 0x06, 0x8348);
		mdio_write(tp, 0x06, 0xE383);
		mdio_write(tp, 0x06, 0x49EF);
		mdio_write(tp, 0x06, 0x65E2);
		mdio_write(tp, 0x06, 0x834A);
		mdio_write(tp, 0x06, 0xE383);
		mdio_write(tp, 0x06, 0x4B1B);
		mdio_write(tp, 0x06, 0x56AA);
		mdio_write(tp, 0x06, 0x0EEF);
		mdio_write(tp, 0x06, 0x56E6);
		mdio_write(tp, 0x06, 0x834A);
		mdio_write(tp, 0x06, 0xE783);
		mdio_write(tp, 0x06, 0x4BE2);
		mdio_write(tp, 0x06, 0x834D);
		mdio_write(tp, 0x06, 0xE683);
		mdio_write(tp, 0x06, 0x4CE0);
		mdio_write(tp, 0x06, 0x834D);
		mdio_write(tp, 0x06, 0xA000);
		mdio_write(tp, 0x06, 0x0CAF);
		mdio_write(tp, 0x06, 0x81DC);
		mdio_write(tp, 0x06, 0xE083);
		mdio_write(tp, 0x06, 0x4D10);
		mdio_write(tp, 0x06, 0xE483);
		mdio_write(tp, 0x06, 0x4DAE);
		mdio_write(tp, 0x06, 0x0480);
		mdio_write(tp, 0x06, 0xE483);
		mdio_write(tp, 0x06, 0x4DE0);
		mdio_write(tp, 0x06, 0x834E);
		mdio_write(tp, 0x06, 0x7803);
		mdio_write(tp, 0x06, 0x9E0B);
		mdio_write(tp, 0x06, 0xE083);
		mdio_write(tp, 0x06, 0x4E78);
		mdio_write(tp, 0x06, 0x049E);
		mdio_write(tp, 0x06, 0x04EE);
		mdio_write(tp, 0x06, 0x834E);
		mdio_write(tp, 0x06, 0x02E0);
		mdio_write(tp, 0x06, 0x8332);
		mdio_write(tp, 0x06, 0xE183);
		mdio_write(tp, 0x06, 0x3359);
		mdio_write(tp, 0x06, 0x0FE2);
		mdio_write(tp, 0x06, 0x834D);
		mdio_write(tp, 0x06, 0x0C24);
		mdio_write(tp, 0x06, 0x5AF0);
		mdio_write(tp, 0x06, 0x1E12);
		mdio_write(tp, 0x06, 0xE4F8);
		mdio_write(tp, 0x06, 0x8CE5);
		mdio_write(tp, 0x06, 0xF88D);
		mdio_write(tp, 0x06, 0xE083);
		mdio_write(tp, 0x06, 0x30E1);
		mdio_write(tp, 0x06, 0x8331);
		mdio_write(tp, 0x06, 0x6801);
		mdio_write(tp, 0x06, 0xE4F8);
		mdio_write(tp, 0x06, 0x8AE5);
		mdio_write(tp, 0x06, 0xF88B);
		mdio_write(tp, 0x06, 0xAE37);
		mdio_write(tp, 0x06, 0xEE83);
		mdio_write(tp, 0x06, 0x4E03);
		mdio_write(tp, 0x06, 0xE083);
		mdio_write(tp, 0x06, 0x4CE1);
		mdio_write(tp, 0x06, 0x834D);
		mdio_write(tp, 0x06, 0x1B01);
		mdio_write(tp, 0x06, 0x9E04);
		mdio_write(tp, 0x06, 0xAAA1);
		mdio_write(tp, 0x06, 0xAEA8);
		mdio_write(tp, 0x06, 0xEE83);
		mdio_write(tp, 0x06, 0x4E04);
		mdio_write(tp, 0x06, 0xEE83);
		mdio_write(tp, 0x06, 0x4F00);
		mdio_write(tp, 0x06, 0xAEAB);
		mdio_write(tp, 0x06, 0xE083);
		mdio_write(tp, 0x06, 0x4F78);
		mdio_write(tp, 0x06, 0x039F);
		mdio_write(tp, 0x06, 0x14EE);
		mdio_write(tp, 0x06, 0x834E);
		mdio_write(tp, 0x06, 0x05D2);
		mdio_write(tp, 0x06, 0x40D6);
		mdio_write(tp, 0x06, 0x5554);
		mdio_write(tp, 0x06, 0x0282);
		mdio_write(tp, 0x06, 0x17D2);
		mdio_write(tp, 0x06, 0xA0D6);
		mdio_write(tp, 0x06, 0xBA00);
		mdio_write(tp, 0x06, 0x0282);
		mdio_write(tp, 0x06, 0x17FE);
		mdio_write(tp, 0x06, 0xFDFC);
		mdio_write(tp, 0x06, 0x05F8);
		mdio_write(tp, 0x06, 0xE0F8);
		mdio_write(tp, 0x06, 0x60E1);
		mdio_write(tp, 0x06, 0xF861);
		mdio_write(tp, 0x06, 0x6802);
		mdio_write(tp, 0x06, 0xE4F8);
		mdio_write(tp, 0x06, 0x60E5);
		mdio_write(tp, 0x06, 0xF861);
		mdio_write(tp, 0x06, 0xE0F8);
		mdio_write(tp, 0x06, 0x48E1);
		mdio_write(tp, 0x06, 0xF849);
		mdio_write(tp, 0x06, 0x580F);
		mdio_write(tp, 0x06, 0x1E02);
		mdio_write(tp, 0x06, 0xE4F8);
		mdio_write(tp, 0x06, 0x48E5);
		mdio_write(tp, 0x06, 0xF849);
		mdio_write(tp, 0x06, 0xD000);
		mdio_write(tp, 0x06, 0x0282);
		mdio_write(tp, 0x06, 0x5BBF);
		mdio_write(tp, 0x06, 0x8350);
		mdio_write(tp, 0x06, 0xEF46);
		mdio_write(tp, 0x06, 0xDC19);
		mdio_write(tp, 0x06, 0xDDD0);
		mdio_write(tp, 0x06, 0x0102);
		mdio_write(tp, 0x06, 0x825B);
		mdio_write(tp, 0x06, 0x0282);
		mdio_write(tp, 0x06, 0x77E0);
		mdio_write(tp, 0x06, 0xF860);
		mdio_write(tp, 0x06, 0xE1F8);
		mdio_write(tp, 0x06, 0x6158);
		mdio_write(tp, 0x06, 0xFDE4);
		mdio_write(tp, 0x06, 0xF860);
		mdio_write(tp, 0x06, 0xE5F8);
		mdio_write(tp, 0x06, 0x61FC);
		mdio_write(tp, 0x06, 0x04F9);
		mdio_write(tp, 0x06, 0xFAFB);
		mdio_write(tp, 0x06, 0xC6BF);
		mdio_write(tp, 0x06, 0xF840);
		mdio_write(tp, 0x06, 0xBE83);
		mdio_write(tp, 0x06, 0x50A0);
		mdio_write(tp, 0x06, 0x0101);
		mdio_write(tp, 0x06, 0x071B);
		mdio_write(tp, 0x06, 0x89CF);
		mdio_write(tp, 0x06, 0xD208);
		mdio_write(tp, 0x06, 0xEBDB);
		mdio_write(tp, 0x06, 0x19B2);
		mdio_write(tp, 0x06, 0xFBFF);
		mdio_write(tp, 0x06, 0xFEFD);
		mdio_write(tp, 0x06, 0x04F8);
		mdio_write(tp, 0x06, 0xE0F8);
		mdio_write(tp, 0x06, 0x48E1);
		mdio_write(tp, 0x06, 0xF849);
		mdio_write(tp, 0x06, 0x6808);
		mdio_write(tp, 0x06, 0xE4F8);
		mdio_write(tp, 0x06, 0x48E5);
		mdio_write(tp, 0x06, 0xF849);
		mdio_write(tp, 0x06, 0x58F7);
		mdio_write(tp, 0x06, 0xE4F8);
		mdio_write(tp, 0x06, 0x48E5);
		mdio_write(tp, 0x06, 0xF849);
		mdio_write(tp, 0x06, 0xFC04);
		mdio_write(tp, 0x06, 0x4D20);
		mdio_write(tp, 0x06, 0x0002);
		mdio_write(tp, 0x06, 0x4E22);
		mdio_write(tp, 0x06, 0x0002);
		mdio_write(tp, 0x06, 0x4DDF);
		mdio_write(tp, 0x06, 0xFF01);
		mdio_write(tp, 0x06, 0x4EDD);
		mdio_write(tp, 0x06, 0xFF01);
			mdio_write(tp, 0x06, 0xf8fa);
			mdio_write(tp, 0x06, 0xfbef);
			mdio_write(tp, 0x06, 0x79bf);
			mdio_write(tp, 0x06, 0xf822);
			mdio_write(tp, 0x06, 0xd819);
			mdio_write(tp, 0x06, 0xd958);
			mdio_write(tp, 0x06, 0x849f);
			mdio_write(tp, 0x06, 0x09bf);
			mdio_write(tp, 0x06, 0x82be);
			mdio_write(tp, 0x06, 0xd682);
			mdio_write(tp, 0x06, 0xc602);
			mdio_write(tp, 0x06, 0x014f);
			mdio_write(tp, 0x06, 0xef97);
			mdio_write(tp, 0x06, 0xfffe);
			mdio_write(tp, 0x06, 0xfc05);
			mdio_write(tp, 0x06, 0x17ff);
			mdio_write(tp, 0x06, 0xfe01);
			mdio_write(tp, 0x06, 0x1700);
			mdio_write(tp, 0x06, 0x0102);
		mdio_write(tp, 0x05, 0x83D8);
		mdio_write(tp, 0x06, 0x8051);
			mdio_write(tp, 0x05, 0x83d6);
			mdio_write(tp, 0x06, 0x82a0);
			mdio_write(tp, 0x05, 0x83d4);
			mdio_write(tp, 0x06, 0x8000);
			mdio_write(tp, 0x02, 0x2010);
		mdio_write(tp, 0x03, 0xDC00);
			mdio_write(tp, 0x1f, 0x0000);
			mdio_write(tp, 0x0b, 0x0600);
			mdio_write(tp, 0x1f, 0x0005);
		mdio_write(tp, 0x05, 0xFFF6);
		mdio_write(tp, 0x06, 0x00FC);
		mdio_write(tp, 0x1F, 0x0000);
		}

		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x0D, 0xF880);
		mdio_write(tp, 0x1F, 0x0000);
	} else if (tp->mcfg == CFG_METHOD_10) {
		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x06, 0x4064);
		mdio_write(tp, 0x07, 0x2863);
		mdio_write(tp, 0x08, 0x059C);
		mdio_write(tp, 0x09, 0x26B4);
		mdio_write(tp, 0x0A, 0x6A19);
		mdio_write(tp, 0x0B, 0xDCC8);
		mdio_write(tp, 0x10, 0xF06D);
		mdio_write(tp, 0x14, 0x7F68);
		mdio_write(tp, 0x18, 0x7FD9);
		mdio_write(tp, 0x1C, 0xF0FF);
		mdio_write(tp, 0x1D, 0x3D9C);
		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x12, 0xF49F);
		mdio_write(tp, 0x13, 0x070B);
		mdio_write(tp, 0x1A, 0x05AD);
		mdio_write(tp, 0x14, 0x94C0);

		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x06, 0x5561);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8332);
		mdio_write(tp, 0x06, 0x5561);

		if (rtl8168_efuse_read(tp, 0x01) == 0xb1) {
			mdio_write(tp, 0x1F, 0x0002);
			mdio_write(tp, 0x05, 0x669A);
			mdio_write(tp, 0x1F, 0x0005);
			mdio_write(tp, 0x05, 0x8330);
			mdio_write(tp, 0x06, 0x669A);

			mdio_write(tp, 0x1F, 0x0002);
			gphy_val = mdio_read(tp, 0x0D);
			if ((gphy_val & 0x00FF) != 0x006C) {
				gphy_val &= 0xFF00;
				mdio_write(tp, 0x1F, 0x0002);
				mdio_write(tp, 0x0D, gphy_val | 0x0065);
				mdio_write(tp, 0x0D, gphy_val | 0x0066);
				mdio_write(tp, 0x0D, gphy_val | 0x0067);
				mdio_write(tp, 0x0D, gphy_val | 0x0068);
				mdio_write(tp, 0x0D, gphy_val | 0x0069);
				mdio_write(tp, 0x0D, gphy_val | 0x006A);
				mdio_write(tp, 0x0D, gphy_val | 0x006B);
				mdio_write(tp, 0x0D, gphy_val | 0x006C);
			}
		} else {
			mdio_write(tp, 0x1F, 0x0002);
			mdio_write(tp, 0x05, 0x2642);
			mdio_write(tp, 0x1F, 0x0005);
			mdio_write(tp, 0x05, 0x8330);
			mdio_write(tp, 0x06, 0x2642);
		}

		if (rtl8168_efuse_read(tp, 0x30) == 0x98) {
			mdio_write(tp, 0x1F, 0x0000);
			mdio_write(tp, 0x11, mdio_read(tp, 0x11) & ~BIT_1);
			mdio_write(tp, 0x1F, 0x0005);
			mdio_write(tp, 0x01, mdio_read(tp, 0x01) | BIT_9);
		} else if (rtl8168_efuse_read(tp, 0x30) == 0x90) {
			mdio_write(tp, 0x1F, 0x0005);
			mdio_write(tp, 0x01, mdio_read(tp, 0x01) & ~BIT_9);
			mdio_write(tp, 0x1F, 0x0000);
			mdio_write(tp, 0x16, 0x5101);
		}
		mdio_write(tp, 0x1F, 0x0002);
		gphy_val = mdio_read(tp, 0x02);
		gphy_val &= ~BIT_10;
		gphy_val &= ~BIT_9;
		gphy_val |= BIT_8;
		mdio_write(tp, 0x02, gphy_val);
		gphy_val = mdio_read(tp, 0x03);
		gphy_val &= ~BIT_15;
		gphy_val &= ~BIT_14;
		gphy_val &= ~BIT_13;
		mdio_write(tp, 0x03, gphy_val);

		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x17, 0x0CC0);

		mdio_write(tp, 0x1F, 0x0002);
		gphy_val = mdio_read(tp, 0x0F);
		gphy_val |= BIT_4;
		gphy_val |= BIT_2;
		gphy_val |= BIT_1;
		gphy_val |= BIT_0;
		mdio_write(tp, 0x0F, gphy_val);

		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x001B);
		if (mdio_read(tp, 0x06) == 0xB300) {
		mdio_write(tp, 0x1F, 0x0005);
			mdio_write(tp, 0x05, 0xfff6);
			mdio_write(tp, 0x06, 0x0080);
		mdio_write(tp, 0x05, 0x8000);
		mdio_write(tp, 0x06, 0xF8F9);
		mdio_write(tp, 0x06, 0xFAEE);
		mdio_write(tp, 0x06, 0xF8EA);
		mdio_write(tp, 0x06, 0x00EE);
		mdio_write(tp, 0x06, 0xF8EB);
		mdio_write(tp, 0x06, 0x00E2);
		mdio_write(tp, 0x06, 0xF87C);
		mdio_write(tp, 0x06, 0xE3F8);
		mdio_write(tp, 0x06, 0x7DA5);
		mdio_write(tp, 0x06, 0x1111);
		mdio_write(tp, 0x06, 0x12D2);
		mdio_write(tp, 0x06, 0x40D6);
		mdio_write(tp, 0x06, 0x4444);
		mdio_write(tp, 0x06, 0x0281);
		mdio_write(tp, 0x06, 0xC6D2);
		mdio_write(tp, 0x06, 0xA0D6);
		mdio_write(tp, 0x06, 0xAAAA);
		mdio_write(tp, 0x06, 0x0281);
		mdio_write(tp, 0x06, 0xC6AE);
		mdio_write(tp, 0x06, 0x0FA5);
		mdio_write(tp, 0x06, 0x4444);
		mdio_write(tp, 0x06, 0x02AE);
		mdio_write(tp, 0x06, 0x4DA5);
		mdio_write(tp, 0x06, 0xAAAA);
		mdio_write(tp, 0x06, 0x02AE);
		mdio_write(tp, 0x06, 0x47AF);
		mdio_write(tp, 0x06, 0x81C2);
		mdio_write(tp, 0x06, 0xEE83);
		mdio_write(tp, 0x06, 0x4E00);
		mdio_write(tp, 0x06, 0xEE83);
		mdio_write(tp, 0x06, 0x4D0F);
		mdio_write(tp, 0x06, 0xEE83);
		mdio_write(tp, 0x06, 0x4C0F);
		mdio_write(tp, 0x06, 0xEE83);
		mdio_write(tp, 0x06, 0x4F00);
		mdio_write(tp, 0x06, 0xEE83);
		mdio_write(tp, 0x06, 0x5100);
		mdio_write(tp, 0x06, 0xEE83);
		mdio_write(tp, 0x06, 0x4AFF);
		mdio_write(tp, 0x06, 0xEE83);
		mdio_write(tp, 0x06, 0x4BFF);
		mdio_write(tp, 0x06, 0xE083);
		mdio_write(tp, 0x06, 0x30E1);
		mdio_write(tp, 0x06, 0x8331);
		mdio_write(tp, 0x06, 0x58FE);
		mdio_write(tp, 0x06, 0xE4F8);
		mdio_write(tp, 0x06, 0x8AE5);
		mdio_write(tp, 0x06, 0xF88B);
		mdio_write(tp, 0x06, 0xE083);
		mdio_write(tp, 0x06, 0x32E1);
		mdio_write(tp, 0x06, 0x8333);
		mdio_write(tp, 0x06, 0x590F);
		mdio_write(tp, 0x06, 0xE283);
		mdio_write(tp, 0x06, 0x4D0C);
		mdio_write(tp, 0x06, 0x245A);
		mdio_write(tp, 0x06, 0xF01E);
		mdio_write(tp, 0x06, 0x12E4);
		mdio_write(tp, 0x06, 0xF88C);
		mdio_write(tp, 0x06, 0xE5F8);
		mdio_write(tp, 0x06, 0x8DAF);
		mdio_write(tp, 0x06, 0x81C2);
		mdio_write(tp, 0x06, 0xE083);
		mdio_write(tp, 0x06, 0x4F10);
		mdio_write(tp, 0x06, 0xE483);
		mdio_write(tp, 0x06, 0x4FE0);
		mdio_write(tp, 0x06, 0x834E);
		mdio_write(tp, 0x06, 0x7800);
		mdio_write(tp, 0x06, 0x9F0A);
		mdio_write(tp, 0x06, 0xE083);
		mdio_write(tp, 0x06, 0x4FA0);
		mdio_write(tp, 0x06, 0x10A5);
		mdio_write(tp, 0x06, 0xEE83);
		mdio_write(tp, 0x06, 0x4E01);
		mdio_write(tp, 0x06, 0xE083);
		mdio_write(tp, 0x06, 0x4E78);
		mdio_write(tp, 0x06, 0x059E);
		mdio_write(tp, 0x06, 0x9AE0);
		mdio_write(tp, 0x06, 0x834E);
		mdio_write(tp, 0x06, 0x7804);
		mdio_write(tp, 0x06, 0x9E10);
		mdio_write(tp, 0x06, 0xE083);
		mdio_write(tp, 0x06, 0x4E78);
		mdio_write(tp, 0x06, 0x039E);
		mdio_write(tp, 0x06, 0x0FE0);
		mdio_write(tp, 0x06, 0x834E);
		mdio_write(tp, 0x06, 0x7801);
		mdio_write(tp, 0x06, 0x9E05);
		mdio_write(tp, 0x06, 0xAE0C);
		mdio_write(tp, 0x06, 0xAF81);
		mdio_write(tp, 0x06, 0xA7AF);
		mdio_write(tp, 0x06, 0x8152);
		mdio_write(tp, 0x06, 0xAF81);
		mdio_write(tp, 0x06, 0x8BAF);
		mdio_write(tp, 0x06, 0x81C2);
		mdio_write(tp, 0x06, 0xEE83);
		mdio_write(tp, 0x06, 0x4800);
		mdio_write(tp, 0x06, 0xEE83);
		mdio_write(tp, 0x06, 0x4900);
		mdio_write(tp, 0x06, 0xE083);
		mdio_write(tp, 0x06, 0x5110);
		mdio_write(tp, 0x06, 0xE483);
		mdio_write(tp, 0x06, 0x5158);
		mdio_write(tp, 0x06, 0x019F);
		mdio_write(tp, 0x06, 0xEAD0);
		mdio_write(tp, 0x06, 0x00D1);
		mdio_write(tp, 0x06, 0x801F);
		mdio_write(tp, 0x06, 0x66E2);
		mdio_write(tp, 0x06, 0xF8EA);
		mdio_write(tp, 0x06, 0xE3F8);
		mdio_write(tp, 0x06, 0xEB5A);
		mdio_write(tp, 0x06, 0xF81E);
		mdio_write(tp, 0x06, 0x20E6);
		mdio_write(tp, 0x06, 0xF8EA);
		mdio_write(tp, 0x06, 0xE5F8);
		mdio_write(tp, 0x06, 0xEBD3);
		mdio_write(tp, 0x06, 0x02B3);
		mdio_write(tp, 0x06, 0xFEE2);
		mdio_write(tp, 0x06, 0xF87C);
		mdio_write(tp, 0x06, 0xEF32);
		mdio_write(tp, 0x06, 0x5B80);
		mdio_write(tp, 0x06, 0xE3F8);
		mdio_write(tp, 0x06, 0x7D9E);
		mdio_write(tp, 0x06, 0x037D);
		mdio_write(tp, 0x06, 0xFFFF);
		mdio_write(tp, 0x06, 0x0D58);
		mdio_write(tp, 0x06, 0x1C55);
		mdio_write(tp, 0x06, 0x1A65);
		mdio_write(tp, 0x06, 0x11A1);
		mdio_write(tp, 0x06, 0x90D3);
		mdio_write(tp, 0x06, 0xE283);
		mdio_write(tp, 0x06, 0x48E3);
		mdio_write(tp, 0x06, 0x8349);
		mdio_write(tp, 0x06, 0x1B56);
		mdio_write(tp, 0x06, 0xAB08);
		mdio_write(tp, 0x06, 0xEF56);
		mdio_write(tp, 0x06, 0xE683);
		mdio_write(tp, 0x06, 0x48E7);
		mdio_write(tp, 0x06, 0x8349);
		mdio_write(tp, 0x06, 0x10D1);
		mdio_write(tp, 0x06, 0x801F);
		mdio_write(tp, 0x06, 0x66A0);
		mdio_write(tp, 0x06, 0x04B9);
		mdio_write(tp, 0x06, 0xE283);
		mdio_write(tp, 0x06, 0x48E3);
		mdio_write(tp, 0x06, 0x8349);
		mdio_write(tp, 0x06, 0xEF65);
		mdio_write(tp, 0x06, 0xE283);
		mdio_write(tp, 0x06, 0x4AE3);
		mdio_write(tp, 0x06, 0x834B);
		mdio_write(tp, 0x06, 0x1B56);
		mdio_write(tp, 0x06, 0xAA0E);
		mdio_write(tp, 0x06, 0xEF56);
		mdio_write(tp, 0x06, 0xE683);
		mdio_write(tp, 0x06, 0x4AE7);
		mdio_write(tp, 0x06, 0x834B);
		mdio_write(tp, 0x06, 0xE283);
		mdio_write(tp, 0x06, 0x4DE6);
		mdio_write(tp, 0x06, 0x834C);
		mdio_write(tp, 0x06, 0xE083);
		mdio_write(tp, 0x06, 0x4DA0);
		mdio_write(tp, 0x06, 0x000C);
		mdio_write(tp, 0x06, 0xAF81);
		mdio_write(tp, 0x06, 0x8BE0);
		mdio_write(tp, 0x06, 0x834D);
		mdio_write(tp, 0x06, 0x10E4);
		mdio_write(tp, 0x06, 0x834D);
		mdio_write(tp, 0x06, 0xAE04);
		mdio_write(tp, 0x06, 0x80E4);
		mdio_write(tp, 0x06, 0x834D);
		mdio_write(tp, 0x06, 0xE083);
		mdio_write(tp, 0x06, 0x4E78);
		mdio_write(tp, 0x06, 0x039E);
		mdio_write(tp, 0x06, 0x0BE0);
		mdio_write(tp, 0x06, 0x834E);
		mdio_write(tp, 0x06, 0x7804);
		mdio_write(tp, 0x06, 0x9E04);
		mdio_write(tp, 0x06, 0xEE83);
		mdio_write(tp, 0x06, 0x4E02);
		mdio_write(tp, 0x06, 0xE083);
		mdio_write(tp, 0x06, 0x32E1);
		mdio_write(tp, 0x06, 0x8333);
		mdio_write(tp, 0x06, 0x590F);
		mdio_write(tp, 0x06, 0xE283);
		mdio_write(tp, 0x06, 0x4D0C);
		mdio_write(tp, 0x06, 0x245A);
		mdio_write(tp, 0x06, 0xF01E);
		mdio_write(tp, 0x06, 0x12E4);
		mdio_write(tp, 0x06, 0xF88C);
		mdio_write(tp, 0x06, 0xE5F8);
		mdio_write(tp, 0x06, 0x8DE0);
		mdio_write(tp, 0x06, 0x8330);
		mdio_write(tp, 0x06, 0xE183);
		mdio_write(tp, 0x06, 0x3168);
		mdio_write(tp, 0x06, 0x01E4);
		mdio_write(tp, 0x06, 0xF88A);
		mdio_write(tp, 0x06, 0xE5F8);
		mdio_write(tp, 0x06, 0x8BAE);
		mdio_write(tp, 0x06, 0x37EE);
		mdio_write(tp, 0x06, 0x834E);
		mdio_write(tp, 0x06, 0x03E0);
		mdio_write(tp, 0x06, 0x834C);
		mdio_write(tp, 0x06, 0xE183);
		mdio_write(tp, 0x06, 0x4D1B);
		mdio_write(tp, 0x06, 0x019E);
		mdio_write(tp, 0x06, 0x04AA);
		mdio_write(tp, 0x06, 0xA1AE);
		mdio_write(tp, 0x06, 0xA8EE);
		mdio_write(tp, 0x06, 0x834E);
		mdio_write(tp, 0x06, 0x04EE);
		mdio_write(tp, 0x06, 0x834F);
		mdio_write(tp, 0x06, 0x00AE);
		mdio_write(tp, 0x06, 0xABE0);
		mdio_write(tp, 0x06, 0x834F);
		mdio_write(tp, 0x06, 0x7803);
		mdio_write(tp, 0x06, 0x9F14);
		mdio_write(tp, 0x06, 0xEE83);
		mdio_write(tp, 0x06, 0x4E05);
		mdio_write(tp, 0x06, 0xD240);
		mdio_write(tp, 0x06, 0xD655);
		mdio_write(tp, 0x06, 0x5402);
		mdio_write(tp, 0x06, 0x81C6);
		mdio_write(tp, 0x06, 0xD2A0);
		mdio_write(tp, 0x06, 0xD6BA);
		mdio_write(tp, 0x06, 0x0002);
		mdio_write(tp, 0x06, 0x81C6);
		mdio_write(tp, 0x06, 0xFEFD);
		mdio_write(tp, 0x06, 0xFC05);
		mdio_write(tp, 0x06, 0xF8E0);
		mdio_write(tp, 0x06, 0xF860);
		mdio_write(tp, 0x06, 0xE1F8);
		mdio_write(tp, 0x06, 0x6168);
		mdio_write(tp, 0x06, 0x02E4);
		mdio_write(tp, 0x06, 0xF860);
		mdio_write(tp, 0x06, 0xE5F8);
		mdio_write(tp, 0x06, 0x61E0);
		mdio_write(tp, 0x06, 0xF848);
		mdio_write(tp, 0x06, 0xE1F8);
		mdio_write(tp, 0x06, 0x4958);
		mdio_write(tp, 0x06, 0x0F1E);
		mdio_write(tp, 0x06, 0x02E4);
		mdio_write(tp, 0x06, 0xF848);
		mdio_write(tp, 0x06, 0xE5F8);
		mdio_write(tp, 0x06, 0x49D0);
		mdio_write(tp, 0x06, 0x0002);
		mdio_write(tp, 0x06, 0x820A);
		mdio_write(tp, 0x06, 0xBF83);
		mdio_write(tp, 0x06, 0x50EF);
		mdio_write(tp, 0x06, 0x46DC);
		mdio_write(tp, 0x06, 0x19DD);
		mdio_write(tp, 0x06, 0xD001);
		mdio_write(tp, 0x06, 0x0282);
		mdio_write(tp, 0x06, 0x0A02);
		mdio_write(tp, 0x06, 0x8226);
		mdio_write(tp, 0x06, 0xE0F8);
		mdio_write(tp, 0x06, 0x60E1);
		mdio_write(tp, 0x06, 0xF861);
		mdio_write(tp, 0x06, 0x58FD);
		mdio_write(tp, 0x06, 0xE4F8);
		mdio_write(tp, 0x06, 0x60E5);
		mdio_write(tp, 0x06, 0xF861);
		mdio_write(tp, 0x06, 0xFC04);
		mdio_write(tp, 0x06, 0xF9FA);
		mdio_write(tp, 0x06, 0xFBC6);
		mdio_write(tp, 0x06, 0xBFF8);
		mdio_write(tp, 0x06, 0x40BE);
		mdio_write(tp, 0x06, 0x8350);
		mdio_write(tp, 0x06, 0xA001);
		mdio_write(tp, 0x06, 0x0107);
		mdio_write(tp, 0x06, 0x1B89);
		mdio_write(tp, 0x06, 0xCFD2);
		mdio_write(tp, 0x06, 0x08EB);
		mdio_write(tp, 0x06, 0xDB19);
		mdio_write(tp, 0x06, 0xB2FB);
		mdio_write(tp, 0x06, 0xFFFE);
		mdio_write(tp, 0x06, 0xFD04);
		mdio_write(tp, 0x06, 0xF8E0);
		mdio_write(tp, 0x06, 0xF848);
		mdio_write(tp, 0x06, 0xE1F8);
		mdio_write(tp, 0x06, 0x4968);
		mdio_write(tp, 0x06, 0x08E4);
		mdio_write(tp, 0x06, 0xF848);
		mdio_write(tp, 0x06, 0xE5F8);
		mdio_write(tp, 0x06, 0x4958);
		mdio_write(tp, 0x06, 0xF7E4);
		mdio_write(tp, 0x06, 0xF848);
		mdio_write(tp, 0x06, 0xE5F8);
		mdio_write(tp, 0x06, 0x49FC);
		mdio_write(tp, 0x06, 0x044D);
		mdio_write(tp, 0x06, 0x2000);
		mdio_write(tp, 0x06, 0x024E);
		mdio_write(tp, 0x06, 0x2200);
		mdio_write(tp, 0x06, 0x024D);
		mdio_write(tp, 0x06, 0xDFFF);
		mdio_write(tp, 0x06, 0x014E);
		mdio_write(tp, 0x06, 0xDDFF);
			mdio_write(tp, 0x06, 0x01f8);
			mdio_write(tp, 0x06, 0xfafb);
			mdio_write(tp, 0x06, 0xef79);
			mdio_write(tp, 0x06, 0xbff8);
			mdio_write(tp, 0x06, 0x22d8);
			mdio_write(tp, 0x06, 0x19d9);
			mdio_write(tp, 0x06, 0x5884);
			mdio_write(tp, 0x06, 0x9f09);
			mdio_write(tp, 0x06, 0xbf82);
			mdio_write(tp, 0x06, 0x6dd6);
			mdio_write(tp, 0x06, 0x8275);
			mdio_write(tp, 0x06, 0x0201);
			mdio_write(tp, 0x06, 0x4fef);
			mdio_write(tp, 0x06, 0x97ff);
			mdio_write(tp, 0x06, 0xfefc);
			mdio_write(tp, 0x06, 0x0517);
			mdio_write(tp, 0x06, 0xfffe);
			mdio_write(tp, 0x06, 0x0117);
			mdio_write(tp, 0x06, 0x0001);
			mdio_write(tp, 0x06, 0x0200);
		mdio_write(tp, 0x05, 0x83D8);
		mdio_write(tp, 0x06, 0x8000);
			mdio_write(tp, 0x05, 0x83d6);
			mdio_write(tp, 0x06, 0x824f);
			mdio_write(tp, 0x02, 0x2010);
		mdio_write(tp, 0x03, 0xDC00);
			mdio_write(tp, 0x1f, 0x0000);
			mdio_write(tp, 0x0b, 0x0600);
			mdio_write(tp, 0x1f, 0x0005);
		mdio_write(tp, 0x05, 0xFFF6);
		mdio_write(tp, 0x06, 0x00FC);
		mdio_write(tp, 0x1F, 0x0000);
		}

		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x0D, 0xF880);
		mdio_write(tp, 0x1F, 0x0000);
	} else if (tp->mcfg == CFG_METHOD_11) {
		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x10, 0x0008);
		mdio_write(tp, 0x0D, 0x006C);


		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x17, 0x0CC0);

		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x0B, 0xA4D8);
		mdio_write(tp, 0x09, 0x281C);
		mdio_write(tp, 0x07, 0x2883);
		mdio_write(tp, 0x0A, 0x6B35);
		mdio_write(tp, 0x1D, 0x3DA4);
		mdio_write(tp, 0x1C, 0xEFFD);
		mdio_write(tp, 0x14, 0x7F52);
		mdio_write(tp, 0x18, 0x7FC6);
		mdio_write(tp, 0x08, 0x0601);
		mdio_write(tp, 0x06, 0x4063);
		mdio_write(tp, 0x10, 0xF074);
		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x13, 0x0789);
		mdio_write(tp, 0x12, 0xF4BD);
		mdio_write(tp, 0x1A, 0x04FD);
		mdio_write(tp, 0x14, 0x84B0);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x00, 0x9200);

		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x01, 0x0340);
		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x04, 0x4000);
		mdio_write(tp, 0x03, 0x1D21);
		mdio_write(tp, 0x02, 0x0C32);
		mdio_write(tp, 0x01, 0x0200);
		mdio_write(tp, 0x00, 0x5554);
		mdio_write(tp, 0x04, 0x4800);
		mdio_write(tp, 0x04, 0x4000);
		mdio_write(tp, 0x04, 0xF000);
		mdio_write(tp, 0x03, 0xDF01);
		mdio_write(tp, 0x02, 0xDF20);
		mdio_write(tp, 0x01, 0x101A);
		mdio_write(tp, 0x00, 0xA0FF);
		mdio_write(tp, 0x04, 0xF800);
		mdio_write(tp, 0x04, 0xF000);
		mdio_write(tp, 0x1F, 0x0000);

		mdio_write(tp, 0x1F, 0x0007);
		mdio_write(tp, 0x1E, 0x0023);
		mdio_write(tp, 0x16, 0x0000);
		mdio_write(tp, 0x1F, 0x0000);
		gphy_val = mdio_read(tp, 0x0D);
		gphy_val |= BIT_5;
		mdio_write(tp, 0x0D, gphy_val);
	} else if (tp->mcfg == CFG_METHOD_12 || tp->mcfg == CFG_METHOD_13) {
		// TO DO:
		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x17, 0x0CC0);
		mdio_write(tp, 0x1F, 0x0007);
		mdio_write(tp, 0x1E, 0x002D);
		mdio_write(tp, 0x18, 0x0040);
		mdio_write(tp, 0x1F, 0x0000);
		gphy_val = mdio_read(tp, 0x0D);
		gphy_val |= BIT_5;
		mdio_write(tp, 0x0D, gphy_val);
		mdio_write(tp, 0x1F, 0x0002);
		gphy_val = mdio_read(tp, 0x0C);
		gphy_val |= BIT_10;
		mdio_write(tp, 0x0C, gphy_val);
	} else if (tp->mcfg == CFG_METHOD_14 || tp->mcfg == CFG_METHOD_15) {
		//struct pci_dev *pdev = tp->pci_dev;
		RTL_W8(tp,0xF3, RTL_R8(tp,0xF3) | BIT_2);
		if (tp->mcfg == CFG_METHOD_14) {

			mdio_write(tp, 0x1f, 0x0000);
			mdio_write(tp, 0x00, 0x1800);
			mdio_write(tp, 0x1f, 0x0007);
			mdio_write(tp, 0x1e, 0x0023);
			mdio_write(tp, 0x17, 0x0117);
			mdio_write(tp, 0x1f, 0x0007);
			mdio_write(tp, 0x1E, 0x002C);
			mdio_write(tp, 0x1B, 0x5000);
			mdio_write(tp, 0x1f, 0x0000);
			mdio_write(tp, 0x16, 0x4104);
			for (i = 0; i < 200; i++) {
				udelay(100);
				gphy_val = mdio_read(tp, 0x1E);
				gphy_val &= 0x03FF;
				if (gphy_val == 0x000C)
					break;
	}
			mdio_write(tp, 0x1f, 0x0005);
			for (i = 0; i < 200; i++) {
				udelay(100);
				gphy_val = mdio_read(tp, 0x07);
				if ((gphy_val & BIT_5) == 0)
					break;
			}
			gphy_val = mdio_read(tp, 0x07);
			if (gphy_val & BIT_5) {
				mdio_write(tp, 0x1f, 0x0007);
				mdio_write(tp, 0x1e, 0x00a1);
				mdio_write(tp, 0x17, 0x1000);
				mdio_write(tp, 0x17, 0x0000);
				mdio_write(tp, 0x17, 0x2000);
				mdio_write(tp, 0x1e, 0x002f);
				mdio_write(tp, 0x18, 0x9bfb);
				mdio_write(tp, 0x1f, 0x0005);
				mdio_write(tp, 0x07, 0x0000);
				mdio_write(tp, 0x1f, 0x0000);
}
			mdio_write(tp, 0x1f, 0x0005);
			mdio_write(tp, 0x05, 0xfff6);
			mdio_write(tp, 0x06, 0x0080);
			gphy_val = mdio_read(tp, 0x00);
			gphy_val &= ~(BIT_7);
			mdio_write(tp, 0x00, gphy_val);
			mdio_write(tp, 0x1f, 0x0002);
			gphy_val = mdio_read(tp, 0x08);
			gphy_val &= ~(BIT_7);
			mdio_write(tp, 0x08, gphy_val);
			mdio_write(tp, 0x1f, 0x0000);
			mdio_write(tp, 0x1f, 0x0007);
			mdio_write(tp, 0x1e, 0x0023);
			mdio_write(tp, 0x16, 0x0306);
			mdio_write(tp, 0x16, 0x0307);
			mdio_write(tp, 0x15, 0x000e);
			mdio_write(tp, 0x19, 0x000a);
			mdio_write(tp, 0x15, 0x0010);
			mdio_write(tp, 0x19, 0x0008);
			mdio_write(tp, 0x15, 0x0018);
			mdio_write(tp, 0x19, 0x4801);
			mdio_write(tp, 0x15, 0x0019);
			mdio_write(tp, 0x19, 0x6801);
			mdio_write(tp, 0x15, 0x001a);
			mdio_write(tp, 0x19, 0x66a1);
			mdio_write(tp, 0x15, 0x001f);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0020);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0021);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0022);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0023);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0024);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0025);
			mdio_write(tp, 0x19, 0x64a1);
			mdio_write(tp, 0x15, 0x0026);
			mdio_write(tp, 0x19, 0x40ea);
			mdio_write(tp, 0x15, 0x0027);
			mdio_write(tp, 0x19, 0x4503);
			mdio_write(tp, 0x15, 0x0028);
			mdio_write(tp, 0x19, 0x9f00);
			mdio_write(tp, 0x15, 0x0029);
			mdio_write(tp, 0x19, 0xa631);
			mdio_write(tp, 0x15, 0x002a);
			mdio_write(tp, 0x19, 0x9717);
			mdio_write(tp, 0x15, 0x002b);
			mdio_write(tp, 0x19, 0x302c);
			mdio_write(tp, 0x15, 0x002c);
			mdio_write(tp, 0x19, 0x4802);
			mdio_write(tp, 0x15, 0x002d);
			mdio_write(tp, 0x19, 0x58da);
			mdio_write(tp, 0x15, 0x002e);
			mdio_write(tp, 0x19, 0x400d);
			mdio_write(tp, 0x15, 0x002f);
			mdio_write(tp, 0x19, 0x4488);
			mdio_write(tp, 0x15, 0x0030);
			mdio_write(tp, 0x19, 0x9e00);
			mdio_write(tp, 0x15, 0x0031);
			mdio_write(tp, 0x19, 0x63c8);
			mdio_write(tp, 0x15, 0x0032);
			mdio_write(tp, 0x19, 0x6481);
			mdio_write(tp, 0x15, 0x0033);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0034);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0035);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0036);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0037);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0038);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0039);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x003a);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x003b);
			mdio_write(tp, 0x19, 0x63e8);
			mdio_write(tp, 0x15, 0x003c);
			mdio_write(tp, 0x19, 0x7d00);
			mdio_write(tp, 0x15, 0x003d);
			mdio_write(tp, 0x19, 0x59d4);
			mdio_write(tp, 0x15, 0x003e);
			mdio_write(tp, 0x19, 0x63f8);
			mdio_write(tp, 0x15, 0x0040);
			mdio_write(tp, 0x19, 0x64a1);
			mdio_write(tp, 0x15, 0x0041);
			mdio_write(tp, 0x19, 0x30de);
			mdio_write(tp, 0x15, 0x0044);
			mdio_write(tp, 0x19, 0x480f);
			mdio_write(tp, 0x15, 0x0045);
			mdio_write(tp, 0x19, 0x6800);
			mdio_write(tp, 0x15, 0x0046);
			mdio_write(tp, 0x19, 0x6680);
			mdio_write(tp, 0x15, 0x0047);
			mdio_write(tp, 0x19, 0x7c10);
			mdio_write(tp, 0x15, 0x0048);
			mdio_write(tp, 0x19, 0x63c8);
			mdio_write(tp, 0x15, 0x0049);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x004a);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x004b);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x004c);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x004d);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x004e);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x004f);
			mdio_write(tp, 0x19, 0x40ea);
			mdio_write(tp, 0x15, 0x0050);
			mdio_write(tp, 0x19, 0x4503);
			mdio_write(tp, 0x15, 0x0051);
			mdio_write(tp, 0x19, 0x58ca);
			mdio_write(tp, 0x15, 0x0052);
			mdio_write(tp, 0x19, 0x63c8);
			mdio_write(tp, 0x15, 0x0053);
			mdio_write(tp, 0x19, 0x63d8);
			mdio_write(tp, 0x15, 0x0054);
			mdio_write(tp, 0x19, 0x66a0);
			mdio_write(tp, 0x15, 0x0055);
			mdio_write(tp, 0x19, 0x9f00);
			mdio_write(tp, 0x15, 0x0056);
			mdio_write(tp, 0x19, 0x3000);
			mdio_write(tp, 0x15, 0x006E);
			mdio_write(tp, 0x19, 0x9afa);
			mdio_write(tp, 0x15, 0x00a1);
			mdio_write(tp, 0x19, 0x3044);
			mdio_write(tp, 0x15, 0x00ab);
			mdio_write(tp, 0x19, 0x5820);
			mdio_write(tp, 0x15, 0x00ac);
			mdio_write(tp, 0x19, 0x5e04);
			mdio_write(tp, 0x15, 0x00ad);
			mdio_write(tp, 0x19, 0xb60c);
			mdio_write(tp, 0x15, 0x00af);
			mdio_write(tp, 0x19, 0x000a);
			mdio_write(tp, 0x15, 0x00b2);
			mdio_write(tp, 0x19, 0x30b9);
			mdio_write(tp, 0x15, 0x00b9);
			mdio_write(tp, 0x19, 0x4408);
			mdio_write(tp, 0x15, 0x00ba);
			mdio_write(tp, 0x19, 0x480b);
			mdio_write(tp, 0x15, 0x00bb);
			mdio_write(tp, 0x19, 0x5e00);
			mdio_write(tp, 0x15, 0x00bc);
			mdio_write(tp, 0x19, 0x405f);
			mdio_write(tp, 0x15, 0x00bd);
			mdio_write(tp, 0x19, 0x4448);
			mdio_write(tp, 0x15, 0x00be);
			mdio_write(tp, 0x19, 0x4020);
			mdio_write(tp, 0x15, 0x00bf);
			mdio_write(tp, 0x19, 0x4468);
			mdio_write(tp, 0x15, 0x00c0);
			mdio_write(tp, 0x19, 0x9c02);
			mdio_write(tp, 0x15, 0x00c1);
			mdio_write(tp, 0x19, 0x58a0);
			mdio_write(tp, 0x15, 0x00c2);
			mdio_write(tp, 0x19, 0xb605);
			mdio_write(tp, 0x15, 0x00c3);
			mdio_write(tp, 0x19, 0xc0d3);
			mdio_write(tp, 0x15, 0x00c4);
			mdio_write(tp, 0x19, 0x00e6);
			mdio_write(tp, 0x15, 0x00c5);
			mdio_write(tp, 0x19, 0xdaec);
			mdio_write(tp, 0x15, 0x00c6);
			mdio_write(tp, 0x19, 0x00fa);
			mdio_write(tp, 0x15, 0x00c7);
			mdio_write(tp, 0x19, 0x9df9);
			mdio_write(tp, 0x15, 0x00c8);
			mdio_write(tp, 0x19, 0x307a);
			mdio_write(tp, 0x15, 0x0112);
			mdio_write(tp, 0x19, 0x6421);
			mdio_write(tp, 0x15, 0x0113);
			mdio_write(tp, 0x19, 0x7c08);
			mdio_write(tp, 0x15, 0x0114);
			mdio_write(tp, 0x19, 0x63f0);
			mdio_write(tp, 0x15, 0x0115);
			mdio_write(tp, 0x19, 0x4003);
			mdio_write(tp, 0x15, 0x0116);
			mdio_write(tp, 0x19, 0x4418);
			mdio_write(tp, 0x15, 0x0117);
			mdio_write(tp, 0x19, 0x9b00);
			mdio_write(tp, 0x15, 0x0118);
			mdio_write(tp, 0x19, 0x6461);
			mdio_write(tp, 0x15, 0x0119);
			mdio_write(tp, 0x19, 0x64e1);
			mdio_write(tp, 0x15, 0x011a);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0150);
			mdio_write(tp, 0x19, 0x7c80);
			mdio_write(tp, 0x15, 0x0151);
			mdio_write(tp, 0x19, 0x6461);
			mdio_write(tp, 0x15, 0x0152);
			mdio_write(tp, 0x19, 0x4003);
			mdio_write(tp, 0x15, 0x0153);
			mdio_write(tp, 0x19, 0x4540);
			mdio_write(tp, 0x15, 0x0154);
			mdio_write(tp, 0x19, 0x9f00);
			mdio_write(tp, 0x15, 0x0155);
			mdio_write(tp, 0x19, 0x9d00);
			mdio_write(tp, 0x15, 0x0156);
			mdio_write(tp, 0x19, 0x7c40);
			mdio_write(tp, 0x15, 0x0157);
			mdio_write(tp, 0x19, 0x6421);
			mdio_write(tp, 0x15, 0x0158);
			mdio_write(tp, 0x19, 0x7c80);
			mdio_write(tp, 0x15, 0x0159);
			mdio_write(tp, 0x19, 0x64a1);
			mdio_write(tp, 0x15, 0x015a);
			mdio_write(tp, 0x19, 0x30fe);
			mdio_write(tp, 0x15, 0x021e);
			mdio_write(tp, 0x19, 0x5410);
			mdio_write(tp, 0x15, 0x0225);
			mdio_write(tp, 0x19, 0x5400);
			mdio_write(tp, 0x15, 0x023D);
			mdio_write(tp, 0x19, 0x4050);
			mdio_write(tp, 0x15, 0x0295);
			mdio_write(tp, 0x19, 0x6c08);
			mdio_write(tp, 0x15, 0x02bd);
			mdio_write(tp, 0x19, 0xa523);
			mdio_write(tp, 0x15, 0x02be);
			mdio_write(tp, 0x19, 0x32ca);
			mdio_write(tp, 0x15, 0x02ca);
			mdio_write(tp, 0x19, 0x48b3);
			mdio_write(tp, 0x15, 0x02cb);
			mdio_write(tp, 0x19, 0x4020);
			mdio_write(tp, 0x15, 0x02cc);
			mdio_write(tp, 0x19, 0x4823);
			mdio_write(tp, 0x15, 0x02cd);
			mdio_write(tp, 0x19, 0x4510);
			mdio_write(tp, 0x15, 0x02ce);
			mdio_write(tp, 0x19, 0xb63a);
			mdio_write(tp, 0x15, 0x02cf);
			mdio_write(tp, 0x19, 0x7dc8);
			mdio_write(tp, 0x15, 0x02d6);
			mdio_write(tp, 0x19, 0x9bf8);
			mdio_write(tp, 0x15, 0x02d8);
			mdio_write(tp, 0x19, 0x85f6);
			mdio_write(tp, 0x15, 0x02d9);
			mdio_write(tp, 0x19, 0x32e0);
			mdio_write(tp, 0x15, 0x02e0);
			mdio_write(tp, 0x19, 0x4834);
			mdio_write(tp, 0x15, 0x02e1);
			mdio_write(tp, 0x19, 0x6c08);
			mdio_write(tp, 0x15, 0x02e2);
			mdio_write(tp, 0x19, 0x4020);
			mdio_write(tp, 0x15, 0x02e3);
			mdio_write(tp, 0x19, 0x4824);
			mdio_write(tp, 0x15, 0x02e4);
			mdio_write(tp, 0x19, 0x4520);
			mdio_write(tp, 0x15, 0x02e5);
			mdio_write(tp, 0x19, 0x4008);
			mdio_write(tp, 0x15, 0x02e6);
			mdio_write(tp, 0x19, 0x4560);
			mdio_write(tp, 0x15, 0x02e7);
			mdio_write(tp, 0x19, 0x9d04);
			mdio_write(tp, 0x15, 0x02e8);
			mdio_write(tp, 0x19, 0x48c4);
			mdio_write(tp, 0x15, 0x02e9);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x02ea);
			mdio_write(tp, 0x19, 0x4844);
			mdio_write(tp, 0x15, 0x02eb);
			mdio_write(tp, 0x19, 0x7dc8);
			mdio_write(tp, 0x15, 0x02f0);
			mdio_write(tp, 0x19, 0x9cf7);
			mdio_write(tp, 0x15, 0x02f1);
			mdio_write(tp, 0x19, 0xdf94);
			mdio_write(tp, 0x15, 0x02f2);
			mdio_write(tp, 0x19, 0x0002);
			mdio_write(tp, 0x15, 0x02f3);
			mdio_write(tp, 0x19, 0x6810);
			mdio_write(tp, 0x15, 0x02f4);
			mdio_write(tp, 0x19, 0xb614);
			mdio_write(tp, 0x15, 0x02f5);
			mdio_write(tp, 0x19, 0xc42b);
			mdio_write(tp, 0x15, 0x02f6);
			mdio_write(tp, 0x19, 0x00d4);
			mdio_write(tp, 0x15, 0x02f7);
			mdio_write(tp, 0x19, 0xc455);
			mdio_write(tp, 0x15, 0x02f8);
			mdio_write(tp, 0x19, 0x0093);
			mdio_write(tp, 0x15, 0x02f9);
			mdio_write(tp, 0x19, 0x92ee);
			mdio_write(tp, 0x15, 0x02fa);
			mdio_write(tp, 0x19, 0xefed);
			mdio_write(tp, 0x15, 0x02fb);
			mdio_write(tp, 0x19, 0x3312);
			mdio_write(tp, 0x15, 0x0312);
			mdio_write(tp, 0x19, 0x49b5);
			mdio_write(tp, 0x15, 0x0313);
			mdio_write(tp, 0x19, 0x7d00);
			mdio_write(tp, 0x15, 0x0314);
			mdio_write(tp, 0x19, 0x4d00);
			mdio_write(tp, 0x15, 0x0315);
			mdio_write(tp, 0x19, 0x6810);
			mdio_write(tp, 0x15, 0x031e);
			mdio_write(tp, 0x19, 0x404f);
			mdio_write(tp, 0x15, 0x031f);
			mdio_write(tp, 0x19, 0x44c8);
			mdio_write(tp, 0x15, 0x0320);
			mdio_write(tp, 0x19, 0xd64f);
			mdio_write(tp, 0x15, 0x0321);
			mdio_write(tp, 0x19, 0x00e7);
			mdio_write(tp, 0x15, 0x0322);
			mdio_write(tp, 0x19, 0x7c08);
			mdio_write(tp, 0x15, 0x0323);
			mdio_write(tp, 0x19, 0x8203);
			mdio_write(tp, 0x15, 0x0324);
			mdio_write(tp, 0x19, 0x4d48);
			mdio_write(tp, 0x15, 0x0325);
			mdio_write(tp, 0x19, 0x3327);
			mdio_write(tp, 0x15, 0x0326);
			mdio_write(tp, 0x19, 0x4d40);
			mdio_write(tp, 0x15, 0x0327);
			mdio_write(tp, 0x19, 0xc8d7);
			mdio_write(tp, 0x15, 0x0328);
			mdio_write(tp, 0x19, 0x0003);
			mdio_write(tp, 0x15, 0x0329);
			mdio_write(tp, 0x19, 0x7c20);
			mdio_write(tp, 0x15, 0x032a);
			mdio_write(tp, 0x19, 0x4c20);
			mdio_write(tp, 0x15, 0x032b);
			mdio_write(tp, 0x19, 0xc8ed);
			mdio_write(tp, 0x15, 0x032c);
			mdio_write(tp, 0x19, 0x00f4);
			mdio_write(tp, 0x15, 0x032d);
			mdio_write(tp, 0x19, 0x82b3);
			mdio_write(tp, 0x15, 0x032e);
			mdio_write(tp, 0x19, 0xd11d);
			mdio_write(tp, 0x15, 0x032f);
			mdio_write(tp, 0x19, 0x00b1);
			mdio_write(tp, 0x15, 0x0330);
			mdio_write(tp, 0x19, 0xde18);
			mdio_write(tp, 0x15, 0x0331);
			mdio_write(tp, 0x19, 0x0008);
			mdio_write(tp, 0x15, 0x0332);
			mdio_write(tp, 0x19, 0x91ee);
			mdio_write(tp, 0x15, 0x0333);
			mdio_write(tp, 0x19, 0x3339);
			mdio_write(tp, 0x15, 0x033a);
			mdio_write(tp, 0x19, 0x4064);
			mdio_write(tp, 0x15, 0x0340);
			mdio_write(tp, 0x19, 0x9e06);
			mdio_write(tp, 0x15, 0x0341);
			mdio_write(tp, 0x19, 0x7c08);
			mdio_write(tp, 0x15, 0x0342);
			mdio_write(tp, 0x19, 0x8203);
			mdio_write(tp, 0x15, 0x0343);
			mdio_write(tp, 0x19, 0x4d48);
			mdio_write(tp, 0x15, 0x0344);
			mdio_write(tp, 0x19, 0x3346);
			mdio_write(tp, 0x15, 0x0345);
			mdio_write(tp, 0x19, 0x4d40);
			mdio_write(tp, 0x15, 0x0346);
			mdio_write(tp, 0x19, 0xd11d);
			mdio_write(tp, 0x15, 0x0347);
			mdio_write(tp, 0x19, 0x0099);
			mdio_write(tp, 0x15, 0x0348);
			mdio_write(tp, 0x19, 0xbb17);
			mdio_write(tp, 0x15, 0x0349);
			mdio_write(tp, 0x19, 0x8102);
			mdio_write(tp, 0x15, 0x034a);
			mdio_write(tp, 0x19, 0x334d);
			mdio_write(tp, 0x15, 0x034b);
			mdio_write(tp, 0x19, 0xa22c);
			mdio_write(tp, 0x15, 0x034c);
			mdio_write(tp, 0x19, 0x3397);
			mdio_write(tp, 0x15, 0x034d);
			mdio_write(tp, 0x19, 0x91f2);
			mdio_write(tp, 0x15, 0x034e);
			mdio_write(tp, 0x19, 0xc218);
			mdio_write(tp, 0x15, 0x034f);
			mdio_write(tp, 0x19, 0x00f0);
			mdio_write(tp, 0x15, 0x0350);
			mdio_write(tp, 0x19, 0x3397);
			mdio_write(tp, 0x15, 0x0351);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0364);
			mdio_write(tp, 0x19, 0xbc05);
			mdio_write(tp, 0x15, 0x0367);
			mdio_write(tp, 0x19, 0xa1fc);
			mdio_write(tp, 0x15, 0x0368);
			mdio_write(tp, 0x19, 0x3377);
			mdio_write(tp, 0x15, 0x0369);
			mdio_write(tp, 0x19, 0x328b);
			mdio_write(tp, 0x15, 0x036a);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0377);
			mdio_write(tp, 0x19, 0x4b97);
			mdio_write(tp, 0x15, 0x0378);
			mdio_write(tp, 0x19, 0x6818);
			mdio_write(tp, 0x15, 0x0379);
			mdio_write(tp, 0x19, 0x4b07);
			mdio_write(tp, 0x15, 0x037a);
			mdio_write(tp, 0x19, 0x40ac);
			mdio_write(tp, 0x15, 0x037b);
			mdio_write(tp, 0x19, 0x4445);
			mdio_write(tp, 0x15, 0x037c);
			mdio_write(tp, 0x19, 0x404e);
			mdio_write(tp, 0x15, 0x037d);
			mdio_write(tp, 0x19, 0x4461);
			mdio_write(tp, 0x15, 0x037e);
			mdio_write(tp, 0x19, 0x9c09);
			mdio_write(tp, 0x15, 0x037f);
			mdio_write(tp, 0x19, 0x63da);
			mdio_write(tp, 0x15, 0x0380);
			mdio_write(tp, 0x19, 0x5440);
			mdio_write(tp, 0x15, 0x0381);
			mdio_write(tp, 0x19, 0x4b98);
			mdio_write(tp, 0x15, 0x0382);
			mdio_write(tp, 0x19, 0x7c60);
			mdio_write(tp, 0x15, 0x0383);
			mdio_write(tp, 0x19, 0x4c00);
			mdio_write(tp, 0x15, 0x0384);
			mdio_write(tp, 0x19, 0x4b08);
			mdio_write(tp, 0x15, 0x0385);
			mdio_write(tp, 0x19, 0x63d8);
			mdio_write(tp, 0x15, 0x0386);
			mdio_write(tp, 0x19, 0x338d);
			mdio_write(tp, 0x15, 0x0387);
			mdio_write(tp, 0x19, 0xd64f);
			mdio_write(tp, 0x15, 0x0388);
			mdio_write(tp, 0x19, 0x0080);
			mdio_write(tp, 0x15, 0x0389);
			mdio_write(tp, 0x19, 0x820c);
			mdio_write(tp, 0x15, 0x038a);
			mdio_write(tp, 0x19, 0xa10b);
			mdio_write(tp, 0x15, 0x038b);
			mdio_write(tp, 0x19, 0x9df3);
			mdio_write(tp, 0x15, 0x038c);
			mdio_write(tp, 0x19, 0x3395);
			mdio_write(tp, 0x15, 0x038d);
			mdio_write(tp, 0x19, 0xd64f);
			mdio_write(tp, 0x15, 0x038e);
			mdio_write(tp, 0x19, 0x00f9);
			mdio_write(tp, 0x15, 0x038f);
			mdio_write(tp, 0x19, 0xc017);
			mdio_write(tp, 0x15, 0x0390);
			mdio_write(tp, 0x19, 0x0005);
			mdio_write(tp, 0x15, 0x0391);
			mdio_write(tp, 0x19, 0x6c0b);
			mdio_write(tp, 0x15, 0x0392);
			mdio_write(tp, 0x19, 0xa103);
			mdio_write(tp, 0x15, 0x0393);
			mdio_write(tp, 0x19, 0x6c08);
			mdio_write(tp, 0x15, 0x0394);
			mdio_write(tp, 0x19, 0x9df9);
			mdio_write(tp, 0x15, 0x0395);
			mdio_write(tp, 0x19, 0x6c08);
			mdio_write(tp, 0x15, 0x0396);
			mdio_write(tp, 0x19, 0x3397);
			mdio_write(tp, 0x15, 0x0399);
			mdio_write(tp, 0x19, 0x6810);
			mdio_write(tp, 0x15, 0x03a4);
			mdio_write(tp, 0x19, 0x7c08);
			mdio_write(tp, 0x15, 0x03a5);
			mdio_write(tp, 0x19, 0x8203);
			mdio_write(tp, 0x15, 0x03a6);
			mdio_write(tp, 0x19, 0x4d08);
			mdio_write(tp, 0x15, 0x03a7);
			mdio_write(tp, 0x19, 0x33a9);
			mdio_write(tp, 0x15, 0x03a8);
			mdio_write(tp, 0x19, 0x4d00);
			mdio_write(tp, 0x15, 0x03a9);
			mdio_write(tp, 0x19, 0x9bfa);
			mdio_write(tp, 0x15, 0x03aa);
			mdio_write(tp, 0x19, 0x33b6);
			mdio_write(tp, 0x15, 0x03bb);
			mdio_write(tp, 0x19, 0x4056);
			mdio_write(tp, 0x15, 0x03bc);
			mdio_write(tp, 0x19, 0x44e9);
			mdio_write(tp, 0x15, 0x03bd);
			mdio_write(tp, 0x19, 0x405e);
			mdio_write(tp, 0x15, 0x03be);
			mdio_write(tp, 0x19, 0x44f8);
			mdio_write(tp, 0x15, 0x03bf);
			mdio_write(tp, 0x19, 0xd64f);
			mdio_write(tp, 0x15, 0x03c0);
			mdio_write(tp, 0x19, 0x0037);
			mdio_write(tp, 0x15, 0x03c1);
			mdio_write(tp, 0x19, 0xbd37);
			mdio_write(tp, 0x15, 0x03c2);
			mdio_write(tp, 0x19, 0x9cfd);
			mdio_write(tp, 0x15, 0x03c3);
			mdio_write(tp, 0x19, 0xc639);
			mdio_write(tp, 0x15, 0x03c4);
			mdio_write(tp, 0x19, 0x0011);
			mdio_write(tp, 0x15, 0x03c5);
			mdio_write(tp, 0x19, 0x9b03);
			mdio_write(tp, 0x15, 0x03c6);
			mdio_write(tp, 0x19, 0x7c01);
			mdio_write(tp, 0x15, 0x03c7);
			mdio_write(tp, 0x19, 0x4c01);
			mdio_write(tp, 0x15, 0x03c8);
			mdio_write(tp, 0x19, 0x9e03);
			mdio_write(tp, 0x15, 0x03c9);
			mdio_write(tp, 0x19, 0x7c20);
			mdio_write(tp, 0x15, 0x03ca);
			mdio_write(tp, 0x19, 0x4c20);
			mdio_write(tp, 0x15, 0x03cb);
			mdio_write(tp, 0x19, 0x9af4);
			mdio_write(tp, 0x15, 0x03cc);
			mdio_write(tp, 0x19, 0x7c12);
			mdio_write(tp, 0x15, 0x03cd);
			mdio_write(tp, 0x19, 0x4c52);
			mdio_write(tp, 0x15, 0x03ce);
			mdio_write(tp, 0x19, 0x4470);
			mdio_write(tp, 0x15, 0x03cf);
			mdio_write(tp, 0x19, 0x7c12);
			mdio_write(tp, 0x15, 0x03d0);
			mdio_write(tp, 0x19, 0x4c40);
			mdio_write(tp, 0x15, 0x03d1);
			mdio_write(tp, 0x19, 0x33bf);
			mdio_write(tp, 0x15, 0x03d6);
			mdio_write(tp, 0x19, 0x4047);
			mdio_write(tp, 0x15, 0x03d7);
			mdio_write(tp, 0x19, 0x4469);
			mdio_write(tp, 0x15, 0x03d8);
			mdio_write(tp, 0x19, 0x492b);
			mdio_write(tp, 0x15, 0x03d9);
			mdio_write(tp, 0x19, 0x4479);
			mdio_write(tp, 0x15, 0x03da);
			mdio_write(tp, 0x19, 0x7c09);
			mdio_write(tp, 0x15, 0x03db);
			mdio_write(tp, 0x19, 0x8203);
			mdio_write(tp, 0x15, 0x03dc);
			mdio_write(tp, 0x19, 0x4d48);
			mdio_write(tp, 0x15, 0x03dd);
			mdio_write(tp, 0x19, 0x33df);
			mdio_write(tp, 0x15, 0x03de);
			mdio_write(tp, 0x19, 0x4d40);
			mdio_write(tp, 0x15, 0x03df);
			mdio_write(tp, 0x19, 0xd64f);
			mdio_write(tp, 0x15, 0x03e0);
			mdio_write(tp, 0x19, 0x0017);
			mdio_write(tp, 0x15, 0x03e1);
			mdio_write(tp, 0x19, 0xbd17);
			mdio_write(tp, 0x15, 0x03e2);
			mdio_write(tp, 0x19, 0x9b03);
			mdio_write(tp, 0x15, 0x03e3);
			mdio_write(tp, 0x19, 0x7c20);
			mdio_write(tp, 0x15, 0x03e4);
			mdio_write(tp, 0x19, 0x4c20);
			mdio_write(tp, 0x15, 0x03e5);
			mdio_write(tp, 0x19, 0x88f5);
			mdio_write(tp, 0x15, 0x03e6);
			mdio_write(tp, 0x19, 0xc428);
			mdio_write(tp, 0x15, 0x03e7);
			mdio_write(tp, 0x19, 0x0008);
			mdio_write(tp, 0x15, 0x03e8);
			mdio_write(tp, 0x19, 0x9af2);
			mdio_write(tp, 0x15, 0x03e9);
			mdio_write(tp, 0x19, 0x7c12);
			mdio_write(tp, 0x15, 0x03ea);
			mdio_write(tp, 0x19, 0x4c52);
			mdio_write(tp, 0x15, 0x03eb);
			mdio_write(tp, 0x19, 0x4470);
			mdio_write(tp, 0x15, 0x03ec);
			mdio_write(tp, 0x19, 0x7c12);
			mdio_write(tp, 0x15, 0x03ed);
			mdio_write(tp, 0x19, 0x4c40);
			mdio_write(tp, 0x15, 0x03ee);
			mdio_write(tp, 0x19, 0x33da);
			mdio_write(tp, 0x15, 0x03ef);
			mdio_write(tp, 0x19, 0x3312);
			mdio_write(tp, 0x16, 0x0306);
			mdio_write(tp, 0x16, 0x0300);
			mdio_write(tp, 0x1f, 0x0000);
			mdio_write(tp, 0x17, 0x2179);
			mdio_write(tp, 0x1f, 0x0007);
			mdio_write(tp, 0x1e, 0x0040);
			mdio_write(tp, 0x18, 0x0645);
			mdio_write(tp, 0x19, 0xe200);
			mdio_write(tp, 0x18, 0x0655);
			mdio_write(tp, 0x19, 0x9000);
			mdio_write(tp, 0x18, 0x0d05);
			mdio_write(tp, 0x19, 0xbe00);
			mdio_write(tp, 0x18, 0x0d15);
			mdio_write(tp, 0x19, 0xd300);
			mdio_write(tp, 0x18, 0x0d25);
			mdio_write(tp, 0x19, 0xfe00);
			mdio_write(tp, 0x18, 0x0d35);
			mdio_write(tp, 0x19, 0x4000);
			mdio_write(tp, 0x18, 0x0d45);
			mdio_write(tp, 0x19, 0x7f00);
			mdio_write(tp, 0x18, 0x0d55);
			mdio_write(tp, 0x19, 0x1000);
			mdio_write(tp, 0x18, 0x0d65);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x18, 0x0d75);
			mdio_write(tp, 0x19, 0x8200);
			mdio_write(tp, 0x18, 0x0d85);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x18, 0x0d95);
			mdio_write(tp, 0x19, 0x7000);
			mdio_write(tp, 0x18, 0x0da5);
			mdio_write(tp, 0x19, 0x0f00);
			mdio_write(tp, 0x18, 0x0db5);
			mdio_write(tp, 0x19, 0x0100);
			mdio_write(tp, 0x18, 0x0dc5);
			mdio_write(tp, 0x19, 0x9b00);
			mdio_write(tp, 0x18, 0x0dd5);
			mdio_write(tp, 0x19, 0x7f00);
			mdio_write(tp, 0x18, 0x0de5);
			mdio_write(tp, 0x19, 0xe000);
			mdio_write(tp, 0x18, 0x0df5);
			mdio_write(tp, 0x19, 0xef00);
			mdio_write(tp, 0x18, 0x16d5);
			mdio_write(tp, 0x19, 0xe200);
			mdio_write(tp, 0x18, 0x16e5);
			mdio_write(tp, 0x19, 0xab00);
			mdio_write(tp, 0x18, 0x2904);
			mdio_write(tp, 0x19, 0x4000);
			mdio_write(tp, 0x18, 0x2914);
			mdio_write(tp, 0x19, 0x7f00);
			mdio_write(tp, 0x18, 0x2924);
			mdio_write(tp, 0x19, 0x0100);
			mdio_write(tp, 0x18, 0x2934);
			mdio_write(tp, 0x19, 0x2000);
			mdio_write(tp, 0x18, 0x2944);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x18, 0x2954);
			mdio_write(tp, 0x19, 0x4600);
			mdio_write(tp, 0x18, 0x2964);
			mdio_write(tp, 0x19, 0xfc00);
			mdio_write(tp, 0x18, 0x2974);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x18, 0x2984);
			mdio_write(tp, 0x19, 0x5000);
			mdio_write(tp, 0x18, 0x2994);
			mdio_write(tp, 0x19, 0x9d00);
			mdio_write(tp, 0x18, 0x29a4);
			mdio_write(tp, 0x19, 0xff00);
			mdio_write(tp, 0x18, 0x29b4);
			mdio_write(tp, 0x19, 0x4000);
			mdio_write(tp, 0x18, 0x29c4);
			mdio_write(tp, 0x19, 0x7f00);
			mdio_write(tp, 0x18, 0x29d4);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x18, 0x29e4);
			mdio_write(tp, 0x19, 0x2000);
			mdio_write(tp, 0x18, 0x29f4);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x18, 0x2a04);
			mdio_write(tp, 0x19, 0xe600);
			mdio_write(tp, 0x18, 0x2a14);
			mdio_write(tp, 0x19, 0xff00);
			mdio_write(tp, 0x18, 0x2a24);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x18, 0x2a34);
			mdio_write(tp, 0x19, 0x5000);
			mdio_write(tp, 0x18, 0x2a44);
			mdio_write(tp, 0x19, 0x8500);
			mdio_write(tp, 0x18, 0x2a54);
			mdio_write(tp, 0x19, 0x7f00);
			mdio_write(tp, 0x18, 0x2a64);
			mdio_write(tp, 0x19, 0xac00);
			mdio_write(tp, 0x18, 0x2a74);
			mdio_write(tp, 0x19, 0x0800);
			mdio_write(tp, 0x18, 0x2a84);
			mdio_write(tp, 0x19, 0xfc00);
			mdio_write(tp, 0x18, 0x2a94);
			mdio_write(tp, 0x19, 0xe000);
			mdio_write(tp, 0x18, 0x2aa4);
			mdio_write(tp, 0x19, 0x7400);
			mdio_write(tp, 0x18, 0x2ab4);
			mdio_write(tp, 0x19, 0x4000);
			mdio_write(tp, 0x18, 0x2ac4);
			mdio_write(tp, 0x19, 0x7f00);
			mdio_write(tp, 0x18, 0x2ad4);
			mdio_write(tp, 0x19, 0x0100);
			mdio_write(tp, 0x18, 0x2ae4);
			mdio_write(tp, 0x19, 0xff00);
			mdio_write(tp, 0x18, 0x2af4);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x18, 0x2b04);
			mdio_write(tp, 0x19, 0x4400);
			mdio_write(tp, 0x18, 0x2b14);
			mdio_write(tp, 0x19, 0xfc00);
			mdio_write(tp, 0x18, 0x2b24);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x18, 0x2b34);
			mdio_write(tp, 0x19, 0x4000);
			mdio_write(tp, 0x18, 0x2b44);
			mdio_write(tp, 0x19, 0x9d00);
			mdio_write(tp, 0x18, 0x2b54);
			mdio_write(tp, 0x19, 0xff00);
			mdio_write(tp, 0x18, 0x2b64);
			mdio_write(tp, 0x19, 0x4000);
			mdio_write(tp, 0x18, 0x2b74);
			mdio_write(tp, 0x19, 0x7f00);
			mdio_write(tp, 0x18, 0x2b84);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x18, 0x2b94);
			mdio_write(tp, 0x19, 0xff00);
			mdio_write(tp, 0x18, 0x2ba4);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x18, 0x2bb4);
			mdio_write(tp, 0x19, 0xfc00);
			mdio_write(tp, 0x18, 0x2bc4);
			mdio_write(tp, 0x19, 0xff00);
			mdio_write(tp, 0x18, 0x2bd4);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x18, 0x2be4);
			mdio_write(tp, 0x19, 0x4000);
			mdio_write(tp, 0x18, 0x2bf4);
			mdio_write(tp, 0x19, 0x8900);
			mdio_write(tp, 0x18, 0x2c04);
			mdio_write(tp, 0x19, 0x8300);
			mdio_write(tp, 0x18, 0x2c14);
			mdio_write(tp, 0x19, 0xe000);
			mdio_write(tp, 0x18, 0x2c24);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x18, 0x2c34);
			mdio_write(tp, 0x19, 0xac00);
			mdio_write(tp, 0x18, 0x2c44);
			mdio_write(tp, 0x19, 0x0800);
			mdio_write(tp, 0x18, 0x2c54);
			mdio_write(tp, 0x19, 0xfa00);
			mdio_write(tp, 0x18, 0x2c64);
			mdio_write(tp, 0x19, 0xe100);
			mdio_write(tp, 0x18, 0x2c74);
			mdio_write(tp, 0x19, 0x7f00);
			mdio_write(tp, 0x18, 0x0001);
			mdio_write(tp, 0x1f, 0x0000);
			mdio_write(tp, 0x17, 0x2100);
			mdio_write(tp, 0x1f, 0x0005);
			mdio_write(tp, 0x05, 0xfff6);
			mdio_write(tp, 0x06, 0x0080);
			mdio_write(tp, 0x05, 0x8b88);
			mdio_write(tp, 0x06, 0x0000);
			mdio_write(tp, 0x06, 0x0000);
			mdio_write(tp, 0x06, 0x0000);
			mdio_write(tp, 0x06, 0x0000);
			mdio_write(tp, 0x05, 0x8000);
			mdio_write(tp, 0x06, 0xd480);
			mdio_write(tp, 0x06, 0xc1e4);
			mdio_write(tp, 0x06, 0x8b9a);
			mdio_write(tp, 0x06, 0xe58b);
			mdio_write(tp, 0x06, 0x9bee);
			mdio_write(tp, 0x06, 0x8b83);
			mdio_write(tp, 0x06, 0x41bf);
			mdio_write(tp, 0x06, 0x8b88);
			mdio_write(tp, 0x06, 0xec00);
			mdio_write(tp, 0x06, 0x19a9);
			mdio_write(tp, 0x06, 0x8b90);
			mdio_write(tp, 0x06, 0xf9ee);
			mdio_write(tp, 0x06, 0xfff6);
			mdio_write(tp, 0x06, 0x00ee);
			mdio_write(tp, 0x06, 0xfff7);
			mdio_write(tp, 0x06, 0xffe0);
			mdio_write(tp, 0x06, 0xe140);
			mdio_write(tp, 0x06, 0xe1e1);
			mdio_write(tp, 0x06, 0x41f7);
			mdio_write(tp, 0x06, 0x2ff6);
			mdio_write(tp, 0x06, 0x28e4);
			mdio_write(tp, 0x06, 0xe140);
			mdio_write(tp, 0x06, 0xe5e1);
			mdio_write(tp, 0x06, 0x41f7);
			mdio_write(tp, 0x06, 0x0002);
			mdio_write(tp, 0x06, 0x020c);
			mdio_write(tp, 0x06, 0x0202);
			mdio_write(tp, 0x06, 0x1d02);
			mdio_write(tp, 0x06, 0x0230);
			mdio_write(tp, 0x06, 0x0202);
			mdio_write(tp, 0x06, 0x4002);
			mdio_write(tp, 0x06, 0x028b);
			mdio_write(tp, 0x06, 0x0280);
			mdio_write(tp, 0x06, 0x6c02);
			mdio_write(tp, 0x06, 0x8085);
			mdio_write(tp, 0x06, 0xe08b);
			mdio_write(tp, 0x06, 0x88e1);
			mdio_write(tp, 0x06, 0x8b89);
			mdio_write(tp, 0x06, 0x1e01);
			mdio_write(tp, 0x06, 0xe18b);
			mdio_write(tp, 0x06, 0x8a1e);
			mdio_write(tp, 0x06, 0x01e1);
			mdio_write(tp, 0x06, 0x8b8b);
			mdio_write(tp, 0x06, 0x1e01);
			mdio_write(tp, 0x06, 0xe18b);
			mdio_write(tp, 0x06, 0x8c1e);
			mdio_write(tp, 0x06, 0x01e1);
			mdio_write(tp, 0x06, 0x8b8d);
			mdio_write(tp, 0x06, 0x1e01);
			mdio_write(tp, 0x06, 0xe18b);
			mdio_write(tp, 0x06, 0x8e1e);
			mdio_write(tp, 0x06, 0x01a0);
			mdio_write(tp, 0x06, 0x00c7);
			mdio_write(tp, 0x06, 0xaec3);
			mdio_write(tp, 0x06, 0xf8e0);
			mdio_write(tp, 0x06, 0x8b8d);
			mdio_write(tp, 0x06, 0xad20);
			mdio_write(tp, 0x06, 0x10ee);
			mdio_write(tp, 0x06, 0x8b8d);
			mdio_write(tp, 0x06, 0x0002);
			mdio_write(tp, 0x06, 0x1310);
			mdio_write(tp, 0x06, 0x021f);
			mdio_write(tp, 0x06, 0x9d02);
			mdio_write(tp, 0x06, 0x1f0c);
			mdio_write(tp, 0x06, 0x0227);
			mdio_write(tp, 0x06, 0x49fc);
			mdio_write(tp, 0x06, 0x04f8);
			mdio_write(tp, 0x06, 0xe08b);
			mdio_write(tp, 0x06, 0x8ead);
			mdio_write(tp, 0x06, 0x200b);
			mdio_write(tp, 0x06, 0xf620);
			mdio_write(tp, 0x06, 0xe48b);
			mdio_write(tp, 0x06, 0x8e02);
			mdio_write(tp, 0x06, 0x830e);
			mdio_write(tp, 0x06, 0x021b);
			mdio_write(tp, 0x06, 0x67ad);
			mdio_write(tp, 0x06, 0x2211);
			mdio_write(tp, 0x06, 0xf622);
			mdio_write(tp, 0x06, 0xe48b);
			mdio_write(tp, 0x06, 0x8e02);
			mdio_write(tp, 0x06, 0x2ba5);
			mdio_write(tp, 0x06, 0x022a);
			mdio_write(tp, 0x06, 0x2402);
			mdio_write(tp, 0x06, 0x80c6);
			mdio_write(tp, 0x06, 0x022a);
			mdio_write(tp, 0x06, 0xf0ad);
			mdio_write(tp, 0x06, 0x2511);
			mdio_write(tp, 0x06, 0xf625);
			mdio_write(tp, 0x06, 0xe48b);
			mdio_write(tp, 0x06, 0x8e02);
			mdio_write(tp, 0x06, 0x8226);
			mdio_write(tp, 0x06, 0x0204);
			mdio_write(tp, 0x06, 0x0302);
			mdio_write(tp, 0x06, 0x19cc);
			mdio_write(tp, 0x06, 0x022b);
			mdio_write(tp, 0x06, 0x5bfc);
			mdio_write(tp, 0x06, 0x04ee);
			mdio_write(tp, 0x06, 0x8b8d);
			mdio_write(tp, 0x06, 0x0105);
			mdio_write(tp, 0x06, 0xf8e0);
			mdio_write(tp, 0x06, 0x8b83);
			mdio_write(tp, 0x06, 0xad24);
			mdio_write(tp, 0x06, 0x44e0);
			mdio_write(tp, 0x06, 0xe022);
			mdio_write(tp, 0x06, 0xe1e0);
			mdio_write(tp, 0x06, 0x23ad);
			mdio_write(tp, 0x06, 0x223b);
			mdio_write(tp, 0x06, 0xe08a);
			mdio_write(tp, 0x06, 0xbea0);
			mdio_write(tp, 0x06, 0x0005);
			mdio_write(tp, 0x06, 0x0228);
			mdio_write(tp, 0x06, 0xdeae);
			mdio_write(tp, 0x06, 0x42a0);
			mdio_write(tp, 0x06, 0x0105);
			mdio_write(tp, 0x06, 0x0228);
			mdio_write(tp, 0x06, 0xf1ae);
			mdio_write(tp, 0x06, 0x3aa0);
			mdio_write(tp, 0x06, 0x0205);
			mdio_write(tp, 0x06, 0x0281);
			mdio_write(tp, 0x06, 0x25ae);
			mdio_write(tp, 0x06, 0x32a0);
			mdio_write(tp, 0x06, 0x0305);
			mdio_write(tp, 0x06, 0x0229);
			mdio_write(tp, 0x06, 0x9aae);
			mdio_write(tp, 0x06, 0x2aa0);
			mdio_write(tp, 0x06, 0x0405);
			mdio_write(tp, 0x06, 0x0229);
			mdio_write(tp, 0x06, 0xaeae);
			mdio_write(tp, 0x06, 0x22a0);
			mdio_write(tp, 0x06, 0x0505);
			mdio_write(tp, 0x06, 0x0229);
			mdio_write(tp, 0x06, 0xd7ae);
			mdio_write(tp, 0x06, 0x1aa0);
			mdio_write(tp, 0x06, 0x0605);
			mdio_write(tp, 0x06, 0x0229);
			mdio_write(tp, 0x06, 0xfeae);
			mdio_write(tp, 0x06, 0x12ee);
			mdio_write(tp, 0x06, 0x8ac0);
			mdio_write(tp, 0x06, 0x00ee);
			mdio_write(tp, 0x06, 0x8ac1);
			mdio_write(tp, 0x06, 0x00ee);
			mdio_write(tp, 0x06, 0x8ac6);
			mdio_write(tp, 0x06, 0x00ee);
			mdio_write(tp, 0x06, 0x8abe);
			mdio_write(tp, 0x06, 0x00ae);
			mdio_write(tp, 0x06, 0x00fc);
			mdio_write(tp, 0x06, 0x04f8);
			mdio_write(tp, 0x06, 0x022a);
			mdio_write(tp, 0x06, 0x67e0);
			mdio_write(tp, 0x06, 0xe022);
			mdio_write(tp, 0x06, 0xe1e0);
			mdio_write(tp, 0x06, 0x230d);
			mdio_write(tp, 0x06, 0x0658);
			mdio_write(tp, 0x06, 0x03a0);
			mdio_write(tp, 0x06, 0x0202);
			mdio_write(tp, 0x06, 0xae2d);
			mdio_write(tp, 0x06, 0xa001);
			mdio_write(tp, 0x06, 0x02ae);
			mdio_write(tp, 0x06, 0x2da0);
			mdio_write(tp, 0x06, 0x004d);
			mdio_write(tp, 0x06, 0xe0e2);
			mdio_write(tp, 0x06, 0x00e1);
			mdio_write(tp, 0x06, 0xe201);
			mdio_write(tp, 0x06, 0xad24);
			mdio_write(tp, 0x06, 0x44e0);
			mdio_write(tp, 0x06, 0x8ac2);
			mdio_write(tp, 0x06, 0xe48a);
			mdio_write(tp, 0x06, 0xc4e0);
			mdio_write(tp, 0x06, 0x8ac3);
			mdio_write(tp, 0x06, 0xe48a);
			mdio_write(tp, 0x06, 0xc5ee);
			mdio_write(tp, 0x06, 0x8abe);
			mdio_write(tp, 0x06, 0x03e0);
			mdio_write(tp, 0x06, 0x8b83);
			mdio_write(tp, 0x06, 0xad25);
			mdio_write(tp, 0x06, 0x3aee);
			mdio_write(tp, 0x06, 0x8abe);
			mdio_write(tp, 0x06, 0x05ae);
			mdio_write(tp, 0x06, 0x34e0);
			mdio_write(tp, 0x06, 0x8ace);
			mdio_write(tp, 0x06, 0xae03);
			mdio_write(tp, 0x06, 0xe08a);
			mdio_write(tp, 0x06, 0xcfe1);
			mdio_write(tp, 0x06, 0x8ac2);
			mdio_write(tp, 0x06, 0x4905);
			mdio_write(tp, 0x06, 0xe58a);
			mdio_write(tp, 0x06, 0xc4e1);
			mdio_write(tp, 0x06, 0x8ac3);
			mdio_write(tp, 0x06, 0x4905);
			mdio_write(tp, 0x06, 0xe58a);
			mdio_write(tp, 0x06, 0xc5ee);
			mdio_write(tp, 0x06, 0x8abe);
			mdio_write(tp, 0x06, 0x0502);
			mdio_write(tp, 0x06, 0x2ab6);
			mdio_write(tp, 0x06, 0xac20);
			mdio_write(tp, 0x06, 0x1202);
			mdio_write(tp, 0x06, 0x819b);
			mdio_write(tp, 0x06, 0xac20);
			mdio_write(tp, 0x06, 0x0cee);
			mdio_write(tp, 0x06, 0x8ac1);
			mdio_write(tp, 0x06, 0x00ee);
			mdio_write(tp, 0x06, 0x8ac6);
			mdio_write(tp, 0x06, 0x00ee);
			mdio_write(tp, 0x06, 0x8abe);
			mdio_write(tp, 0x06, 0x02fc);
			mdio_write(tp, 0x06, 0x04d0);
			mdio_write(tp, 0x06, 0x0002);
			mdio_write(tp, 0x06, 0x81ad);
			mdio_write(tp, 0x06, 0x590f);
			mdio_write(tp, 0x06, 0x3902);
			mdio_write(tp, 0x06, 0xaa04);
			mdio_write(tp, 0x06, 0xd001);
			mdio_write(tp, 0x06, 0xae02);
			mdio_write(tp, 0x06, 0xd000);
			mdio_write(tp, 0x06, 0x04f9);
			mdio_write(tp, 0x06, 0xfae2);
			mdio_write(tp, 0x06, 0xe2d2);
			mdio_write(tp, 0x06, 0xe3e2);
			mdio_write(tp, 0x06, 0xd3f9);
			mdio_write(tp, 0x06, 0x5af7);
			mdio_write(tp, 0x06, 0xe6e2);
			mdio_write(tp, 0x06, 0xd2e7);
			mdio_write(tp, 0x06, 0xe2d3);
			mdio_write(tp, 0x06, 0xe2e0);
			mdio_write(tp, 0x06, 0x2ce3);
			mdio_write(tp, 0x06, 0xe02d);
			mdio_write(tp, 0x06, 0xf95b);
			mdio_write(tp, 0x06, 0xe01e);
			mdio_write(tp, 0x06, 0x30e6);
			mdio_write(tp, 0x06, 0xe02c);
			mdio_write(tp, 0x06, 0xe7e0);
			mdio_write(tp, 0x06, 0x2de2);
			mdio_write(tp, 0x06, 0xe2cc);
			mdio_write(tp, 0x06, 0xe3e2);
			mdio_write(tp, 0x06, 0xcdf9);
			mdio_write(tp, 0x06, 0x5a0f);
			mdio_write(tp, 0x06, 0x6a50);
			mdio_write(tp, 0x06, 0xe6e2);
			mdio_write(tp, 0x06, 0xcce7);
			mdio_write(tp, 0x06, 0xe2cd);
			mdio_write(tp, 0x06, 0xe0e0);
			mdio_write(tp, 0x06, 0x3ce1);
			mdio_write(tp, 0x06, 0xe03d);
			mdio_write(tp, 0x06, 0xef64);
			mdio_write(tp, 0x06, 0xfde0);
			mdio_write(tp, 0x06, 0xe2cc);
			mdio_write(tp, 0x06, 0xe1e2);
			mdio_write(tp, 0x06, 0xcd58);
			mdio_write(tp, 0x06, 0x0f5a);
			mdio_write(tp, 0x06, 0xf01e);
			mdio_write(tp, 0x06, 0x02e4);
			mdio_write(tp, 0x06, 0xe2cc);
			mdio_write(tp, 0x06, 0xe5e2);
			mdio_write(tp, 0x06, 0xcdfd);
			mdio_write(tp, 0x06, 0xe0e0);
			mdio_write(tp, 0x06, 0x2ce1);
			mdio_write(tp, 0x06, 0xe02d);
			mdio_write(tp, 0x06, 0x59e0);
			mdio_write(tp, 0x06, 0x5b1f);
			mdio_write(tp, 0x06, 0x1e13);
			mdio_write(tp, 0x06, 0xe4e0);
			mdio_write(tp, 0x06, 0x2ce5);
			mdio_write(tp, 0x06, 0xe02d);
			mdio_write(tp, 0x06, 0xfde0);
			mdio_write(tp, 0x06, 0xe2d2);
			mdio_write(tp, 0x06, 0xe1e2);
			mdio_write(tp, 0x06, 0xd358);
			mdio_write(tp, 0x06, 0xf75a);
			mdio_write(tp, 0x06, 0x081e);
			mdio_write(tp, 0x06, 0x02e4);
			mdio_write(tp, 0x06, 0xe2d2);
			mdio_write(tp, 0x06, 0xe5e2);
			mdio_write(tp, 0x06, 0xd3ef);
			mdio_write(tp, 0x06, 0x46fe);
			mdio_write(tp, 0x06, 0xfd04);
			mdio_write(tp, 0x06, 0xf8f9);
			mdio_write(tp, 0x06, 0xfaef);
			mdio_write(tp, 0x06, 0x69e0);
			mdio_write(tp, 0x06, 0xe022);
			mdio_write(tp, 0x06, 0xe1e0);
			mdio_write(tp, 0x06, 0x2358);
			mdio_write(tp, 0x06, 0xc4e1);
			mdio_write(tp, 0x06, 0x8b6e);
			mdio_write(tp, 0x06, 0x1f10);
			mdio_write(tp, 0x06, 0x9e58);
			mdio_write(tp, 0x06, 0xe48b);
			mdio_write(tp, 0x06, 0x6ead);
			mdio_write(tp, 0x06, 0x2222);
			mdio_write(tp, 0x06, 0xac27);
			mdio_write(tp, 0x06, 0x55ac);
			mdio_write(tp, 0x06, 0x2602);
			mdio_write(tp, 0x06, 0xae1a);
			mdio_write(tp, 0x06, 0xd106);
			mdio_write(tp, 0x06, 0xbf3b);
			mdio_write(tp, 0x06, 0xba02);
			mdio_write(tp, 0x06, 0x2dc1);
			mdio_write(tp, 0x06, 0xd107);
			mdio_write(tp, 0x06, 0xbf3b);
			mdio_write(tp, 0x06, 0xbd02);
			mdio_write(tp, 0x06, 0x2dc1);
			mdio_write(tp, 0x06, 0xd107);
			mdio_write(tp, 0x06, 0xbf3b);
			mdio_write(tp, 0x06, 0xc002);
			mdio_write(tp, 0x06, 0x2dc1);
			mdio_write(tp, 0x06, 0xae30);
			mdio_write(tp, 0x06, 0xd103);
			mdio_write(tp, 0x06, 0xbf3b);
			mdio_write(tp, 0x06, 0xc302);
			mdio_write(tp, 0x06, 0x2dc1);
			mdio_write(tp, 0x06, 0xd100);
			mdio_write(tp, 0x06, 0xbf3b);
			mdio_write(tp, 0x06, 0xc602);
			mdio_write(tp, 0x06, 0x2dc1);
			mdio_write(tp, 0x06, 0xd100);
			mdio_write(tp, 0x06, 0xbf82);
			mdio_write(tp, 0x06, 0xca02);
			mdio_write(tp, 0x06, 0x2dc1);
			mdio_write(tp, 0x06, 0xd10f);
			mdio_write(tp, 0x06, 0xbf3b);
			mdio_write(tp, 0x06, 0xba02);
			mdio_write(tp, 0x06, 0x2dc1);
			mdio_write(tp, 0x06, 0xd101);
			mdio_write(tp, 0x06, 0xbf3b);
			mdio_write(tp, 0x06, 0xbd02);
			mdio_write(tp, 0x06, 0x2dc1);
			mdio_write(tp, 0x06, 0xd101);
			mdio_write(tp, 0x06, 0xbf3b);
			mdio_write(tp, 0x06, 0xc002);
			mdio_write(tp, 0x06, 0x2dc1);
			mdio_write(tp, 0x06, 0xef96);
			mdio_write(tp, 0x06, 0xfefd);
			mdio_write(tp, 0x06, 0xfc04);
			mdio_write(tp, 0x06, 0xd100);
			mdio_write(tp, 0x06, 0xbf3b);
			mdio_write(tp, 0x06, 0xc302);
			mdio_write(tp, 0x06, 0x2dc1);
			mdio_write(tp, 0x06, 0xd011);
			mdio_write(tp, 0x06, 0x022b);
			mdio_write(tp, 0x06, 0xfb59);
			mdio_write(tp, 0x06, 0x03ef);
			mdio_write(tp, 0x06, 0x01d1);
			mdio_write(tp, 0x06, 0x00a0);
			mdio_write(tp, 0x06, 0x0002);
			mdio_write(tp, 0x06, 0xd101);
			mdio_write(tp, 0x06, 0xbf3b);
			mdio_write(tp, 0x06, 0xc602);
			mdio_write(tp, 0x06, 0x2dc1);
			mdio_write(tp, 0x06, 0xd111);
			mdio_write(tp, 0x06, 0xad20);
			mdio_write(tp, 0x06, 0x020c);
			mdio_write(tp, 0x06, 0x11ad);
			mdio_write(tp, 0x06, 0x2102);
			mdio_write(tp, 0x06, 0x0c12);
			mdio_write(tp, 0x06, 0xbf82);
			mdio_write(tp, 0x06, 0xca02);
			mdio_write(tp, 0x06, 0x2dc1);
			mdio_write(tp, 0x06, 0xaec8);
			mdio_write(tp, 0x06, 0x70e4);
			mdio_write(tp, 0x06, 0x2602);
			mdio_write(tp, 0x06, 0x82d1);
			mdio_write(tp, 0x06, 0x05f8);
			mdio_write(tp, 0x06, 0xfaef);
			mdio_write(tp, 0x06, 0x69e0);
			mdio_write(tp, 0x06, 0xe2fe);
			mdio_write(tp, 0x06, 0xe1e2);
			mdio_write(tp, 0x06, 0xffad);
			mdio_write(tp, 0x06, 0x2d1a);
			mdio_write(tp, 0x06, 0xe0e1);
			mdio_write(tp, 0x06, 0x4ee1);
			mdio_write(tp, 0x06, 0xe14f);
			mdio_write(tp, 0x06, 0xac2d);
			mdio_write(tp, 0x06, 0x22f6);
			mdio_write(tp, 0x06, 0x0302);
			mdio_write(tp, 0x06, 0x033b);
			mdio_write(tp, 0x06, 0xf703);
			mdio_write(tp, 0x06, 0xf706);
			mdio_write(tp, 0x06, 0xbf84);
			mdio_write(tp, 0x06, 0x4402);
			mdio_write(tp, 0x06, 0x2d21);
			mdio_write(tp, 0x06, 0xae11);
			mdio_write(tp, 0x06, 0xe0e1);
			mdio_write(tp, 0x06, 0x4ee1);
			mdio_write(tp, 0x06, 0xe14f);
			mdio_write(tp, 0x06, 0xad2d);
			mdio_write(tp, 0x06, 0x08bf);
			mdio_write(tp, 0x06, 0x844f);
			mdio_write(tp, 0x06, 0x022d);
			mdio_write(tp, 0x06, 0x21f6);
			mdio_write(tp, 0x06, 0x06ef);
			mdio_write(tp, 0x06, 0x96fe);
			mdio_write(tp, 0x06, 0xfc04);
			mdio_write(tp, 0x06, 0xf8fa);
			mdio_write(tp, 0x06, 0xef69);
			mdio_write(tp, 0x06, 0x0283);
			mdio_write(tp, 0x06, 0x4502);
			mdio_write(tp, 0x06, 0x83a2);
			mdio_write(tp, 0x06, 0xe0e0);
			mdio_write(tp, 0x06, 0x00e1);
			mdio_write(tp, 0x06, 0xe001);
			mdio_write(tp, 0x06, 0xad27);
			mdio_write(tp, 0x06, 0x1fd1);
			mdio_write(tp, 0x06, 0x01bf);
			mdio_write(tp, 0x06, 0x843b);
			mdio_write(tp, 0x06, 0x022d);
			mdio_write(tp, 0x06, 0xc1e0);
			mdio_write(tp, 0x06, 0xe020);
			mdio_write(tp, 0x06, 0xe1e0);
			mdio_write(tp, 0x06, 0x21ad);
			mdio_write(tp, 0x06, 0x200e);
			mdio_write(tp, 0x06, 0xd100);
			mdio_write(tp, 0x06, 0xbf84);
			mdio_write(tp, 0x06, 0x3b02);
			mdio_write(tp, 0x06, 0x2dc1);
			mdio_write(tp, 0x06, 0xbf3b);
			mdio_write(tp, 0x06, 0x9602);
			mdio_write(tp, 0x06, 0x2d21);
			mdio_write(tp, 0x06, 0xef96);
			mdio_write(tp, 0x06, 0xfefc);
			mdio_write(tp, 0x06, 0x04f8);
			mdio_write(tp, 0x06, 0xf9fa);
			mdio_write(tp, 0x06, 0xef69);
			mdio_write(tp, 0x06, 0xe08b);
			mdio_write(tp, 0x06, 0x87ad);
			mdio_write(tp, 0x06, 0x204c);
			mdio_write(tp, 0x06, 0xd200);
			mdio_write(tp, 0x06, 0xe0e2);
			mdio_write(tp, 0x06, 0x0058);
			mdio_write(tp, 0x06, 0x010c);
			mdio_write(tp, 0x06, 0x021e);
			mdio_write(tp, 0x06, 0x20e0);
			mdio_write(tp, 0x06, 0xe000);
			mdio_write(tp, 0x06, 0x5810);
			mdio_write(tp, 0x06, 0x1e20);
			mdio_write(tp, 0x06, 0xe0e0);
			mdio_write(tp, 0x06, 0x3658);
			mdio_write(tp, 0x06, 0x031e);
			mdio_write(tp, 0x06, 0x20e0);
			mdio_write(tp, 0x06, 0xe022);
			mdio_write(tp, 0x06, 0xe1e0);
			mdio_write(tp, 0x06, 0x2358);
			mdio_write(tp, 0x06, 0xe01e);
			mdio_write(tp, 0x06, 0x20e0);
			mdio_write(tp, 0x06, 0x8b64);
			mdio_write(tp, 0x06, 0x1f02);
			mdio_write(tp, 0x06, 0x9e22);
			mdio_write(tp, 0x06, 0xe68b);
			mdio_write(tp, 0x06, 0x64ad);
			mdio_write(tp, 0x06, 0x3214);
			mdio_write(tp, 0x06, 0xad34);
			mdio_write(tp, 0x06, 0x11ef);
			mdio_write(tp, 0x06, 0x0258);
			mdio_write(tp, 0x06, 0x039e);
			mdio_write(tp, 0x06, 0x07ad);
			mdio_write(tp, 0x06, 0x3508);
			mdio_write(tp, 0x06, 0x5ac0);
			mdio_write(tp, 0x06, 0x9f04);
			mdio_write(tp, 0x06, 0xd101);
			mdio_write(tp, 0x06, 0xae02);
			mdio_write(tp, 0x06, 0xd100);
			mdio_write(tp, 0x06, 0xbf84);
			mdio_write(tp, 0x06, 0x3e02);
			mdio_write(tp, 0x06, 0x2dc1);
			mdio_write(tp, 0x06, 0xef96);
			mdio_write(tp, 0x06, 0xfefd);
			mdio_write(tp, 0x06, 0xfc04);
			mdio_write(tp, 0x06, 0xf8f9);
			mdio_write(tp, 0x06, 0xfbe0);
			mdio_write(tp, 0x06, 0x8b85);
			mdio_write(tp, 0x06, 0xad25);
			mdio_write(tp, 0x06, 0x22e0);
			mdio_write(tp, 0x06, 0xe022);
			mdio_write(tp, 0x06, 0xe1e0);
			mdio_write(tp, 0x06, 0x23e2);
			mdio_write(tp, 0x06, 0xe036);
			mdio_write(tp, 0x06, 0xe3e0);
			mdio_write(tp, 0x06, 0x375a);
			mdio_write(tp, 0x06, 0xc40d);
			mdio_write(tp, 0x06, 0x0158);
			mdio_write(tp, 0x06, 0x021e);
			mdio_write(tp, 0x06, 0x20e3);
			mdio_write(tp, 0x06, 0x8ae7);
			mdio_write(tp, 0x06, 0xac31);
			mdio_write(tp, 0x06, 0x60ac);
			mdio_write(tp, 0x06, 0x3a08);
			mdio_write(tp, 0x06, 0xac3e);
			mdio_write(tp, 0x06, 0x26ae);
			mdio_write(tp, 0x06, 0x67af);
			mdio_write(tp, 0x06, 0x8437);
			mdio_write(tp, 0x06, 0xad37);
			mdio_write(tp, 0x06, 0x61e0);
			mdio_write(tp, 0x06, 0x8ae8);
			mdio_write(tp, 0x06, 0x10e4);
			mdio_write(tp, 0x06, 0x8ae8);
			mdio_write(tp, 0x06, 0xe18a);
			mdio_write(tp, 0x06, 0xe91b);
			mdio_write(tp, 0x06, 0x109e);
			mdio_write(tp, 0x06, 0x02ae);
			mdio_write(tp, 0x06, 0x51d1);
			mdio_write(tp, 0x06, 0x00bf);
			mdio_write(tp, 0x06, 0x8441);
			mdio_write(tp, 0x06, 0x022d);
			mdio_write(tp, 0x06, 0xc1ee);
			mdio_write(tp, 0x06, 0x8ae8);
			mdio_write(tp, 0x06, 0x00ae);
			mdio_write(tp, 0x06, 0x43ad);
			mdio_write(tp, 0x06, 0x3627);
			mdio_write(tp, 0x06, 0xe08a);
			mdio_write(tp, 0x06, 0xeee1);
			mdio_write(tp, 0x06, 0x8aef);
			mdio_write(tp, 0x06, 0xef74);
			mdio_write(tp, 0x06, 0xe08a);
			mdio_write(tp, 0x06, 0xeae1);
			mdio_write(tp, 0x06, 0x8aeb);
			mdio_write(tp, 0x06, 0x1b74);
			mdio_write(tp, 0x06, 0x9e2e);
			mdio_write(tp, 0x06, 0x14e4);
			mdio_write(tp, 0x06, 0x8aea);
			mdio_write(tp, 0x06, 0xe58a);
			mdio_write(tp, 0x06, 0xebef);
			mdio_write(tp, 0x06, 0x74e0);
			mdio_write(tp, 0x06, 0x8aee);
			mdio_write(tp, 0x06, 0xe18a);
			mdio_write(tp, 0x06, 0xef1b);
			mdio_write(tp, 0x06, 0x479e);
			mdio_write(tp, 0x06, 0x0fae);
			mdio_write(tp, 0x06, 0x19ee);
			mdio_write(tp, 0x06, 0x8aea);
			mdio_write(tp, 0x06, 0x00ee);
			mdio_write(tp, 0x06, 0x8aeb);
			mdio_write(tp, 0x06, 0x00ae);
			mdio_write(tp, 0x06, 0x0fac);
			mdio_write(tp, 0x06, 0x390c);
			mdio_write(tp, 0x06, 0xd101);
			mdio_write(tp, 0x06, 0xbf84);
			mdio_write(tp, 0x06, 0x4102);
			mdio_write(tp, 0x06, 0x2dc1);
			mdio_write(tp, 0x06, 0xee8a);
			mdio_write(tp, 0x06, 0xe800);
			mdio_write(tp, 0x06, 0xe68a);
			mdio_write(tp, 0x06, 0xe7ff);
			mdio_write(tp, 0x06, 0xfdfc);
			mdio_write(tp, 0x06, 0x0400);
			mdio_write(tp, 0x06, 0xe234);
			mdio_write(tp, 0x06, 0xcce2);
			mdio_write(tp, 0x06, 0x0088);
			mdio_write(tp, 0x06, 0xe200);
			mdio_write(tp, 0x06, 0xa725);
			mdio_write(tp, 0x06, 0xe50a);
			mdio_write(tp, 0x06, 0x1de5);
			mdio_write(tp, 0x06, 0x0a2c);
			mdio_write(tp, 0x06, 0xe50a);
			mdio_write(tp, 0x06, 0x6de5);
			mdio_write(tp, 0x06, 0x0a1d);
			mdio_write(tp, 0x06, 0xe50a);
			mdio_write(tp, 0x06, 0x1ce5);
			mdio_write(tp, 0x06, 0x0a2d);
			mdio_write(tp, 0x06, 0xa755);
			mdio_write(tp, 0x05, 0x8b64);
			mdio_write(tp, 0x06, 0x0000);
			mdio_write(tp, 0x05, 0x8b94);
			mdio_write(tp, 0x06, 0x82cd);
			mdio_write(tp, 0x05, 0x8b85);
			mdio_write(tp, 0x06, 0x2000);
			mdio_write(tp, 0x05, 0x8aee);
			mdio_write(tp, 0x06, 0x03b8);
			mdio_write(tp, 0x05, 0x8ae8);
			mdio_write(tp, 0x06, 0x0002);
			gphy_val = mdio_read(tp, 0x01);
			gphy_val |= BIT_0;
			mdio_write(tp, 0x01, gphy_val);
			gphy_val = mdio_read(tp, 0x00);
			gphy_val |= BIT_0;
			mdio_write(tp, 0x00, gphy_val);
			mdio_write(tp, 0x1f, 0x0000);
			mdio_write(tp, 0x1f, 0x0005);
			for (i = 0; i < 200; i++) {
				udelay(100);
				gphy_val = mdio_read(tp, 0x00);
				if (gphy_val & BIT_7)
					break;
			}
			mdio_write(tp, 0x1f, 0x0007);
			mdio_write(tp, 0x1e, 0x0023);
			gphy_val = mdio_read(tp, 0x17);
			gphy_val &= ~(BIT_0);
			if ((subsystem_vendor == 0x144d &&
			     subsystem_device == 0xc098) ||
			    (subsystem_vendor == 0x144d &&
			     subsystem_device == 0xc0b1)) {
				gphy_val &= ~(BIT_2);
			}
			mdio_write(tp, 0x17, gphy_val);
			mdio_write(tp, 0x1f, 0x0007);
			mdio_write(tp, 0x1e, 0x0028);
			mdio_write(tp, 0x15, 0x0010);
			mdio_write(tp, 0x1f, 0x0007);
			mdio_write(tp, 0x1e, 0x0041);
			mdio_write(tp, 0x15, 0x0802);
			mdio_write(tp, 0x16, 0x2185);
			mdio_write(tp, 0x1f, 0x0000);
		} else {
			mdio_write(tp, 0x1f, 0x0000);
			mdio_write(tp, 0x00, 0x1800);
			mdio_write(tp, 0x1f, 0x0007);
			mdio_write(tp, 0x1e, 0x0023);
			mdio_write(tp, 0x17, 0x0117);
			mdio_write(tp, 0x1f, 0x0007);
			mdio_write(tp, 0x1E, 0x002C);
			mdio_write(tp, 0x1B, 0x5000);
			mdio_write(tp, 0x1f, 0x0000);
			mdio_write(tp, 0x16, 0x4104);
			for (i = 0; i < 200; i++) {
				udelay(100);
				gphy_val = mdio_read(tp, 0x1E);
				gphy_val &= 0x03FF;
				if (gphy_val==0x000C)
					break;
			}
			mdio_write(tp, 0x1f, 0x0005);
			for (i = 0; i < 200; i++) {
				udelay(100);
				gphy_val = mdio_read(tp, 0x07);
				if ((gphy_val & BIT_5) == 0)
					break;
			}
			gphy_val = mdio_read(tp, 0x07);
			if (gphy_val & BIT_5) {
				mdio_write(tp, 0x1f, 0x0007);
				mdio_write(tp, 0x1e, 0x00a1);
				mdio_write(tp, 0x17, 0x1000);
				mdio_write(tp, 0x17, 0x0000);
				mdio_write(tp, 0x17, 0x2000);
				mdio_write(tp, 0x1e, 0x002f);
				mdio_write(tp, 0x18, 0x9bfb);
				mdio_write(tp, 0x1f, 0x0005);
				mdio_write(tp, 0x07, 0x0000);
				mdio_write(tp, 0x1f, 0x0000);
			}
			mdio_write(tp, 0x1f, 0x0005);
			mdio_write(tp, 0x05, 0xfff6);
			mdio_write(tp, 0x06, 0x0080);
			gphy_val = mdio_read(tp, 0x00);
			gphy_val &= ~(BIT_7);
			mdio_write(tp, 0x00, gphy_val);
			mdio_write(tp, 0x1f, 0x0002);
			gphy_val = mdio_read(tp, 0x08);
			gphy_val &= ~(BIT_7);
			mdio_write(tp, 0x08, gphy_val);
			mdio_write(tp, 0x1f, 0x0000);
			mdio_write(tp, 0x1f, 0x0007);
			mdio_write(tp, 0x1e, 0x0023);
			mdio_write(tp, 0x16, 0x0306);
			mdio_write(tp, 0x16, 0x0307);
			mdio_write(tp, 0x15, 0x000e);
			mdio_write(tp, 0x19, 0x000a);
			mdio_write(tp, 0x15, 0x0010);
			mdio_write(tp, 0x19, 0x0008);
			mdio_write(tp, 0x15, 0x0018);
			mdio_write(tp, 0x19, 0x4801);
			mdio_write(tp, 0x15, 0x0019);
			mdio_write(tp, 0x19, 0x6801);
			mdio_write(tp, 0x15, 0x001a);
			mdio_write(tp, 0x19, 0x66a1);
			mdio_write(tp, 0x15, 0x001f);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0020);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0021);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0022);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0023);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0024);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0025);
			mdio_write(tp, 0x19, 0x64a1);
			mdio_write(tp, 0x15, 0x0026);
			mdio_write(tp, 0x19, 0x40ea);
			mdio_write(tp, 0x15, 0x0027);
			mdio_write(tp, 0x19, 0x4503);
			mdio_write(tp, 0x15, 0x0028);
			mdio_write(tp, 0x19, 0x9f00);
			mdio_write(tp, 0x15, 0x0029);
			mdio_write(tp, 0x19, 0xa631);
			mdio_write(tp, 0x15, 0x002a);
			mdio_write(tp, 0x19, 0x9717);
			mdio_write(tp, 0x15, 0x002b);
			mdio_write(tp, 0x19, 0x302c);
			mdio_write(tp, 0x15, 0x002c);
			mdio_write(tp, 0x19, 0x4802);
			mdio_write(tp, 0x15, 0x002d);
			mdio_write(tp, 0x19, 0x58da);
			mdio_write(tp, 0x15, 0x002e);
			mdio_write(tp, 0x19, 0x400d);
			mdio_write(tp, 0x15, 0x002f);
			mdio_write(tp, 0x19, 0x4488);
			mdio_write(tp, 0x15, 0x0030);
			mdio_write(tp, 0x19, 0x9e00);
			mdio_write(tp, 0x15, 0x0031);
			mdio_write(tp, 0x19, 0x63c8);
			mdio_write(tp, 0x15, 0x0032);
			mdio_write(tp, 0x19, 0x6481);
			mdio_write(tp, 0x15, 0x0033);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0034);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0035);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0036);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0037);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0038);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0039);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x003a);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x003b);
			mdio_write(tp, 0x19, 0x63e8);
			mdio_write(tp, 0x15, 0x003c);
			mdio_write(tp, 0x19, 0x7d00);
			mdio_write(tp, 0x15, 0x003d);
			mdio_write(tp, 0x19, 0x59d4);
			mdio_write(tp, 0x15, 0x003e);
			mdio_write(tp, 0x19, 0x63f8);
			mdio_write(tp, 0x15, 0x0040);
			mdio_write(tp, 0x19, 0x64a1);
			mdio_write(tp, 0x15, 0x0041);
			mdio_write(tp, 0x19, 0x30de);
			mdio_write(tp, 0x15, 0x0044);
			mdio_write(tp, 0x19, 0x480f);
			mdio_write(tp, 0x15, 0x0045);
			mdio_write(tp, 0x19, 0x6800);
			mdio_write(tp, 0x15, 0x0046);
			mdio_write(tp, 0x19, 0x6680);
			mdio_write(tp, 0x15, 0x0047);
			mdio_write(tp, 0x19, 0x7c10);
			mdio_write(tp, 0x15, 0x0048);
			mdio_write(tp, 0x19, 0x63c8);
			mdio_write(tp, 0x15, 0x0049);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x004a);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x004b);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x004c);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x004d);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x004e);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x004f);
			mdio_write(tp, 0x19, 0x40ea);
			mdio_write(tp, 0x15, 0x0050);
			mdio_write(tp, 0x19, 0x4503);
			mdio_write(tp, 0x15, 0x0051);
			mdio_write(tp, 0x19, 0x58ca);
			mdio_write(tp, 0x15, 0x0052);
			mdio_write(tp, 0x19, 0x63c8);
			mdio_write(tp, 0x15, 0x0053);
			mdio_write(tp, 0x19, 0x63d8);
			mdio_write(tp, 0x15, 0x0054);
			mdio_write(tp, 0x19, 0x66a0);
			mdio_write(tp, 0x15, 0x0055);
			mdio_write(tp, 0x19, 0x9f00);
			mdio_write(tp, 0x15, 0x0056);
			mdio_write(tp, 0x19, 0x3000);
			mdio_write(tp, 0x15, 0x00a1);
			mdio_write(tp, 0x19, 0x3044);
			mdio_write(tp, 0x15, 0x00ab);
			mdio_write(tp, 0x19, 0x5820);
			mdio_write(tp, 0x15, 0x00ac);
			mdio_write(tp, 0x19, 0x5e04);
			mdio_write(tp, 0x15, 0x00ad);
			mdio_write(tp, 0x19, 0xb60c);
			mdio_write(tp, 0x15, 0x00af);
			mdio_write(tp, 0x19, 0x000a);
			mdio_write(tp, 0x15, 0x00b2);
			mdio_write(tp, 0x19, 0x30b9);
			mdio_write(tp, 0x15, 0x00b9);
			mdio_write(tp, 0x19, 0x4408);
			mdio_write(tp, 0x15, 0x00ba);
			mdio_write(tp, 0x19, 0x480b);
			mdio_write(tp, 0x15, 0x00bb);
			mdio_write(tp, 0x19, 0x5e00);
			mdio_write(tp, 0x15, 0x00bc);
			mdio_write(tp, 0x19, 0x405f);
			mdio_write(tp, 0x15, 0x00bd);
			mdio_write(tp, 0x19, 0x4448);
			mdio_write(tp, 0x15, 0x00be);
			mdio_write(tp, 0x19, 0x4020);
			mdio_write(tp, 0x15, 0x00bf);
			mdio_write(tp, 0x19, 0x4468);
			mdio_write(tp, 0x15, 0x00c0);
			mdio_write(tp, 0x19, 0x9c02);
			mdio_write(tp, 0x15, 0x00c1);
			mdio_write(tp, 0x19, 0x58a0);
			mdio_write(tp, 0x15, 0x00c2);
			mdio_write(tp, 0x19, 0xb605);
			mdio_write(tp, 0x15, 0x00c3);
			mdio_write(tp, 0x19, 0xc0d3);
			mdio_write(tp, 0x15, 0x00c4);
			mdio_write(tp, 0x19, 0x00e6);
			mdio_write(tp, 0x15, 0x00c5);
			mdio_write(tp, 0x19, 0xdaec);
			mdio_write(tp, 0x15, 0x00c6);
			mdio_write(tp, 0x19, 0x00fa);
			mdio_write(tp, 0x15, 0x00c7);
			mdio_write(tp, 0x19, 0x9df9);
			mdio_write(tp, 0x15, 0x0112);
			mdio_write(tp, 0x19, 0x6421);
			mdio_write(tp, 0x15, 0x0113);
			mdio_write(tp, 0x19, 0x7c08);
			mdio_write(tp, 0x15, 0x0114);
			mdio_write(tp, 0x19, 0x63f0);
			mdio_write(tp, 0x15, 0x0115);
			mdio_write(tp, 0x19, 0x4003);
			mdio_write(tp, 0x15, 0x0116);
			mdio_write(tp, 0x19, 0x4418);
			mdio_write(tp, 0x15, 0x0117);
			mdio_write(tp, 0x19, 0x9b00);
			mdio_write(tp, 0x15, 0x0118);
			mdio_write(tp, 0x19, 0x6461);
			mdio_write(tp, 0x15, 0x0119);
			mdio_write(tp, 0x19, 0x64e1);
			mdio_write(tp, 0x15, 0x011a);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0150);
			mdio_write(tp, 0x19, 0x7c80);
			mdio_write(tp, 0x15, 0x0151);
			mdio_write(tp, 0x19, 0x6461);
			mdio_write(tp, 0x15, 0x0152);
			mdio_write(tp, 0x19, 0x4003);
			mdio_write(tp, 0x15, 0x0153);
			mdio_write(tp, 0x19, 0x4540);
			mdio_write(tp, 0x15, 0x0154);
			mdio_write(tp, 0x19, 0x9f00);
			mdio_write(tp, 0x15, 0x0155);
			mdio_write(tp, 0x19, 0x9d00);
			mdio_write(tp, 0x15, 0x0156);
			mdio_write(tp, 0x19, 0x7c40);
			mdio_write(tp, 0x15, 0x0157);
			mdio_write(tp, 0x19, 0x6421);
			mdio_write(tp, 0x15, 0x0158);
			mdio_write(tp, 0x19, 0x7c80);
			mdio_write(tp, 0x15, 0x0159);
			mdio_write(tp, 0x19, 0x64a1);
			mdio_write(tp, 0x15, 0x015a);
			mdio_write(tp, 0x19, 0x30fe);
			mdio_write(tp, 0x15, 0x02e7);
			mdio_write(tp, 0x19, 0x0000);
			mdio_write(tp, 0x15, 0x0329);
			mdio_write(tp, 0x19, 0x7c00);
			mdio_write(tp, 0x15, 0x0382);
			mdio_write(tp, 0x19, 0x7c40);
			mdio_write(tp, 0x15, 0x03bd);
			mdio_write(tp, 0x19, 0x405e);
			mdio_write(tp, 0x15, 0x03c9);
			mdio_write(tp, 0x19, 0x7c00);
			mdio_write(tp, 0x15, 0x03e3);
			mdio_write(tp, 0x19, 0x7c00);
			mdio_write(tp, 0x16, 0x0306);
			mdio_write(tp, 0x16, 0x0300);
			mdio_write(tp, 0x1f, 0x0005);
			mdio_write(tp, 0x05, 0xfff6);
			mdio_write(tp, 0x06, 0x0080);
			mdio_write(tp, 0x05, 0x8000);
			mdio_write(tp, 0x06, 0x0280);
			mdio_write(tp, 0x06, 0x48f7);
			mdio_write(tp, 0x06, 0x00e0);
			mdio_write(tp, 0x06, 0xfff7);
			mdio_write(tp, 0x06, 0xa080);
			mdio_write(tp, 0x06, 0x02ae);
			mdio_write(tp, 0x06, 0xf602);
			mdio_write(tp, 0x06, 0x0200);
			mdio_write(tp, 0x06, 0x0280);
			mdio_write(tp, 0x06, 0x8c02);
			mdio_write(tp, 0x06, 0x0224);
			mdio_write(tp, 0x06, 0x0202);
			mdio_write(tp, 0x06, 0x3402);
			mdio_write(tp, 0x06, 0x027f);
			mdio_write(tp, 0x06, 0x0280);
			mdio_write(tp, 0x06, 0xa202);
			mdio_write(tp, 0x06, 0x80bb);
			mdio_write(tp, 0x06, 0xe08b);
			mdio_write(tp, 0x06, 0x88e1);
			mdio_write(tp, 0x06, 0x8b89);
			mdio_write(tp, 0x06, 0x1e01);
			mdio_write(tp, 0x06, 0xe18b);
			mdio_write(tp, 0x06, 0x8a1e);
			mdio_write(tp, 0x06, 0x01e1);
			mdio_write(tp, 0x06, 0x8b8b);
			mdio_write(tp, 0x06, 0x1e01);
			mdio_write(tp, 0x06, 0xe18b);
			mdio_write(tp, 0x06, 0x8c1e);
			mdio_write(tp, 0x06, 0x01e1);
			mdio_write(tp, 0x06, 0x8b8d);
			mdio_write(tp, 0x06, 0x1e01);
			mdio_write(tp, 0x06, 0xe18b);
			mdio_write(tp, 0x06, 0x8e1e);
			mdio_write(tp, 0x06, 0x01a0);
			mdio_write(tp, 0x06, 0x00c7);
			mdio_write(tp, 0x06, 0xaebb);
			mdio_write(tp, 0x06, 0xee8a);
			mdio_write(tp, 0x06, 0xe600);
			mdio_write(tp, 0x06, 0xee8a);
			mdio_write(tp, 0x06, 0xee03);
			mdio_write(tp, 0x06, 0xee8a);
			mdio_write(tp, 0x06, 0xefb8);
			mdio_write(tp, 0x06, 0xee8a);
			mdio_write(tp, 0x06, 0xe902);
			mdio_write(tp, 0x06, 0xee8b);
			mdio_write(tp, 0x06, 0x8520);
			mdio_write(tp, 0x06, 0xee8b);
			mdio_write(tp, 0x06, 0x8701);
			mdio_write(tp, 0x06, 0xd481);
			mdio_write(tp, 0x06, 0x31e4);
			mdio_write(tp, 0x06, 0x8b94);
			mdio_write(tp, 0x06, 0xe58b);
			mdio_write(tp, 0x06, 0x95bf);
			mdio_write(tp, 0x06, 0x8b88);
			mdio_write(tp, 0x06, 0xec00);
			mdio_write(tp, 0x06, 0x19a9);
			mdio_write(tp, 0x06, 0x8b90);
			mdio_write(tp, 0x06, 0xf9ee);
			mdio_write(tp, 0x06, 0xfff6);
			mdio_write(tp, 0x06, 0x00ee);
			mdio_write(tp, 0x06, 0xfff7);
			mdio_write(tp, 0x06, 0xffe0);
			mdio_write(tp, 0x06, 0xe140);
			mdio_write(tp, 0x06, 0xe1e1);
			mdio_write(tp, 0x06, 0x41f7);
			mdio_write(tp, 0x06, 0x2ff6);
			mdio_write(tp, 0x06, 0x28e4);
			mdio_write(tp, 0x06, 0xe140);
			mdio_write(tp, 0x06, 0xe5e1);
			mdio_write(tp, 0x06, 0x4104);
			mdio_write(tp, 0x06, 0xf8e0);
			mdio_write(tp, 0x06, 0x8b89);
			mdio_write(tp, 0x06, 0xad20);
			mdio_write(tp, 0x06, 0x0dee);
			mdio_write(tp, 0x06, 0x8b89);
			mdio_write(tp, 0x06, 0x0002);
			mdio_write(tp, 0x06, 0x82ed);
			mdio_write(tp, 0x06, 0x021f);
			mdio_write(tp, 0x06, 0x4102);
			mdio_write(tp, 0x06, 0x2812);
			mdio_write(tp, 0x06, 0xfc04);
			mdio_write(tp, 0x06, 0xf8e0);
			mdio_write(tp, 0x06, 0x8b8d);
			mdio_write(tp, 0x06, 0xad20);
			mdio_write(tp, 0x06, 0x10ee);
			mdio_write(tp, 0x06, 0x8b8d);
			mdio_write(tp, 0x06, 0x0002);
			mdio_write(tp, 0x06, 0x139d);
			mdio_write(tp, 0x06, 0x0281);
			mdio_write(tp, 0x06, 0xcf02);
			mdio_write(tp, 0x06, 0x1f99);
			mdio_write(tp, 0x06, 0x0227);
			mdio_write(tp, 0x06, 0xeafc);
			mdio_write(tp, 0x06, 0x04f8);
			mdio_write(tp, 0x06, 0xe08b);
			mdio_write(tp, 0x06, 0x8ead);
			mdio_write(tp, 0x06, 0x2014);
			mdio_write(tp, 0x06, 0xf620);
			mdio_write(tp, 0x06, 0xe48b);
			mdio_write(tp, 0x06, 0x8e02);
			mdio_write(tp, 0x06, 0x8100);
			mdio_write(tp, 0x06, 0x021b);
			mdio_write(tp, 0x06, 0xf402);
			mdio_write(tp, 0x06, 0x2c9c);
			mdio_write(tp, 0x06, 0x0281);
			mdio_write(tp, 0x06, 0x7202);
			mdio_write(tp, 0x06, 0x843c);
			mdio_write(tp, 0x06, 0xad22);
			mdio_write(tp, 0x06, 0x11f6);
			mdio_write(tp, 0x06, 0x22e4);
			mdio_write(tp, 0x06, 0x8b8e);
			mdio_write(tp, 0x06, 0x022c);
			mdio_write(tp, 0x06, 0x4602);
			mdio_write(tp, 0x06, 0x2ac5);
			mdio_write(tp, 0x06, 0x0229);
			mdio_write(tp, 0x06, 0x2002);
			mdio_write(tp, 0x06, 0x2b91);
			mdio_write(tp, 0x06, 0xad25);
			mdio_write(tp, 0x06, 0x11f6);
			mdio_write(tp, 0x06, 0x25e4);
			mdio_write(tp, 0x06, 0x8b8e);
			mdio_write(tp, 0x06, 0x0203);
			mdio_write(tp, 0x06, 0x5a02);
			mdio_write(tp, 0x06, 0x043a);
			mdio_write(tp, 0x06, 0x021a);
			mdio_write(tp, 0x06, 0x5902);
			mdio_write(tp, 0x06, 0x2bfc);
			mdio_write(tp, 0x06, 0xfc04);
			mdio_write(tp, 0x06, 0xf8fa);
			mdio_write(tp, 0x06, 0xef69);
			mdio_write(tp, 0x06, 0xe0e0);
			mdio_write(tp, 0x06, 0x00e1);
			mdio_write(tp, 0x06, 0xe001);
			mdio_write(tp, 0x06, 0xad27);
			mdio_write(tp, 0x06, 0x1fd1);
			mdio_write(tp, 0x06, 0x01bf);
			mdio_write(tp, 0x06, 0x84eb);
			mdio_write(tp, 0x06, 0x022f);
			mdio_write(tp, 0x06, 0x50e0);
			mdio_write(tp, 0x06, 0xe020);
			mdio_write(tp, 0x06, 0xe1e0);
			mdio_write(tp, 0x06, 0x21ad);
			mdio_write(tp, 0x06, 0x200e);
			mdio_write(tp, 0x06, 0xd100);
			mdio_write(tp, 0x06, 0xbf84);
			mdio_write(tp, 0x06, 0xeb02);
			mdio_write(tp, 0x06, 0x2f50);
			mdio_write(tp, 0x06, 0xbf3d);
			mdio_write(tp, 0x06, 0x3902);
			mdio_write(tp, 0x06, 0x2eb0);
			mdio_write(tp, 0x06, 0xef96);
			mdio_write(tp, 0x06, 0xfefc);
			mdio_write(tp, 0x06, 0x0402);
			mdio_write(tp, 0x06, 0x8135);
			mdio_write(tp, 0x06, 0x05f8);
			mdio_write(tp, 0x06, 0xfaef);
			mdio_write(tp, 0x06, 0x69e0);
			mdio_write(tp, 0x06, 0xe2fe);
			mdio_write(tp, 0x06, 0xe1e2);
			mdio_write(tp, 0x06, 0xffad);
			mdio_write(tp, 0x06, 0x2d1a);
			mdio_write(tp, 0x06, 0xe0e1);
			mdio_write(tp, 0x06, 0x4ee1);
			mdio_write(tp, 0x06, 0xe14f);
			mdio_write(tp, 0x06, 0xac2d);
			mdio_write(tp, 0x06, 0x22f6);
			mdio_write(tp, 0x06, 0x0302);
			mdio_write(tp, 0x06, 0x0336);
			mdio_write(tp, 0x06, 0xf703);
			mdio_write(tp, 0x06, 0xf706);
			mdio_write(tp, 0x06, 0xbf84);
			mdio_write(tp, 0x06, 0xd502);
			mdio_write(tp, 0x06, 0x2eb0);
			mdio_write(tp, 0x06, 0xae11);
			mdio_write(tp, 0x06, 0xe0e1);
			mdio_write(tp, 0x06, 0x4ee1);
			mdio_write(tp, 0x06, 0xe14f);
			mdio_write(tp, 0x06, 0xad2d);
			mdio_write(tp, 0x06, 0x08bf);
			mdio_write(tp, 0x06, 0x84e0);
			mdio_write(tp, 0x06, 0x022e);
			mdio_write(tp, 0x06, 0xb0f6);
			mdio_write(tp, 0x06, 0x06ef);
			mdio_write(tp, 0x06, 0x96fe);
			mdio_write(tp, 0x06, 0xfc04);
			mdio_write(tp, 0x06, 0xf8f9);
			mdio_write(tp, 0x06, 0xfaef);
			mdio_write(tp, 0x06, 0x69e0);
			mdio_write(tp, 0x06, 0x8b87);
			mdio_write(tp, 0x06, 0xad20);
			mdio_write(tp, 0x06, 0x4cd2);
			mdio_write(tp, 0x06, 0x00e0);
			mdio_write(tp, 0x06, 0xe200);
			mdio_write(tp, 0x06, 0x5801);
			mdio_write(tp, 0x06, 0x0c02);
			mdio_write(tp, 0x06, 0x1e20);
			mdio_write(tp, 0x06, 0xe0e0);
			mdio_write(tp, 0x06, 0x0058);
			mdio_write(tp, 0x06, 0x101e);
			mdio_write(tp, 0x06, 0x20e0);
			mdio_write(tp, 0x06, 0xe036);
			mdio_write(tp, 0x06, 0x5803);
			mdio_write(tp, 0x06, 0x1e20);
			mdio_write(tp, 0x06, 0xe0e0);
			mdio_write(tp, 0x06, 0x22e1);
			mdio_write(tp, 0x06, 0xe023);
			mdio_write(tp, 0x06, 0x58e0);
			mdio_write(tp, 0x06, 0x1e20);
			mdio_write(tp, 0x06, 0xe08a);
			mdio_write(tp, 0x06, 0xe61f);
			mdio_write(tp, 0x06, 0x029e);
			mdio_write(tp, 0x06, 0x22e6);
			mdio_write(tp, 0x06, 0x8ae6);
			mdio_write(tp, 0x06, 0xad32);
			mdio_write(tp, 0x06, 0x14ad);
			mdio_write(tp, 0x06, 0x3411);
			mdio_write(tp, 0x06, 0xef02);
			mdio_write(tp, 0x06, 0x5803);
			mdio_write(tp, 0x06, 0x9e07);
			mdio_write(tp, 0x06, 0xad35);
			mdio_write(tp, 0x06, 0x085a);
			mdio_write(tp, 0x06, 0xc09f);
			mdio_write(tp, 0x06, 0x04d1);
			mdio_write(tp, 0x06, 0x01ae);
			mdio_write(tp, 0x06, 0x02d1);
			mdio_write(tp, 0x06, 0x00bf);
			mdio_write(tp, 0x06, 0x84f1);
			mdio_write(tp, 0x06, 0x022f);
			mdio_write(tp, 0x06, 0x50ef);
			mdio_write(tp, 0x06, 0x96fe);
			mdio_write(tp, 0x06, 0xfdfc);
			mdio_write(tp, 0x06, 0x04f8);
			mdio_write(tp, 0x06, 0xf9fa);
			mdio_write(tp, 0x06, 0xe08b);
			mdio_write(tp, 0x06, 0x81ac);
			mdio_write(tp, 0x06, 0x260e);
			mdio_write(tp, 0x06, 0xe08b);
			mdio_write(tp, 0x06, 0x81ac);
			mdio_write(tp, 0x06, 0x2108);
			mdio_write(tp, 0x06, 0xe08b);
			mdio_write(tp, 0x06, 0x87ac);
			mdio_write(tp, 0x06, 0x2402);
			mdio_write(tp, 0x06, 0xae6b);
			mdio_write(tp, 0x06, 0xeee0);
			mdio_write(tp, 0x06, 0xea00);
			mdio_write(tp, 0x06, 0xeee0);
			mdio_write(tp, 0x06, 0xeb00);
			mdio_write(tp, 0x06, 0xe2e0);
			mdio_write(tp, 0x06, 0x7ce3);
			mdio_write(tp, 0x06, 0xe07d);
			mdio_write(tp, 0x06, 0xa511);
			mdio_write(tp, 0x06, 0x1115);
			mdio_write(tp, 0x06, 0xd260);
			mdio_write(tp, 0x06, 0xd666);
			mdio_write(tp, 0x06, 0x6602);
			mdio_write(tp, 0x06, 0x07f9);
			mdio_write(tp, 0x06, 0xd2a0);
			mdio_write(tp, 0x06, 0xd6aa);
			mdio_write(tp, 0x06, 0xaa02);
			mdio_write(tp, 0x06, 0x07f9);
			mdio_write(tp, 0x06, 0x0282);
			mdio_write(tp, 0x06, 0x55ae);
			mdio_write(tp, 0x06, 0x44a5);
			mdio_write(tp, 0x06, 0x6666);
			mdio_write(tp, 0x06, 0x02ae);
			mdio_write(tp, 0x06, 0x38a5);
			mdio_write(tp, 0x06, 0xaaaa);
			mdio_write(tp, 0x06, 0x02ae);
			mdio_write(tp, 0x06, 0x32ee);
			mdio_write(tp, 0x06, 0xe0ea);
			mdio_write(tp, 0x06, 0x04ee);
			mdio_write(tp, 0x06, 0xe0eb);
			mdio_write(tp, 0x06, 0x06e2);
			mdio_write(tp, 0x06, 0xe07c);
			mdio_write(tp, 0x06, 0xe3e0);
			mdio_write(tp, 0x06, 0x7de0);
			mdio_write(tp, 0x06, 0xe038);
			mdio_write(tp, 0x06, 0xe1e0);
			mdio_write(tp, 0x06, 0x39ad);
			mdio_write(tp, 0x06, 0x2e21);
			mdio_write(tp, 0x06, 0xad3f);
			mdio_write(tp, 0x06, 0x13e0);
			mdio_write(tp, 0x06, 0xe414);
			mdio_write(tp, 0x06, 0xe1e4);
			mdio_write(tp, 0x06, 0x1568);
			mdio_write(tp, 0x06, 0x80e4);
			mdio_write(tp, 0x06, 0xe414);
			mdio_write(tp, 0x06, 0xe5e4);
			mdio_write(tp, 0x06, 0x1502);
			mdio_write(tp, 0x06, 0x8255);
			mdio_write(tp, 0x06, 0xae0b);
			mdio_write(tp, 0x06, 0xac3e);
			mdio_write(tp, 0x06, 0x02ae);
			mdio_write(tp, 0x06, 0x0602);
			mdio_write(tp, 0x06, 0x827f);
			mdio_write(tp, 0x06, 0x0282);
			mdio_write(tp, 0x06, 0xa9fe);
			mdio_write(tp, 0x06, 0xfdfc);
			mdio_write(tp, 0x06, 0x04f8);
			mdio_write(tp, 0x06, 0xe18b);
			mdio_write(tp, 0x06, 0x2ee0);
			mdio_write(tp, 0x06, 0x8b81);
			mdio_write(tp, 0x06, 0xad26);
			mdio_write(tp, 0x06, 0x0502);
			mdio_write(tp, 0x06, 0x21f3);
			mdio_write(tp, 0x06, 0xf728);
			mdio_write(tp, 0x06, 0xe08b);
			mdio_write(tp, 0x06, 0x81ad);
			mdio_write(tp, 0x06, 0x2105);
			mdio_write(tp, 0x06, 0x0222);
			mdio_write(tp, 0x06, 0xf8f7);
			mdio_write(tp, 0x06, 0x29e0);
			mdio_write(tp, 0x06, 0x8b87);
			mdio_write(tp, 0x06, 0xad24);
			mdio_write(tp, 0x06, 0x0502);
			mdio_write(tp, 0x06, 0x82e4);
			mdio_write(tp, 0x06, 0xf72a);
			mdio_write(tp, 0x06, 0xe58b);
			mdio_write(tp, 0x06, 0x2efc);
			mdio_write(tp, 0x06, 0x04f8);
			mdio_write(tp, 0x06, 0xe08b);
			mdio_write(tp, 0x06, 0x81ad);
			mdio_write(tp, 0x06, 0x2603);
			mdio_write(tp, 0x06, 0x0221);
			mdio_write(tp, 0x06, 0x34e0);
			mdio_write(tp, 0x06, 0x8b81);
			mdio_write(tp, 0x06, 0xad21);
			mdio_write(tp, 0x06, 0x09e0);
			mdio_write(tp, 0x06, 0x8b2e);
			mdio_write(tp, 0x06, 0xac20);
			mdio_write(tp, 0x06, 0x0302);
			mdio_write(tp, 0x06, 0x834b);
			mdio_write(tp, 0x06, 0xe08b);
			mdio_write(tp, 0x06, 0x87ad);
			mdio_write(tp, 0x06, 0x2409);
			mdio_write(tp, 0x06, 0xe08b);
			mdio_write(tp, 0x06, 0x2eac);
			mdio_write(tp, 0x06, 0x2103);
			mdio_write(tp, 0x06, 0x0283);
			mdio_write(tp, 0x06, 0x30fc);
			mdio_write(tp, 0x06, 0x04f8);
			mdio_write(tp, 0x06, 0xe18b);
			mdio_write(tp, 0x06, 0x2ee0);
			mdio_write(tp, 0x06, 0x8b81);
			mdio_write(tp, 0x06, 0xad26);
			mdio_write(tp, 0x06, 0x08e0);
			mdio_write(tp, 0x06, 0x85d2);
			mdio_write(tp, 0x06, 0xad25);
			mdio_write(tp, 0x06, 0x02f6);
			mdio_write(tp, 0x06, 0x28e0);
			mdio_write(tp, 0x06, 0x8b81);
			mdio_write(tp, 0x06, 0xad21);
			mdio_write(tp, 0x06, 0x0ae0);
			mdio_write(tp, 0x06, 0x860a);
			mdio_write(tp, 0x06, 0xf627);
			mdio_write(tp, 0x06, 0xa005);
			mdio_write(tp, 0x06, 0x02f6);
			mdio_write(tp, 0x06, 0x29e0);
			mdio_write(tp, 0x06, 0x8b87);
			mdio_write(tp, 0x06, 0xad24);
			mdio_write(tp, 0x06, 0x08e0);
			mdio_write(tp, 0x06, 0x8aed);
			mdio_write(tp, 0x06, 0xad20);
			mdio_write(tp, 0x06, 0x02f6);
			mdio_write(tp, 0x06, 0x2ae5);
			mdio_write(tp, 0x06, 0x8b2e);
			mdio_write(tp, 0x06, 0xa100);
			mdio_write(tp, 0x06, 0x0302);
			mdio_write(tp, 0x06, 0x2111);
			mdio_write(tp, 0x06, 0xfc04);
			mdio_write(tp, 0x06, 0xee8a);
			mdio_write(tp, 0x06, 0xed00);
			mdio_write(tp, 0x06, 0xee8a);
			mdio_write(tp, 0x06, 0xec00);
			mdio_write(tp, 0x06, 0x04f8);
			mdio_write(tp, 0x06, 0xe08b);
			mdio_write(tp, 0x06, 0x87ad);
			mdio_write(tp, 0x06, 0x243a);
			mdio_write(tp, 0x06, 0xe0e0);
			mdio_write(tp, 0x06, 0xeae1);
			mdio_write(tp, 0x06, 0xe0eb);
			mdio_write(tp, 0x06, 0x58f8);
			mdio_write(tp, 0x06, 0xd101);
			mdio_write(tp, 0x06, 0xe4e0);
			mdio_write(tp, 0x06, 0xeae5);
			mdio_write(tp, 0x06, 0xe0eb);
			mdio_write(tp, 0x06, 0xe0e0);
			mdio_write(tp, 0x06, 0x7ce1);
			mdio_write(tp, 0x06, 0xe07d);
			mdio_write(tp, 0x06, 0x5c00);
			mdio_write(tp, 0x06, 0xff3c);
			mdio_write(tp, 0x06, 0x001e);
			mdio_write(tp, 0x06, 0xab1c);
			mdio_write(tp, 0x06, 0xe0e0);
			mdio_write(tp, 0x06, 0x4ce1);
			mdio_write(tp, 0x06, 0xe04d);
			mdio_write(tp, 0x06, 0x58c1);
			mdio_write(tp, 0x06, 0xe4e0);
			mdio_write(tp, 0x06, 0x4ce5);
			mdio_write(tp, 0x06, 0xe04d);
			mdio_write(tp, 0x06, 0xe0e0);
			mdio_write(tp, 0x06, 0xeee1);
			mdio_write(tp, 0x06, 0xe0ef);
			mdio_write(tp, 0x06, 0x693c);
			mdio_write(tp, 0x06, 0xe4e0);
			mdio_write(tp, 0x06, 0xeee5);
			mdio_write(tp, 0x06, 0xe0ef);
			mdio_write(tp, 0x06, 0xfc04);
			mdio_write(tp, 0x06, 0xf8e0);
			mdio_write(tp, 0x06, 0x8b87);
			mdio_write(tp, 0x06, 0xad24);
			mdio_write(tp, 0x06, 0x12e0);
			mdio_write(tp, 0x06, 0xe0ee);
			mdio_write(tp, 0x06, 0xe1e0);
			mdio_write(tp, 0x06, 0xef59);
			mdio_write(tp, 0x06, 0xc3e4);
			mdio_write(tp, 0x06, 0xe0ee);
			mdio_write(tp, 0x06, 0xe5e0);
			mdio_write(tp, 0x06, 0xefee);
			mdio_write(tp, 0x06, 0x8aed);
			mdio_write(tp, 0x06, 0x01fc);
			mdio_write(tp, 0x06, 0x04f8);
			mdio_write(tp, 0x06, 0xe08b);
			mdio_write(tp, 0x06, 0x81ac);
			mdio_write(tp, 0x06, 0x2505);
			mdio_write(tp, 0x06, 0x0283);
			mdio_write(tp, 0x06, 0x5cae);
			mdio_write(tp, 0x06, 0x0302);
			mdio_write(tp, 0x06, 0x2516);
			mdio_write(tp, 0x06, 0xfc04);
			mdio_write(tp, 0x06, 0xf8f9);
			mdio_write(tp, 0x06, 0xfaef);
			mdio_write(tp, 0x06, 0x69fa);
			mdio_write(tp, 0x06, 0xe086);
			mdio_write(tp, 0x06, 0x0aa0);
			mdio_write(tp, 0x06, 0x0019);
			mdio_write(tp, 0x06, 0xe086);
			mdio_write(tp, 0x06, 0x0be1);
			mdio_write(tp, 0x06, 0x8b33);
			mdio_write(tp, 0x06, 0x1b10);
			mdio_write(tp, 0x06, 0x9e04);
			mdio_write(tp, 0x06, 0xaa02);
			mdio_write(tp, 0x06, 0xae06);
			mdio_write(tp, 0x06, 0xee86);
			mdio_write(tp, 0x06, 0x0a01);
			mdio_write(tp, 0x06, 0xaee6);
			mdio_write(tp, 0x06, 0x0224);
			mdio_write(tp, 0x06, 0x1eae);
			mdio_write(tp, 0x06, 0x14a0);
			mdio_write(tp, 0x06, 0x0114);
			mdio_write(tp, 0x06, 0x0224);
			mdio_write(tp, 0x06, 0x26bf);
			mdio_write(tp, 0x06, 0x266d);
			mdio_write(tp, 0x06, 0x022e);
			mdio_write(tp, 0x06, 0xb0ee);
			mdio_write(tp, 0x06, 0x860b);
			mdio_write(tp, 0x06, 0x00ee);
			mdio_write(tp, 0x06, 0x860a);
			mdio_write(tp, 0x06, 0x02af);
			mdio_write(tp, 0x06, 0x8435);
			mdio_write(tp, 0x06, 0xa002);
			mdio_write(tp, 0x06, 0x52ee);
			mdio_write(tp, 0x06, 0x8604);
			mdio_write(tp, 0x06, 0x00ee);
			mdio_write(tp, 0x06, 0x8605);
			mdio_write(tp, 0x06, 0x00e0);
			mdio_write(tp, 0x06, 0x860b);
			mdio_write(tp, 0x06, 0xe18b);
			mdio_write(tp, 0x06, 0x321b);
			mdio_write(tp, 0x06, 0x109e);
			mdio_write(tp, 0x06, 0x04aa);
			mdio_write(tp, 0x06, 0x02ae);
			mdio_write(tp, 0x06, 0xcbee);
			mdio_write(tp, 0x06, 0x860b);
			mdio_write(tp, 0x06, 0x0002);
			mdio_write(tp, 0x06, 0x243a);
			mdio_write(tp, 0x06, 0xe286);
			mdio_write(tp, 0x06, 0x04e3);
			mdio_write(tp, 0x06, 0x8605);
			mdio_write(tp, 0x06, 0xef65);
			mdio_write(tp, 0x06, 0xe286);
			mdio_write(tp, 0x06, 0x06e3);
			mdio_write(tp, 0x06, 0x8607);
			mdio_write(tp, 0x06, 0x1b56);
			mdio_write(tp, 0x06, 0xaa0e);
			mdio_write(tp, 0x06, 0xef56);
			mdio_write(tp, 0x06, 0xe686);
			mdio_write(tp, 0x06, 0x06e7);
			mdio_write(tp, 0x06, 0x8607);
			mdio_write(tp, 0x06, 0xe286);
			mdio_write(tp, 0x06, 0x09e6);
			mdio_write(tp, 0x06, 0x8608);
			mdio_write(tp, 0x06, 0xe086);
			mdio_write(tp, 0x06, 0x09a0);
			mdio_write(tp, 0x06, 0x0007);
			mdio_write(tp, 0x06, 0xee86);
			mdio_write(tp, 0x06, 0x0a03);
			mdio_write(tp, 0x06, 0xaf83);
			mdio_write(tp, 0x06, 0x6202);
			mdio_write(tp, 0x06, 0x248e);
			mdio_write(tp, 0x06, 0x0224);
			mdio_write(tp, 0x06, 0x26ae);
			mdio_write(tp, 0x06, 0x48a0);
			mdio_write(tp, 0x06, 0x0321);
			mdio_write(tp, 0x06, 0xe086);
			mdio_write(tp, 0x06, 0x08e1);
			mdio_write(tp, 0x06, 0x8609);
			mdio_write(tp, 0x06, 0x1b01);
			mdio_write(tp, 0x06, 0x9e0c);
			mdio_write(tp, 0x06, 0xaa05);
			mdio_write(tp, 0x06, 0x0224);
			mdio_write(tp, 0x06, 0x9dae);
			mdio_write(tp, 0x06, 0xe702);
			mdio_write(tp, 0x06, 0x248e);
			mdio_write(tp, 0x06, 0xaee2);
			mdio_write(tp, 0x06, 0xee86);
			mdio_write(tp, 0x06, 0x0a04);
			mdio_write(tp, 0x06, 0xee86);
			mdio_write(tp, 0x06, 0x0b00);
			mdio_write(tp, 0x06, 0xaf83);
			mdio_write(tp, 0x06, 0x62a0);
			mdio_write(tp, 0x06, 0x0415);
			mdio_write(tp, 0x06, 0xe086);
			mdio_write(tp, 0x06, 0x0be1);
			mdio_write(tp, 0x06, 0x8b34);
			mdio_write(tp, 0x06, 0x1b10);
			mdio_write(tp, 0x06, 0x9e05);
			mdio_write(tp, 0x06, 0xaa03);
			mdio_write(tp, 0x06, 0xaf83);
			mdio_write(tp, 0x06, 0x7cee);
			mdio_write(tp, 0x06, 0x860a);
			mdio_write(tp, 0x06, 0x05ae);
			mdio_write(tp, 0x06, 0x0ca0);
			mdio_write(tp, 0x06, 0x0502);
			mdio_write(tp, 0x06, 0xae07);
			mdio_write(tp, 0x06, 0x0223);
			mdio_write(tp, 0x06, 0x09ee);
			mdio_write(tp, 0x06, 0x860a);
			mdio_write(tp, 0x06, 0x00fe);
			mdio_write(tp, 0x06, 0xef96);
			mdio_write(tp, 0x06, 0xfefd);
			mdio_write(tp, 0x06, 0xfc04);
			mdio_write(tp, 0x06, 0xf8f9);
			mdio_write(tp, 0x06, 0xfbe0);
			mdio_write(tp, 0x06, 0x8b85);
			mdio_write(tp, 0x06, 0xad25);
			mdio_write(tp, 0x06, 0x22e0);
			mdio_write(tp, 0x06, 0xe022);
			mdio_write(tp, 0x06, 0xe1e0);
			mdio_write(tp, 0x06, 0x23e2);
			mdio_write(tp, 0x06, 0xe036);
			mdio_write(tp, 0x06, 0xe3e0);
			mdio_write(tp, 0x06, 0x375a);
			mdio_write(tp, 0x06, 0xc40d);
			mdio_write(tp, 0x06, 0x0158);
			mdio_write(tp, 0x06, 0x021e);
			mdio_write(tp, 0x06, 0x20e3);
			mdio_write(tp, 0x06, 0x8ae7);
			mdio_write(tp, 0x06, 0xac31);
			mdio_write(tp, 0x06, 0x60ac);
			mdio_write(tp, 0x06, 0x3a08);
			mdio_write(tp, 0x06, 0xac3e);
			mdio_write(tp, 0x06, 0x26ae);
			mdio_write(tp, 0x06, 0x67af);
			mdio_write(tp, 0x06, 0x84d1);
			mdio_write(tp, 0x06, 0xad37);
			mdio_write(tp, 0x06, 0x61e0);
			mdio_write(tp, 0x06, 0x8ae8);
			mdio_write(tp, 0x06, 0x10e4);
			mdio_write(tp, 0x06, 0x8ae8);
			mdio_write(tp, 0x06, 0xe18a);
			mdio_write(tp, 0x06, 0xe91b);
			mdio_write(tp, 0x06, 0x109e);
			mdio_write(tp, 0x06, 0x02ae);
			mdio_write(tp, 0x06, 0x51d1);
			mdio_write(tp, 0x06, 0x00bf);
			mdio_write(tp, 0x06, 0x84ee);
			mdio_write(tp, 0x06, 0x022f);
			mdio_write(tp, 0x06, 0x50ee);
			mdio_write(tp, 0x06, 0x8ae8);
			mdio_write(tp, 0x06, 0x00ae);
			mdio_write(tp, 0x06, 0x43ad);
			mdio_write(tp, 0x06, 0x3627);
			mdio_write(tp, 0x06, 0xe08a);
			mdio_write(tp, 0x06, 0xeee1);
			mdio_write(tp, 0x06, 0x8aef);
			mdio_write(tp, 0x06, 0xef74);
			mdio_write(tp, 0x06, 0xe08a);
			mdio_write(tp, 0x06, 0xeae1);
			mdio_write(tp, 0x06, 0x8aeb);
			mdio_write(tp, 0x06, 0x1b74);
			mdio_write(tp, 0x06, 0x9e2e);
			mdio_write(tp, 0x06, 0x14e4);
			mdio_write(tp, 0x06, 0x8aea);
			mdio_write(tp, 0x06, 0xe58a);
			mdio_write(tp, 0x06, 0xebef);
			mdio_write(tp, 0x06, 0x74e0);
			mdio_write(tp, 0x06, 0x8aee);
			mdio_write(tp, 0x06, 0xe18a);
			mdio_write(tp, 0x06, 0xef1b);
			mdio_write(tp, 0x06, 0x479e);
			mdio_write(tp, 0x06, 0x0fae);
			mdio_write(tp, 0x06, 0x19ee);
			mdio_write(tp, 0x06, 0x8aea);
			mdio_write(tp, 0x06, 0x00ee);
			mdio_write(tp, 0x06, 0x8aeb);
			mdio_write(tp, 0x06, 0x00ae);
			mdio_write(tp, 0x06, 0x0fac);
			mdio_write(tp, 0x06, 0x390c);
			mdio_write(tp, 0x06, 0xd101);
			mdio_write(tp, 0x06, 0xbf84);
			mdio_write(tp, 0x06, 0xee02);
			mdio_write(tp, 0x06, 0x2f50);
			mdio_write(tp, 0x06, 0xee8a);
			mdio_write(tp, 0x06, 0xe800);
			mdio_write(tp, 0x06, 0xe68a);
			mdio_write(tp, 0x06, 0xe7ff);
			mdio_write(tp, 0x06, 0xfdfc);
			mdio_write(tp, 0x06, 0x04a7);
			mdio_write(tp, 0x06, 0x25e5);
			mdio_write(tp, 0x06, 0x0a1d);
			mdio_write(tp, 0x06, 0xe50a);
			mdio_write(tp, 0x06, 0x2ce5);
			mdio_write(tp, 0x06, 0x0a6d);
			mdio_write(tp, 0x06, 0xe50a);
			mdio_write(tp, 0x06, 0x1de5);
			mdio_write(tp, 0x06, 0x0a1c);
			mdio_write(tp, 0x06, 0xe50a);
			mdio_write(tp, 0x06, 0x2da7);
			mdio_write(tp, 0x06, 0x5500);
			mdio_write(tp, 0x06, 0xe234);
			mdio_write(tp, 0x06, 0x88e2);
			mdio_write(tp, 0x06, 0x00cc);
			mdio_write(tp, 0x06, 0xe200);
			gphy_val = mdio_read(tp, 0x01);
			gphy_val |= BIT_0;
			mdio_write(tp, 0x01, gphy_val);
			gphy_val = mdio_read(tp, 0x00);
			gphy_val |= BIT_0;
			mdio_write(tp, 0x00, gphy_val);
			mdio_write(tp, 0x1f, 0x0000);
			mdio_write(tp, 0x1f, 0x0000);
			mdio_write(tp, 0x17, 0x2179);
			mdio_write(tp, 0x1f, 0x0001);
			mdio_write(tp, 0x10, 0xf274);
			mdio_write(tp, 0x1f, 0x0007);
			mdio_write(tp, 0x1e, 0x0042);
			mdio_write(tp, 0x15, 0x0f00);
			mdio_write(tp, 0x15, 0x0f00);
			mdio_write(tp, 0x16, 0x7408);
			mdio_write(tp, 0x15, 0x0e00);
			mdio_write(tp, 0x15, 0x0f00);
			mdio_write(tp, 0x15, 0x0f01);
			mdio_write(tp, 0x16, 0x4000);
			mdio_write(tp, 0x15, 0x0e01);
			mdio_write(tp, 0x15, 0x0f01);
			mdio_write(tp, 0x15, 0x0f02);
			mdio_write(tp, 0x16, 0x9400);
			mdio_write(tp, 0x15, 0x0e02);
			mdio_write(tp, 0x15, 0x0f02);
			mdio_write(tp, 0x15, 0x0f03);
			mdio_write(tp, 0x16, 0x7408);
			mdio_write(tp, 0x15, 0x0e03);
			mdio_write(tp, 0x15, 0x0f03);
			mdio_write(tp, 0x15, 0x0f04);
			mdio_write(tp, 0x16, 0x4008);
			mdio_write(tp, 0x15, 0x0e04);
			mdio_write(tp, 0x15, 0x0f04);
			mdio_write(tp, 0x15, 0x0f05);
			mdio_write(tp, 0x16, 0x9400);
			mdio_write(tp, 0x15, 0x0e05);
			mdio_write(tp, 0x15, 0x0f05);
			mdio_write(tp, 0x15, 0x0f06);
			mdio_write(tp, 0x16, 0x0803);
			mdio_write(tp, 0x15, 0x0e06);
			mdio_write(tp, 0x15, 0x0f06);
			mdio_write(tp, 0x15, 0x0d00);
			mdio_write(tp, 0x15, 0x0100);
			mdio_write(tp, 0x1f, 0x0001);
			mdio_write(tp, 0x10, 0xf074);
			mdio_write(tp, 0x1f, 0x0000);
			mdio_write(tp, 0x17, 0x2149);
			mdio_write(tp, 0x1f, 0x0005);
			for (i = 0; i < 200; i++) {
				udelay(100);
				gphy_val = mdio_read(tp, 0x00);
				if (gphy_val & BIT_7)
					break;
			}
			mdio_write(tp, 0x1f, 0x0007);
			mdio_write(tp, 0x1e, 0x0023);
			gphy_val = mdio_read(tp, 0x17);
			gphy_val &= ~(BIT_0);
			if ((subsystem_vendor == 0x144d &&
			     subsystem_device == 0xc098) ||
			    (subsystem_vendor == 0x144d &&
			     subsystem_device == 0xc0b1)) {
				gphy_val &= ~(BIT_2);
			}
			mdio_write(tp, 0x17, gphy_val);
			mdio_write(tp, 0x1f, 0x0000);
			mdio_write(tp, 0x1f, 0x0007);
			mdio_write(tp, 0x1e, 0x0023);
			gphy_val = mdio_read(tp, 0x17);
			gphy_val |= BIT_14;
			mdio_write(tp, 0x17, gphy_val);
			mdio_write(tp, 0x1e, 0x0020);
			gphy_val = mdio_read(tp, 0x1b);
			gphy_val |= BIT_7;
			mdio_write(tp, 0x1b, gphy_val);
			mdio_write(tp, 0x1e, 0x0041);
			mdio_write(tp, 0x15, 0x0e02);
			mdio_write(tp, 0x1e, 0x0028);
			gphy_val = mdio_read(tp, 0x19);
			gphy_val |= BIT_15;
			mdio_write(tp, 0x19, gphy_val);
			mdio_write(tp, 0x1f, 0x0000);
			if (subsystem_vendor == 0x104d &&
			    subsystem_device == 0x907b) {
				mdio_write(tp, 0x1F, 0x0005);
				mdio_write(tp, 0x05, 0x8B86);
				gphy_val = mdio_read(tp, 0x06) | BIT_4;
				mdio_write(tp, 0x06, gphy_val);
				mdio_write(tp, 0x1f, 0x0000);
			}
		}
		mdio_write(tp, 0x1F, 0x0007);
		mdio_write(tp, 0x1E, 0x0023);
		gphy_val = mdio_read(tp, 0x17)|0x06;
		mdio_write(tp, 0x17, gphy_val);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1f, 0x0005);
		mdio_write(tp, 0x05, 0x8b80);
		mdio_write(tp, 0x06, 0xc896);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x0B, 0x6C20);
		mdio_write(tp, 0x07, 0x2872);
		mdio_write(tp, 0x1C, 0xEFFF);
		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x14, 0x6420);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0002);
		gphy_val = mdio_read(tp, 0x08) & 0x00FF;
		mdio_write(tp, 0x08, gphy_val | 0x8000);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0007);
		mdio_write(tp, 0x1E, 0x002D);
		gphy_val = mdio_read(tp, 0x18);
		mdio_write(tp, 0x18, gphy_val | 0x0050);
		mdio_write(tp, 0x1F, 0x0000);
		gphy_val = mdio_read(tp, 0x14);
		mdio_write(tp, 0x14, gphy_val | 0x8000);
		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x00, 0x080B);
		mdio_write(tp, 0x0B, 0x09D7);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x15, 0x1006);
		mdio_write(tp, 0x1F, 0x0007);
		mdio_write(tp, 0x1E, 0x002F);
		mdio_write(tp, 0x15, 0x1919);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8B86);
		gphy_val = mdio_read(tp, 0x06);
		mdio_write(tp, 0x06, gphy_val | 0x0001);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0007);
		mdio_write(tp, 0x1E, 0x00AC);
		mdio_write(tp, 0x18, 0x0006);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x19, 0x7F46);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8AD2);
		mdio_write(tp, 0x06, 0x6810);
		mdio_write(tp, 0x05, 0x8AD4);
		mdio_write(tp, 0x06, 0x8002);
		mdio_write(tp, 0x05, 0x8ADE);
		mdio_write(tp, 0x06, 0x8025);
		mdio_write(tp, 0x1F, 0x0000);
	} else if (tp->mcfg == CFG_METHOD_16) {
		RTL_W8(tp,0x6E, RTL_R8(tp,0x6E) | BIT_6);
		rtl8168_eri_write(tp, 0x1AE, 2, 0x0403, ERIAR_ExGMAC);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x00, 0x1800);
		gphy_val = mdio_read(tp, 0x15);
		gphy_val &= ~(BIT_12);
		mdio_write(tp, 0x15, gphy_val);
		mdelay(20);
		mdio_write(tp, 0x1f, 0x0004);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x0023);
		gphy_val = mdio_read(tp, 0x17);
		if ((gphy_val & BIT_11) == 0x0000)
		{
			gphy_val |= BIT_0;
			mdio_write(tp, 0x17, gphy_val);
			for (i=0;i<200;i++)
			{
				udelay(100);
				gphy_val = mdio_read(tp, 0x17);
				if (gphy_val & BIT_11)
					break;
			}
		}
		gphy_val = mdio_read(tp, 0x17);
		gphy_val |= BIT_0;
		mdio_write(tp, 0x17, gphy_val);
		mdio_write(tp, 0x1f, 0x0004);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1E, 0x002C);
		mdio_write(tp, 0x1B, 0x5000);
		mdio_write(tp, 0x1E, 0x002d);
		mdio_write(tp, 0x19, 0x0004);
		mdio_write(tp, 0x1f, 0x0002);
		mdio_write(tp, 0x1f, 0x0000);
		for (i=0;i<200;i++)
		{
			udelay(100);
			gphy_val = mdio_read(tp, 0x1E);
			if ((gphy_val & 0x03FF) == 0x0014)
				break;
		}
		mdio_write(tp, 0x1f, 0x0005);
		for (i=0;i<200;i++)
		{
			udelay(100);
			gphy_val = mdio_read(tp, 0x07);
			if ((gphy_val & BIT_5) == 0)
				break;
		}
		gphy_val = mdio_read(tp, 0x07);
		if (gphy_val & BIT_5)
		{
			mdio_write(tp, 0x1f, 0x0004);
			mdio_write(tp, 0x1f, 0x0007);
			mdio_write(tp, 0x1e, 0x00a1);
			mdio_write(tp, 0x17, 0x1000);
			mdio_write(tp, 0x17, 0x0000);
			mdio_write(tp, 0x17, 0x2000);
			mdio_write(tp, 0x1e, 0x002f);
			mdio_write(tp, 0x18, 0x9bfb);
			mdio_write(tp, 0x1f, 0x0005);
			mdio_write(tp, 0x07, 0x0000);
			mdio_write(tp, 0x1f, 0x0002);
			mdio_write(tp, 0x1f, 0x0000);
		}
		mdio_write(tp, 0x1f, 0x0005);
		mdio_write(tp, 0x05, 0xfff6);
		mdio_write(tp, 0x06, 0x0080);
		gphy_val = mdio_read(tp, 0x00);
		gphy_val &= ~(BIT_7);
		mdio_write(tp, 0x00, gphy_val);
		mdio_write(tp, 0x1f, 0x0004);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x0023);
		mdio_write(tp, 0x16, 0x0306);
		mdio_write(tp, 0x16, 0x0307);
		mdio_write(tp, 0x15, 0x0000);
		mdio_write(tp, 0x19, 0x407d);
		mdio_write(tp, 0x15, 0x0001);
		mdio_write(tp, 0x19, 0x440f);
		mdio_write(tp, 0x15, 0x0002);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x0003);
		mdio_write(tp, 0x19, 0x6c03);
		mdio_write(tp, 0x15, 0x0004);
		mdio_write(tp, 0x19, 0xc4d5);
		mdio_write(tp, 0x15, 0x0005);
		mdio_write(tp, 0x19, 0x00ff);
		mdio_write(tp, 0x15, 0x0006);
		mdio_write(tp, 0x19, 0x74f0);
		mdio_write(tp, 0x15, 0x0007);
		mdio_write(tp, 0x19, 0x4880);
		mdio_write(tp, 0x15, 0x0008);
		mdio_write(tp, 0x19, 0x4c00);
		mdio_write(tp, 0x15, 0x0009);
		mdio_write(tp, 0x19, 0x4800);
		mdio_write(tp, 0x15, 0x000a);
		mdio_write(tp, 0x19, 0x5000);
		mdio_write(tp, 0x15, 0x000b);
		mdio_write(tp, 0x19, 0x4400);
		mdio_write(tp, 0x15, 0x000c);
		mdio_write(tp, 0x19, 0x7801);
		mdio_write(tp, 0x15, 0x000d);
		mdio_write(tp, 0x19, 0x4000);
		mdio_write(tp, 0x15, 0x000e);
		mdio_write(tp, 0x19, 0x7800);
		mdio_write(tp, 0x15, 0x000f);
		mdio_write(tp, 0x19, 0x7010);
		mdio_write(tp, 0x15, 0x0010);
		mdio_write(tp, 0x19, 0x6804);
		mdio_write(tp, 0x15, 0x0011);
		mdio_write(tp, 0x19, 0x64a0);
		mdio_write(tp, 0x15, 0x0012);
		mdio_write(tp, 0x19, 0x63da);
		mdio_write(tp, 0x15, 0x0013);
		mdio_write(tp, 0x19, 0x63d8);
		mdio_write(tp, 0x15, 0x0014);
		mdio_write(tp, 0x19, 0x6f05);
		mdio_write(tp, 0x15, 0x0015);
		mdio_write(tp, 0x19, 0x5420);
		mdio_write(tp, 0x15, 0x0016);
		mdio_write(tp, 0x19, 0x58ce);
		mdio_write(tp, 0x15, 0x0017);
		mdio_write(tp, 0x19, 0x5cf3);
		mdio_write(tp, 0x15, 0x0018);
		mdio_write(tp, 0x19, 0xb600);
		mdio_write(tp, 0x15, 0x0019);
		mdio_write(tp, 0x19, 0xc659);
		mdio_write(tp, 0x15, 0x001a);
		mdio_write(tp, 0x19, 0x0018);
		mdio_write(tp, 0x15, 0x001b);
		mdio_write(tp, 0x19, 0xc403);
		mdio_write(tp, 0x15, 0x001c);
		mdio_write(tp, 0x19, 0x0016);
		mdio_write(tp, 0x15, 0x001d);
		mdio_write(tp, 0x19, 0xaa05);
		mdio_write(tp, 0x15, 0x001e);
		mdio_write(tp, 0x19, 0xc503);
		mdio_write(tp, 0x15, 0x001f);
		mdio_write(tp, 0x19, 0x0003);
		mdio_write(tp, 0x15, 0x0020);
		mdio_write(tp, 0x19, 0x89f8);
		mdio_write(tp, 0x15, 0x0021);
		mdio_write(tp, 0x19, 0x32ae);
		mdio_write(tp, 0x15, 0x0022);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x0023);
		mdio_write(tp, 0x19, 0x6c03);
		mdio_write(tp, 0x15, 0x0024);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x0025);
		mdio_write(tp, 0x19, 0x6801);
		mdio_write(tp, 0x15, 0x0026);
		mdio_write(tp, 0x19, 0x66a0);
		mdio_write(tp, 0x15, 0x0027);
		mdio_write(tp, 0x19, 0xa300);
		mdio_write(tp, 0x15, 0x0028);
		mdio_write(tp, 0x19, 0x64a0);
		mdio_write(tp, 0x15, 0x0029);
		mdio_write(tp, 0x19, 0x76f0);
		mdio_write(tp, 0x15, 0x002a);
		mdio_write(tp, 0x19, 0x7670);
		mdio_write(tp, 0x15, 0x002b);
		mdio_write(tp, 0x19, 0x7630);
		mdio_write(tp, 0x15, 0x002c);
		mdio_write(tp, 0x19, 0x31a6);
		mdio_write(tp, 0x15, 0x002d);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x002e);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x002f);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0030);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0031);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0032);
		mdio_write(tp, 0x19, 0x4801);
		mdio_write(tp, 0x15, 0x0033);
		mdio_write(tp, 0x19, 0x6803);
		mdio_write(tp, 0x15, 0x0034);
		mdio_write(tp, 0x19, 0x66a1);
		mdio_write(tp, 0x15, 0x0035);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x0036);
		mdio_write(tp, 0x19, 0x6c03);
		mdio_write(tp, 0x15, 0x0037);
		mdio_write(tp, 0x19, 0xa300);
		mdio_write(tp, 0x15, 0x0038);
		mdio_write(tp, 0x19, 0x64a1);
		mdio_write(tp, 0x15, 0x0039);
		mdio_write(tp, 0x19, 0x7c08);
		mdio_write(tp, 0x15, 0x003a);
		mdio_write(tp, 0x19, 0x74f8);
		mdio_write(tp, 0x15, 0x003b);
		mdio_write(tp, 0x19, 0x63d0);
		mdio_write(tp, 0x15, 0x003c);
		mdio_write(tp, 0x19, 0x7ff0);
		mdio_write(tp, 0x15, 0x003d);
		mdio_write(tp, 0x19, 0x77f0);
		mdio_write(tp, 0x15, 0x003e);
		mdio_write(tp, 0x19, 0x7ff0);
		mdio_write(tp, 0x15, 0x003f);
		mdio_write(tp, 0x19, 0x7750);
		mdio_write(tp, 0x15, 0x0040);
		mdio_write(tp, 0x19, 0x63d8);
		mdio_write(tp, 0x15, 0x0041);
		mdio_write(tp, 0x19, 0x7cf0);
		mdio_write(tp, 0x15, 0x0042);
		mdio_write(tp, 0x19, 0x7708);
		mdio_write(tp, 0x15, 0x0043);
		mdio_write(tp, 0x19, 0xa654);
		mdio_write(tp, 0x15, 0x0044);
		mdio_write(tp, 0x19, 0x304a);
		mdio_write(tp, 0x15, 0x0045);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0046);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0047);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0048);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0049);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x004a);
		mdio_write(tp, 0x19, 0x4802);
		mdio_write(tp, 0x15, 0x004b);
		mdio_write(tp, 0x19, 0x4003);
		mdio_write(tp, 0x15, 0x004c);
		mdio_write(tp, 0x19, 0x4440);
		mdio_write(tp, 0x15, 0x004d);
		mdio_write(tp, 0x19, 0x63c8);
		mdio_write(tp, 0x15, 0x004e);
		mdio_write(tp, 0x19, 0x6481);
		mdio_write(tp, 0x15, 0x004f);
		mdio_write(tp, 0x19, 0x9d00);
		mdio_write(tp, 0x15, 0x0050);
		mdio_write(tp, 0x19, 0x63e8);
		mdio_write(tp, 0x15, 0x0051);
		mdio_write(tp, 0x19, 0x7d00);
		mdio_write(tp, 0x15, 0x0052);
		mdio_write(tp, 0x19, 0x5900);
		mdio_write(tp, 0x15, 0x0053);
		mdio_write(tp, 0x19, 0x63f8);
		mdio_write(tp, 0x15, 0x0054);
		mdio_write(tp, 0x19, 0x64a1);
		mdio_write(tp, 0x15, 0x0055);
		mdio_write(tp, 0x19, 0x3116);
		mdio_write(tp, 0x15, 0x0056);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0057);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0058);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0059);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x005a);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x005b);
		mdio_write(tp, 0x19, 0x6c03);
		mdio_write(tp, 0x15, 0x005c);
		mdio_write(tp, 0x19, 0x7c08);
		mdio_write(tp, 0x15, 0x005d);
		mdio_write(tp, 0x19, 0x6000);
		mdio_write(tp, 0x15, 0x005e);
		mdio_write(tp, 0x19, 0x59ce);
		mdio_write(tp, 0x15, 0x005f);
		mdio_write(tp, 0x19, 0x4400);
		mdio_write(tp, 0x15, 0x0060);
		mdio_write(tp, 0x19, 0x7d00);
		mdio_write(tp, 0x15, 0x0061);
		mdio_write(tp, 0x19, 0x72b0);
		mdio_write(tp, 0x15, 0x0062);
		mdio_write(tp, 0x19, 0x400e);
		mdio_write(tp, 0x15, 0x0063);
		mdio_write(tp, 0x19, 0x4440);
		mdio_write(tp, 0x15, 0x0064);
		mdio_write(tp, 0x19, 0x9d00);
		mdio_write(tp, 0x15, 0x0065);
		mdio_write(tp, 0x19, 0x7f00);
		mdio_write(tp, 0x15, 0x0066);
		mdio_write(tp, 0x19, 0x70b0);
		mdio_write(tp, 0x15, 0x0067);
		mdio_write(tp, 0x19, 0x7c08);
		mdio_write(tp, 0x15, 0x0068);
		mdio_write(tp, 0x19, 0x6008);
		mdio_write(tp, 0x15, 0x0069);
		mdio_write(tp, 0x19, 0x7cf0);
		mdio_write(tp, 0x15, 0x006a);
		mdio_write(tp, 0x19, 0x7750);
		mdio_write(tp, 0x15, 0x006b);
		mdio_write(tp, 0x19, 0x4007);
		mdio_write(tp, 0x15, 0x006c);
		mdio_write(tp, 0x19, 0x4500);
		mdio_write(tp, 0x15, 0x006d);
		mdio_write(tp, 0x19, 0x4023);
		mdio_write(tp, 0x15, 0x006e);
		mdio_write(tp, 0x19, 0x4580);
		mdio_write(tp, 0x15, 0x006f);
		mdio_write(tp, 0x19, 0x9f00);
		mdio_write(tp, 0x15, 0x0070);
		mdio_write(tp, 0x19, 0xcd78);
		mdio_write(tp, 0x15, 0x0071);
		mdio_write(tp, 0x19, 0x0003);
		mdio_write(tp, 0x15, 0x0072);
		mdio_write(tp, 0x19, 0xbe02);
		mdio_write(tp, 0x15, 0x0073);
		mdio_write(tp, 0x19, 0x3070);
		mdio_write(tp, 0x15, 0x0074);
		mdio_write(tp, 0x19, 0x7cf0);
		mdio_write(tp, 0x15, 0x0075);
		mdio_write(tp, 0x19, 0x77f0);
		mdio_write(tp, 0x15, 0x0076);
		mdio_write(tp, 0x19, 0x4400);
		mdio_write(tp, 0x15, 0x0077);
		mdio_write(tp, 0x19, 0x4007);
		mdio_write(tp, 0x15, 0x0078);
		mdio_write(tp, 0x19, 0x4500);
		mdio_write(tp, 0x15, 0x0079);
		mdio_write(tp, 0x19, 0x4023);
		mdio_write(tp, 0x15, 0x007a);
		mdio_write(tp, 0x19, 0x4580);
		mdio_write(tp, 0x15, 0x007b);
		mdio_write(tp, 0x19, 0x9f00);
		mdio_write(tp, 0x15, 0x007c);
		mdio_write(tp, 0x19, 0xce80);
		mdio_write(tp, 0x15, 0x007d);
		mdio_write(tp, 0x19, 0x0004);
		mdio_write(tp, 0x15, 0x007e);
		mdio_write(tp, 0x19, 0xce80);
		mdio_write(tp, 0x15, 0x007f);
		mdio_write(tp, 0x19, 0x0002);
		mdio_write(tp, 0x15, 0x0080);
		mdio_write(tp, 0x19, 0x307c);
		mdio_write(tp, 0x15, 0x0081);
		mdio_write(tp, 0x19, 0x4400);
		mdio_write(tp, 0x15, 0x0082);
		mdio_write(tp, 0x19, 0x480f);
		mdio_write(tp, 0x15, 0x0083);
		mdio_write(tp, 0x19, 0x6802);
		mdio_write(tp, 0x15, 0x0084);
		mdio_write(tp, 0x19, 0x6680);
		mdio_write(tp, 0x15, 0x0085);
		mdio_write(tp, 0x19, 0x7c10);
		mdio_write(tp, 0x15, 0x0086);
		mdio_write(tp, 0x19, 0x6010);
		mdio_write(tp, 0x15, 0x0087);
		mdio_write(tp, 0x19, 0x400a);
		mdio_write(tp, 0x15, 0x0088);
		mdio_write(tp, 0x19, 0x4580);
		mdio_write(tp, 0x15, 0x0089);
		mdio_write(tp, 0x19, 0x9e00);
		mdio_write(tp, 0x15, 0x008a);
		mdio_write(tp, 0x19, 0x7d00);
		mdio_write(tp, 0x15, 0x008b);
		mdio_write(tp, 0x19, 0x5800);
		mdio_write(tp, 0x15, 0x008c);
		mdio_write(tp, 0x19, 0x63c8);
		mdio_write(tp, 0x15, 0x008d);
		mdio_write(tp, 0x19, 0x63d8);
		mdio_write(tp, 0x15, 0x008e);
		mdio_write(tp, 0x19, 0x66a0);
		mdio_write(tp, 0x15, 0x008f);
		mdio_write(tp, 0x19, 0x8300);
		mdio_write(tp, 0x15, 0x0090);
		mdio_write(tp, 0x19, 0x7ff0);
		mdio_write(tp, 0x15, 0x0091);
		mdio_write(tp, 0x19, 0x74f0);
		mdio_write(tp, 0x15, 0x0092);
		mdio_write(tp, 0x19, 0x3006);
		mdio_write(tp, 0x15, 0x0093);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0094);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0095);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0096);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0097);
		mdio_write(tp, 0x19, 0x4803);
		mdio_write(tp, 0x15, 0x0098);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x0099);
		mdio_write(tp, 0x19, 0x6c03);
		mdio_write(tp, 0x15, 0x009a);
		mdio_write(tp, 0x19, 0xa203);
		mdio_write(tp, 0x15, 0x009b);
		mdio_write(tp, 0x19, 0x64b1);
		mdio_write(tp, 0x15, 0x009c);
		mdio_write(tp, 0x19, 0x309e);
		mdio_write(tp, 0x15, 0x009d);
		mdio_write(tp, 0x19, 0x64b3);
		mdio_write(tp, 0x15, 0x009e);
		mdio_write(tp, 0x19, 0x4030);
		mdio_write(tp, 0x15, 0x009f);
		mdio_write(tp, 0x19, 0x440e);
		mdio_write(tp, 0x15, 0x00a0);
		mdio_write(tp, 0x19, 0x4020);
		mdio_write(tp, 0x15, 0x00a1);
		mdio_write(tp, 0x19, 0x4419);
		mdio_write(tp, 0x15, 0x00a2);
		mdio_write(tp, 0x19, 0x7801);
		mdio_write(tp, 0x15, 0x00a3);
		mdio_write(tp, 0x19, 0xc520);
		mdio_write(tp, 0x15, 0x00a4);
		mdio_write(tp, 0x19, 0x000b);
		mdio_write(tp, 0x15, 0x00a5);
		mdio_write(tp, 0x19, 0x4020);
		mdio_write(tp, 0x15, 0x00a6);
		mdio_write(tp, 0x19, 0x7800);
		mdio_write(tp, 0x15, 0x00a7);
		mdio_write(tp, 0x19, 0x58a4);
		mdio_write(tp, 0x15, 0x00a8);
		mdio_write(tp, 0x19, 0x63da);
		mdio_write(tp, 0x15, 0x00a9);
		mdio_write(tp, 0x19, 0x5cb0);
		mdio_write(tp, 0x15, 0x00aa);
		mdio_write(tp, 0x19, 0x7d00);
		mdio_write(tp, 0x15, 0x00ab);
		mdio_write(tp, 0x19, 0x72b0);
		mdio_write(tp, 0x15, 0x00ac);
		mdio_write(tp, 0x19, 0x7f00);
		mdio_write(tp, 0x15, 0x00ad);
		mdio_write(tp, 0x19, 0x70b0);
		mdio_write(tp, 0x15, 0x00ae);
		mdio_write(tp, 0x19, 0x30b8);
		mdio_write(tp, 0x15, 0x00AF);
		mdio_write(tp, 0x19, 0x4060);
		mdio_write(tp, 0x15, 0x00B0);
		mdio_write(tp, 0x19, 0x7800);
		mdio_write(tp, 0x15, 0x00B1);
		mdio_write(tp, 0x19, 0x7e00);
		mdio_write(tp, 0x15, 0x00B2);
		mdio_write(tp, 0x19, 0x72B0);
		mdio_write(tp, 0x15, 0x00B3);
		mdio_write(tp, 0x19, 0x7F00);
		mdio_write(tp, 0x15, 0x00B4);
		mdio_write(tp, 0x19, 0x73B0);
		mdio_write(tp, 0x15, 0x00b5);
		mdio_write(tp, 0x19, 0x58a0);
		mdio_write(tp, 0x15, 0x00b6);
		mdio_write(tp, 0x19, 0x63d2);
		mdio_write(tp, 0x15, 0x00b7);
		mdio_write(tp, 0x19, 0x5c00);
		mdio_write(tp, 0x15, 0x00b8);
		mdio_write(tp, 0x19, 0x5780);
		mdio_write(tp, 0x15, 0x00b9);
		mdio_write(tp, 0x19, 0xb60d);
		mdio_write(tp, 0x15, 0x00ba);
		mdio_write(tp, 0x19, 0x9bff);
		mdio_write(tp, 0x15, 0x00bb);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x00bc);
		mdio_write(tp, 0x19, 0x6001);
		mdio_write(tp, 0x15, 0x00bd);
		mdio_write(tp, 0x19, 0xc020);
		mdio_write(tp, 0x15, 0x00be);
		mdio_write(tp, 0x19, 0x002b);
		mdio_write(tp, 0x15, 0x00bf);
		mdio_write(tp, 0x19, 0xc137);
		mdio_write(tp, 0x15, 0x00c0);
		mdio_write(tp, 0x19, 0x0006);
		mdio_write(tp, 0x15, 0x00c1);
		mdio_write(tp, 0x19, 0x9af8);
		mdio_write(tp, 0x15, 0x00c2);
		mdio_write(tp, 0x19, 0x30c6);
		mdio_write(tp, 0x15, 0x00c3);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x00c4);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x00c5);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x00c6);
		mdio_write(tp, 0x19, 0x7d00);
		mdio_write(tp, 0x15, 0x00c7);
		mdio_write(tp, 0x19, 0x70b0);
		mdio_write(tp, 0x15, 0x00c8);
		mdio_write(tp, 0x19, 0x4400);
		mdio_write(tp, 0x15, 0x00c9);
		mdio_write(tp, 0x19, 0x4804);
		mdio_write(tp, 0x15, 0x00ca);
		mdio_write(tp, 0x19, 0x7c80);
		mdio_write(tp, 0x15, 0x00cb);
		mdio_write(tp, 0x19, 0x5c80);
		mdio_write(tp, 0x15, 0x00cc);
		mdio_write(tp, 0x19, 0x4010);
		mdio_write(tp, 0x15, 0x00cd);
		mdio_write(tp, 0x19, 0x4415);
		mdio_write(tp, 0x15, 0x00ce);
		mdio_write(tp, 0x19, 0x9b00);
		mdio_write(tp, 0x15, 0x00cf);
		mdio_write(tp, 0x19, 0x7f00);
		mdio_write(tp, 0x15, 0x00d0);
		mdio_write(tp, 0x19, 0x70b0);
		mdio_write(tp, 0x15, 0x00d1);
		mdio_write(tp, 0x19, 0x3177);
		mdio_write(tp, 0x15, 0x00d2);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x00d3);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x00d4);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x00d5);
		mdio_write(tp, 0x19, 0x4808);
		mdio_write(tp, 0x15, 0x00d6);
		mdio_write(tp, 0x19, 0x4007);
		mdio_write(tp, 0x15, 0x00d7);
		mdio_write(tp, 0x19, 0x4420);
		mdio_write(tp, 0x15, 0x00d8);
		mdio_write(tp, 0x19, 0x63d8);
		mdio_write(tp, 0x15, 0x00d9);
		mdio_write(tp, 0x19, 0xb608);
		mdio_write(tp, 0x15, 0x00da);
		mdio_write(tp, 0x19, 0xbcbd);
		mdio_write(tp, 0x15, 0x00db);
		mdio_write(tp, 0x19, 0xc60b);
		mdio_write(tp, 0x15, 0x00dc);
		mdio_write(tp, 0x19, 0x00fd);
		mdio_write(tp, 0x15, 0x00dd);
		mdio_write(tp, 0x19, 0x30e1);
		mdio_write(tp, 0x15, 0x00de);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x00df);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x00e0);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x00e1);
		mdio_write(tp, 0x19, 0x4809);
		mdio_write(tp, 0x15, 0x00e2);
		mdio_write(tp, 0x19, 0x7e40);
		mdio_write(tp, 0x15, 0x00e3);
		mdio_write(tp, 0x19, 0x5a40);
		mdio_write(tp, 0x15, 0x00e4);
		mdio_write(tp, 0x19, 0x305a);
		mdio_write(tp, 0x15, 0x00e5);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x00e6);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x00e7);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x00e8);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x00e9);
		mdio_write(tp, 0x19, 0x480a);
		mdio_write(tp, 0x15, 0x00ea);
		mdio_write(tp, 0x19, 0x5820);
		mdio_write(tp, 0x15, 0x00eb);
		mdio_write(tp, 0x19, 0x6c03);
		mdio_write(tp, 0x15, 0x00ec);
		mdio_write(tp, 0x19, 0xb60a);
		mdio_write(tp, 0x15, 0x00ed);
		mdio_write(tp, 0x19, 0xda07);
		mdio_write(tp, 0x15, 0x00ee);
		mdio_write(tp, 0x19, 0x0008);
		mdio_write(tp, 0x15, 0x00ef);
		mdio_write(tp, 0x19, 0xc60b);
		mdio_write(tp, 0x15, 0x00f0);
		mdio_write(tp, 0x19, 0x00fc);
		mdio_write(tp, 0x15, 0x00f1);
		mdio_write(tp, 0x19, 0x30f6);
		mdio_write(tp, 0x15, 0x00f2);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x00f3);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x00f4);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x00f5);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x00f6);
		mdio_write(tp, 0x19, 0x4408);
		mdio_write(tp, 0x15, 0x00f7);
		mdio_write(tp, 0x19, 0x480b);
		mdio_write(tp, 0x15, 0x00f8);
		mdio_write(tp, 0x19, 0x6f03);
		mdio_write(tp, 0x15, 0x00f9);
		mdio_write(tp, 0x19, 0x405f);
		mdio_write(tp, 0x15, 0x00fa);
		mdio_write(tp, 0x19, 0x4448);
		mdio_write(tp, 0x15, 0x00fb);
		mdio_write(tp, 0x19, 0x4020);
		mdio_write(tp, 0x15, 0x00fc);
		mdio_write(tp, 0x19, 0x4468);
		mdio_write(tp, 0x15, 0x00fd);
		mdio_write(tp, 0x19, 0x9c03);
		mdio_write(tp, 0x15, 0x00fe);
		mdio_write(tp, 0x19, 0x6f07);
		mdio_write(tp, 0x15, 0x00ff);
		mdio_write(tp, 0x19, 0x58a0);
		mdio_write(tp, 0x15, 0x0100);
		mdio_write(tp, 0x19, 0xd6d1);
		mdio_write(tp, 0x15, 0x0101);
		mdio_write(tp, 0x19, 0x0004);
		mdio_write(tp, 0x15, 0x0102);
		mdio_write(tp, 0x19, 0xc137);
		mdio_write(tp, 0x15, 0x0103);
		mdio_write(tp, 0x19, 0x0002);
		mdio_write(tp, 0x15, 0x0104);
		mdio_write(tp, 0x19, 0xa0e5);
		mdio_write(tp, 0x15, 0x0105);
		mdio_write(tp, 0x19, 0x9df8);
		mdio_write(tp, 0x15, 0x0106);
		mdio_write(tp, 0x19, 0x30c6);
		mdio_write(tp, 0x15, 0x0107);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0108);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0109);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x010a);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x010b);
		mdio_write(tp, 0x19, 0x4808);
		mdio_write(tp, 0x15, 0x010c);
		mdio_write(tp, 0x19, 0xc32d);
		mdio_write(tp, 0x15, 0x010d);
		mdio_write(tp, 0x19, 0x0003);
		mdio_write(tp, 0x15, 0x010e);
		mdio_write(tp, 0x19, 0xc8b3);
		mdio_write(tp, 0x15, 0x010f);
		mdio_write(tp, 0x19, 0x00fc);
		mdio_write(tp, 0x15, 0x0110);
		mdio_write(tp, 0x19, 0x4400);
		mdio_write(tp, 0x15, 0x0111);
		mdio_write(tp, 0x19, 0x3116);
		mdio_write(tp, 0x15, 0x0112);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0113);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0114);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0115);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0116);
		mdio_write(tp, 0x19, 0x4803);
		mdio_write(tp, 0x15, 0x0117);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x0118);
		mdio_write(tp, 0x19, 0x6c02);
		mdio_write(tp, 0x15, 0x0119);
		mdio_write(tp, 0x19, 0x7c04);
		mdio_write(tp, 0x15, 0x011a);
		mdio_write(tp, 0x19, 0x6000);
		mdio_write(tp, 0x15, 0x011b);
		mdio_write(tp, 0x19, 0x5cf7);
		mdio_write(tp, 0x15, 0x011c);
		mdio_write(tp, 0x19, 0x7c2a);
		mdio_write(tp, 0x15, 0x011d);
		mdio_write(tp, 0x19, 0x5800);
		mdio_write(tp, 0x15, 0x011e);
		mdio_write(tp, 0x19, 0x5400);
		mdio_write(tp, 0x15, 0x011f);
		mdio_write(tp, 0x19, 0x7c08);
		mdio_write(tp, 0x15, 0x0120);
		mdio_write(tp, 0x19, 0x74f0);
		mdio_write(tp, 0x15, 0x0121);
		mdio_write(tp, 0x19, 0x4019);
		mdio_write(tp, 0x15, 0x0122);
		mdio_write(tp, 0x19, 0x440d);
		mdio_write(tp, 0x15, 0x0123);
		mdio_write(tp, 0x19, 0xb6c1);
		mdio_write(tp, 0x15, 0x0124);
		mdio_write(tp, 0x19, 0xc05b);
		mdio_write(tp, 0x15, 0x0125);
		mdio_write(tp, 0x19, 0x00bf);
		mdio_write(tp, 0x15, 0x0126);
		mdio_write(tp, 0x19, 0xc025);
		mdio_write(tp, 0x15, 0x0127);
		mdio_write(tp, 0x19, 0x00bd);
		mdio_write(tp, 0x15, 0x0128);
		mdio_write(tp, 0x19, 0xc603);
		mdio_write(tp, 0x15, 0x0129);
		mdio_write(tp, 0x19, 0x00bb);
		mdio_write(tp, 0x15, 0x012a);
		mdio_write(tp, 0x19, 0x8805);
		mdio_write(tp, 0x15, 0x012b);
		mdio_write(tp, 0x19, 0x7801);
		mdio_write(tp, 0x15, 0x012c);
		mdio_write(tp, 0x19, 0x4001);
		mdio_write(tp, 0x15, 0x012d);
		mdio_write(tp, 0x19, 0x7800);
		mdio_write(tp, 0x15, 0x012e);
		mdio_write(tp, 0x19, 0xa3dd);
		mdio_write(tp, 0x15, 0x012f);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x0130);
		mdio_write(tp, 0x19, 0x6c03);
		mdio_write(tp, 0x15, 0x0131);
		mdio_write(tp, 0x19, 0x8407);
		mdio_write(tp, 0x15, 0x0132);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x0133);
		mdio_write(tp, 0x19, 0x6c02);
		mdio_write(tp, 0x15, 0x0134);
		mdio_write(tp, 0x19, 0xd9b8);
		mdio_write(tp, 0x15, 0x0135);
		mdio_write(tp, 0x19, 0x0003);
		mdio_write(tp, 0x15, 0x0136);
		mdio_write(tp, 0x19, 0xc240);
		mdio_write(tp, 0x15, 0x0137);
		mdio_write(tp, 0x19, 0x0015);
		mdio_write(tp, 0x15, 0x0138);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x0139);
		mdio_write(tp, 0x19, 0x6c02);
		mdio_write(tp, 0x15, 0x013a);
		mdio_write(tp, 0x19, 0x9ae9);
		mdio_write(tp, 0x15, 0x013b);
		mdio_write(tp, 0x19, 0x3140);
		mdio_write(tp, 0x15, 0x013c);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x013d);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x013e);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x013f);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0140);
		mdio_write(tp, 0x19, 0x4807);
		mdio_write(tp, 0x15, 0x0141);
		mdio_write(tp, 0x19, 0x4004);
		mdio_write(tp, 0x15, 0x0142);
		mdio_write(tp, 0x19, 0x4410);
		mdio_write(tp, 0x15, 0x0143);
		mdio_write(tp, 0x19, 0x7c0c);
		mdio_write(tp, 0x15, 0x0144);
		mdio_write(tp, 0x19, 0x600c);
		mdio_write(tp, 0x15, 0x0145);
		mdio_write(tp, 0x19, 0x9b00);
		mdio_write(tp, 0x15, 0x0146);
		mdio_write(tp, 0x19, 0xa68f);
		mdio_write(tp, 0x15, 0x0147);
		mdio_write(tp, 0x19, 0x3116);
		mdio_write(tp, 0x15, 0x0148);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0149);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x014a);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x014b);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x014c);
		mdio_write(tp, 0x19, 0x4804);
		mdio_write(tp, 0x15, 0x014d);
		mdio_write(tp, 0x19, 0x54c0);
		mdio_write(tp, 0x15, 0x014e);
		mdio_write(tp, 0x19, 0xb703);
		mdio_write(tp, 0x15, 0x014f);
		mdio_write(tp, 0x19, 0x5cff);
		mdio_write(tp, 0x15, 0x0150);
		mdio_write(tp, 0x19, 0x315f);
		mdio_write(tp, 0x15, 0x0151);
		mdio_write(tp, 0x19, 0x7c08);
		mdio_write(tp, 0x15, 0x0152);
		mdio_write(tp, 0x19, 0x74f8);
		mdio_write(tp, 0x15, 0x0153);
		mdio_write(tp, 0x19, 0x6421);
		mdio_write(tp, 0x15, 0x0154);
		mdio_write(tp, 0x19, 0x7c08);
		mdio_write(tp, 0x15, 0x0155);
		mdio_write(tp, 0x19, 0x6000);
		mdio_write(tp, 0x15, 0x0156);
		mdio_write(tp, 0x19, 0x4003);
		mdio_write(tp, 0x15, 0x0157);
		mdio_write(tp, 0x19, 0x4418);
		mdio_write(tp, 0x15, 0x0158);
		mdio_write(tp, 0x19, 0x9b00);
		mdio_write(tp, 0x15, 0x0159);
		mdio_write(tp, 0x19, 0x6461);
		mdio_write(tp, 0x15, 0x015a);
		mdio_write(tp, 0x19, 0x64e1);
		mdio_write(tp, 0x15, 0x015b);
		mdio_write(tp, 0x19, 0x7c20);
		mdio_write(tp, 0x15, 0x015c);
		mdio_write(tp, 0x19, 0x5820);
		mdio_write(tp, 0x15, 0x015d);
		mdio_write(tp, 0x19, 0x5ccf);
		mdio_write(tp, 0x15, 0x015e);
		mdio_write(tp, 0x19, 0x7050);
		mdio_write(tp, 0x15, 0x015f);
		mdio_write(tp, 0x19, 0xd9b8);
		mdio_write(tp, 0x15, 0x0160);
		mdio_write(tp, 0x19, 0x0008);
		mdio_write(tp, 0x15, 0x0161);
		mdio_write(tp, 0x19, 0xdab1);
		mdio_write(tp, 0x15, 0x0162);
		mdio_write(tp, 0x19, 0x0015);
		mdio_write(tp, 0x15, 0x0163);
		mdio_write(tp, 0x19, 0xc244);
		mdio_write(tp, 0x15, 0x0164);
		mdio_write(tp, 0x19, 0x0013);
		mdio_write(tp, 0x15, 0x0165);
		mdio_write(tp, 0x19, 0xc021);
		mdio_write(tp, 0x15, 0x0166);
		mdio_write(tp, 0x19, 0x00f9);
		mdio_write(tp, 0x15, 0x0167);
		mdio_write(tp, 0x19, 0x3177);
		mdio_write(tp, 0x15, 0x0168);
		mdio_write(tp, 0x19, 0x5cf7);
		mdio_write(tp, 0x15, 0x0169);
		mdio_write(tp, 0x19, 0x4010);
		mdio_write(tp, 0x15, 0x016a);
		mdio_write(tp, 0x19, 0x4428);
		mdio_write(tp, 0x15, 0x016b);
		mdio_write(tp, 0x19, 0x9c00);
		mdio_write(tp, 0x15, 0x016c);
		mdio_write(tp, 0x19, 0x7c08);
		mdio_write(tp, 0x15, 0x016d);
		mdio_write(tp, 0x19, 0x6008);
		mdio_write(tp, 0x15, 0x016e);
		mdio_write(tp, 0x19, 0x7c08);
		mdio_write(tp, 0x15, 0x016f);
		mdio_write(tp, 0x19, 0x74f0);
		mdio_write(tp, 0x15, 0x0170);
		mdio_write(tp, 0x19, 0x6461);
		mdio_write(tp, 0x15, 0x0171);
		mdio_write(tp, 0x19, 0x6421);
		mdio_write(tp, 0x15, 0x0172);
		mdio_write(tp, 0x19, 0x64a1);
		mdio_write(tp, 0x15, 0x0173);
		mdio_write(tp, 0x19, 0x3116);
		mdio_write(tp, 0x15, 0x0174);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0175);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0176);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0177);
		mdio_write(tp, 0x19, 0x4805);
		mdio_write(tp, 0x15, 0x0178);
		mdio_write(tp, 0x19, 0xa103);
		mdio_write(tp, 0x15, 0x0179);
		mdio_write(tp, 0x19, 0x7c02);
		mdio_write(tp, 0x15, 0x017a);
		mdio_write(tp, 0x19, 0x6002);
		mdio_write(tp, 0x15, 0x017b);
		mdio_write(tp, 0x19, 0x7e00);
		mdio_write(tp, 0x15, 0x017c);
		mdio_write(tp, 0x19, 0x5400);
		mdio_write(tp, 0x15, 0x017d);
		mdio_write(tp, 0x19, 0x7c6b);
		mdio_write(tp, 0x15, 0x017e);
		mdio_write(tp, 0x19, 0x5c63);
		mdio_write(tp, 0x15, 0x017f);
		mdio_write(tp, 0x19, 0x407d);
		mdio_write(tp, 0x15, 0x0180);
		mdio_write(tp, 0x19, 0xa602);
		mdio_write(tp, 0x15, 0x0181);
		mdio_write(tp, 0x19, 0x4001);
		mdio_write(tp, 0x15, 0x0182);
		mdio_write(tp, 0x19, 0x4420);
		mdio_write(tp, 0x15, 0x0183);
		mdio_write(tp, 0x19, 0x4020);
		mdio_write(tp, 0x15, 0x0184);
		mdio_write(tp, 0x19, 0x44a1);
		mdio_write(tp, 0x15, 0x0185);
		mdio_write(tp, 0x19, 0xd6e0);
		mdio_write(tp, 0x15, 0x0186);
		mdio_write(tp, 0x19, 0x0009);
		mdio_write(tp, 0x15, 0x0187);
		mdio_write(tp, 0x19, 0x9efe);
		mdio_write(tp, 0x15, 0x0188);
		mdio_write(tp, 0x19, 0x7c02);
		mdio_write(tp, 0x15, 0x0189);
		mdio_write(tp, 0x19, 0x6000);
		mdio_write(tp, 0x15, 0x018a);
		mdio_write(tp, 0x19, 0x9c00);
		mdio_write(tp, 0x15, 0x018b);
		mdio_write(tp, 0x19, 0x318f);
		mdio_write(tp, 0x15, 0x018c);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x018d);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x018e);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x018f);
		mdio_write(tp, 0x19, 0x4806);
		mdio_write(tp, 0x15, 0x0190);
		mdio_write(tp, 0x19, 0x7c10);
		mdio_write(tp, 0x15, 0x0191);
		mdio_write(tp, 0x19, 0x5c10);
		mdio_write(tp, 0x15, 0x0192);
		mdio_write(tp, 0x19, 0x40fa);
		mdio_write(tp, 0x15, 0x0193);
		mdio_write(tp, 0x19, 0xa602);
		mdio_write(tp, 0x15, 0x0194);
		mdio_write(tp, 0x19, 0x4010);
		mdio_write(tp, 0x15, 0x0195);
		mdio_write(tp, 0x19, 0x4440);
		mdio_write(tp, 0x15, 0x0196);
		mdio_write(tp, 0x19, 0x9d00);
		mdio_write(tp, 0x15, 0x0197);
		mdio_write(tp, 0x19, 0x7c80);
		mdio_write(tp, 0x15, 0x0198);
		mdio_write(tp, 0x19, 0x6400);
		mdio_write(tp, 0x15, 0x0199);
		mdio_write(tp, 0x19, 0x4003);
		mdio_write(tp, 0x15, 0x019a);
		mdio_write(tp, 0x19, 0x4540);
		mdio_write(tp, 0x15, 0x019b);
		mdio_write(tp, 0x19, 0x7c08);
		mdio_write(tp, 0x15, 0x019c);
		mdio_write(tp, 0x19, 0x6008);
		mdio_write(tp, 0x15, 0x019d);
		mdio_write(tp, 0x19, 0x9f00);
		mdio_write(tp, 0x15, 0x019e);
		mdio_write(tp, 0x19, 0x7c40);
		mdio_write(tp, 0x15, 0x019f);
		mdio_write(tp, 0x19, 0x6400);
		mdio_write(tp, 0x15, 0x01a0);
		mdio_write(tp, 0x19, 0x7c80);
		mdio_write(tp, 0x15, 0x01a1);
		mdio_write(tp, 0x19, 0x6480);
		mdio_write(tp, 0x15, 0x01a2);
		mdio_write(tp, 0x19, 0x3140);
		mdio_write(tp, 0x15, 0x01a3);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x01a4);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x01a5);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x01a6);
		mdio_write(tp, 0x19, 0x4400);
		mdio_write(tp, 0x15, 0x01a7);
		mdio_write(tp, 0x19, 0x7c0b);
		mdio_write(tp, 0x15, 0x01a8);
		mdio_write(tp, 0x19, 0x6c01);
		mdio_write(tp, 0x15, 0x01a9);
		mdio_write(tp, 0x19, 0x64a8);
		mdio_write(tp, 0x15, 0x01aa);
		mdio_write(tp, 0x19, 0x6800);
		mdio_write(tp, 0x15, 0x01ab);
		mdio_write(tp, 0x19, 0x5cf0);
		mdio_write(tp, 0x15, 0x01ac);
		mdio_write(tp, 0x19, 0x588f);
		mdio_write(tp, 0x15, 0x01ad);
		mdio_write(tp, 0x19, 0xb628);
		mdio_write(tp, 0x15, 0x01ae);
		mdio_write(tp, 0x19, 0xc053);
		mdio_write(tp, 0x15, 0x01af);
		mdio_write(tp, 0x19, 0x0026);
		mdio_write(tp, 0x15, 0x01b0);
		mdio_write(tp, 0x19, 0xc02d);
		mdio_write(tp, 0x15, 0x01b1);
		mdio_write(tp, 0x19, 0x0024);
		mdio_write(tp, 0x15, 0x01b2);
		mdio_write(tp, 0x19, 0xc603);
		mdio_write(tp, 0x15, 0x01b3);
		mdio_write(tp, 0x19, 0x0022);
		mdio_write(tp, 0x15, 0x01b4);
		mdio_write(tp, 0x19, 0x8cf9);
		mdio_write(tp, 0x15, 0x01b5);
		mdio_write(tp, 0x19, 0x31ba);
		mdio_write(tp, 0x15, 0x01b6);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x01b7);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x01b8);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x01b9);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x01ba);
		mdio_write(tp, 0x19, 0x4400);
		mdio_write(tp, 0x15, 0x01bb);
		mdio_write(tp, 0x19, 0x5420);
		mdio_write(tp, 0x15, 0x01bc);
		mdio_write(tp, 0x19, 0x4811);
		mdio_write(tp, 0x15, 0x01bd);
		mdio_write(tp, 0x19, 0x5000);
		mdio_write(tp, 0x15, 0x01be);
		mdio_write(tp, 0x19, 0x4801);
		mdio_write(tp, 0x15, 0x01bf);
		mdio_write(tp, 0x19, 0x6800);
		mdio_write(tp, 0x15, 0x01c0);
		mdio_write(tp, 0x19, 0x31f5);
		mdio_write(tp, 0x15, 0x01c1);
		mdio_write(tp, 0x19, 0xb614);
		mdio_write(tp, 0x15, 0x01c2);
		mdio_write(tp, 0x19, 0x8ce4);
		mdio_write(tp, 0x15, 0x01c3);
		mdio_write(tp, 0x19, 0xb30c);
		mdio_write(tp, 0x15, 0x01c4);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x01c5);
		mdio_write(tp, 0x19, 0x6c02);
		mdio_write(tp, 0x15, 0x01c6);
		mdio_write(tp, 0x19, 0x8206);
		mdio_write(tp, 0x15, 0x01c7);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x01c8);
		mdio_write(tp, 0x19, 0x6c00);
		mdio_write(tp, 0x15, 0x01c9);
		mdio_write(tp, 0x19, 0x7c04);
		mdio_write(tp, 0x15, 0x01ca);
		mdio_write(tp, 0x19, 0x7404);
		mdio_write(tp, 0x15, 0x01cb);
		mdio_write(tp, 0x19, 0x31c0);
		mdio_write(tp, 0x15, 0x01cc);
		mdio_write(tp, 0x19, 0x7c04);
		mdio_write(tp, 0x15, 0x01cd);
		mdio_write(tp, 0x19, 0x7400);
		mdio_write(tp, 0x15, 0x01ce);
		mdio_write(tp, 0x19, 0x31c0);
		mdio_write(tp, 0x15, 0x01cf);
		mdio_write(tp, 0x19, 0x8df1);
		mdio_write(tp, 0x15, 0x01d0);
		mdio_write(tp, 0x19, 0x3248);
		mdio_write(tp, 0x15, 0x01d1);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x01d2);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x01d3);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x01d4);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x01d5);
		mdio_write(tp, 0x19, 0x4400);
		mdio_write(tp, 0x15, 0x01d6);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x01d7);
		mdio_write(tp, 0x19, 0x6c03);
		mdio_write(tp, 0x15, 0x01d8);
		mdio_write(tp, 0x19, 0x7670);
		mdio_write(tp, 0x15, 0x01d9);
		mdio_write(tp, 0x19, 0x4023);
		mdio_write(tp, 0x15, 0x01da);
		mdio_write(tp, 0x19, 0x4500);
		mdio_write(tp, 0x15, 0x01db);
		mdio_write(tp, 0x19, 0x4069);
		mdio_write(tp, 0x15, 0x01dc);
		mdio_write(tp, 0x19, 0x4580);
		mdio_write(tp, 0x15, 0x01dd);
		mdio_write(tp, 0x19, 0x9f00);
		mdio_write(tp, 0x15, 0x01de);
		mdio_write(tp, 0x19, 0xcff5);
		mdio_write(tp, 0x15, 0x01df);
		mdio_write(tp, 0x19, 0x00ff);
		mdio_write(tp, 0x15, 0x01e0);
		mdio_write(tp, 0x19, 0x76f0);
		mdio_write(tp, 0x15, 0x01e1);
		mdio_write(tp, 0x19, 0x4400);
		mdio_write(tp, 0x15, 0x01e2);
		mdio_write(tp, 0x19, 0x4023);
		mdio_write(tp, 0x15, 0x01e3);
		mdio_write(tp, 0x19, 0x4500);
		mdio_write(tp, 0x15, 0x01e4);
		mdio_write(tp, 0x19, 0x4069);
		mdio_write(tp, 0x15, 0x01e5);
		mdio_write(tp, 0x19, 0x4580);
		mdio_write(tp, 0x15, 0x01e6);
		mdio_write(tp, 0x19, 0x9f00);
		mdio_write(tp, 0x15, 0x01e7);
		mdio_write(tp, 0x19, 0xd0f5);
		mdio_write(tp, 0x15, 0x01e8);
		mdio_write(tp, 0x19, 0x00ff);
		mdio_write(tp, 0x15, 0x01e9);
		mdio_write(tp, 0x19, 0x4400);
		mdio_write(tp, 0x15, 0x01ea);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x01eb);
		mdio_write(tp, 0x19, 0x6800);
		mdio_write(tp, 0x15, 0x01ec);
		mdio_write(tp, 0x19, 0x66a0);
		mdio_write(tp, 0x15, 0x01ed);
		mdio_write(tp, 0x19, 0x8300);
		mdio_write(tp, 0x15, 0x01ee);
		mdio_write(tp, 0x19, 0x74f0);
		mdio_write(tp, 0x15, 0x01ef);
		mdio_write(tp, 0x19, 0x3006);
		mdio_write(tp, 0x15, 0x01f0);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x01f1);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x01f2);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x01f3);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x01f4);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x01f5);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x01f6);
		mdio_write(tp, 0x19, 0x6c02);
		mdio_write(tp, 0x15, 0x01f7);
		mdio_write(tp, 0x19, 0x409d);
		mdio_write(tp, 0x15, 0x01f8);
		mdio_write(tp, 0x19, 0x7c87);
		mdio_write(tp, 0x15, 0x01f9);
		mdio_write(tp, 0x19, 0xae14);
		mdio_write(tp, 0x15, 0x01fa);
		mdio_write(tp, 0x19, 0x4400);
		mdio_write(tp, 0x15, 0x01fb);
		mdio_write(tp, 0x19, 0x7c40);
		mdio_write(tp, 0x15, 0x01fc);
		mdio_write(tp, 0x19, 0x6800);
		mdio_write(tp, 0x15, 0x01fd);
		mdio_write(tp, 0x19, 0x7801);
		mdio_write(tp, 0x15, 0x01fe);
		mdio_write(tp, 0x19, 0x980e);
		mdio_write(tp, 0x15, 0x01ff);
		mdio_write(tp, 0x19, 0x930c);
		mdio_write(tp, 0x15, 0x0200);
		mdio_write(tp, 0x19, 0x9206);
		mdio_write(tp, 0x15, 0x0201);
		mdio_write(tp, 0x19, 0x4002);
		mdio_write(tp, 0x15, 0x0202);
		mdio_write(tp, 0x19, 0x7800);
		mdio_write(tp, 0x15, 0x0203);
		mdio_write(tp, 0x19, 0x588f);
		mdio_write(tp, 0x15, 0x0204);
		mdio_write(tp, 0x19, 0x5520);
		mdio_write(tp, 0x15, 0x0205);
		mdio_write(tp, 0x19, 0x320c);
		mdio_write(tp, 0x15, 0x0206);
		mdio_write(tp, 0x19, 0x4000);
		mdio_write(tp, 0x15, 0x0207);
		mdio_write(tp, 0x19, 0x7800);
		mdio_write(tp, 0x15, 0x0208);
		mdio_write(tp, 0x19, 0x588d);
		mdio_write(tp, 0x15, 0x0209);
		mdio_write(tp, 0x19, 0x5500);
		mdio_write(tp, 0x15, 0x020a);
		mdio_write(tp, 0x19, 0x320c);
		mdio_write(tp, 0x15, 0x020b);
		mdio_write(tp, 0x19, 0x4002);
		mdio_write(tp, 0x15, 0x020c);
		mdio_write(tp, 0x19, 0x3220);
		mdio_write(tp, 0x15, 0x020d);
		mdio_write(tp, 0x19, 0x4480);
		mdio_write(tp, 0x15, 0x020e);
		mdio_write(tp, 0x19, 0x9e03);
		mdio_write(tp, 0x15, 0x020f);
		mdio_write(tp, 0x19, 0x7c40);
		mdio_write(tp, 0x15, 0x0210);
		mdio_write(tp, 0x19, 0x6840);
		mdio_write(tp, 0x15, 0x0211);
		mdio_write(tp, 0x19, 0x7801);
		mdio_write(tp, 0x15, 0x0212);
		mdio_write(tp, 0x19, 0x980e);
		mdio_write(tp, 0x15, 0x0213);
		mdio_write(tp, 0x19, 0x930c);
		mdio_write(tp, 0x15, 0x0214);
		mdio_write(tp, 0x19, 0x9206);
		mdio_write(tp, 0x15, 0x0215);
		mdio_write(tp, 0x19, 0x4000);
		mdio_write(tp, 0x15, 0x0216);
		mdio_write(tp, 0x19, 0x7800);
		mdio_write(tp, 0x15, 0x0217);
		mdio_write(tp, 0x19, 0x588f);
		mdio_write(tp, 0x15, 0x0218);
		mdio_write(tp, 0x19, 0x5520);
		mdio_write(tp, 0x15, 0x0219);
		mdio_write(tp, 0x19, 0x3220);
		mdio_write(tp, 0x15, 0x021a);
		mdio_write(tp, 0x19, 0x4002);
		mdio_write(tp, 0x15, 0x021b);
		mdio_write(tp, 0x19, 0x7800);
		mdio_write(tp, 0x15, 0x021c);
		mdio_write(tp, 0x19, 0x588d);
		mdio_write(tp, 0x15, 0x021d);
		mdio_write(tp, 0x19, 0x5540);
		mdio_write(tp, 0x15, 0x021e);
		mdio_write(tp, 0x19, 0x3220);
		mdio_write(tp, 0x15, 0x021f);
		mdio_write(tp, 0x19, 0x4000);
		mdio_write(tp, 0x15, 0x0220);
		mdio_write(tp, 0x19, 0x7800);
		mdio_write(tp, 0x15, 0x0221);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x0222);
		mdio_write(tp, 0x19, 0x6c00);
		mdio_write(tp, 0x15, 0x0223);
		mdio_write(tp, 0x19, 0x3231);
		mdio_write(tp, 0x15, 0x0224);
		mdio_write(tp, 0x19, 0xab06);
		mdio_write(tp, 0x15, 0x0225);
		mdio_write(tp, 0x19, 0xbf08);
		mdio_write(tp, 0x15, 0x0226);
		mdio_write(tp, 0x19, 0x4076);
		mdio_write(tp, 0x15, 0x0227);
		mdio_write(tp, 0x19, 0x7d07);
		mdio_write(tp, 0x15, 0x0228);
		mdio_write(tp, 0x19, 0x4502);
		mdio_write(tp, 0x15, 0x0229);
		mdio_write(tp, 0x19, 0x3231);
		mdio_write(tp, 0x15, 0x022a);
		mdio_write(tp, 0x19, 0x7d80);
		mdio_write(tp, 0x15, 0x022b);
		mdio_write(tp, 0x19, 0x5180);
		mdio_write(tp, 0x15, 0x022c);
		mdio_write(tp, 0x19, 0x322f);
		mdio_write(tp, 0x15, 0x022d);
		mdio_write(tp, 0x19, 0x7d80);
		mdio_write(tp, 0x15, 0x022e);
		mdio_write(tp, 0x19, 0x5000);
		mdio_write(tp, 0x15, 0x022f);
		mdio_write(tp, 0x19, 0x7d07);
		mdio_write(tp, 0x15, 0x0230);
		mdio_write(tp, 0x19, 0x4402);
		mdio_write(tp, 0x15, 0x0231);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x0232);
		mdio_write(tp, 0x19, 0x6c02);
		mdio_write(tp, 0x15, 0x0233);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x0234);
		mdio_write(tp, 0x19, 0xb309);
		mdio_write(tp, 0x15, 0x0235);
		mdio_write(tp, 0x19, 0xb204);
		mdio_write(tp, 0x15, 0x0236);
		mdio_write(tp, 0x19, 0xb105);
		mdio_write(tp, 0x15, 0x0237);
		mdio_write(tp, 0x19, 0x6c00);
		mdio_write(tp, 0x15, 0x0238);
		mdio_write(tp, 0x19, 0x31c1);
		mdio_write(tp, 0x15, 0x0239);
		mdio_write(tp, 0x19, 0x6c00);
		mdio_write(tp, 0x15, 0x023a);
		mdio_write(tp, 0x19, 0x3261);
		mdio_write(tp, 0x15, 0x023b);
		mdio_write(tp, 0x19, 0x6c00);
		mdio_write(tp, 0x15, 0x023c);
		mdio_write(tp, 0x19, 0x3250);
		mdio_write(tp, 0x15, 0x023d);
		mdio_write(tp, 0x19, 0xb203);
		mdio_write(tp, 0x15, 0x023e);
		mdio_write(tp, 0x19, 0x6c00);
		mdio_write(tp, 0x15, 0x023f);
		mdio_write(tp, 0x19, 0x327a);
		mdio_write(tp, 0x15, 0x0240);
		mdio_write(tp, 0x19, 0x6c00);
		mdio_write(tp, 0x15, 0x0241);
		mdio_write(tp, 0x19, 0x3293);
		mdio_write(tp, 0x15, 0x0242);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0243);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0244);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0245);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0246);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0247);
		mdio_write(tp, 0x19, 0x32a3);
		mdio_write(tp, 0x15, 0x0248);
		mdio_write(tp, 0x19, 0x5520);
		mdio_write(tp, 0x15, 0x0249);
		mdio_write(tp, 0x19, 0x403d);
		mdio_write(tp, 0x15, 0x024a);
		mdio_write(tp, 0x19, 0x440c);
		mdio_write(tp, 0x15, 0x024b);
		mdio_write(tp, 0x19, 0x4812);
		mdio_write(tp, 0x15, 0x024c);
		mdio_write(tp, 0x19, 0x5001);
		mdio_write(tp, 0x15, 0x024d);
		mdio_write(tp, 0x19, 0x4802);
		mdio_write(tp, 0x15, 0x024e);
		mdio_write(tp, 0x19, 0x6880);
		mdio_write(tp, 0x15, 0x024f);
		mdio_write(tp, 0x19, 0x31f5);
		mdio_write(tp, 0x15, 0x0250);
		mdio_write(tp, 0x19, 0xb685);
		mdio_write(tp, 0x15, 0x0251);
		mdio_write(tp, 0x19, 0x801c);
		mdio_write(tp, 0x15, 0x0252);
		mdio_write(tp, 0x19, 0xbaf5);
		mdio_write(tp, 0x15, 0x0253);
		mdio_write(tp, 0x19, 0xc07c);
		mdio_write(tp, 0x15, 0x0254);
		mdio_write(tp, 0x19, 0x00fb);
		mdio_write(tp, 0x15, 0x0255);
		mdio_write(tp, 0x19, 0x325a);
		mdio_write(tp, 0x15, 0x0256);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0257);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0258);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0259);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x025a);
		mdio_write(tp, 0x19, 0x481a);
		mdio_write(tp, 0x15, 0x025b);
		mdio_write(tp, 0x19, 0x5001);
		mdio_write(tp, 0x15, 0x025c);
		mdio_write(tp, 0x19, 0x401b);
		mdio_write(tp, 0x15, 0x025d);
		mdio_write(tp, 0x19, 0x480a);
		mdio_write(tp, 0x15, 0x025e);
		mdio_write(tp, 0x19, 0x4418);
		mdio_write(tp, 0x15, 0x025f);
		mdio_write(tp, 0x19, 0x6900);
		mdio_write(tp, 0x15, 0x0260);
		mdio_write(tp, 0x19, 0x31f5);
		mdio_write(tp, 0x15, 0x0261);
		mdio_write(tp, 0x19, 0xb64b);
		mdio_write(tp, 0x15, 0x0262);
		mdio_write(tp, 0x19, 0xdb00);
		mdio_write(tp, 0x15, 0x0263);
		mdio_write(tp, 0x19, 0x0048);
		mdio_write(tp, 0x15, 0x0264);
		mdio_write(tp, 0x19, 0xdb7d);
		mdio_write(tp, 0x15, 0x0265);
		mdio_write(tp, 0x19, 0x0002);
		mdio_write(tp, 0x15, 0x0266);
		mdio_write(tp, 0x19, 0xa0fa);
		mdio_write(tp, 0x15, 0x0267);
		mdio_write(tp, 0x19, 0x4408);
		mdio_write(tp, 0x15, 0x0268);
		mdio_write(tp, 0x19, 0x3248);
		mdio_write(tp, 0x15, 0x0269);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x026a);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x026b);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x026c);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x026d);
		mdio_write(tp, 0x19, 0xb806);
		mdio_write(tp, 0x15, 0x026e);
		mdio_write(tp, 0x19, 0x588d);
		mdio_write(tp, 0x15, 0x026f);
		mdio_write(tp, 0x19, 0x5500);
		mdio_write(tp, 0x15, 0x0270);
		mdio_write(tp, 0x19, 0x7801);
		mdio_write(tp, 0x15, 0x0271);
		mdio_write(tp, 0x19, 0x4002);
		mdio_write(tp, 0x15, 0x0272);
		mdio_write(tp, 0x19, 0x7800);
		mdio_write(tp, 0x15, 0x0273);
		mdio_write(tp, 0x19, 0x4814);
		mdio_write(tp, 0x15, 0x0274);
		mdio_write(tp, 0x19, 0x500b);
		mdio_write(tp, 0x15, 0x0275);
		mdio_write(tp, 0x19, 0x4804);
		mdio_write(tp, 0x15, 0x0276);
		mdio_write(tp, 0x19, 0x40c4);
		mdio_write(tp, 0x15, 0x0277);
		mdio_write(tp, 0x19, 0x4425);
		mdio_write(tp, 0x15, 0x0278);
		mdio_write(tp, 0x19, 0x6a00);
		mdio_write(tp, 0x15, 0x0279);
		mdio_write(tp, 0x19, 0x31f5);
		mdio_write(tp, 0x15, 0x027a);
		mdio_write(tp, 0x19, 0xb632);
		mdio_write(tp, 0x15, 0x027b);
		mdio_write(tp, 0x19, 0xdc03);
		mdio_write(tp, 0x15, 0x027c);
		mdio_write(tp, 0x19, 0x0027);
		mdio_write(tp, 0x15, 0x027d);
		mdio_write(tp, 0x19, 0x80fc);
		mdio_write(tp, 0x15, 0x027e);
		mdio_write(tp, 0x19, 0x3283);
		mdio_write(tp, 0x15, 0x027f);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0280);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0281);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0282);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0283);
		mdio_write(tp, 0x19, 0xb806);
		mdio_write(tp, 0x15, 0x0284);
		mdio_write(tp, 0x19, 0x588f);
		mdio_write(tp, 0x15, 0x0285);
		mdio_write(tp, 0x19, 0x5520);
		mdio_write(tp, 0x15, 0x0286);
		mdio_write(tp, 0x19, 0x7801);
		mdio_write(tp, 0x15, 0x0287);
		mdio_write(tp, 0x19, 0x4000);
		mdio_write(tp, 0x15, 0x0288);
		mdio_write(tp, 0x19, 0x7800);
		mdio_write(tp, 0x15, 0x0289);
		mdio_write(tp, 0x19, 0x4818);
		mdio_write(tp, 0x15, 0x028a);
		mdio_write(tp, 0x19, 0x5051);
		mdio_write(tp, 0x15, 0x028b);
		mdio_write(tp, 0x19, 0x4808);
		mdio_write(tp, 0x15, 0x028c);
		mdio_write(tp, 0x19, 0x4050);
		mdio_write(tp, 0x15, 0x028d);
		mdio_write(tp, 0x19, 0x4462);
		mdio_write(tp, 0x15, 0x028e);
		mdio_write(tp, 0x19, 0x40c4);
		mdio_write(tp, 0x15, 0x028f);
		mdio_write(tp, 0x19, 0x4473);
		mdio_write(tp, 0x15, 0x0290);
		mdio_write(tp, 0x19, 0x5041);
		mdio_write(tp, 0x15, 0x0291);
		mdio_write(tp, 0x19, 0x6b00);
		mdio_write(tp, 0x15, 0x0292);
		mdio_write(tp, 0x19, 0x31f5);
		mdio_write(tp, 0x15, 0x0293);
		mdio_write(tp, 0x19, 0xb619);
		mdio_write(tp, 0x15, 0x0294);
		mdio_write(tp, 0x19, 0x80d9);
		mdio_write(tp, 0x15, 0x0295);
		mdio_write(tp, 0x19, 0xbd06);
		mdio_write(tp, 0x15, 0x0296);
		mdio_write(tp, 0x19, 0xbb0d);
		mdio_write(tp, 0x15, 0x0297);
		mdio_write(tp, 0x19, 0xaf14);
		mdio_write(tp, 0x15, 0x0298);
		mdio_write(tp, 0x19, 0x8efa);
		mdio_write(tp, 0x15, 0x0299);
		mdio_write(tp, 0x19, 0x5049);
		mdio_write(tp, 0x15, 0x029a);
		mdio_write(tp, 0x19, 0x3248);
		mdio_write(tp, 0x15, 0x029b);
		mdio_write(tp, 0x19, 0x4c10);
		mdio_write(tp, 0x15, 0x029c);
		mdio_write(tp, 0x19, 0x44b0);
		mdio_write(tp, 0x15, 0x029d);
		mdio_write(tp, 0x19, 0x4c00);
		mdio_write(tp, 0x15, 0x029e);
		mdio_write(tp, 0x19, 0x3292);
		mdio_write(tp, 0x15, 0x029f);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02a0);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02a1);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02a2);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02a3);
		mdio_write(tp, 0x19, 0x481f);
		mdio_write(tp, 0x15, 0x02a4);
		mdio_write(tp, 0x19, 0x5005);
		mdio_write(tp, 0x15, 0x02a5);
		mdio_write(tp, 0x19, 0x480f);
		mdio_write(tp, 0x15, 0x02a6);
		mdio_write(tp, 0x19, 0xac00);
		mdio_write(tp, 0x15, 0x02a7);
		mdio_write(tp, 0x19, 0x31a6);
		mdio_write(tp, 0x15, 0x02a8);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02a9);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02aa);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02ab);
		mdio_write(tp, 0x19, 0x31ba);
		mdio_write(tp, 0x15, 0x02ac);
		mdio_write(tp, 0x19, 0x31d5);
		mdio_write(tp, 0x15, 0x02ad);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02ae);
		mdio_write(tp, 0x19, 0x5cf0);
		mdio_write(tp, 0x15, 0x02af);
		mdio_write(tp, 0x19, 0x588c);
		mdio_write(tp, 0x15, 0x02b0);
		mdio_write(tp, 0x19, 0x542f);
		mdio_write(tp, 0x15, 0x02b1);
		mdio_write(tp, 0x19, 0x7ffb);
		mdio_write(tp, 0x15, 0x02b2);
		mdio_write(tp, 0x19, 0x6ff8);
		mdio_write(tp, 0x15, 0x02b3);
		mdio_write(tp, 0x19, 0x64a4);
		mdio_write(tp, 0x15, 0x02b4);
		mdio_write(tp, 0x19, 0x64a0);
		mdio_write(tp, 0x15, 0x02b5);
		mdio_write(tp, 0x19, 0x6800);
		mdio_write(tp, 0x15, 0x02b6);
		mdio_write(tp, 0x19, 0x4400);
		mdio_write(tp, 0x15, 0x02b7);
		mdio_write(tp, 0x19, 0x4020);
		mdio_write(tp, 0x15, 0x02b8);
		mdio_write(tp, 0x19, 0x4480);
		mdio_write(tp, 0x15, 0x02b9);
		mdio_write(tp, 0x19, 0x9e00);
		mdio_write(tp, 0x15, 0x02ba);
		mdio_write(tp, 0x19, 0x4891);
		mdio_write(tp, 0x15, 0x02bb);
		mdio_write(tp, 0x19, 0x4cc0);
		mdio_write(tp, 0x15, 0x02bc);
		mdio_write(tp, 0x19, 0x4801);
		mdio_write(tp, 0x15, 0x02bd);
		mdio_write(tp, 0x19, 0xa609);
		mdio_write(tp, 0x15, 0x02be);
		mdio_write(tp, 0x19, 0xd64f);
		mdio_write(tp, 0x15, 0x02bf);
		mdio_write(tp, 0x19, 0x004e);
		mdio_write(tp, 0x15, 0x02c0);
		mdio_write(tp, 0x19, 0x87fe);
		mdio_write(tp, 0x15, 0x02c1);
		mdio_write(tp, 0x19, 0x32c6);
		mdio_write(tp, 0x15, 0x02c2);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02c3);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02c4);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02c5);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02c6);
		mdio_write(tp, 0x19, 0x48b2);
		mdio_write(tp, 0x15, 0x02c7);
		mdio_write(tp, 0x19, 0x4020);
		mdio_write(tp, 0x15, 0x02c8);
		mdio_write(tp, 0x19, 0x4822);
		mdio_write(tp, 0x15, 0x02c9);
		mdio_write(tp, 0x19, 0x4488);
		mdio_write(tp, 0x15, 0x02ca);
		mdio_write(tp, 0x19, 0xd64f);
		mdio_write(tp, 0x15, 0x02cb);
		mdio_write(tp, 0x19, 0x0042);
		mdio_write(tp, 0x15, 0x02cc);
		mdio_write(tp, 0x19, 0x8203);
		mdio_write(tp, 0x15, 0x02cd);
		mdio_write(tp, 0x19, 0x4cc8);
		mdio_write(tp, 0x15, 0x02ce);
		mdio_write(tp, 0x19, 0x32d0);
		mdio_write(tp, 0x15, 0x02cf);
		mdio_write(tp, 0x19, 0x4cc0);
		mdio_write(tp, 0x15, 0x02d0);
		mdio_write(tp, 0x19, 0xc4d4);
		mdio_write(tp, 0x15, 0x02d1);
		mdio_write(tp, 0x19, 0x00f9);
		mdio_write(tp, 0x15, 0x02d2);
		mdio_write(tp, 0x19, 0xa51a);
		mdio_write(tp, 0x15, 0x02d3);
		mdio_write(tp, 0x19, 0x32d9);
		mdio_write(tp, 0x15, 0x02d4);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02d5);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02d6);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02d7);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02d8);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02d9);
		mdio_write(tp, 0x19, 0x48b3);
		mdio_write(tp, 0x15, 0x02da);
		mdio_write(tp, 0x19, 0x4020);
		mdio_write(tp, 0x15, 0x02db);
		mdio_write(tp, 0x19, 0x4823);
		mdio_write(tp, 0x15, 0x02dc);
		mdio_write(tp, 0x19, 0x4410);
		mdio_write(tp, 0x15, 0x02dd);
		mdio_write(tp, 0x19, 0xb630);
		mdio_write(tp, 0x15, 0x02de);
		mdio_write(tp, 0x19, 0x7dc8);
		mdio_write(tp, 0x15, 0x02df);
		mdio_write(tp, 0x19, 0x8203);
		mdio_write(tp, 0x15, 0x02e0);
		mdio_write(tp, 0x19, 0x4c48);
		mdio_write(tp, 0x15, 0x02e1);
		mdio_write(tp, 0x19, 0x32e3);
		mdio_write(tp, 0x15, 0x02e2);
		mdio_write(tp, 0x19, 0x4c40);
		mdio_write(tp, 0x15, 0x02e3);
		mdio_write(tp, 0x19, 0x9bfa);
		mdio_write(tp, 0x15, 0x02e4);
		mdio_write(tp, 0x19, 0x84ca);
		mdio_write(tp, 0x15, 0x02e5);
		mdio_write(tp, 0x19, 0x85f8);
		mdio_write(tp, 0x15, 0x02e6);
		mdio_write(tp, 0x19, 0x32ec);
		mdio_write(tp, 0x15, 0x02e7);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02e8);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02e9);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02ea);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02eb);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x02ec);
		mdio_write(tp, 0x19, 0x48d4);
		mdio_write(tp, 0x15, 0x02ed);
		mdio_write(tp, 0x19, 0x4020);
		mdio_write(tp, 0x15, 0x02ee);
		mdio_write(tp, 0x19, 0x4844);
		mdio_write(tp, 0x15, 0x02ef);
		mdio_write(tp, 0x19, 0x4420);
		mdio_write(tp, 0x15, 0x02f0);
		mdio_write(tp, 0x19, 0x6800);
		mdio_write(tp, 0x15, 0x02f1);
		mdio_write(tp, 0x19, 0x7dc0);
		mdio_write(tp, 0x15, 0x02f2);
		mdio_write(tp, 0x19, 0x4c40);
		mdio_write(tp, 0x15, 0x02f3);
		mdio_write(tp, 0x19, 0x7c0b);
		mdio_write(tp, 0x15, 0x02f4);
		mdio_write(tp, 0x19, 0x6c08);
		mdio_write(tp, 0x15, 0x02f5);
		mdio_write(tp, 0x19, 0x3311);
		mdio_write(tp, 0x15, 0x02f6);
		mdio_write(tp, 0x19, 0x9cfd);
		mdio_write(tp, 0x15, 0x02f7);
		mdio_write(tp, 0x19, 0xb616);
		mdio_write(tp, 0x15, 0x02f8);
		mdio_write(tp, 0x19, 0xc42b);
		mdio_write(tp, 0x15, 0x02f9);
		mdio_write(tp, 0x19, 0x00e0);
		mdio_write(tp, 0x15, 0x02fa);
		mdio_write(tp, 0x19, 0xc455);
		mdio_write(tp, 0x15, 0x02fb);
		mdio_write(tp, 0x19, 0x00b3);
		mdio_write(tp, 0x15, 0x02fc);
		mdio_write(tp, 0x19, 0xb20a);
		mdio_write(tp, 0x15, 0x02fd);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x02fe);
		mdio_write(tp, 0x19, 0x6c02);
		mdio_write(tp, 0x15, 0x02ff);
		mdio_write(tp, 0x19, 0x8204);
		mdio_write(tp, 0x15, 0x0300);
		mdio_write(tp, 0x19, 0x7c04);
		mdio_write(tp, 0x15, 0x0301);
		mdio_write(tp, 0x19, 0x7404);
		mdio_write(tp, 0x15, 0x0302);
		mdio_write(tp, 0x19, 0x32f3);
		mdio_write(tp, 0x15, 0x0303);
		mdio_write(tp, 0x19, 0x7c04);
		mdio_write(tp, 0x15, 0x0304);
		mdio_write(tp, 0x19, 0x7400);
		mdio_write(tp, 0x15, 0x0305);
		mdio_write(tp, 0x19, 0x32f3);
		mdio_write(tp, 0x15, 0x0306);
		mdio_write(tp, 0x19, 0xefed);
		mdio_write(tp, 0x15, 0x0307);
		mdio_write(tp, 0x19, 0x3342);
		mdio_write(tp, 0x15, 0x0308);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0309);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x030a);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x030b);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x030c);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x030d);
		mdio_write(tp, 0x19, 0x3006);
		mdio_write(tp, 0x15, 0x030e);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x030f);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0310);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0311);
		mdio_write(tp, 0x19, 0x7c08);
		mdio_write(tp, 0x15, 0x0312);
		mdio_write(tp, 0x19, 0xa207);
		mdio_write(tp, 0x15, 0x0313);
		mdio_write(tp, 0x19, 0x4c00);
		mdio_write(tp, 0x15, 0x0314);
		mdio_write(tp, 0x19, 0x3322);
		mdio_write(tp, 0x15, 0x0315);
		mdio_write(tp, 0x19, 0x4041);
		mdio_write(tp, 0x15, 0x0316);
		mdio_write(tp, 0x19, 0x7d07);
		mdio_write(tp, 0x15, 0x0317);
		mdio_write(tp, 0x19, 0x4502);
		mdio_write(tp, 0x15, 0x0318);
		mdio_write(tp, 0x19, 0x3322);
		mdio_write(tp, 0x15, 0x0319);
		mdio_write(tp, 0x19, 0x4c08);
		mdio_write(tp, 0x15, 0x031a);
		mdio_write(tp, 0x19, 0x3322);
		mdio_write(tp, 0x15, 0x031b);
		mdio_write(tp, 0x19, 0x7d80);
		mdio_write(tp, 0x15, 0x031c);
		mdio_write(tp, 0x19, 0x5180);
		mdio_write(tp, 0x15, 0x031d);
		mdio_write(tp, 0x19, 0x3320);
		mdio_write(tp, 0x15, 0x031e);
		mdio_write(tp, 0x19, 0x7d80);
		mdio_write(tp, 0x15, 0x031f);
		mdio_write(tp, 0x19, 0x5000);
		mdio_write(tp, 0x15, 0x0320);
		mdio_write(tp, 0x19, 0x7d07);
		mdio_write(tp, 0x15, 0x0321);
		mdio_write(tp, 0x19, 0x4402);
		mdio_write(tp, 0x15, 0x0322);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x0323);
		mdio_write(tp, 0x19, 0x6c02);
		mdio_write(tp, 0x15, 0x0324);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x0325);
		mdio_write(tp, 0x19, 0xb30c);
		mdio_write(tp, 0x15, 0x0326);
		mdio_write(tp, 0x19, 0xb206);
		mdio_write(tp, 0x15, 0x0327);
		mdio_write(tp, 0x19, 0xb103);
		mdio_write(tp, 0x15, 0x0328);
		mdio_write(tp, 0x19, 0x6c00);
		mdio_write(tp, 0x15, 0x0329);
		mdio_write(tp, 0x19, 0x32f6);
		mdio_write(tp, 0x15, 0x032a);
		mdio_write(tp, 0x19, 0x6c00);
		mdio_write(tp, 0x15, 0x032b);
		mdio_write(tp, 0x19, 0x3352);
		mdio_write(tp, 0x15, 0x032c);
		mdio_write(tp, 0x19, 0xb103);
		mdio_write(tp, 0x15, 0x032d);
		mdio_write(tp, 0x19, 0x6c00);
		mdio_write(tp, 0x15, 0x032e);
		mdio_write(tp, 0x19, 0x336a);
		mdio_write(tp, 0x15, 0x032f);
		mdio_write(tp, 0x19, 0x6c00);
		mdio_write(tp, 0x15, 0x0330);
		mdio_write(tp, 0x19, 0x3382);
		mdio_write(tp, 0x15, 0x0331);
		mdio_write(tp, 0x19, 0xb206);
		mdio_write(tp, 0x15, 0x0332);
		mdio_write(tp, 0x19, 0xb103);
		mdio_write(tp, 0x15, 0x0333);
		mdio_write(tp, 0x19, 0x6c00);
		mdio_write(tp, 0x15, 0x0334);
		mdio_write(tp, 0x19, 0x3395);
		mdio_write(tp, 0x15, 0x0335);
		mdio_write(tp, 0x19, 0x6c00);
		mdio_write(tp, 0x15, 0x0336);
		mdio_write(tp, 0x19, 0x33c6);
		mdio_write(tp, 0x15, 0x0337);
		mdio_write(tp, 0x19, 0xb103);
		mdio_write(tp, 0x15, 0x0338);
		mdio_write(tp, 0x19, 0x6c00);
		mdio_write(tp, 0x15, 0x0339);
		mdio_write(tp, 0x19, 0x33d7);
		mdio_write(tp, 0x15, 0x033a);
		mdio_write(tp, 0x19, 0x6c00);
		mdio_write(tp, 0x15, 0x033b);
		mdio_write(tp, 0x19, 0x33f2);
		mdio_write(tp, 0x15, 0x033c);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x033d);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x033e);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x033f);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0340);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0341);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0342);
		mdio_write(tp, 0x19, 0x49b5);
		mdio_write(tp, 0x15, 0x0343);
		mdio_write(tp, 0x19, 0x7d00);
		mdio_write(tp, 0x15, 0x0344);
		mdio_write(tp, 0x19, 0x4d00);
		mdio_write(tp, 0x15, 0x0345);
		mdio_write(tp, 0x19, 0x6880);
		mdio_write(tp, 0x15, 0x0346);
		mdio_write(tp, 0x19, 0x7c08);
		mdio_write(tp, 0x15, 0x0347);
		mdio_write(tp, 0x19, 0x6c08);
		mdio_write(tp, 0x15, 0x0348);
		mdio_write(tp, 0x19, 0x4925);
		mdio_write(tp, 0x15, 0x0349);
		mdio_write(tp, 0x19, 0x403b);
		mdio_write(tp, 0x15, 0x034a);
		mdio_write(tp, 0x19, 0xa602);
		mdio_write(tp, 0x15, 0x034b);
		mdio_write(tp, 0x19, 0x402f);
		mdio_write(tp, 0x15, 0x034c);
		mdio_write(tp, 0x19, 0x4484);
		mdio_write(tp, 0x15, 0x034d);
		mdio_write(tp, 0x19, 0x40c8);
		mdio_write(tp, 0x15, 0x034e);
		mdio_write(tp, 0x19, 0x44c4);
		mdio_write(tp, 0x15, 0x034f);
		mdio_write(tp, 0x19, 0xd64f);
		mdio_write(tp, 0x15, 0x0350);
		mdio_write(tp, 0x19, 0x00bd);
		mdio_write(tp, 0x15, 0x0351);
		mdio_write(tp, 0x19, 0x3311);
		mdio_write(tp, 0x15, 0x0352);
		mdio_write(tp, 0x19, 0xc8ed);
		mdio_write(tp, 0x15, 0x0353);
		mdio_write(tp, 0x19, 0x00fc);
		mdio_write(tp, 0x15, 0x0354);
		mdio_write(tp, 0x19, 0x8221);
		mdio_write(tp, 0x15, 0x0355);
		mdio_write(tp, 0x19, 0xd11d);
		mdio_write(tp, 0x15, 0x0356);
		mdio_write(tp, 0x19, 0x001f);
		mdio_write(tp, 0x15, 0x0357);
		mdio_write(tp, 0x19, 0xde18);
		mdio_write(tp, 0x15, 0x0358);
		mdio_write(tp, 0x19, 0x0008);
		mdio_write(tp, 0x15, 0x0359);
		mdio_write(tp, 0x19, 0x91f6);
		mdio_write(tp, 0x15, 0x035a);
		mdio_write(tp, 0x19, 0x3360);
		mdio_write(tp, 0x15, 0x035b);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x035c);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x035d);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x035e);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x035f);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0360);
		mdio_write(tp, 0x19, 0x4bb6);
		mdio_write(tp, 0x15, 0x0361);
		mdio_write(tp, 0x19, 0x4064);
		mdio_write(tp, 0x15, 0x0362);
		mdio_write(tp, 0x19, 0x4b26);
		mdio_write(tp, 0x15, 0x0363);
		mdio_write(tp, 0x19, 0x4410);
		mdio_write(tp, 0x15, 0x0364);
		mdio_write(tp, 0x19, 0x4006);
		mdio_write(tp, 0x15, 0x0365);
		mdio_write(tp, 0x19, 0x4490);
		mdio_write(tp, 0x15, 0x0366);
		mdio_write(tp, 0x19, 0x6900);
		mdio_write(tp, 0x15, 0x0367);
		mdio_write(tp, 0x19, 0xb6a6);
		mdio_write(tp, 0x15, 0x0368);
		mdio_write(tp, 0x19, 0x9e02);
		mdio_write(tp, 0x15, 0x0369);
		mdio_write(tp, 0x19, 0x3311);
		mdio_write(tp, 0x15, 0x036a);
		mdio_write(tp, 0x19, 0xd11d);
		mdio_write(tp, 0x15, 0x036b);
		mdio_write(tp, 0x19, 0x000a);
		mdio_write(tp, 0x15, 0x036c);
		mdio_write(tp, 0x19, 0xbb0f);
		mdio_write(tp, 0x15, 0x036d);
		mdio_write(tp, 0x19, 0x8102);
		mdio_write(tp, 0x15, 0x036e);
		mdio_write(tp, 0x19, 0x3371);
		mdio_write(tp, 0x15, 0x036f);
		mdio_write(tp, 0x19, 0xa21e);
		mdio_write(tp, 0x15, 0x0370);
		mdio_write(tp, 0x19, 0x33b6);
		mdio_write(tp, 0x15, 0x0371);
		mdio_write(tp, 0x19, 0x91f6);
		mdio_write(tp, 0x15, 0x0372);
		mdio_write(tp, 0x19, 0xc218);
		mdio_write(tp, 0x15, 0x0373);
		mdio_write(tp, 0x19, 0x00f4);
		mdio_write(tp, 0x15, 0x0374);
		mdio_write(tp, 0x19, 0x33b6);
		mdio_write(tp, 0x15, 0x0375);
		mdio_write(tp, 0x19, 0x32ec);
		mdio_write(tp, 0x15, 0x0376);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0377);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0378);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x0379);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x037a);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x037b);
		mdio_write(tp, 0x19, 0x4b97);
		mdio_write(tp, 0x15, 0x037c);
		mdio_write(tp, 0x19, 0x402b);
		mdio_write(tp, 0x15, 0x037d);
		mdio_write(tp, 0x19, 0x4b07);
		mdio_write(tp, 0x15, 0x037e);
		mdio_write(tp, 0x19, 0x4422);
		mdio_write(tp, 0x15, 0x037f);
		mdio_write(tp, 0x19, 0x6980);
		mdio_write(tp, 0x15, 0x0380);
		mdio_write(tp, 0x19, 0xb608);
		mdio_write(tp, 0x15, 0x0381);
		mdio_write(tp, 0x19, 0x3311);
		mdio_write(tp, 0x15, 0x0382);
		mdio_write(tp, 0x19, 0xbc05);
		mdio_write(tp, 0x15, 0x0383);
		mdio_write(tp, 0x19, 0xc21c);
		mdio_write(tp, 0x15, 0x0384);
		mdio_write(tp, 0x19, 0x0032);
		mdio_write(tp, 0x15, 0x0385);
		mdio_write(tp, 0x19, 0xa1fb);
		mdio_write(tp, 0x15, 0x0386);
		mdio_write(tp, 0x19, 0x338d);
		mdio_write(tp, 0x15, 0x0387);
		mdio_write(tp, 0x19, 0x32ae);
		mdio_write(tp, 0x15, 0x0388);
		mdio_write(tp, 0x19, 0x330d);
		mdio_write(tp, 0x15, 0x0389);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x038a);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x038b);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x038c);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x038d);
		mdio_write(tp, 0x19, 0x4b97);
		mdio_write(tp, 0x15, 0x038e);
		mdio_write(tp, 0x19, 0x6a08);
		mdio_write(tp, 0x15, 0x038f);
		mdio_write(tp, 0x19, 0x4b07);
		mdio_write(tp, 0x15, 0x0390);
		mdio_write(tp, 0x19, 0x40ac);
		mdio_write(tp, 0x15, 0x0391);
		mdio_write(tp, 0x19, 0x4445);
		mdio_write(tp, 0x15, 0x0392);
		mdio_write(tp, 0x19, 0x404e);
		mdio_write(tp, 0x15, 0x0393);
		mdio_write(tp, 0x19, 0x4461);
		mdio_write(tp, 0x15, 0x0394);
		mdio_write(tp, 0x19, 0x3311);
		mdio_write(tp, 0x15, 0x0395);
		mdio_write(tp, 0x19, 0x9c0a);
		mdio_write(tp, 0x15, 0x0396);
		mdio_write(tp, 0x19, 0x63da);
		mdio_write(tp, 0x15, 0x0397);
		mdio_write(tp, 0x19, 0x6f0c);
		mdio_write(tp, 0x15, 0x0398);
		mdio_write(tp, 0x19, 0x5440);
		mdio_write(tp, 0x15, 0x0399);
		mdio_write(tp, 0x19, 0x4b98);
		mdio_write(tp, 0x15, 0x039a);
		mdio_write(tp, 0x19, 0x7c40);
		mdio_write(tp, 0x15, 0x039b);
		mdio_write(tp, 0x19, 0x4c00);
		mdio_write(tp, 0x15, 0x039c);
		mdio_write(tp, 0x19, 0x4b08);
		mdio_write(tp, 0x15, 0x039d);
		mdio_write(tp, 0x19, 0x63d8);
		mdio_write(tp, 0x15, 0x039e);
		mdio_write(tp, 0x19, 0x33a5);
		mdio_write(tp, 0x15, 0x039f);
		mdio_write(tp, 0x19, 0xd64f);
		mdio_write(tp, 0x15, 0x03a0);
		mdio_write(tp, 0x19, 0x00e8);
		mdio_write(tp, 0x15, 0x03a1);
		mdio_write(tp, 0x19, 0x820e);
		mdio_write(tp, 0x15, 0x03a2);
		mdio_write(tp, 0x19, 0xa10d);
		mdio_write(tp, 0x15, 0x03a3);
		mdio_write(tp, 0x19, 0x9df1);
		mdio_write(tp, 0x15, 0x03a4);
		mdio_write(tp, 0x19, 0x33af);
		mdio_write(tp, 0x15, 0x03a5);
		mdio_write(tp, 0x19, 0xd64f);
		mdio_write(tp, 0x15, 0x03a6);
		mdio_write(tp, 0x19, 0x00f9);
		mdio_write(tp, 0x15, 0x03a7);
		mdio_write(tp, 0x19, 0xc017);
		mdio_write(tp, 0x15, 0x03a8);
		mdio_write(tp, 0x19, 0x0007);
		mdio_write(tp, 0x15, 0x03a9);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x03aa);
		mdio_write(tp, 0x19, 0x6c03);
		mdio_write(tp, 0x15, 0x03ab);
		mdio_write(tp, 0x19, 0xa104);
		mdio_write(tp, 0x15, 0x03ac);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x03ad);
		mdio_write(tp, 0x19, 0x6c00);
		mdio_write(tp, 0x15, 0x03ae);
		mdio_write(tp, 0x19, 0x9df7);
		mdio_write(tp, 0x15, 0x03af);
		mdio_write(tp, 0x19, 0x7c03);
		mdio_write(tp, 0x15, 0x03b0);
		mdio_write(tp, 0x19, 0x6c08);
		mdio_write(tp, 0x15, 0x03b1);
		mdio_write(tp, 0x19, 0x33b6);
		mdio_write(tp, 0x15, 0x03b2);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x03b3);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x03b4);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x03b5);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x03b6);
		mdio_write(tp, 0x19, 0x55af);
		mdio_write(tp, 0x15, 0x03b7);
		mdio_write(tp, 0x19, 0x7ff0);
		mdio_write(tp, 0x15, 0x03b8);
		mdio_write(tp, 0x19, 0x6ff0);
		mdio_write(tp, 0x15, 0x03b9);
		mdio_write(tp, 0x19, 0x4bb9);
		mdio_write(tp, 0x15, 0x03ba);
		mdio_write(tp, 0x19, 0x6a80);
		mdio_write(tp, 0x15, 0x03bb);
		mdio_write(tp, 0x19, 0x4b29);
		mdio_write(tp, 0x15, 0x03bc);
		mdio_write(tp, 0x19, 0x4041);
		mdio_write(tp, 0x15, 0x03bd);
		mdio_write(tp, 0x19, 0x440a);
		mdio_write(tp, 0x15, 0x03be);
		mdio_write(tp, 0x19, 0x4029);
		mdio_write(tp, 0x15, 0x03bf);
		mdio_write(tp, 0x19, 0x4418);
		mdio_write(tp, 0x15, 0x03c0);
		mdio_write(tp, 0x19, 0x4090);
		mdio_write(tp, 0x15, 0x03c1);
		mdio_write(tp, 0x19, 0x4438);
		mdio_write(tp, 0x15, 0x03c2);
		mdio_write(tp, 0x19, 0x40c4);
		mdio_write(tp, 0x15, 0x03c3);
		mdio_write(tp, 0x19, 0x447b);
		mdio_write(tp, 0x15, 0x03c4);
		mdio_write(tp, 0x19, 0xb6c4);
		mdio_write(tp, 0x15, 0x03c5);
		mdio_write(tp, 0x19, 0x3311);
		mdio_write(tp, 0x15, 0x03c6);
		mdio_write(tp, 0x19, 0x9bfe);
		mdio_write(tp, 0x15, 0x03c7);
		mdio_write(tp, 0x19, 0x33cc);
		mdio_write(tp, 0x15, 0x03c8);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x03c9);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x03ca);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x03cb);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x03cc);
		mdio_write(tp, 0x19, 0x542f);
		mdio_write(tp, 0x15, 0x03cd);
		mdio_write(tp, 0x19, 0x499a);
		mdio_write(tp, 0x15, 0x03ce);
		mdio_write(tp, 0x19, 0x7c40);
		mdio_write(tp, 0x15, 0x03cf);
		mdio_write(tp, 0x19, 0x4c40);
		mdio_write(tp, 0x15, 0x03d0);
		mdio_write(tp, 0x19, 0x490a);
		mdio_write(tp, 0x15, 0x03d1);
		mdio_write(tp, 0x19, 0x405e);
		mdio_write(tp, 0x15, 0x03d2);
		mdio_write(tp, 0x19, 0x44f8);
		mdio_write(tp, 0x15, 0x03d3);
		mdio_write(tp, 0x19, 0x6b00);
		mdio_write(tp, 0x15, 0x03d4);
		mdio_write(tp, 0x19, 0xd64f);
		mdio_write(tp, 0x15, 0x03d5);
		mdio_write(tp, 0x19, 0x0028);
		mdio_write(tp, 0x15, 0x03d6);
		mdio_write(tp, 0x19, 0x3311);
		mdio_write(tp, 0x15, 0x03d7);
		mdio_write(tp, 0x19, 0xbd27);
		mdio_write(tp, 0x15, 0x03d8);
		mdio_write(tp, 0x19, 0x9cfc);
		mdio_write(tp, 0x15, 0x03d9);
		mdio_write(tp, 0x19, 0xc639);
		mdio_write(tp, 0x15, 0x03da);
		mdio_write(tp, 0x19, 0x000f);
		mdio_write(tp, 0x15, 0x03db);
		mdio_write(tp, 0x19, 0x9e03);
		mdio_write(tp, 0x15, 0x03dc);
		mdio_write(tp, 0x19, 0x7c01);
		mdio_write(tp, 0x15, 0x03dd);
		mdio_write(tp, 0x19, 0x4c01);
		mdio_write(tp, 0x15, 0x03de);
		mdio_write(tp, 0x19, 0x9af6);
		mdio_write(tp, 0x15, 0x03df);
		mdio_write(tp, 0x19, 0x7c12);
		mdio_write(tp, 0x15, 0x03e0);
		mdio_write(tp, 0x19, 0x4c52);
		mdio_write(tp, 0x15, 0x03e1);
		mdio_write(tp, 0x19, 0x4470);
		mdio_write(tp, 0x15, 0x03e2);
		mdio_write(tp, 0x19, 0x7c12);
		mdio_write(tp, 0x15, 0x03e3);
		mdio_write(tp, 0x19, 0x4c40);
		mdio_write(tp, 0x15, 0x03e4);
		mdio_write(tp, 0x19, 0x33d4);
		mdio_write(tp, 0x15, 0x03e5);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x03e6);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x03e7);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x03e8);
		mdio_write(tp, 0x19, 0x0000);
		mdio_write(tp, 0x15, 0x03e9);
		mdio_write(tp, 0x19, 0x49bb);
		mdio_write(tp, 0x15, 0x03ea);
		mdio_write(tp, 0x19, 0x4478);
		mdio_write(tp, 0x15, 0x03eb);
		mdio_write(tp, 0x19, 0x492b);
		mdio_write(tp, 0x15, 0x03ec);
		mdio_write(tp, 0x19, 0x6b80);
		mdio_write(tp, 0x15, 0x03ed);
		mdio_write(tp, 0x19, 0x7c01);
		mdio_write(tp, 0x15, 0x03ee);
		mdio_write(tp, 0x19, 0x4c00);
		mdio_write(tp, 0x15, 0x03ef);
		mdio_write(tp, 0x19, 0xd64f);
		mdio_write(tp, 0x15, 0x03f0);
		mdio_write(tp, 0x19, 0x000d);
		mdio_write(tp, 0x15, 0x03f1);
		mdio_write(tp, 0x19, 0x3311);
		mdio_write(tp, 0x15, 0x03f2);
		mdio_write(tp, 0x19, 0xbd0c);
		mdio_write(tp, 0x15, 0x03f3);
		mdio_write(tp, 0x19, 0xc428);
		mdio_write(tp, 0x15, 0x03f4);
		mdio_write(tp, 0x19, 0x0008);
		mdio_write(tp, 0x15, 0x03f5);
		mdio_write(tp, 0x19, 0x9afa);
		mdio_write(tp, 0x15, 0x03f6);
		mdio_write(tp, 0x19, 0x7c12);
		mdio_write(tp, 0x15, 0x03f7);
		mdio_write(tp, 0x19, 0x4c52);
		mdio_write(tp, 0x15, 0x03f8);
		mdio_write(tp, 0x19, 0x4470);
		mdio_write(tp, 0x15, 0x03f9);
		mdio_write(tp, 0x19, 0x7c12);
		mdio_write(tp, 0x15, 0x03fa);
		mdio_write(tp, 0x19, 0x4c40);
		mdio_write(tp, 0x15, 0x03fb);
		mdio_write(tp, 0x19, 0x33ef);
		mdio_write(tp, 0x15, 0x03fc);
		mdio_write(tp, 0x19, 0x3342);
		mdio_write(tp, 0x15, 0x03fd);
		mdio_write(tp, 0x19, 0x330d);
		mdio_write(tp, 0x15, 0x03fe);
		mdio_write(tp, 0x19, 0x32ae);
		mdio_write(tp, 0x15, 0x0000);
		mdio_write(tp, 0x16, 0x0306);
		mdio_write(tp, 0x16, 0x0300);
		mdio_write(tp, 0x1f, 0x0002);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1f, 0x0005);
		mdio_write(tp, 0x05, 0xfff6);
		mdio_write(tp, 0x06, 0x0080);
		mdio_write(tp, 0x05, 0x8000);
		mdio_write(tp, 0x06, 0x0280);
		mdio_write(tp, 0x06, 0x48f7);
		mdio_write(tp, 0x06, 0x00e0);
		mdio_write(tp, 0x06, 0xfff7);
		mdio_write(tp, 0x06, 0xa080);
		mdio_write(tp, 0x06, 0x02ae);
		mdio_write(tp, 0x06, 0xf602);
		mdio_write(tp, 0x06, 0x0112);
		mdio_write(tp, 0x06, 0x0201);
		mdio_write(tp, 0x06, 0x1f02);
		mdio_write(tp, 0x06, 0x012c);
		mdio_write(tp, 0x06, 0x0201);
		mdio_write(tp, 0x06, 0x3c02);
		mdio_write(tp, 0x06, 0x0156);
		mdio_write(tp, 0x06, 0x0201);
		mdio_write(tp, 0x06, 0x6d02);
		mdio_write(tp, 0x06, 0x809d);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x88e1);
		mdio_write(tp, 0x06, 0x8b89);
		mdio_write(tp, 0x06, 0x1e01);
		mdio_write(tp, 0x06, 0xe18b);
		mdio_write(tp, 0x06, 0x8a1e);
		mdio_write(tp, 0x06, 0x01e1);
		mdio_write(tp, 0x06, 0x8b8b);
		mdio_write(tp, 0x06, 0x1e01);
		mdio_write(tp, 0x06, 0xe18b);
		mdio_write(tp, 0x06, 0x8c1e);
		mdio_write(tp, 0x06, 0x01e1);
		mdio_write(tp, 0x06, 0x8b8d);
		mdio_write(tp, 0x06, 0x1e01);
		mdio_write(tp, 0x06, 0xe18b);
		mdio_write(tp, 0x06, 0x8e1e);
		mdio_write(tp, 0x06, 0x01a0);
		mdio_write(tp, 0x06, 0x00c7);
		mdio_write(tp, 0x06, 0xaebb);
		mdio_write(tp, 0x06, 0xd100);
		mdio_write(tp, 0x06, 0xbf82);
		mdio_write(tp, 0x06, 0xc702);
		mdio_write(tp, 0x06, 0x320a);
		mdio_write(tp, 0x06, 0xd105);
		mdio_write(tp, 0x06, 0xbf82);
		mdio_write(tp, 0x06, 0xcd02);
		mdio_write(tp, 0x06, 0x320a);
		mdio_write(tp, 0x06, 0xd100);
		mdio_write(tp, 0x06, 0xbf82);
		mdio_write(tp, 0x06, 0xca02);
		mdio_write(tp, 0x06, 0x320a);
		mdio_write(tp, 0x06, 0xd105);
		mdio_write(tp, 0x06, 0xbf82);
		mdio_write(tp, 0x06, 0xd002);
		mdio_write(tp, 0x06, 0x320a);
		mdio_write(tp, 0x06, 0xd481);
		mdio_write(tp, 0x06, 0xc9e4);
		mdio_write(tp, 0x06, 0x8b90);
		mdio_write(tp, 0x06, 0xe58b);
		mdio_write(tp, 0x06, 0x91d4);
		mdio_write(tp, 0x06, 0x81b8);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x92e5);
		mdio_write(tp, 0x06, 0x8b93);
		mdio_write(tp, 0x06, 0xbf8b);
		mdio_write(tp, 0x06, 0x88ec);
		mdio_write(tp, 0x06, 0x0019);
		mdio_write(tp, 0x06, 0xa98b);
		mdio_write(tp, 0x06, 0x90f9);
		mdio_write(tp, 0x06, 0xeeff);
		mdio_write(tp, 0x06, 0xf600);
		mdio_write(tp, 0x06, 0xeeff);
		mdio_write(tp, 0x06, 0xf7fc);
		mdio_write(tp, 0x06, 0xd100);
		mdio_write(tp, 0x06, 0xbf82);
		mdio_write(tp, 0x06, 0xc102);
		mdio_write(tp, 0x06, 0x320a);
		mdio_write(tp, 0x06, 0xd101);
		mdio_write(tp, 0x06, 0xbf82);
		mdio_write(tp, 0x06, 0xc402);
		mdio_write(tp, 0x06, 0x320a);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8ead);
		mdio_write(tp, 0x06, 0x201a);
		mdio_write(tp, 0x06, 0xf620);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x8e02);
		mdio_write(tp, 0x06, 0x824b);
		mdio_write(tp, 0x06, 0x0281);
		mdio_write(tp, 0x06, 0x1902);
		mdio_write(tp, 0x06, 0x2c9d);
		mdio_write(tp, 0x06, 0x0203);
		mdio_write(tp, 0x06, 0x9602);
		mdio_write(tp, 0x06, 0x0473);
		mdio_write(tp, 0x06, 0x022e);
		mdio_write(tp, 0x06, 0x3902);
		mdio_write(tp, 0x06, 0x044d);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8ead);
		mdio_write(tp, 0x06, 0x210b);
		mdio_write(tp, 0x06, 0xf621);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x8e02);
		mdio_write(tp, 0x06, 0x0416);
		mdio_write(tp, 0x06, 0x021b);
		mdio_write(tp, 0x06, 0xa4e0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad22);
		mdio_write(tp, 0x06, 0x05f6);
		mdio_write(tp, 0x06, 0x22e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8ead);
		mdio_write(tp, 0x06, 0x2305);
		mdio_write(tp, 0x06, 0xf623);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x8ee0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad24);
		mdio_write(tp, 0x06, 0x05f6);
		mdio_write(tp, 0x06, 0x24e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8ead);
		mdio_write(tp, 0x06, 0x2505);
		mdio_write(tp, 0x06, 0xf625);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x8ee0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad26);
		mdio_write(tp, 0x06, 0x08f6);
		mdio_write(tp, 0x06, 0x26e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0x0281);
		mdio_write(tp, 0x06, 0xdae0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad27);
		mdio_write(tp, 0x06, 0x05f6);
		mdio_write(tp, 0x06, 0x27e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0x0203);
		mdio_write(tp, 0x06, 0x5cfc);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xfaef);
		mdio_write(tp, 0x06, 0x69e0);
		mdio_write(tp, 0x06, 0x8b85);
		mdio_write(tp, 0x06, 0xad21);
		mdio_write(tp, 0x06, 0x57e0);
		mdio_write(tp, 0x06, 0xe022);
		mdio_write(tp, 0x06, 0xe1e0);
		mdio_write(tp, 0x06, 0x2358);
		mdio_write(tp, 0x06, 0xc059);
		mdio_write(tp, 0x06, 0x021e);
		mdio_write(tp, 0x06, 0x01e1);
		mdio_write(tp, 0x06, 0x8b3c);
		mdio_write(tp, 0x06, 0x1f10);
		mdio_write(tp, 0x06, 0x9e44);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x3cad);
		mdio_write(tp, 0x06, 0x211d);
		mdio_write(tp, 0x06, 0xe18b);
		mdio_write(tp, 0x06, 0x84f7);
		mdio_write(tp, 0x06, 0x29e5);
		mdio_write(tp, 0x06, 0x8b84);
		mdio_write(tp, 0x06, 0xac27);
		mdio_write(tp, 0x06, 0x0dac);
		mdio_write(tp, 0x06, 0x2605);
		mdio_write(tp, 0x06, 0x0281);
		mdio_write(tp, 0x06, 0x7fae);
		mdio_write(tp, 0x06, 0x2b02);
		mdio_write(tp, 0x06, 0x2c23);
		mdio_write(tp, 0x06, 0xae26);
		mdio_write(tp, 0x06, 0x022c);
		mdio_write(tp, 0x06, 0x41ae);
		mdio_write(tp, 0x06, 0x21e0);
		mdio_write(tp, 0x06, 0x8b87);
		mdio_write(tp, 0x06, 0xad22);
		mdio_write(tp, 0x06, 0x18e0);
		mdio_write(tp, 0x06, 0xfff7);
		mdio_write(tp, 0x06, 0x58fc);
		mdio_write(tp, 0x06, 0xe4ff);
		mdio_write(tp, 0x06, 0xf7d1);
		mdio_write(tp, 0x06, 0x00bf);
		mdio_write(tp, 0x06, 0x2eee);
		mdio_write(tp, 0x06, 0x0232);
		mdio_write(tp, 0x06, 0x0ad1);
		mdio_write(tp, 0x06, 0x00bf);
		mdio_write(tp, 0x06, 0x82e8);
		mdio_write(tp, 0x06, 0x0232);
		mdio_write(tp, 0x06, 0x0a02);
		mdio_write(tp, 0x06, 0x2bdf);
		mdio_write(tp, 0x06, 0xef96);
		mdio_write(tp, 0x06, 0xfefc);
		mdio_write(tp, 0x06, 0x04d0);
		mdio_write(tp, 0x06, 0x0202);
		mdio_write(tp, 0x06, 0x1e97);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x87ad);
		mdio_write(tp, 0x06, 0x2228);
		mdio_write(tp, 0x06, 0xd100);
		mdio_write(tp, 0x06, 0xbf82);
		mdio_write(tp, 0x06, 0xd302);
		mdio_write(tp, 0x06, 0x320a);
		mdio_write(tp, 0x06, 0xd10c);
		mdio_write(tp, 0x06, 0xbf82);
		mdio_write(tp, 0x06, 0xd602);
		mdio_write(tp, 0x06, 0x320a);
		mdio_write(tp, 0x06, 0xd104);
		mdio_write(tp, 0x06, 0xbf82);
		mdio_write(tp, 0x06, 0xd902);
		mdio_write(tp, 0x06, 0x320a);
		mdio_write(tp, 0x06, 0xd101);
		mdio_write(tp, 0x06, 0xbf82);
		mdio_write(tp, 0x06, 0xe802);
		mdio_write(tp, 0x06, 0x320a);
		mdio_write(tp, 0x06, 0xe0ff);
		mdio_write(tp, 0x06, 0xf768);
		mdio_write(tp, 0x06, 0x03e4);
		mdio_write(tp, 0x06, 0xfff7);
		mdio_write(tp, 0x06, 0xd004);
		mdio_write(tp, 0x06, 0x0228);
		mdio_write(tp, 0x06, 0x7a04);
		mdio_write(tp, 0x06, 0xf8e0);
		mdio_write(tp, 0x06, 0xe234);
		mdio_write(tp, 0x06, 0xe1e2);
		mdio_write(tp, 0x06, 0x35f6);
		mdio_write(tp, 0x06, 0x2be4);
		mdio_write(tp, 0x06, 0xe234);
		mdio_write(tp, 0x06, 0xe5e2);
		mdio_write(tp, 0x06, 0x35fc);
		mdio_write(tp, 0x06, 0x05f8);
		mdio_write(tp, 0x06, 0xe0e2);
		mdio_write(tp, 0x06, 0x34e1);
		mdio_write(tp, 0x06, 0xe235);
		mdio_write(tp, 0x06, 0xf72b);
		mdio_write(tp, 0x06, 0xe4e2);
		mdio_write(tp, 0x06, 0x34e5);
		mdio_write(tp, 0x06, 0xe235);
		mdio_write(tp, 0x06, 0xfc05);
		mdio_write(tp, 0x06, 0xf8f9);
		mdio_write(tp, 0x06, 0xfaef);
		mdio_write(tp, 0x06, 0x69ac);
		mdio_write(tp, 0x06, 0x1b4c);
		mdio_write(tp, 0x06, 0xbf2e);
		mdio_write(tp, 0x06, 0x3002);
		mdio_write(tp, 0x06, 0x31dd);
		mdio_write(tp, 0x06, 0xef01);
		mdio_write(tp, 0x06, 0xe28a);
		mdio_write(tp, 0x06, 0x76e4);
		mdio_write(tp, 0x06, 0x8a76);
		mdio_write(tp, 0x06, 0x1f12);
		mdio_write(tp, 0x06, 0x9e3a);
		mdio_write(tp, 0x06, 0xef12);
		mdio_write(tp, 0x06, 0x5907);
		mdio_write(tp, 0x06, 0x9f12);
		mdio_write(tp, 0x06, 0xf8e0);
		mdio_write(tp, 0x06, 0x8b40);
		mdio_write(tp, 0x06, 0xf721);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x40d0);
		mdio_write(tp, 0x06, 0x0302);
		mdio_write(tp, 0x06, 0x287a);
		mdio_write(tp, 0x06, 0x0282);
		mdio_write(tp, 0x06, 0x34fc);
		mdio_write(tp, 0x06, 0xa000);
		mdio_write(tp, 0x06, 0x1002);
		mdio_write(tp, 0x06, 0x2dc3);
		mdio_write(tp, 0x06, 0x022e);
		mdio_write(tp, 0x06, 0x21e0);
		mdio_write(tp, 0x06, 0x8b40);
		mdio_write(tp, 0x06, 0xf621);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x40ae);
		mdio_write(tp, 0x06, 0x0fbf);
		mdio_write(tp, 0x06, 0x3fa5);
		mdio_write(tp, 0x06, 0x0231);
		mdio_write(tp, 0x06, 0x6cbf);
		mdio_write(tp, 0x06, 0x3fa2);
		mdio_write(tp, 0x06, 0x0231);
		mdio_write(tp, 0x06, 0x6c02);
		mdio_write(tp, 0x06, 0x2dc3);
		mdio_write(tp, 0x06, 0xef96);
		mdio_write(tp, 0x06, 0xfefd);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8e0);
		mdio_write(tp, 0x06, 0xe2f4);
		mdio_write(tp, 0x06, 0xe1e2);
		mdio_write(tp, 0x06, 0xf5e4);
		mdio_write(tp, 0x06, 0x8a78);
		mdio_write(tp, 0x06, 0xe58a);
		mdio_write(tp, 0x06, 0x79ee);
		mdio_write(tp, 0x06, 0xe2f4);
		mdio_write(tp, 0x06, 0xd8ee);
		mdio_write(tp, 0x06, 0xe2f5);
		mdio_write(tp, 0x06, 0x20fc);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xf9fa);
		mdio_write(tp, 0x06, 0xef69);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x87ad);
		mdio_write(tp, 0x06, 0x2065);
		mdio_write(tp, 0x06, 0xd200);
		mdio_write(tp, 0x06, 0xbf2e);
		mdio_write(tp, 0x06, 0xe802);
		mdio_write(tp, 0x06, 0x31dd);
		mdio_write(tp, 0x06, 0x1e21);
		mdio_write(tp, 0x06, 0xbf82);
		mdio_write(tp, 0x06, 0xdf02);
		mdio_write(tp, 0x06, 0x31dd);
		mdio_write(tp, 0x06, 0x0c11);
		mdio_write(tp, 0x06, 0x1e21);
		mdio_write(tp, 0x06, 0xbf82);
		mdio_write(tp, 0x06, 0xe202);
		mdio_write(tp, 0x06, 0x31dd);
		mdio_write(tp, 0x06, 0x0c12);
		mdio_write(tp, 0x06, 0x1e21);
		mdio_write(tp, 0x06, 0xbf82);
		mdio_write(tp, 0x06, 0xe502);
		mdio_write(tp, 0x06, 0x31dd);
		mdio_write(tp, 0x06, 0x0c13);
		mdio_write(tp, 0x06, 0x1e21);
		mdio_write(tp, 0x06, 0xbf1f);
		mdio_write(tp, 0x06, 0x5302);
		mdio_write(tp, 0x06, 0x31dd);
		mdio_write(tp, 0x06, 0x0c14);
		mdio_write(tp, 0x06, 0x1e21);
		mdio_write(tp, 0x06, 0xbf82);
		mdio_write(tp, 0x06, 0xeb02);
		mdio_write(tp, 0x06, 0x31dd);
		mdio_write(tp, 0x06, 0x0c16);
		mdio_write(tp, 0x06, 0x1e21);
		mdio_write(tp, 0x06, 0xe083);
		mdio_write(tp, 0x06, 0xe01f);
		mdio_write(tp, 0x06, 0x029e);
		mdio_write(tp, 0x06, 0x22e6);
		mdio_write(tp, 0x06, 0x83e0);
		mdio_write(tp, 0x06, 0xad31);
		mdio_write(tp, 0x06, 0x14ad);
		mdio_write(tp, 0x06, 0x3011);
		mdio_write(tp, 0x06, 0xef02);
		mdio_write(tp, 0x06, 0x580c);
		mdio_write(tp, 0x06, 0x9e07);
		mdio_write(tp, 0x06, 0xad36);
		mdio_write(tp, 0x06, 0x085a);
		mdio_write(tp, 0x06, 0x309f);
		mdio_write(tp, 0x06, 0x04d1);
		mdio_write(tp, 0x06, 0x01ae);
		mdio_write(tp, 0x06, 0x02d1);
		mdio_write(tp, 0x06, 0x00bf);
		mdio_write(tp, 0x06, 0x82dc);
		mdio_write(tp, 0x06, 0x0232);
		mdio_write(tp, 0x06, 0x0aef);
		mdio_write(tp, 0x06, 0x96fe);
		mdio_write(tp, 0x06, 0xfdfc);
		mdio_write(tp, 0x06, 0x0400);
		mdio_write(tp, 0x06, 0xe140);
		mdio_write(tp, 0x06, 0x77e1);
		mdio_write(tp, 0x06, 0x4010);
		mdio_write(tp, 0x06, 0xe150);
		mdio_write(tp, 0x06, 0x32e1);
		mdio_write(tp, 0x06, 0x5030);
		mdio_write(tp, 0x06, 0xe144);
		mdio_write(tp, 0x06, 0x74e1);
		mdio_write(tp, 0x06, 0x44bb);
		mdio_write(tp, 0x06, 0xe2d2);
		mdio_write(tp, 0x06, 0x40e0);
		mdio_write(tp, 0x06, 0x2cfc);
		mdio_write(tp, 0x06, 0xe2cc);
		mdio_write(tp, 0x06, 0xcce2);
		mdio_write(tp, 0x06, 0x00cc);
		mdio_write(tp, 0x06, 0xe000);
		mdio_write(tp, 0x06, 0x99e0);
		mdio_write(tp, 0x06, 0x3688);
		mdio_write(tp, 0x06, 0xe036);
		mdio_write(tp, 0x06, 0x99e1);
		mdio_write(tp, 0x06, 0x40dd);
		mdio_write(tp, 0x06, 0xe022);
		mdio_write(tp, 0x05, 0xe142);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_0;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x05, 0xe140);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_0;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1f, 0x0005);
		for (i=0;i<200;i++)
		{
			udelay(100);
			gphy_val = mdio_read(tp, 0x00);
			if (gphy_val & BIT_7)
				break;
		}
		mdio_write(tp, 0x1f, 0x0004);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x0023);
		gphy_val = mdio_read(tp, 0x17);
		gphy_val &= ~(BIT_0);
		gphy_val |= BIT_2;
		mdio_write(tp, 0x17, gphy_val);
		mdio_write(tp, 0x1f, 0x0002);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8B80);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_2 | BIT_1;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1f, 0x0004);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x002D);
		gphy_val = mdio_read(tp, 0x18);
		gphy_val |= BIT_4;
		mdio_write(tp, 0x18, gphy_val);
		mdio_write(tp, 0x1f, 0x0002);
		mdio_write(tp, 0x1f, 0x0000);
		gphy_val = mdio_read(tp, 0x14);
		gphy_val |= BIT_15;
		mdio_write(tp, 0x14, gphy_val);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x15, 0x1006);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8B86);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_0;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0001);
		mdio_write(tp, 0x0B, 0x6C14);
		mdio_write(tp, 0x14, 0x7F3D);
		mdio_write(tp, 0x1C, 0xFAFE);
		mdio_write(tp, 0x08, 0x07C5);
		mdio_write(tp, 0x10, 0xF090);
		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x14, 0x641A);
		mdio_write(tp, 0x1A, 0x0606);
		mdio_write(tp, 0x12, 0xF480);
		mdio_write(tp, 0x13, 0x0747);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0004);
		mdio_write(tp, 0x1F, 0x0007);
		mdio_write(tp, 0x1E, 0x0078);
		mdio_write(tp, 0x15, 0xA408);
		mdio_write(tp, 0x17, 0x5100);
		mdio_write(tp, 0x19, 0x0008);
		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x0D, 0x0207);
		mdio_write(tp, 0x02, 0x5FD0);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0004);
		mdio_write(tp, 0x1F, 0x0007);
		mdio_write(tp, 0x1E, 0x00A1);
		gphy_val = mdio_read(tp, 0x1A);
		gphy_val &= ~BIT_2;
		mdio_write(tp, 0x1A, gphy_val);
		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0004);
		mdio_write(tp, 0x1F, 0x0007);
		mdio_write(tp, 0x1E, 0x002D);
		gphy_val = mdio_read(tp, 0x16);
		gphy_val |= BIT_5;
		mdio_write(tp, 0x16, gphy_val);
		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0004);
		mdio_write(tp, 0x1F, 0x0007);
		mdio_write(tp, 0x1E, 0x00AC);
		mdio_write(tp, 0x18, 0x0006);
		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x09, 0xA20F);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8B5B);
		mdio_write(tp, 0x06, 0x9222);
		mdio_write(tp, 0x05, 0x8B6D);
		mdio_write(tp, 0x06, 0x8000);
		mdio_write(tp, 0x05, 0x8B76);
		mdio_write(tp, 0x06, 0x8000);
		mdio_write(tp, 0x1F, 0x0000);
	} else if (tp->mcfg == CFG_METHOD_17) {
		//struct pci_dev *pdev = tp->pci_dev;
		RTL_W8(tp,0x6E, RTL_R8(tp,0x6E) | BIT_6);
		rtl8168_eri_write(tp, 0x1AE, 2, 0x0403, ERIAR_ExGMAC);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x00, 0x1800);
		gphy_val = mdio_read(tp, 0x15);
		gphy_val &= ~(BIT_12);
		mdio_write(tp, 0x15, gphy_val);
		mdio_write(tp, 0x00, 0x4800);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x002f);
		for (i = 0; i < 1000; i++) {
			udelay(100);
			gphy_val = mdio_read(tp, 0x1c);
			if ((gphy_val & 0x0080) == 0x0080)
				break;
		}
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x00, 0x1800);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x0023);
		for (i = 0; i < 200; i++) {
			udelay(100);
			gphy_val = mdio_read(tp, 0x17);
			if (!(gphy_val & 0x0001))
				break;
		}
		mdio_write(tp, 0x1f, 0x0005);
		mdio_write(tp, 0x05, 0xfff6);
		mdio_write(tp, 0x06, 0x0080);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x0023);
		mdio_write(tp, 0x16, 0x0306);
		mdio_write(tp, 0x16, 0x0307);
		mdio_write(tp, 0x15, 0x00AF);
		mdio_write(tp, 0x19, 0x4060);
		mdio_write(tp, 0x15, 0x00B0);
		mdio_write(tp, 0x19, 0x7800);
		mdio_write(tp, 0x15, 0x00B1);
		mdio_write(tp, 0x19, 0x7e00);
		mdio_write(tp, 0x15, 0x00B2);
		mdio_write(tp, 0x19, 0x72B0);
		mdio_write(tp, 0x15, 0x00B3);
		mdio_write(tp, 0x19, 0x7F00);
		mdio_write(tp, 0x15, 0x00B4);
		mdio_write(tp, 0x19, 0x73B0);
		mdio_write(tp, 0x15, 0x0101);
		mdio_write(tp, 0x19, 0x0005);
		mdio_write(tp, 0x15, 0x0103);
		mdio_write(tp, 0x19, 0x0003);
		mdio_write(tp, 0x15, 0x0105);
		mdio_write(tp, 0x19, 0x30FD);
		mdio_write(tp, 0x15, 0x0106);
		mdio_write(tp, 0x19, 0x9DF7);
		mdio_write(tp, 0x15, 0x0107);
		mdio_write(tp, 0x19, 0x30C6);
		mdio_write(tp, 0x15, 0x0098);
		mdio_write(tp, 0x19, 0x7c0b);
		mdio_write(tp, 0x15, 0x0099);
		mdio_write(tp, 0x19, 0x6c0b);
		mdio_write(tp, 0x15, 0x00eb);
		mdio_write(tp, 0x19, 0x6c0b);
		mdio_write(tp, 0x15, 0x00f8);
		mdio_write(tp, 0x19, 0x6f0b);
		mdio_write(tp, 0x15, 0x00fe);
		mdio_write(tp, 0x19, 0x6f0f);
		mdio_write(tp, 0x15, 0x00db);
		mdio_write(tp, 0x19, 0x6f09);
		mdio_write(tp, 0x15, 0x00dc);
		mdio_write(tp, 0x19, 0xaefd);
		mdio_write(tp, 0x15, 0x00dd);
		mdio_write(tp, 0x19, 0x6f0b);
		mdio_write(tp, 0x15, 0x00de);
		mdio_write(tp, 0x19, 0xc60b);
		mdio_write(tp, 0x15, 0x00df);
		mdio_write(tp, 0x19, 0x00fa);
		mdio_write(tp, 0x15, 0x00e0);
		mdio_write(tp, 0x19, 0x30e1);
		mdio_write(tp, 0x15, 0x0000);
		mdio_write(tp, 0x16, 0x0306);
		mdio_write(tp, 0x16, 0x0300);
		mdio_write(tp, 0x1f, 0x0002);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x17, 0x2160);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x0040);
		mdio_write(tp, 0x18, 0x0004);
		if (subsystem_vendor == 0x144d &&
		    subsystem_device == 0xc0a6) {
			mdio_write(tp, 0x18, 0x0724);
			mdio_write(tp, 0x19, 0xfe00);
			mdio_write(tp, 0x18, 0x0734);
			mdio_write(tp, 0x19, 0xfd00);
			mdio_write(tp, 0x18, 0x1824);
			mdio_write(tp, 0x19, 0xfc00);
			mdio_write(tp, 0x18, 0x1834);
			mdio_write(tp, 0x19, 0xfd00);
		}
		mdio_write(tp, 0x18, 0x09d4);
		mdio_write(tp, 0x19, 0x4000);
		mdio_write(tp, 0x18, 0x09e4);
		mdio_write(tp, 0x19, 0x0800);
		mdio_write(tp, 0x18, 0x09f4);
		mdio_write(tp, 0x19, 0xff00);
		mdio_write(tp, 0x18, 0x0a04);
		mdio_write(tp, 0x19, 0x4000);
		mdio_write(tp, 0x18, 0x0a14);
		mdio_write(tp, 0x19, 0x0c00);
		mdio_write(tp, 0x18, 0x0a24);
		mdio_write(tp, 0x19, 0xff00);
		mdio_write(tp, 0x18, 0x0a74);
		mdio_write(tp, 0x19, 0xf600);
		mdio_write(tp, 0x18, 0x1a24);
		mdio_write(tp, 0x19, 0x7d00);
		mdio_write(tp, 0x18, 0x1a64);
		mdio_write(tp, 0x19, 0x0500);
		mdio_write(tp, 0x18, 0x1a74);
		mdio_write(tp, 0x19, 0x9500);
		mdio_write(tp, 0x18, 0x1a84);
		mdio_write(tp, 0x19, 0x8000);
		mdio_write(tp, 0x18, 0x1a94);
		mdio_write(tp, 0x19, 0x7d00);
		mdio_write(tp, 0x18, 0x1aa4);
		mdio_write(tp, 0x19, 0x9600);
		mdio_write(tp, 0x18, 0x1ac4);
		mdio_write(tp, 0x19, 0x4000);
		mdio_write(tp, 0x18, 0x1ad4);
		mdio_write(tp, 0x19, 0x0800);
		mdio_write(tp, 0x18, 0x1af4);
		mdio_write(tp, 0x19, 0xc400);
		mdio_write(tp, 0x18, 0x1b04);
		mdio_write(tp, 0x19, 0x4000);
		mdio_write(tp, 0x18, 0x1b14);
		mdio_write(tp, 0x19, 0x0800);
		mdio_write(tp, 0x18, 0x1b24);
		mdio_write(tp, 0x19, 0xfd00);
		mdio_write(tp, 0x18, 0x1b34);
		mdio_write(tp, 0x19, 0x4000);
		mdio_write(tp, 0x18, 0x1b44);
		mdio_write(tp, 0x19, 0x0400);
		mdio_write(tp, 0x18, 0x1b94);
		mdio_write(tp, 0x19, 0xf100);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x17, 0x2100);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x0040);
		mdio_write(tp, 0x18, 0x0000);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1f, 0x0005);
		mdio_write(tp, 0x05, 0xfff6);
		mdio_write(tp, 0x06, 0x0080);
		mdio_write(tp, 0x05, 0x8000);
		mdio_write(tp, 0x06, 0x0280);
		mdio_write(tp, 0x06, 0x48f7);
		mdio_write(tp, 0x06, 0x00e0);
		mdio_write(tp, 0x06, 0xfff7);
		mdio_write(tp, 0x06, 0xa080);
		mdio_write(tp, 0x06, 0x02ae);
		mdio_write(tp, 0x06, 0xf602);
		mdio_write(tp, 0x06, 0x0115);
		mdio_write(tp, 0x06, 0x0201);
		mdio_write(tp, 0x06, 0x2202);
		mdio_write(tp, 0x06, 0x80a0);
		mdio_write(tp, 0x06, 0x0201);
		mdio_write(tp, 0x06, 0x3f02);
		mdio_write(tp, 0x06, 0x0159);
		mdio_write(tp, 0x06, 0x0280);
		mdio_write(tp, 0x06, 0xbd02);
		mdio_write(tp, 0x06, 0x80da);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x88e1);
		mdio_write(tp, 0x06, 0x8b89);
		mdio_write(tp, 0x06, 0x1e01);
		mdio_write(tp, 0x06, 0xe18b);
		mdio_write(tp, 0x06, 0x8a1e);
		mdio_write(tp, 0x06, 0x01e1);
		mdio_write(tp, 0x06, 0x8b8b);
		mdio_write(tp, 0x06, 0x1e01);
		mdio_write(tp, 0x06, 0xe18b);
		mdio_write(tp, 0x06, 0x8c1e);
		mdio_write(tp, 0x06, 0x01e1);
		mdio_write(tp, 0x06, 0x8b8d);
		mdio_write(tp, 0x06, 0x1e01);
		mdio_write(tp, 0x06, 0xe18b);
		mdio_write(tp, 0x06, 0x8e1e);
		mdio_write(tp, 0x06, 0x01a0);
		mdio_write(tp, 0x06, 0x00c7);
		mdio_write(tp, 0x06, 0xaebb);
		mdio_write(tp, 0x06, 0xd481);
		mdio_write(tp, 0x06, 0xd2e4);
		mdio_write(tp, 0x06, 0x8b92);
		mdio_write(tp, 0x06, 0xe58b);
		mdio_write(tp, 0x06, 0x93d1);
		mdio_write(tp, 0x06, 0x03bf);
		mdio_write(tp, 0x06, 0x8598);
		mdio_write(tp, 0x06, 0x0237);
		mdio_write(tp, 0x06, 0x23d1);
		mdio_write(tp, 0x06, 0x02bf);
		mdio_write(tp, 0x06, 0x859b);
		mdio_write(tp, 0x06, 0x0237);
		mdio_write(tp, 0x06, 0x23ee);
		mdio_write(tp, 0x06, 0x8608);
		mdio_write(tp, 0x06, 0x03ee);
		mdio_write(tp, 0x06, 0x860a);
		mdio_write(tp, 0x06, 0x60ee);
		mdio_write(tp, 0x06, 0x8610);
		mdio_write(tp, 0x06, 0x00ee);
		mdio_write(tp, 0x06, 0x8611);
		mdio_write(tp, 0x06, 0x00ee);
		mdio_write(tp, 0x06, 0x8abe);
		mdio_write(tp, 0x06, 0x07ee);
		mdio_write(tp, 0x06, 0x8abf);
		mdio_write(tp, 0x06, 0x73ee);
		mdio_write(tp, 0x06, 0x8a95);
		mdio_write(tp, 0x06, 0x02bf);
		mdio_write(tp, 0x06, 0x8b88);
		mdio_write(tp, 0x06, 0xec00);
		mdio_write(tp, 0x06, 0x19a9);
		mdio_write(tp, 0x06, 0x8b90);
		mdio_write(tp, 0x06, 0xf9ee);
		mdio_write(tp, 0x06, 0xfff6);
		mdio_write(tp, 0x06, 0x00ee);
		mdio_write(tp, 0x06, 0xfff7);
		mdio_write(tp, 0x06, 0xfed1);
		mdio_write(tp, 0x06, 0x00bf);
		mdio_write(tp, 0x06, 0x858f);
		mdio_write(tp, 0x06, 0x0237);
		mdio_write(tp, 0x06, 0x23d1);
		mdio_write(tp, 0x06, 0x01bf);
		mdio_write(tp, 0x06, 0x8592);
		mdio_write(tp, 0x06, 0x0237);
		mdio_write(tp, 0x06, 0x2304);
		mdio_write(tp, 0x06, 0xf8e0);
		mdio_write(tp, 0x06, 0x8b8a);
		mdio_write(tp, 0x06, 0xad20);
		mdio_write(tp, 0x06, 0x14ee);
		mdio_write(tp, 0x06, 0x8b8a);
		mdio_write(tp, 0x06, 0x0002);
		mdio_write(tp, 0x06, 0x1f9a);
		mdio_write(tp, 0x06, 0xe0e4);
		mdio_write(tp, 0x06, 0x26e1);
		mdio_write(tp, 0x06, 0xe427);
		mdio_write(tp, 0x06, 0xeee4);
		mdio_write(tp, 0x06, 0x2623);
		mdio_write(tp, 0x06, 0xe5e4);
		mdio_write(tp, 0x06, 0x27fc);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8dad);
		mdio_write(tp, 0x06, 0x2014);
		mdio_write(tp, 0x06, 0xee8b);
		mdio_write(tp, 0x06, 0x8d00);
		mdio_write(tp, 0x06, 0xe08a);
		mdio_write(tp, 0x06, 0x5a78);
		mdio_write(tp, 0x06, 0x039e);
		mdio_write(tp, 0x06, 0x0902);
		mdio_write(tp, 0x06, 0x05db);
		mdio_write(tp, 0x06, 0x0282);
		mdio_write(tp, 0x06, 0x7b02);
		mdio_write(tp, 0x06, 0x3231);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8e0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad20);
		mdio_write(tp, 0x06, 0x1df6);
		mdio_write(tp, 0x06, 0x20e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0x0281);
		mdio_write(tp, 0x06, 0x5c02);
		mdio_write(tp, 0x06, 0x2bcb);
		mdio_write(tp, 0x06, 0x022d);
		mdio_write(tp, 0x06, 0x2902);
		mdio_write(tp, 0x06, 0x03b4);
		mdio_write(tp, 0x06, 0x0285);
		mdio_write(tp, 0x06, 0x5e02);
		mdio_write(tp, 0x06, 0x2eca);
		mdio_write(tp, 0x06, 0x0284);
		mdio_write(tp, 0x06, 0xc702);
		mdio_write(tp, 0x06, 0x046f);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8ead);
		mdio_write(tp, 0x06, 0x210b);
		mdio_write(tp, 0x06, 0xf621);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x8e02);
		mdio_write(tp, 0x06, 0x851a);
		mdio_write(tp, 0x06, 0x021b);
		mdio_write(tp, 0x06, 0xe8e0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad22);
		mdio_write(tp, 0x06, 0x05f6);
		mdio_write(tp, 0x06, 0x22e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8ead);
		mdio_write(tp, 0x06, 0x2308);
		mdio_write(tp, 0x06, 0xf623);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x8e02);
		mdio_write(tp, 0x06, 0x311c);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8ead);
		mdio_write(tp, 0x06, 0x2405);
		mdio_write(tp, 0x06, 0xf624);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x8ee0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad25);
		mdio_write(tp, 0x06, 0x05f6);
		mdio_write(tp, 0x06, 0x25e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8ead);
		mdio_write(tp, 0x06, 0x2608);
		mdio_write(tp, 0x06, 0xf626);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x8e02);
		mdio_write(tp, 0x06, 0x2df5);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8ead);
		mdio_write(tp, 0x06, 0x2705);
		mdio_write(tp, 0x06, 0xf627);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x8e02);
		mdio_write(tp, 0x06, 0x037a);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8f9);
		mdio_write(tp, 0x06, 0xfaef);
		mdio_write(tp, 0x06, 0x69e0);
		mdio_write(tp, 0x06, 0x8b87);
		mdio_write(tp, 0x06, 0xad20);
		mdio_write(tp, 0x06, 0x65d2);
		mdio_write(tp, 0x06, 0x00bf);
		mdio_write(tp, 0x06, 0x2fe9);
		mdio_write(tp, 0x06, 0x0236);
		mdio_write(tp, 0x06, 0xf61e);
		mdio_write(tp, 0x06, 0x21bf);
		mdio_write(tp, 0x06, 0x2ff5);
		mdio_write(tp, 0x06, 0x0236);
		mdio_write(tp, 0x06, 0xf60c);
		mdio_write(tp, 0x06, 0x111e);
		mdio_write(tp, 0x06, 0x21bf);
		mdio_write(tp, 0x06, 0x2ff8);
		mdio_write(tp, 0x06, 0x0236);
		mdio_write(tp, 0x06, 0xf60c);
		mdio_write(tp, 0x06, 0x121e);
		mdio_write(tp, 0x06, 0x21bf);
		mdio_write(tp, 0x06, 0x2ffb);
		mdio_write(tp, 0x06, 0x0236);
		mdio_write(tp, 0x06, 0xf60c);
		mdio_write(tp, 0x06, 0x131e);
		mdio_write(tp, 0x06, 0x21bf);
		mdio_write(tp, 0x06, 0x1f97);
		mdio_write(tp, 0x06, 0x0236);
		mdio_write(tp, 0x06, 0xf60c);
		mdio_write(tp, 0x06, 0x141e);
		mdio_write(tp, 0x06, 0x21bf);
		mdio_write(tp, 0x06, 0x8595);
		mdio_write(tp, 0x06, 0x0236);
		mdio_write(tp, 0x06, 0xf60c);
		mdio_write(tp, 0x06, 0x161e);
		mdio_write(tp, 0x06, 0x21e0);
		mdio_write(tp, 0x06, 0x8a8c);
		mdio_write(tp, 0x06, 0x1f02);
		mdio_write(tp, 0x06, 0x9e22);
		mdio_write(tp, 0x06, 0xe68a);
		mdio_write(tp, 0x06, 0x8cad);
		mdio_write(tp, 0x06, 0x3114);
		mdio_write(tp, 0x06, 0xad30);
		mdio_write(tp, 0x06, 0x11ef);
		mdio_write(tp, 0x06, 0x0258);
		mdio_write(tp, 0x06, 0x0c9e);
		mdio_write(tp, 0x06, 0x07ad);
		mdio_write(tp, 0x06, 0x3608);
		mdio_write(tp, 0x06, 0x5a30);
		mdio_write(tp, 0x06, 0x9f04);
		mdio_write(tp, 0x06, 0xd101);
		mdio_write(tp, 0x06, 0xae02);
		mdio_write(tp, 0x06, 0xd100);
		mdio_write(tp, 0x06, 0xbf2f);
		mdio_write(tp, 0x06, 0xf202);
		mdio_write(tp, 0x06, 0x3723);
		mdio_write(tp, 0x06, 0xef96);
		mdio_write(tp, 0x06, 0xfefd);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8f9);
		mdio_write(tp, 0x06, 0xface);
		mdio_write(tp, 0x06, 0xfaef);
		mdio_write(tp, 0x06, 0x69fa);
		mdio_write(tp, 0x06, 0xd401);
		mdio_write(tp, 0x06, 0x55b4);
		mdio_write(tp, 0x06, 0xfebf);
		mdio_write(tp, 0x06, 0x85a1);
		mdio_write(tp, 0x06, 0x0236);
		mdio_write(tp, 0x06, 0xf6ac);
		mdio_write(tp, 0x06, 0x280b);
		mdio_write(tp, 0x06, 0xbf85);
		mdio_write(tp, 0x06, 0x9e02);
		mdio_write(tp, 0x06, 0x36f6);
		mdio_write(tp, 0x06, 0xac28);
		mdio_write(tp, 0x06, 0x49ae);
		mdio_write(tp, 0x06, 0x64bf);
		mdio_write(tp, 0x06, 0x859e);
		mdio_write(tp, 0x06, 0x0236);
		mdio_write(tp, 0x06, 0xf6ac);
		mdio_write(tp, 0x06, 0x285b);
		mdio_write(tp, 0x06, 0xd000);
		mdio_write(tp, 0x06, 0x0282);
		mdio_write(tp, 0x06, 0x60ac);
		mdio_write(tp, 0x06, 0x2105);
		mdio_write(tp, 0x06, 0xac22);
		mdio_write(tp, 0x06, 0x02ae);
		mdio_write(tp, 0x06, 0x4ebf);
		mdio_write(tp, 0x06, 0xe0c4);
		mdio_write(tp, 0x06, 0xbe86);
		mdio_write(tp, 0x06, 0x14d2);
		mdio_write(tp, 0x06, 0x04d8);
		mdio_write(tp, 0x06, 0x19d9);
		mdio_write(tp, 0x06, 0x1907);
		mdio_write(tp, 0x06, 0xdc19);
		mdio_write(tp, 0x06, 0xdd19);
		mdio_write(tp, 0x06, 0x0789);
		mdio_write(tp, 0x06, 0x89ef);
		mdio_write(tp, 0x06, 0x645e);
		mdio_write(tp, 0x06, 0x07ff);
		mdio_write(tp, 0x06, 0x0d65);
		mdio_write(tp, 0x06, 0x5cf8);
		mdio_write(tp, 0x06, 0x001e);
		mdio_write(tp, 0x06, 0x46dc);
		mdio_write(tp, 0x06, 0x19dd);
		mdio_write(tp, 0x06, 0x19b2);
		mdio_write(tp, 0x06, 0xe2d4);
		mdio_write(tp, 0x06, 0x0001);
		mdio_write(tp, 0x06, 0xbf85);
		mdio_write(tp, 0x06, 0x9e02);
		mdio_write(tp, 0x06, 0x3723);
		mdio_write(tp, 0x06, 0xae1d);
		mdio_write(tp, 0x06, 0xbee0);
		mdio_write(tp, 0x06, 0xc4bf);
		mdio_write(tp, 0x06, 0x8614);
		mdio_write(tp, 0x06, 0xd204);
		mdio_write(tp, 0x06, 0xd819);
		mdio_write(tp, 0x06, 0xd919);
		mdio_write(tp, 0x06, 0x07dc);
		mdio_write(tp, 0x06, 0x19dd);
		mdio_write(tp, 0x06, 0x1907);
		mdio_write(tp, 0x06, 0xb2f4);
		mdio_write(tp, 0x06, 0xd400);
		mdio_write(tp, 0x06, 0x00bf);
		mdio_write(tp, 0x06, 0x859e);
		mdio_write(tp, 0x06, 0x0237);
		mdio_write(tp, 0x06, 0x23fe);
		mdio_write(tp, 0x06, 0xef96);
		mdio_write(tp, 0x06, 0xfec6);
		mdio_write(tp, 0x06, 0xfefd);
		mdio_write(tp, 0x06, 0xfc05);
		mdio_write(tp, 0x06, 0xf9e2);
		mdio_write(tp, 0x06, 0xe0ea);
		mdio_write(tp, 0x06, 0xe3e0);
		mdio_write(tp, 0x06, 0xeb5a);
		mdio_write(tp, 0x06, 0x070c);
		mdio_write(tp, 0x06, 0x031e);
		mdio_write(tp, 0x06, 0x20e6);
		mdio_write(tp, 0x06, 0xe0ea);
		mdio_write(tp, 0x06, 0xe7e0);
		mdio_write(tp, 0x06, 0xebe0);
		mdio_write(tp, 0x06, 0xe0fc);
		mdio_write(tp, 0x06, 0xe1e0);
		mdio_write(tp, 0x06, 0xfdfd);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xf9e0);
		mdio_write(tp, 0x06, 0x8b81);
		mdio_write(tp, 0x06, 0xac26);
		mdio_write(tp, 0x06, 0x1ae0);
		mdio_write(tp, 0x06, 0x8b81);
		mdio_write(tp, 0x06, 0xac21);
		mdio_write(tp, 0x06, 0x14e0);
		mdio_write(tp, 0x06, 0x8b85);
		mdio_write(tp, 0x06, 0xac20);
		mdio_write(tp, 0x06, 0x0ee0);
		mdio_write(tp, 0x06, 0x8b85);
		mdio_write(tp, 0x06, 0xac23);
		mdio_write(tp, 0x06, 0x08e0);
		mdio_write(tp, 0x06, 0x8b87);
		mdio_write(tp, 0x06, 0xac24);
		mdio_write(tp, 0x06, 0x02ae);
		mdio_write(tp, 0x06, 0x32ee);
		mdio_write(tp, 0x06, 0xe41c);
		mdio_write(tp, 0x06, 0x04ee);
		mdio_write(tp, 0x06, 0xe41d);
		mdio_write(tp, 0x06, 0x04e2);
		mdio_write(tp, 0x06, 0xe07c);
		mdio_write(tp, 0x06, 0xe3e0);
		mdio_write(tp, 0x06, 0x7de0);
		mdio_write(tp, 0x06, 0xe038);
		mdio_write(tp, 0x06, 0xe1e0);
		mdio_write(tp, 0x06, 0x39ad);
		mdio_write(tp, 0x06, 0x2e1b);
		mdio_write(tp, 0x06, 0xad39);
		mdio_write(tp, 0x06, 0x0dd1);
		mdio_write(tp, 0x06, 0x01bf);
		mdio_write(tp, 0x06, 0x21d5);
		mdio_write(tp, 0x06, 0x0237);
		mdio_write(tp, 0x06, 0x2302);
		mdio_write(tp, 0x06, 0x82d2);
		mdio_write(tp, 0x06, 0xae0b);
		mdio_write(tp, 0x06, 0xac38);
		mdio_write(tp, 0x06, 0x02ae);
		mdio_write(tp, 0x06, 0x0602);
		mdio_write(tp, 0x06, 0x8312);
		mdio_write(tp, 0x06, 0x0283);
		mdio_write(tp, 0x06, 0x5afd);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8e1);
		mdio_write(tp, 0x06, 0x8af4);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x81ad);
		mdio_write(tp, 0x06, 0x2605);
		mdio_write(tp, 0x06, 0x0222);
		mdio_write(tp, 0x06, 0xa4f7);
		mdio_write(tp, 0x06, 0x28e0);
		mdio_write(tp, 0x06, 0x8b81);
		mdio_write(tp, 0x06, 0xad21);
		mdio_write(tp, 0x06, 0x0502);
		mdio_write(tp, 0x06, 0x23a9);
		mdio_write(tp, 0x06, 0xf729);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x85ad);
		mdio_write(tp, 0x06, 0x2005);
		mdio_write(tp, 0x06, 0x0214);
		mdio_write(tp, 0x06, 0xabf7);
		mdio_write(tp, 0x06, 0x2ae0);
		mdio_write(tp, 0x06, 0x8b85);
		mdio_write(tp, 0x06, 0xad23);
		mdio_write(tp, 0x06, 0x0502);
		mdio_write(tp, 0x06, 0x12e7);
		mdio_write(tp, 0x06, 0xf72b);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x87ad);
		mdio_write(tp, 0x06, 0x2405);
		mdio_write(tp, 0x06, 0x0283);
		mdio_write(tp, 0x06, 0xb6f7);
		mdio_write(tp, 0x06, 0x2ce5);
		mdio_write(tp, 0x06, 0x8af4);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8e0);
		mdio_write(tp, 0x06, 0x8b81);
		mdio_write(tp, 0x06, 0xad26);
		mdio_write(tp, 0x06, 0x0302);
		mdio_write(tp, 0x06, 0x21e5);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x81ad);
		mdio_write(tp, 0x06, 0x2109);
		mdio_write(tp, 0x06, 0xe08a);
		mdio_write(tp, 0x06, 0xf4ac);
		mdio_write(tp, 0x06, 0x2003);
		mdio_write(tp, 0x06, 0x0223);
		mdio_write(tp, 0x06, 0x98e0);
		mdio_write(tp, 0x06, 0x8b85);
		mdio_write(tp, 0x06, 0xad20);
		mdio_write(tp, 0x06, 0x09e0);
		mdio_write(tp, 0x06, 0x8af4);
		mdio_write(tp, 0x06, 0xac21);
		mdio_write(tp, 0x06, 0x0302);
		mdio_write(tp, 0x06, 0x13fb);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x85ad);
		mdio_write(tp, 0x06, 0x2309);
		mdio_write(tp, 0x06, 0xe08a);
		mdio_write(tp, 0x06, 0xf4ac);
		mdio_write(tp, 0x06, 0x2203);
		mdio_write(tp, 0x06, 0x0212);
		mdio_write(tp, 0x06, 0xfae0);
		mdio_write(tp, 0x06, 0x8b87);
		mdio_write(tp, 0x06, 0xad24);
		mdio_write(tp, 0x06, 0x09e0);
		mdio_write(tp, 0x06, 0x8af4);
		mdio_write(tp, 0x06, 0xac23);
		mdio_write(tp, 0x06, 0x0302);
		mdio_write(tp, 0x06, 0x83bb);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8e1);
		mdio_write(tp, 0x06, 0x8af4);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x81ad);
		mdio_write(tp, 0x06, 0x2608);
		mdio_write(tp, 0x06, 0xe083);
		mdio_write(tp, 0x06, 0xd2ad);
		mdio_write(tp, 0x06, 0x2502);
		mdio_write(tp, 0x06, 0xf628);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x81ad);
		mdio_write(tp, 0x06, 0x210a);
		mdio_write(tp, 0x06, 0xe084);
		mdio_write(tp, 0x06, 0x0af6);
		mdio_write(tp, 0x06, 0x27a0);
		mdio_write(tp, 0x06, 0x0502);
		mdio_write(tp, 0x06, 0xf629);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x85ad);
		mdio_write(tp, 0x06, 0x2008);
		mdio_write(tp, 0x06, 0xe08a);
		mdio_write(tp, 0x06, 0xe8ad);
		mdio_write(tp, 0x06, 0x2102);
		mdio_write(tp, 0x06, 0xf62a);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x85ad);
		mdio_write(tp, 0x06, 0x2308);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x20a0);
		mdio_write(tp, 0x06, 0x0302);
		mdio_write(tp, 0x06, 0xf62b);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x87ad);
		mdio_write(tp, 0x06, 0x2408);
		mdio_write(tp, 0x06, 0xe086);
		mdio_write(tp, 0x06, 0x02a0);
		mdio_write(tp, 0x06, 0x0302);
		mdio_write(tp, 0x06, 0xf62c);
		mdio_write(tp, 0x06, 0xe58a);
		mdio_write(tp, 0x06, 0xf4a1);
		mdio_write(tp, 0x06, 0x0008);
		mdio_write(tp, 0x06, 0xd100);
		mdio_write(tp, 0x06, 0xbf21);
		mdio_write(tp, 0x06, 0xd502);
		mdio_write(tp, 0x06, 0x3723);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xee86);
		mdio_write(tp, 0x06, 0x0200);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x87ad);
		mdio_write(tp, 0x06, 0x241e);
		mdio_write(tp, 0x06, 0xe086);
		mdio_write(tp, 0x06, 0x02a0);
		mdio_write(tp, 0x06, 0x0005);
		mdio_write(tp, 0x06, 0x0283);
		mdio_write(tp, 0x06, 0xe2ae);
		mdio_write(tp, 0x06, 0xf5a0);
		mdio_write(tp, 0x06, 0x0105);
		mdio_write(tp, 0x06, 0x0283);
		mdio_write(tp, 0x06, 0xf2ae);
		mdio_write(tp, 0x06, 0x0ba0);
		mdio_write(tp, 0x06, 0x0205);
		mdio_write(tp, 0x06, 0x0284);
		mdio_write(tp, 0x06, 0x0eae);
		mdio_write(tp, 0x06, 0x03a0);
		mdio_write(tp, 0x06, 0x0300);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8fa);
		mdio_write(tp, 0x06, 0xef69);
		mdio_write(tp, 0x06, 0x0284);
		mdio_write(tp, 0x06, 0x25ee);
		mdio_write(tp, 0x06, 0x8602);
		mdio_write(tp, 0x06, 0x01ef);
		mdio_write(tp, 0x06, 0x96fe);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8ee);
		mdio_write(tp, 0x06, 0x8609);
		mdio_write(tp, 0x06, 0x0002);
		mdio_write(tp, 0x06, 0x845b);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xae10);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0xf8e0);
		mdio_write(tp, 0x06, 0x8608);
		mdio_write(tp, 0x06, 0xe186);
		mdio_write(tp, 0x06, 0x091f);
		mdio_write(tp, 0x06, 0x019e);
		mdio_write(tp, 0x06, 0x0611);
		mdio_write(tp, 0x06, 0xe586);
		mdio_write(tp, 0x06, 0x09ae);
		mdio_write(tp, 0x06, 0x04ee);
		mdio_write(tp, 0x06, 0x8602);
		mdio_write(tp, 0x06, 0x01fc);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xf9fa);
		mdio_write(tp, 0x06, 0xef69);
		mdio_write(tp, 0x06, 0xfbbf);
		mdio_write(tp, 0x06, 0x8604);
		mdio_write(tp, 0x06, 0xef79);
		mdio_write(tp, 0x06, 0xd200);
		mdio_write(tp, 0x06, 0xd400);
		mdio_write(tp, 0x06, 0x221e);
		mdio_write(tp, 0x06, 0x02bf);
		mdio_write(tp, 0x06, 0x2fec);
		mdio_write(tp, 0x06, 0x0237);
		mdio_write(tp, 0x06, 0x23bf);
		mdio_write(tp, 0x06, 0x13f2);
		mdio_write(tp, 0x06, 0x0236);
		mdio_write(tp, 0x06, 0xf60d);
		mdio_write(tp, 0x06, 0x4559);
		mdio_write(tp, 0x06, 0x1fef);
		mdio_write(tp, 0x06, 0x97dd);
		mdio_write(tp, 0x06, 0xd308);
		mdio_write(tp, 0x06, 0x1a93);
		mdio_write(tp, 0x06, 0xdd12);
		mdio_write(tp, 0x06, 0x17a2);
		mdio_write(tp, 0x06, 0x04de);
		mdio_write(tp, 0x06, 0xffef);
		mdio_write(tp, 0x06, 0x96fe);
		mdio_write(tp, 0x06, 0xfdfc);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xf9fa);
		mdio_write(tp, 0x06, 0xef69);
		mdio_write(tp, 0x06, 0xfbee);
		mdio_write(tp, 0x06, 0x8602);
		mdio_write(tp, 0x06, 0x03d5);
		mdio_write(tp, 0x06, 0x0080);
		mdio_write(tp, 0x06, 0xbf86);
		mdio_write(tp, 0x06, 0x04ef);
		mdio_write(tp, 0x06, 0x79ef);
		mdio_write(tp, 0x06, 0x45bf);
		mdio_write(tp, 0x06, 0x2fec);
		mdio_write(tp, 0x06, 0x0237);
		mdio_write(tp, 0x06, 0x23bf);
		mdio_write(tp, 0x06, 0x13f2);
		mdio_write(tp, 0x06, 0x0236);
		mdio_write(tp, 0x06, 0xf6ad);
		mdio_write(tp, 0x06, 0x2702);
		mdio_write(tp, 0x06, 0x78ff);
		mdio_write(tp, 0x06, 0xe186);
		mdio_write(tp, 0x06, 0x0a1b);
		mdio_write(tp, 0x06, 0x01aa);
		mdio_write(tp, 0x06, 0x2eef);
		mdio_write(tp, 0x06, 0x97d9);
		mdio_write(tp, 0x06, 0x7900);
		mdio_write(tp, 0x06, 0x9e2b);
		mdio_write(tp, 0x06, 0x81dd);
		mdio_write(tp, 0x06, 0xbf85);
		mdio_write(tp, 0x06, 0xa702);
		mdio_write(tp, 0x06, 0x3723);
		mdio_write(tp, 0x06, 0xd101);
		mdio_write(tp, 0x06, 0xef02);
		mdio_write(tp, 0x06, 0x100c);
		mdio_write(tp, 0x06, 0x11b0);
		mdio_write(tp, 0x06, 0xfc0d);
		mdio_write(tp, 0x06, 0x11bf);
		mdio_write(tp, 0x06, 0x85a4);
		mdio_write(tp, 0x06, 0x0237);
		mdio_write(tp, 0x06, 0x23d1);
		mdio_write(tp, 0x06, 0x00bf);
		mdio_write(tp, 0x06, 0x85a4);
		mdio_write(tp, 0x06, 0x0237);
		mdio_write(tp, 0x06, 0x23ee);
		mdio_write(tp, 0x06, 0x8602);
		mdio_write(tp, 0x06, 0x02ae);
		mdio_write(tp, 0x06, 0x0413);
		mdio_write(tp, 0x06, 0xa38b);
		mdio_write(tp, 0x06, 0xb4d3);
		mdio_write(tp, 0x06, 0x8012);
		mdio_write(tp, 0x06, 0x17a2);
		mdio_write(tp, 0x06, 0x04ad);
		mdio_write(tp, 0x06, 0xffef);
		mdio_write(tp, 0x06, 0x96fe);
		mdio_write(tp, 0x06, 0xfdfc);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xf9e0);
		mdio_write(tp, 0x06, 0x8b85);
		mdio_write(tp, 0x06, 0xad25);
		mdio_write(tp, 0x06, 0x48e0);
		mdio_write(tp, 0x06, 0x8a96);
		mdio_write(tp, 0x06, 0xe18a);
		mdio_write(tp, 0x06, 0x977c);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x9e35);
		mdio_write(tp, 0x06, 0xee8a);
		mdio_write(tp, 0x06, 0x9600);
		mdio_write(tp, 0x06, 0xee8a);
		mdio_write(tp, 0x06, 0x9700);
		mdio_write(tp, 0x06, 0xe08a);
		mdio_write(tp, 0x06, 0xbee1);
		mdio_write(tp, 0x06, 0x8abf);
		mdio_write(tp, 0x06, 0xe286);
		mdio_write(tp, 0x06, 0x10e3);
		mdio_write(tp, 0x06, 0x8611);
		mdio_write(tp, 0x06, 0x0236);
		mdio_write(tp, 0x06, 0x1aad);
		mdio_write(tp, 0x06, 0x2012);
		mdio_write(tp, 0x06, 0xee8a);
		mdio_write(tp, 0x06, 0x9603);
		mdio_write(tp, 0x06, 0xee8a);
		mdio_write(tp, 0x06, 0x97b7);
		mdio_write(tp, 0x06, 0xee86);
		mdio_write(tp, 0x06, 0x1000);
		mdio_write(tp, 0x06, 0xee86);
		mdio_write(tp, 0x06, 0x1100);
		mdio_write(tp, 0x06, 0xae11);
		mdio_write(tp, 0x06, 0x15e6);
		mdio_write(tp, 0x06, 0x8610);
		mdio_write(tp, 0x06, 0xe786);
		mdio_write(tp, 0x06, 0x11ae);
		mdio_write(tp, 0x06, 0x08ee);
		mdio_write(tp, 0x06, 0x8610);
		mdio_write(tp, 0x06, 0x00ee);
		mdio_write(tp, 0x06, 0x8611);
		mdio_write(tp, 0x06, 0x00fd);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8fa);
		mdio_write(tp, 0x06, 0xef69);
		mdio_write(tp, 0x06, 0xe0e0);
		mdio_write(tp, 0x06, 0x00e1);
		mdio_write(tp, 0x06, 0xe001);
		mdio_write(tp, 0x06, 0xad27);
		mdio_write(tp, 0x06, 0x32e0);
		mdio_write(tp, 0x06, 0x8b40);
		mdio_write(tp, 0x06, 0xf720);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x40bf);
		mdio_write(tp, 0x06, 0x31f5);
		mdio_write(tp, 0x06, 0x0236);
		mdio_write(tp, 0x06, 0xf6ad);
		mdio_write(tp, 0x06, 0x2821);
		mdio_write(tp, 0x06, 0xe0e0);
		mdio_write(tp, 0x06, 0x20e1);
		mdio_write(tp, 0x06, 0xe021);
		mdio_write(tp, 0x06, 0xad20);
		mdio_write(tp, 0x06, 0x18e0);
		mdio_write(tp, 0x06, 0x8b40);
		mdio_write(tp, 0x06, 0xf620);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x40ee);
		mdio_write(tp, 0x06, 0x8b3b);
		mdio_write(tp, 0x06, 0xffe0);
		mdio_write(tp, 0x06, 0x8a8a);
		mdio_write(tp, 0x06, 0xe18a);
		mdio_write(tp, 0x06, 0x8be4);
		mdio_write(tp, 0x06, 0xe000);
		mdio_write(tp, 0x06, 0xe5e0);
		mdio_write(tp, 0x06, 0x01ef);
		mdio_write(tp, 0x06, 0x96fe);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8fa);
		mdio_write(tp, 0x06, 0xef69);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x80ad);
		mdio_write(tp, 0x06, 0x2722);
		mdio_write(tp, 0x06, 0xbf44);
		mdio_write(tp, 0x06, 0xfc02);
		mdio_write(tp, 0x06, 0x36f6);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x441f);
		mdio_write(tp, 0x06, 0x019e);
		mdio_write(tp, 0x06, 0x15e5);
		mdio_write(tp, 0x06, 0x8b44);
		mdio_write(tp, 0x06, 0xad29);
		mdio_write(tp, 0x06, 0x07ac);
		mdio_write(tp, 0x06, 0x2804);
		mdio_write(tp, 0x06, 0xd101);
		mdio_write(tp, 0x06, 0xae02);
		mdio_write(tp, 0x06, 0xd100);
		mdio_write(tp, 0x06, 0xbf85);
		mdio_write(tp, 0x06, 0xaa02);
		mdio_write(tp, 0x06, 0x3723);
		mdio_write(tp, 0x06, 0xef96);
		mdio_write(tp, 0x06, 0xfefc);
		mdio_write(tp, 0x06, 0x0400);
		mdio_write(tp, 0x06, 0xe140);
		mdio_write(tp, 0x06, 0x77e1);
		mdio_write(tp, 0x06, 0x40dd);
		mdio_write(tp, 0x06, 0xe022);
		mdio_write(tp, 0x06, 0x32e1);
		mdio_write(tp, 0x06, 0x5074);
		mdio_write(tp, 0x06, 0xe144);
		mdio_write(tp, 0x06, 0xffe0);
		mdio_write(tp, 0x06, 0xdaff);
		mdio_write(tp, 0x06, 0xe0c0);
		mdio_write(tp, 0x06, 0x52e0);
		mdio_write(tp, 0x06, 0xeed9);
		mdio_write(tp, 0x06, 0xe04c);
		mdio_write(tp, 0x06, 0xbbe0);
		mdio_write(tp, 0x06, 0x2a00);
		mdio_write(tp, 0x05, 0xe142);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_0;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x05, 0xe140);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_0;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1f, 0x0005);
		for (i = 0; i < 200; i++) {
			udelay(100);
			gphy_val = mdio_read(tp, 0x00);
			if (gphy_val & BIT_7)
				break;
		}
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x0042);
		mdio_write(tp, 0x18, 0x2300);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x0023);
		if ((subsystem_vendor == 0x144d &&
		     subsystem_device == 0xc098) ||
		    (subsystem_vendor == 0x144d &&
		     subsystem_device == 0xc0b1)) {
			gphy_val = mdio_read(tp, 0x17);
			gphy_val &= ~(BIT_2);
			mdio_write(tp, 0x17, gphy_val);
		}
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x00, 0x9200);
		if (subsystem_vendor == 0x144d &&
		    subsystem_device == 0xc0a6) {
			mdio_write(tp, 0x1F, 0x0001);
			mdio_write(tp, 0x0e, 0x6b7f);
			mdio_write(tp, 0x1f, 0x0000);
			mdio_write(tp, 0x1F, 0x0005);
			mdio_write(tp, 0x05, 0x8B86);
			gphy_val = mdio_read(tp, 0x06);
			gphy_val |= BIT_4;
			mdio_write(tp, 0x06, gphy_val);
			mdio_write(tp, 0x1f, 0x0000);
		} else {
			mdio_write(tp, 0x1F, 0x0005);
			mdio_write(tp, 0x05, 0x8B80);
			gphy_val = mdio_read(tp, 0x06);
			gphy_val |= BIT_2 | BIT_1;
			mdio_write(tp, 0x06, gphy_val);
			mdio_write(tp, 0x1f, 0x0000);
			mdio_write(tp, 0x1F, 0x0005);
			mdio_write(tp, 0x05, 0x8B86);
			gphy_val = mdio_read(tp, 0x06);
			gphy_val &= ~BIT_4;
			mdio_write(tp, 0x06, gphy_val);
			mdio_write(tp, 0x1f, 0x0000);
		}
		mdio_write(tp, 0x1f, 0x0004);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x002D);
		gphy_val = mdio_read(tp, 0x18);
		gphy_val |= BIT_4;
		mdio_write(tp, 0x18, gphy_val);
		mdio_write(tp, 0x1f, 0x0002);
		mdio_write(tp, 0x1f, 0x0000);
		gphy_val = mdio_read(tp, 0x14);
		gphy_val |= BIT_15;
		mdio_write(tp, 0x14, gphy_val);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8B86);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_0;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1F, 0x0004);
		mdio_write(tp, 0x1F, 0x0007);
		mdio_write(tp, 0x1E, 0x00AC);
		mdio_write(tp, 0x18, 0x0006);
		mdio_write(tp, 0x1F, 0x0002);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x09, 0xA20F);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8B85);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_14;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8B5B);
		mdio_write(tp, 0x06, 0x9222);
		mdio_write(tp, 0x05, 0x8B6D);
		mdio_write(tp, 0x06, 0x8000);
		mdio_write(tp, 0x05, 0x8B76);
		mdio_write(tp, 0x06, 0x8000);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1f, 0x0000);
		gphy_val = mdio_read(tp, 0x15);
		gphy_val |= BIT_12;
		mdio_write(tp, 0x15, gphy_val);
	} else if (tp->mcfg == CFG_METHOD_18) {
		RTL_W8(tp,0x6E, RTL_R8(tp,0x6E) | BIT_6);
		rtl8168_eri_write(tp, 0x1AE, 2, 0x0403, ERIAR_ExGMAC);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x00, 0x1800);
		gphy_val = mdio_read(tp, 0x15);
		gphy_val &= ~(BIT_12);
		mdio_write(tp, 0x15, gphy_val);
		mdio_write(tp, 0x00, 0x4800);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x002f);
		for (i = 0; i < 1000; i++) {
			udelay(100);
			gphy_val = mdio_read(tp, 0x1c);
			if (gphy_val & 0x0080)
				break;
		}
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x00, 0x1800);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x0023);
		for (i = 0; i < 200; i++) {
			udelay(100);
			gphy_val = mdio_read(tp, 0x18);
			if (!(gphy_val & 0x0001))
				break;
		}
		mdio_write(tp, 0x1f, 0x0005);
		mdio_write(tp, 0x05, 0xfff6);
		mdio_write(tp, 0x06, 0x0080);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x0023);
		mdio_write(tp, 0x16, 0x0306);
		mdio_write(tp, 0x16, 0x0307);
		mdio_write(tp, 0x15, 0x0194);
		mdio_write(tp, 0x19, 0x407D);
		mdio_write(tp, 0x15, 0x0098);
		mdio_write(tp, 0x19, 0x7c0b);
		mdio_write(tp, 0x15, 0x0099);
		mdio_write(tp, 0x19, 0x6c0b);
		mdio_write(tp, 0x15, 0x00eb);
		mdio_write(tp, 0x19, 0x6c0b);
		mdio_write(tp, 0x15, 0x00f8);
		mdio_write(tp, 0x19, 0x6f0b);
		mdio_write(tp, 0x15, 0x00fe);
		mdio_write(tp, 0x19, 0x6f0f);
		mdio_write(tp, 0x15, 0x00db);
		mdio_write(tp, 0x19, 0x6f09);
		mdio_write(tp, 0x15, 0x00dc);
		mdio_write(tp, 0x19, 0xaefd);
		mdio_write(tp, 0x15, 0x00dd);
		mdio_write(tp, 0x19, 0x6f0b);
		mdio_write(tp, 0x15, 0x00de);
		mdio_write(tp, 0x19, 0xc60b);
		mdio_write(tp, 0x15, 0x00df);
		mdio_write(tp, 0x19, 0x00fa);
		mdio_write(tp, 0x15, 0x00e0);
		mdio_write(tp, 0x19, 0x30e1);
		mdio_write(tp, 0x15, 0x0000);
		mdio_write(tp, 0x16, 0x0306);
		mdio_write(tp, 0x16, 0x0300);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1f, 0x0005);
		mdio_write(tp, 0x05, 0xfff6);
		mdio_write(tp, 0x06, 0x0080);
		mdio_write(tp, 0x05, 0x8000);
		mdio_write(tp, 0x06, 0x0280);
		mdio_write(tp, 0x06, 0x48f7);
		mdio_write(tp, 0x06, 0x00e0);
		mdio_write(tp, 0x06, 0xfff7);
		mdio_write(tp, 0x06, 0xa080);
		mdio_write(tp, 0x06, 0x02ae);
		mdio_write(tp, 0x06, 0xf602);
		mdio_write(tp, 0x06, 0x0118);
		mdio_write(tp, 0x06, 0x0201);
		mdio_write(tp, 0x06, 0x2502);
		mdio_write(tp, 0x06, 0x8090);
		mdio_write(tp, 0x06, 0x0201);
		mdio_write(tp, 0x06, 0x4202);
		mdio_write(tp, 0x06, 0x015c);
		mdio_write(tp, 0x06, 0x0280);
		mdio_write(tp, 0x06, 0xad02);
		mdio_write(tp, 0x06, 0x80ca);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x88e1);
		mdio_write(tp, 0x06, 0x8b89);
		mdio_write(tp, 0x06, 0x1e01);
		mdio_write(tp, 0x06, 0xe18b);
		mdio_write(tp, 0x06, 0x8a1e);
		mdio_write(tp, 0x06, 0x01e1);
		mdio_write(tp, 0x06, 0x8b8b);
		mdio_write(tp, 0x06, 0x1e01);
		mdio_write(tp, 0x06, 0xe18b);
		mdio_write(tp, 0x06, 0x8c1e);
		mdio_write(tp, 0x06, 0x01e1);
		mdio_write(tp, 0x06, 0x8b8d);
		mdio_write(tp, 0x06, 0x1e01);
		mdio_write(tp, 0x06, 0xe18b);
		mdio_write(tp, 0x06, 0x8e1e);
		mdio_write(tp, 0x06, 0x01a0);
		mdio_write(tp, 0x06, 0x00c7);
		mdio_write(tp, 0x06, 0xaebb);
		mdio_write(tp, 0x06, 0xd484);
		mdio_write(tp, 0x06, 0x36e4);
		mdio_write(tp, 0x06, 0x8b92);
		mdio_write(tp, 0x06, 0xe58b);
		mdio_write(tp, 0x06, 0x93ee);
		mdio_write(tp, 0x06, 0x8ac8);
		mdio_write(tp, 0x06, 0x03ee);
		mdio_write(tp, 0x06, 0x8aca);
		mdio_write(tp, 0x06, 0x60ee);
		mdio_write(tp, 0x06, 0x8ac0);
		mdio_write(tp, 0x06, 0x00ee);
		mdio_write(tp, 0x06, 0x8ac1);
		mdio_write(tp, 0x06, 0x00ee);
		mdio_write(tp, 0x06, 0x8abe);
		mdio_write(tp, 0x06, 0x07ee);
		mdio_write(tp, 0x06, 0x8abf);
		mdio_write(tp, 0x06, 0x73ee);
		mdio_write(tp, 0x06, 0x8a95);
		mdio_write(tp, 0x06, 0x02bf);
		mdio_write(tp, 0x06, 0x8b88);
		mdio_write(tp, 0x06, 0xec00);
		mdio_write(tp, 0x06, 0x19a9);
		mdio_write(tp, 0x06, 0x8b90);
		mdio_write(tp, 0x06, 0xf9ee);
		mdio_write(tp, 0x06, 0xfff6);
		mdio_write(tp, 0x06, 0x00ee);
		mdio_write(tp, 0x06, 0xfff7);
		mdio_write(tp, 0x06, 0xfed1);
		mdio_write(tp, 0x06, 0x00bf);
		mdio_write(tp, 0x06, 0x8510);
		mdio_write(tp, 0x06, 0x0238);
		mdio_write(tp, 0x06, 0x7dd1);
		mdio_write(tp, 0x06, 0x01bf);
		mdio_write(tp, 0x06, 0x8513);
		mdio_write(tp, 0x06, 0x0238);
		mdio_write(tp, 0x06, 0x7d04);
		mdio_write(tp, 0x06, 0xf8e0);
		mdio_write(tp, 0x06, 0x8b8a);
		mdio_write(tp, 0x06, 0xad20);
		mdio_write(tp, 0x06, 0x14ee);
		mdio_write(tp, 0x06, 0x8b8a);
		mdio_write(tp, 0x06, 0x0002);
		mdio_write(tp, 0x06, 0x204b);
		mdio_write(tp, 0x06, 0xe0e4);
		mdio_write(tp, 0x06, 0x26e1);
		mdio_write(tp, 0x06, 0xe427);
		mdio_write(tp, 0x06, 0xeee4);
		mdio_write(tp, 0x06, 0x2623);
		mdio_write(tp, 0x06, 0xe5e4);
		mdio_write(tp, 0x06, 0x27fc);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8dad);
		mdio_write(tp, 0x06, 0x2014);
		mdio_write(tp, 0x06, 0xee8b);
		mdio_write(tp, 0x06, 0x8d00);
		mdio_write(tp, 0x06, 0xe08a);
		mdio_write(tp, 0x06, 0x5a78);
		mdio_write(tp, 0x06, 0x039e);
		mdio_write(tp, 0x06, 0x0902);
		mdio_write(tp, 0x06, 0x05e8);
		mdio_write(tp, 0x06, 0x0281);
		mdio_write(tp, 0x06, 0x4f02);
		mdio_write(tp, 0x06, 0x326c);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8e0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad20);
		mdio_write(tp, 0x06, 0x1df6);
		mdio_write(tp, 0x06, 0x20e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0x022f);
		mdio_write(tp, 0x06, 0x0902);
		mdio_write(tp, 0x06, 0x2ab0);
		mdio_write(tp, 0x06, 0x022c);
		mdio_write(tp, 0x06, 0x0e02);
		mdio_write(tp, 0x06, 0x03ba);
		mdio_write(tp, 0x06, 0x0284);
		mdio_write(tp, 0x06, 0xdf02);
		mdio_write(tp, 0x06, 0x2df1);
		mdio_write(tp, 0x06, 0x0283);
		mdio_write(tp, 0x06, 0x7d02);
		mdio_write(tp, 0x06, 0x0475);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8ead);
		mdio_write(tp, 0x06, 0x210b);
		mdio_write(tp, 0x06, 0xf621);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x8e02);
		mdio_write(tp, 0x06, 0x83d0);
		mdio_write(tp, 0x06, 0x021c);
		mdio_write(tp, 0x06, 0x99e0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad22);
		mdio_write(tp, 0x06, 0x08f6);
		mdio_write(tp, 0x06, 0x22e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0x0235);
		mdio_write(tp, 0x06, 0x63e0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad23);
		mdio_write(tp, 0x06, 0x08f6);
		mdio_write(tp, 0x06, 0x23e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0x0231);
		mdio_write(tp, 0x06, 0x57e0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad24);
		mdio_write(tp, 0x06, 0x05f6);
		mdio_write(tp, 0x06, 0x24e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8ead);
		mdio_write(tp, 0x06, 0x2505);
		mdio_write(tp, 0x06, 0xf625);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x8ee0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad26);
		mdio_write(tp, 0x06, 0x08f6);
		mdio_write(tp, 0x06, 0x26e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0x022d);
		mdio_write(tp, 0x06, 0x1ce0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad27);
		mdio_write(tp, 0x06, 0x05f6);
		mdio_write(tp, 0x06, 0x27e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0x0203);
		mdio_write(tp, 0x06, 0x80fc);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xf9e0);
		mdio_write(tp, 0x06, 0x8b81);
		mdio_write(tp, 0x06, 0xac26);
		mdio_write(tp, 0x06, 0x1ae0);
		mdio_write(tp, 0x06, 0x8b81);
		mdio_write(tp, 0x06, 0xac21);
		mdio_write(tp, 0x06, 0x14e0);
		mdio_write(tp, 0x06, 0x8b85);
		mdio_write(tp, 0x06, 0xac20);
		mdio_write(tp, 0x06, 0x0ee0);
		mdio_write(tp, 0x06, 0x8b85);
		mdio_write(tp, 0x06, 0xac23);
		mdio_write(tp, 0x06, 0x08e0);
		mdio_write(tp, 0x06, 0x8b87);
		mdio_write(tp, 0x06, 0xac24);
		mdio_write(tp, 0x06, 0x02ae);
		mdio_write(tp, 0x06, 0x32ee);
		mdio_write(tp, 0x06, 0xe41c);
		mdio_write(tp, 0x06, 0x04ee);
		mdio_write(tp, 0x06, 0xe41d);
		mdio_write(tp, 0x06, 0x04e2);
		mdio_write(tp, 0x06, 0xe07c);
		mdio_write(tp, 0x06, 0xe3e0);
		mdio_write(tp, 0x06, 0x7de0);
		mdio_write(tp, 0x06, 0xe038);
		mdio_write(tp, 0x06, 0xe1e0);
		mdio_write(tp, 0x06, 0x39ad);
		mdio_write(tp, 0x06, 0x2e1b);
		mdio_write(tp, 0x06, 0xad39);
		mdio_write(tp, 0x06, 0x0dd1);
		mdio_write(tp, 0x06, 0x01bf);
		mdio_write(tp, 0x06, 0x227a);
		mdio_write(tp, 0x06, 0x0238);
		mdio_write(tp, 0x06, 0x7d02);
		mdio_write(tp, 0x06, 0x81a6);
		mdio_write(tp, 0x06, 0xae0b);
		mdio_write(tp, 0x06, 0xac38);
		mdio_write(tp, 0x06, 0x02ae);
		mdio_write(tp, 0x06, 0x0602);
		mdio_write(tp, 0x06, 0x81e3);
		mdio_write(tp, 0x06, 0x0282);
		mdio_write(tp, 0x06, 0x28fd);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8e1);
		mdio_write(tp, 0x06, 0x8af4);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x81ad);
		mdio_write(tp, 0x06, 0x2602);
		mdio_write(tp, 0x06, 0xf728);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x81ad);
		mdio_write(tp, 0x06, 0x2105);
		mdio_write(tp, 0x06, 0x0222);
		mdio_write(tp, 0x06, 0x8ef7);
		mdio_write(tp, 0x06, 0x29e0);
		mdio_write(tp, 0x06, 0x8b85);
		mdio_write(tp, 0x06, 0xad20);
		mdio_write(tp, 0x06, 0x0502);
		mdio_write(tp, 0x06, 0x14b8);
		mdio_write(tp, 0x06, 0xf72a);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x85ad);
		mdio_write(tp, 0x06, 0x2305);
		mdio_write(tp, 0x06, 0x0212);
		mdio_write(tp, 0x06, 0xf4f7);
		mdio_write(tp, 0x06, 0x2be0);
		mdio_write(tp, 0x06, 0x8b87);
		mdio_write(tp, 0x06, 0xad24);
		mdio_write(tp, 0x06, 0x0502);
		mdio_write(tp, 0x06, 0x827e);
		mdio_write(tp, 0x06, 0xf72c);
		mdio_write(tp, 0x06, 0xe58a);
		mdio_write(tp, 0x06, 0xf4fc);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x81ad);
		mdio_write(tp, 0x06, 0x2600);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x81ad);
		mdio_write(tp, 0x06, 0x2109);
		mdio_write(tp, 0x06, 0xe08a);
		mdio_write(tp, 0x06, 0xf4ac);
		mdio_write(tp, 0x06, 0x2003);
		mdio_write(tp, 0x06, 0x0222);
		mdio_write(tp, 0x06, 0x7de0);
		mdio_write(tp, 0x06, 0x8b85);
		mdio_write(tp, 0x06, 0xad20);
		mdio_write(tp, 0x06, 0x09e0);
		mdio_write(tp, 0x06, 0x8af4);
		mdio_write(tp, 0x06, 0xac21);
		mdio_write(tp, 0x06, 0x0302);
		mdio_write(tp, 0x06, 0x1408);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x85ad);
		mdio_write(tp, 0x06, 0x2309);
		mdio_write(tp, 0x06, 0xe08a);
		mdio_write(tp, 0x06, 0xf4ac);
		mdio_write(tp, 0x06, 0x2203);
		mdio_write(tp, 0x06, 0x0213);
		mdio_write(tp, 0x06, 0x07e0);
		mdio_write(tp, 0x06, 0x8b87);
		mdio_write(tp, 0x06, 0xad24);
		mdio_write(tp, 0x06, 0x09e0);
		mdio_write(tp, 0x06, 0x8af4);
		mdio_write(tp, 0x06, 0xac23);
		mdio_write(tp, 0x06, 0x0302);
		mdio_write(tp, 0x06, 0x8283);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8e1);
		mdio_write(tp, 0x06, 0x8af4);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x81ad);
		mdio_write(tp, 0x06, 0x2602);
		mdio_write(tp, 0x06, 0xf628);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x81ad);
		mdio_write(tp, 0x06, 0x210a);
		mdio_write(tp, 0x06, 0xe083);
		mdio_write(tp, 0x06, 0xecf6);
		mdio_write(tp, 0x06, 0x27a0);
		mdio_write(tp, 0x06, 0x0502);
		mdio_write(tp, 0x06, 0xf629);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x85ad);
		mdio_write(tp, 0x06, 0x2008);
		mdio_write(tp, 0x06, 0xe08a);
		mdio_write(tp, 0x06, 0xe8ad);
		mdio_write(tp, 0x06, 0x2102);
		mdio_write(tp, 0x06, 0xf62a);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x85ad);
		mdio_write(tp, 0x06, 0x2308);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x20a0);
		mdio_write(tp, 0x06, 0x0302);
		mdio_write(tp, 0x06, 0xf62b);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x87ad);
		mdio_write(tp, 0x06, 0x2408);
		mdio_write(tp, 0x06, 0xe08a);
		mdio_write(tp, 0x06, 0xc2a0);
		mdio_write(tp, 0x06, 0x0302);
		mdio_write(tp, 0x06, 0xf62c);
		mdio_write(tp, 0x06, 0xe58a);
		mdio_write(tp, 0x06, 0xf4a1);
		mdio_write(tp, 0x06, 0x0008);
		mdio_write(tp, 0x06, 0xd100);
		mdio_write(tp, 0x06, 0xbf22);
		mdio_write(tp, 0x06, 0x7a02);
		mdio_write(tp, 0x06, 0x387d);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xee8a);
		mdio_write(tp, 0x06, 0xc200);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x87ad);
		mdio_write(tp, 0x06, 0x241e);
		mdio_write(tp, 0x06, 0xe08a);
		mdio_write(tp, 0x06, 0xc2a0);
		mdio_write(tp, 0x06, 0x0005);
		mdio_write(tp, 0x06, 0x0282);
		mdio_write(tp, 0x06, 0xaaae);
		mdio_write(tp, 0x06, 0xf5a0);
		mdio_write(tp, 0x06, 0x0105);
		mdio_write(tp, 0x06, 0x0282);
		mdio_write(tp, 0x06, 0xbaae);
		mdio_write(tp, 0x06, 0x0ba0);
		mdio_write(tp, 0x06, 0x0205);
		mdio_write(tp, 0x06, 0x0282);
		mdio_write(tp, 0x06, 0xc4ae);
		mdio_write(tp, 0x06, 0x03a0);
		mdio_write(tp, 0x06, 0x0300);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8fa);
		mdio_write(tp, 0x06, 0xef69);
		mdio_write(tp, 0x06, 0x0282);
		mdio_write(tp, 0x06, 0xdbee);
		mdio_write(tp, 0x06, 0x8ac2);
		mdio_write(tp, 0x06, 0x01ef);
		mdio_write(tp, 0x06, 0x96fe);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8ee);
		mdio_write(tp, 0x06, 0x8ac9);
		mdio_write(tp, 0x06, 0x0002);
		mdio_write(tp, 0x06, 0x8311);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8e0);
		mdio_write(tp, 0x06, 0x8ac8);
		mdio_write(tp, 0x06, 0xe18a);
		mdio_write(tp, 0x06, 0xc91f);
		mdio_write(tp, 0x06, 0x019e);
		mdio_write(tp, 0x06, 0x0611);
		mdio_write(tp, 0x06, 0xe58a);
		mdio_write(tp, 0x06, 0xc9ae);
		mdio_write(tp, 0x06, 0x04ee);
		mdio_write(tp, 0x06, 0x8ac2);
		mdio_write(tp, 0x06, 0x01fc);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xf9fa);
		mdio_write(tp, 0x06, 0xef69);
		mdio_write(tp, 0x06, 0xfbbf);
		mdio_write(tp, 0x06, 0x8ac4);
		mdio_write(tp, 0x06, 0xef79);
		mdio_write(tp, 0x06, 0xd200);
		mdio_write(tp, 0x06, 0xd400);
		mdio_write(tp, 0x06, 0x221e);
		mdio_write(tp, 0x06, 0x02bf);
		mdio_write(tp, 0x06, 0x3024);
		mdio_write(tp, 0x06, 0x0238);
		mdio_write(tp, 0x06, 0x7dbf);
		mdio_write(tp, 0x06, 0x13ff);
		mdio_write(tp, 0x06, 0x0238);
		mdio_write(tp, 0x06, 0x500d);
		mdio_write(tp, 0x06, 0x4559);
		mdio_write(tp, 0x06, 0x1fef);
		mdio_write(tp, 0x06, 0x97dd);
		mdio_write(tp, 0x06, 0xd308);
		mdio_write(tp, 0x06, 0x1a93);
		mdio_write(tp, 0x06, 0xdd12);
		mdio_write(tp, 0x06, 0x17a2);
		mdio_write(tp, 0x06, 0x04de);
		mdio_write(tp, 0x06, 0xffef);
		mdio_write(tp, 0x06, 0x96fe);
		mdio_write(tp, 0x06, 0xfdfc);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xf9fa);
		mdio_write(tp, 0x06, 0xef69);
		mdio_write(tp, 0x06, 0xfbee);
		mdio_write(tp, 0x06, 0x8ac2);
		mdio_write(tp, 0x06, 0x03d5);
		mdio_write(tp, 0x06, 0x0080);
		mdio_write(tp, 0x06, 0xbf8a);
		mdio_write(tp, 0x06, 0xc4ef);
		mdio_write(tp, 0x06, 0x79ef);
		mdio_write(tp, 0x06, 0x45bf);
		mdio_write(tp, 0x06, 0x3024);
		mdio_write(tp, 0x06, 0x0238);
		mdio_write(tp, 0x06, 0x7dbf);
		mdio_write(tp, 0x06, 0x13ff);
		mdio_write(tp, 0x06, 0x0238);
		mdio_write(tp, 0x06, 0x50ad);
		mdio_write(tp, 0x06, 0x2702);
		mdio_write(tp, 0x06, 0x78ff);
		mdio_write(tp, 0x06, 0xe18a);
		mdio_write(tp, 0x06, 0xca1b);
		mdio_write(tp, 0x06, 0x01aa);
		mdio_write(tp, 0x06, 0x2eef);
		mdio_write(tp, 0x06, 0x97d9);
		mdio_write(tp, 0x06, 0x7900);
		mdio_write(tp, 0x06, 0x9e2b);
		mdio_write(tp, 0x06, 0x81dd);
		mdio_write(tp, 0x06, 0xbf85);
		mdio_write(tp, 0x06, 0x1902);
		mdio_write(tp, 0x06, 0x387d);
		mdio_write(tp, 0x06, 0xd101);
		mdio_write(tp, 0x06, 0xef02);
		mdio_write(tp, 0x06, 0x100c);
		mdio_write(tp, 0x06, 0x11b0);
		mdio_write(tp, 0x06, 0xfc0d);
		mdio_write(tp, 0x06, 0x11bf);
		mdio_write(tp, 0x06, 0x8516);
		mdio_write(tp, 0x06, 0x0238);
		mdio_write(tp, 0x06, 0x7dd1);
		mdio_write(tp, 0x06, 0x00bf);
		mdio_write(tp, 0x06, 0x8516);
		mdio_write(tp, 0x06, 0x0238);
		mdio_write(tp, 0x06, 0x7dee);
		mdio_write(tp, 0x06, 0x8ac2);
		mdio_write(tp, 0x06, 0x02ae);
		mdio_write(tp, 0x06, 0x0413);
		mdio_write(tp, 0x06, 0xa38b);
		mdio_write(tp, 0x06, 0xb4d3);
		mdio_write(tp, 0x06, 0x8012);
		mdio_write(tp, 0x06, 0x17a2);
		mdio_write(tp, 0x06, 0x04ad);
		mdio_write(tp, 0x06, 0xffef);
		mdio_write(tp, 0x06, 0x96fe);
		mdio_write(tp, 0x06, 0xfdfc);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xf9e0);
		mdio_write(tp, 0x06, 0x8b85);
		mdio_write(tp, 0x06, 0xad25);
		mdio_write(tp, 0x06, 0x48e0);
		mdio_write(tp, 0x06, 0x8a96);
		mdio_write(tp, 0x06, 0xe18a);
		mdio_write(tp, 0x06, 0x977c);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x9e35);
		mdio_write(tp, 0x06, 0xee8a);
		mdio_write(tp, 0x06, 0x9600);
		mdio_write(tp, 0x06, 0xee8a);
		mdio_write(tp, 0x06, 0x9700);
		mdio_write(tp, 0x06, 0xe08a);
		mdio_write(tp, 0x06, 0xbee1);
		mdio_write(tp, 0x06, 0x8abf);
		mdio_write(tp, 0x06, 0xe28a);
		mdio_write(tp, 0x06, 0xc0e3);
		mdio_write(tp, 0x06, 0x8ac1);
		mdio_write(tp, 0x06, 0x0237);
		mdio_write(tp, 0x06, 0x74ad);
		mdio_write(tp, 0x06, 0x2012);
		mdio_write(tp, 0x06, 0xee8a);
		mdio_write(tp, 0x06, 0x9603);
		mdio_write(tp, 0x06, 0xee8a);
		mdio_write(tp, 0x06, 0x97b7);
		mdio_write(tp, 0x06, 0xee8a);
		mdio_write(tp, 0x06, 0xc000);
		mdio_write(tp, 0x06, 0xee8a);
		mdio_write(tp, 0x06, 0xc100);
		mdio_write(tp, 0x06, 0xae11);
		mdio_write(tp, 0x06, 0x15e6);
		mdio_write(tp, 0x06, 0x8ac0);
		mdio_write(tp, 0x06, 0xe78a);
		mdio_write(tp, 0x06, 0xc1ae);
		mdio_write(tp, 0x06, 0x08ee);
		mdio_write(tp, 0x06, 0x8ac0);
		mdio_write(tp, 0x06, 0x00ee);
		mdio_write(tp, 0x06, 0x8ac1);
		mdio_write(tp, 0x06, 0x00fd);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8fa);
		mdio_write(tp, 0x06, 0xef69);
		mdio_write(tp, 0x06, 0xae20);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x06, 0xe0e0);
		mdio_write(tp, 0x06, 0x00e1);
		mdio_write(tp, 0x06, 0xe001);
		mdio_write(tp, 0x06, 0xad27);
		mdio_write(tp, 0x06, 0x32e0);
		mdio_write(tp, 0x06, 0x8b40);
		mdio_write(tp, 0x06, 0xf720);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x40bf);
		mdio_write(tp, 0x06, 0x3230);
		mdio_write(tp, 0x06, 0x0238);
		mdio_write(tp, 0x06, 0x50ad);
		mdio_write(tp, 0x06, 0x2821);
		mdio_write(tp, 0x06, 0xe0e0);
		mdio_write(tp, 0x06, 0x20e1);
		mdio_write(tp, 0x06, 0xe021);
		mdio_write(tp, 0x06, 0xad20);
		mdio_write(tp, 0x06, 0x18e0);
		mdio_write(tp, 0x06, 0x8b40);
		mdio_write(tp, 0x06, 0xf620);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x40ee);
		mdio_write(tp, 0x06, 0x8b3b);
		mdio_write(tp, 0x06, 0xffe0);
		mdio_write(tp, 0x06, 0x8a8a);
		mdio_write(tp, 0x06, 0xe18a);
		mdio_write(tp, 0x06, 0x8be4);
		mdio_write(tp, 0x06, 0xe000);
		mdio_write(tp, 0x06, 0xe5e0);
		mdio_write(tp, 0x06, 0x01ef);
		mdio_write(tp, 0x06, 0x96fe);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8f9);
		mdio_write(tp, 0x06, 0xface);
		mdio_write(tp, 0x06, 0xfaef);
		mdio_write(tp, 0x06, 0x69fa);
		mdio_write(tp, 0x06, 0xd401);
		mdio_write(tp, 0x06, 0x55b4);
		mdio_write(tp, 0x06, 0xfebf);
		mdio_write(tp, 0x06, 0x1c1e);
		mdio_write(tp, 0x06, 0x0238);
		mdio_write(tp, 0x06, 0x50ac);
		mdio_write(tp, 0x06, 0x280b);
		mdio_write(tp, 0x06, 0xbf1c);
		mdio_write(tp, 0x06, 0x1b02);
		mdio_write(tp, 0x06, 0x3850);
		mdio_write(tp, 0x06, 0xac28);
		mdio_write(tp, 0x06, 0x49ae);
		mdio_write(tp, 0x06, 0x64bf);
		mdio_write(tp, 0x06, 0x1c1b);
		mdio_write(tp, 0x06, 0x0238);
		mdio_write(tp, 0x06, 0x50ac);
		mdio_write(tp, 0x06, 0x285b);
		mdio_write(tp, 0x06, 0xd000);
		mdio_write(tp, 0x06, 0x0284);
		mdio_write(tp, 0x06, 0xc4ac);
		mdio_write(tp, 0x06, 0x2105);
		mdio_write(tp, 0x06, 0xac22);
		mdio_write(tp, 0x06, 0x02ae);
		mdio_write(tp, 0x06, 0x4ebf);
		mdio_write(tp, 0x06, 0xe0c4);
		mdio_write(tp, 0x06, 0xbe85);
		mdio_write(tp, 0x06, 0xf6d2);
		mdio_write(tp, 0x06, 0x04d8);
		mdio_write(tp, 0x06, 0x19d9);
		mdio_write(tp, 0x06, 0x1907);
		mdio_write(tp, 0x06, 0xdc19);
		mdio_write(tp, 0x06, 0xdd19);
		mdio_write(tp, 0x06, 0x0789);
		mdio_write(tp, 0x06, 0x89ef);
		mdio_write(tp, 0x06, 0x645e);
		mdio_write(tp, 0x06, 0x07ff);
		mdio_write(tp, 0x06, 0x0d65);
		mdio_write(tp, 0x06, 0x5cf8);
		mdio_write(tp, 0x06, 0x001e);
		mdio_write(tp, 0x06, 0x46dc);
		mdio_write(tp, 0x06, 0x19dd);
		mdio_write(tp, 0x06, 0x19b2);
		mdio_write(tp, 0x06, 0xe2d4);
		mdio_write(tp, 0x06, 0x0001);
		mdio_write(tp, 0x06, 0xbf1c);
		mdio_write(tp, 0x06, 0x1b02);
		mdio_write(tp, 0x06, 0x387d);
		mdio_write(tp, 0x06, 0xae1d);
		mdio_write(tp, 0x06, 0xbee0);
		mdio_write(tp, 0x06, 0xc4bf);
		mdio_write(tp, 0x06, 0x85f6);
		mdio_write(tp, 0x06, 0xd204);
		mdio_write(tp, 0x06, 0xd819);
		mdio_write(tp, 0x06, 0xd919);
		mdio_write(tp, 0x06, 0x07dc);
		mdio_write(tp, 0x06, 0x19dd);
		mdio_write(tp, 0x06, 0x1907);
		mdio_write(tp, 0x06, 0xb2f4);
		mdio_write(tp, 0x06, 0xd400);
		mdio_write(tp, 0x06, 0x00bf);
		mdio_write(tp, 0x06, 0x1c1b);
		mdio_write(tp, 0x06, 0x0238);
		mdio_write(tp, 0x06, 0x7dfe);
		mdio_write(tp, 0x06, 0xef96);
		mdio_write(tp, 0x06, 0xfec6);
		mdio_write(tp, 0x06, 0xfefd);
		mdio_write(tp, 0x06, 0xfc05);
		mdio_write(tp, 0x06, 0xf9e2);
		mdio_write(tp, 0x06, 0xe0ea);
		mdio_write(tp, 0x06, 0xe3e0);
		mdio_write(tp, 0x06, 0xeb5a);
		mdio_write(tp, 0x06, 0x070c);
		mdio_write(tp, 0x06, 0x031e);
		mdio_write(tp, 0x06, 0x20e6);
		mdio_write(tp, 0x06, 0xe0ea);
		mdio_write(tp, 0x06, 0xe7e0);
		mdio_write(tp, 0x06, 0xebe0);
		mdio_write(tp, 0x06, 0xe0fc);
		mdio_write(tp, 0x06, 0xe1e0);
		mdio_write(tp, 0x06, 0xfdfd);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xfaef);
		mdio_write(tp, 0x06, 0x69e0);
		mdio_write(tp, 0x06, 0x8b80);
		mdio_write(tp, 0x06, 0xad27);
		mdio_write(tp, 0x06, 0x22bf);
		mdio_write(tp, 0x06, 0x4616);
		mdio_write(tp, 0x06, 0x0238);
		mdio_write(tp, 0x06, 0x50e0);
		mdio_write(tp, 0x06, 0x8b44);
		mdio_write(tp, 0x06, 0x1f01);
		mdio_write(tp, 0x06, 0x9e15);
		mdio_write(tp, 0x06, 0xe58b);
		mdio_write(tp, 0x06, 0x44ad);
		mdio_write(tp, 0x06, 0x2907);
		mdio_write(tp, 0x06, 0xac28);
		mdio_write(tp, 0x06, 0x04d1);
		mdio_write(tp, 0x06, 0x01ae);
		mdio_write(tp, 0x06, 0x02d1);
		mdio_write(tp, 0x06, 0x00bf);
		mdio_write(tp, 0x06, 0x851c);
		mdio_write(tp, 0x06, 0x0238);
		mdio_write(tp, 0x06, 0x7def);
		mdio_write(tp, 0x06, 0x96fe);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0x00e1);
		mdio_write(tp, 0x06, 0x4077);
		mdio_write(tp, 0x06, 0xe140);
		mdio_write(tp, 0x06, 0x52e0);
		mdio_write(tp, 0x06, 0xeed9);
		mdio_write(tp, 0x06, 0xe04c);
		mdio_write(tp, 0x06, 0xbbe0);
		mdio_write(tp, 0x06, 0x2a00);
		mdio_write(tp, 0x05, 0xe142);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_0;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x05, 0xe140);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_0;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1f, 0x0005);
		for (i = 0; i < 200; i++) {
			udelay(100);
			gphy_val = mdio_read(tp, 0x00);
			if (gphy_val & BIT_7)
				break;
		}
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x0023);
		gphy_val = mdio_read(tp, 0x17);
		gphy_val |= BIT_1;
		mdio_write(tp, 0x17, gphy_val);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x00, 0x9200);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8b80);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_2 | BIT_1;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x002D);
		gphy_val = mdio_read(tp, 0x18);
		gphy_val |= BIT_4;
		mdio_write(tp, 0x18, gphy_val);
		mdio_write(tp, 0x1f, 0x0000);
		gphy_val = mdio_read(tp, 0x14);
		gphy_val |= BIT_15;
		mdio_write(tp, 0x14, gphy_val);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8B86);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_0;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8B85);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_14;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x09, 0xA20F);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8B55);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x05, 0x8B5E);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x05, 0x8B67);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x05, 0x8B70);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0007);
		mdio_write(tp, 0x1E, 0x0078);
		mdio_write(tp, 0x17, 0x0000);
		mdio_write(tp, 0x19, 0x00FB);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8B79);
		mdio_write(tp, 0x06, 0xAA00);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1f, 0x0003);
		mdio_write(tp, 0x01, 0x328A);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1f, 0x0000);
		gphy_val = mdio_read(tp, 0x15);
		gphy_val |= BIT_12;
		mdio_write(tp, 0x15, gphy_val);
	} else if (tp->mcfg == CFG_METHOD_19) {
		RTL_W8(tp,0x6E, RTL_R8(tp,0x6E) | BIT_6);
		rtl8168_eri_write(tp, 0x1AE, 2, 0x0403, ERIAR_ExGMAC);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x00, 0x1800);
		gphy_val = mdio_read(tp, 0x15);
		gphy_val &= ~(BIT_12);
		mdio_write(tp, 0x15, gphy_val);
		mdio_write(tp, 0x00, 0x9800);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x002f);
		for (i = 0; i < 1000; i++) {
			udelay(100);
			gphy_val = mdio_read(tp, 0x1c);
			if (gphy_val & 0x0080)
				break;
		}
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x0023);
		mdio_write(tp, 0x16, 0x0306);
		mdio_write(tp, 0x16, 0x0307);
		mdio_write(tp, 0x15, 0x0098);
		mdio_write(tp, 0x19, 0x7c0b);
		mdio_write(tp, 0x15, 0x0099);
		mdio_write(tp, 0x19, 0x6c0b);
		mdio_write(tp, 0x15, 0x00eb);
		mdio_write(tp, 0x19, 0x6c0b);
		mdio_write(tp, 0x15, 0x00f8);
		mdio_write(tp, 0x19, 0x6f0b);
		mdio_write(tp, 0x15, 0x00fe);
		mdio_write(tp, 0x19, 0x6f0f);
		mdio_write(tp, 0x15, 0x00db);
		mdio_write(tp, 0x19, 0x6f09);
		mdio_write(tp, 0x15, 0x00dc);
		mdio_write(tp, 0x19, 0xaefd);
		mdio_write(tp, 0x15, 0x00dd);
		mdio_write(tp, 0x19, 0x6f0b);
		mdio_write(tp, 0x15, 0x00de);
		mdio_write(tp, 0x19, 0xc60b);
		mdio_write(tp, 0x15, 0x00df);
		mdio_write(tp, 0x19, 0x00fa);
		mdio_write(tp, 0x15, 0x00e0);
		mdio_write(tp, 0x19, 0x30e1);
		mdio_write(tp, 0x15, 0x0000);
		mdio_write(tp, 0x16, 0x0306);
		mdio_write(tp, 0x16, 0x0300);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1f, 0x0005);
		mdio_write(tp, 0x05, 0xfff6);
		mdio_write(tp, 0x06, 0x0080);
		mdio_write(tp, 0x05, 0x8000);
		mdio_write(tp, 0x06, 0x0280);
		mdio_write(tp, 0x06, 0x48f7);
		mdio_write(tp, 0x06, 0x00e0);
		mdio_write(tp, 0x06, 0xfff7);
		mdio_write(tp, 0x06, 0xa080);
		mdio_write(tp, 0x06, 0x02ae);
		mdio_write(tp, 0x06, 0xf602);
		mdio_write(tp, 0x06, 0x011b);
		mdio_write(tp, 0x06, 0x0201);
		mdio_write(tp, 0x06, 0x2802);
		mdio_write(tp, 0x06, 0x0135);
		mdio_write(tp, 0x06, 0x0201);
		mdio_write(tp, 0x06, 0x4502);
		mdio_write(tp, 0x06, 0x015f);
		mdio_write(tp, 0x06, 0x0201);
		mdio_write(tp, 0x06, 0x7602);
		mdio_write(tp, 0x06, 0x806b);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x88e1);
		mdio_write(tp, 0x06, 0x8b89);
		mdio_write(tp, 0x06, 0x1e01);
		mdio_write(tp, 0x06, 0xe18b);
		mdio_write(tp, 0x06, 0x8a1e);
		mdio_write(tp, 0x06, 0x01e1);
		mdio_write(tp, 0x06, 0x8b8b);
		mdio_write(tp, 0x06, 0x1e01);
		mdio_write(tp, 0x06, 0xe18b);
		mdio_write(tp, 0x06, 0x8c1e);
		mdio_write(tp, 0x06, 0x01e1);
		mdio_write(tp, 0x06, 0x8b8d);
		mdio_write(tp, 0x06, 0x1e01);
		mdio_write(tp, 0x06, 0xe18b);
		mdio_write(tp, 0x06, 0x8e1e);
		mdio_write(tp, 0x06, 0x01a0);
		mdio_write(tp, 0x06, 0x00c7);
		mdio_write(tp, 0x06, 0xaebb);
		mdio_write(tp, 0x06, 0xbf8b);
		mdio_write(tp, 0x06, 0x88ec);
		mdio_write(tp, 0x06, 0x0019);
		mdio_write(tp, 0x06, 0xa98b);
		mdio_write(tp, 0x06, 0x90f9);
		mdio_write(tp, 0x06, 0xeeff);
		mdio_write(tp, 0x06, 0xf600);
		mdio_write(tp, 0x06, 0xeeff);
		mdio_write(tp, 0x06, 0xf7fe);
		mdio_write(tp, 0x06, 0xd100);
		mdio_write(tp, 0x06, 0xbf81);
		mdio_write(tp, 0x06, 0x1e02);
		mdio_write(tp, 0x06, 0x39f3);
		mdio_write(tp, 0x06, 0xd101);
		mdio_write(tp, 0x06, 0xbf81);
		mdio_write(tp, 0x06, 0x2102);
		mdio_write(tp, 0x06, 0x39f3);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8ead);
		mdio_write(tp, 0x06, 0x201a);
		mdio_write(tp, 0x06, 0xf620);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x8e02);
		mdio_write(tp, 0x06, 0x2afe);
		mdio_write(tp, 0x06, 0x022c);
		mdio_write(tp, 0x06, 0x5c02);
		mdio_write(tp, 0x06, 0x03c5);
		mdio_write(tp, 0x06, 0x0280);
		mdio_write(tp, 0x06, 0xed02);
		mdio_write(tp, 0x06, 0x2e4f);
		mdio_write(tp, 0x06, 0x0204);
		mdio_write(tp, 0x06, 0x8902);
		mdio_write(tp, 0x06, 0x2f7a);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8ead);
		mdio_write(tp, 0x06, 0x210b);
		mdio_write(tp, 0x06, 0xf621);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x8e02);
		mdio_write(tp, 0x06, 0x0445);
		mdio_write(tp, 0x06, 0x021c);
		mdio_write(tp, 0x06, 0xb8e0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad22);
		mdio_write(tp, 0x06, 0x08f6);
		mdio_write(tp, 0x06, 0x22e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0x0235);
		mdio_write(tp, 0x06, 0xd4e0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad23);
		mdio_write(tp, 0x06, 0x08f6);
		mdio_write(tp, 0x06, 0x23e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0x0231);
		mdio_write(tp, 0x06, 0xc8e0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad24);
		mdio_write(tp, 0x06, 0x05f6);
		mdio_write(tp, 0x06, 0x24e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8ead);
		mdio_write(tp, 0x06, 0x2505);
		mdio_write(tp, 0x06, 0xf625);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x8ee0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad26);
		mdio_write(tp, 0x06, 0x08f6);
		mdio_write(tp, 0x06, 0x26e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0x022d);
		mdio_write(tp, 0x06, 0x6ae0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad27);
		mdio_write(tp, 0x06, 0x05f6);
		mdio_write(tp, 0x06, 0x27e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0x0203);
		mdio_write(tp, 0x06, 0x8bfc);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xfaef);
		mdio_write(tp, 0x06, 0x69e0);
		mdio_write(tp, 0x06, 0x8b80);
		mdio_write(tp, 0x06, 0xad27);
		mdio_write(tp, 0x06, 0x22bf);
		mdio_write(tp, 0x06, 0x479a);
		mdio_write(tp, 0x06, 0x0239);
		mdio_write(tp, 0x06, 0xc6e0);
		mdio_write(tp, 0x06, 0x8b44);
		mdio_write(tp, 0x06, 0x1f01);
		mdio_write(tp, 0x06, 0x9e15);
		mdio_write(tp, 0x06, 0xe58b);
		mdio_write(tp, 0x06, 0x44ad);
		mdio_write(tp, 0x06, 0x2907);
		mdio_write(tp, 0x06, 0xac28);
		mdio_write(tp, 0x06, 0x04d1);
		mdio_write(tp, 0x06, 0x01ae);
		mdio_write(tp, 0x06, 0x02d1);
		mdio_write(tp, 0x06, 0x00bf);
		mdio_write(tp, 0x06, 0x8124);
		mdio_write(tp, 0x06, 0x0239);
		mdio_write(tp, 0x06, 0xf3ef);
		mdio_write(tp, 0x06, 0x96fe);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0x00e1);
		mdio_write(tp, 0x06, 0x4077);
		mdio_write(tp, 0x06, 0xe140);
		mdio_write(tp, 0x06, 0xbbe0);
		mdio_write(tp, 0x06, 0x2a00);
		mdio_write(tp, 0x05, 0xe142);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_0;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x05, 0xe140);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_0;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x0023);
		gphy_val = mdio_read(tp, 0x17);
		gphy_val |= BIT_1;
		mdio_write(tp, 0x17, gphy_val);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x00, 0x9200);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8b80);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_2 | BIT_1;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x002D);
		gphy_val = mdio_read(tp, 0x18);
		gphy_val |= BIT_4;
		mdio_write(tp, 0x18, gphy_val);
		mdio_write(tp, 0x1f, 0x0000);
		gphy_val = mdio_read(tp, 0x14);
		gphy_val |= BIT_15;
		mdio_write(tp, 0x14, gphy_val);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8B86);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_0;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1f, 0x0000);
		gphy_val = mdio_read(tp, 0x15);
		gphy_val |= BIT_12;
		mdio_write(tp, 0x15, gphy_val);
	} else if (tp->mcfg == CFG_METHOD_20) {
		RTL_W8(tp,0x6E, RTL_R8(tp,0x6E) | BIT_6);
		rtl8168_eri_write(tp, 0x1AE, 2, 0x0403, ERIAR_ExGMAC);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x00, 0x1800);
		gphy_val = mdio_read(tp, 0x15);
		gphy_val &= ~(BIT_12);
		mdio_write(tp, 0x15, gphy_val);
		mdio_write(tp, 0x00, 0x4800);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x002f);
		for (i = 0; i < 1000; i++) {
			udelay(100);
			gphy_val = mdio_read(tp, 0x1c);
			if (gphy_val & 0x0080)
				break;
		}
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x00, 0x1800);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x0023);
		for (i = 0; i < 200; i++) {
			udelay(100);
			gphy_val = mdio_read(tp, 0x18);
			if (!(gphy_val & 0x0001))
				break;
		}
		mdio_write(tp, 0x1f, 0x0005);
		mdio_write(tp, 0x05, 0xfff6);
		mdio_write(tp, 0x06, 0x0080);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x0023);
		mdio_write(tp, 0x16, 0x0306);
		mdio_write(tp, 0x16, 0x0307);
		mdio_write(tp, 0x15, 0x0098);
		mdio_write(tp, 0x19, 0x7c0b);
		mdio_write(tp, 0x15, 0x0099);
		mdio_write(tp, 0x19, 0x6c0b);
		mdio_write(tp, 0x15, 0x00eb);
		mdio_write(tp, 0x19, 0x6c0b);
		mdio_write(tp, 0x15, 0x00f8);
		mdio_write(tp, 0x19, 0x6f0b);
		mdio_write(tp, 0x15, 0x00fe);
		mdio_write(tp, 0x19, 0x6f0f);
		mdio_write(tp, 0x15, 0x00db);
		mdio_write(tp, 0x19, 0x6f09);
		mdio_write(tp, 0x15, 0x00dc);
		mdio_write(tp, 0x19, 0xaefd);
		mdio_write(tp, 0x15, 0x00dd);
		mdio_write(tp, 0x19, 0x6f0b);
		mdio_write(tp, 0x15, 0x00de);
		mdio_write(tp, 0x19, 0xc60b);
		mdio_write(tp, 0x15, 0x00df);
		mdio_write(tp, 0x19, 0x00fa);
		mdio_write(tp, 0x15, 0x00e0);
		mdio_write(tp, 0x19, 0x30e1);
		mdio_write(tp, 0x15, 0x0000);
		mdio_write(tp, 0x16, 0x0306);
		mdio_write(tp, 0x16, 0x0300);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1f, 0x0005);
		mdio_write(tp, 0x05, 0xfff6);
		mdio_write(tp, 0x06, 0x0080);
		mdio_write(tp, 0x05, 0x8000);
		mdio_write(tp, 0x06, 0x0280);
		mdio_write(tp, 0x06, 0x48f7);
		mdio_write(tp, 0x06, 0x00e0);
		mdio_write(tp, 0x06, 0xfff7);
		mdio_write(tp, 0x06, 0xa080);
		mdio_write(tp, 0x06, 0x02ae);
		mdio_write(tp, 0x06, 0xf602);
		mdio_write(tp, 0x06, 0x011e);
		mdio_write(tp, 0x06, 0x0201);
		mdio_write(tp, 0x06, 0x2b02);
		mdio_write(tp, 0x06, 0x8077);
		mdio_write(tp, 0x06, 0x0201);
		mdio_write(tp, 0x06, 0x4802);
		mdio_write(tp, 0x06, 0x0162);
		mdio_write(tp, 0x06, 0x0201);
		mdio_write(tp, 0x06, 0x7902);
		mdio_write(tp, 0x06, 0x8094);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x88e1);
		mdio_write(tp, 0x06, 0x8b89);
		mdio_write(tp, 0x06, 0x1e01);
		mdio_write(tp, 0x06, 0xe18b);
		mdio_write(tp, 0x06, 0x8a1e);
		mdio_write(tp, 0x06, 0x01e1);
		mdio_write(tp, 0x06, 0x8b8b);
		mdio_write(tp, 0x06, 0x1e01);
		mdio_write(tp, 0x06, 0xe18b);
		mdio_write(tp, 0x06, 0x8c1e);
		mdio_write(tp, 0x06, 0x01e1);
		mdio_write(tp, 0x06, 0x8b8d);
		mdio_write(tp, 0x06, 0x1e01);
		mdio_write(tp, 0x06, 0xe18b);
		mdio_write(tp, 0x06, 0x8e1e);
		mdio_write(tp, 0x06, 0x01a0);
		mdio_write(tp, 0x06, 0x00c7);
		mdio_write(tp, 0x06, 0xaebb);
		mdio_write(tp, 0x06, 0xd481);
		mdio_write(tp, 0x06, 0x5ae4);
		mdio_write(tp, 0x06, 0x8b92);
		mdio_write(tp, 0x06, 0xe58b);
		mdio_write(tp, 0x06, 0x9302);
		mdio_write(tp, 0x06, 0x2e5a);
		mdio_write(tp, 0x06, 0xbf8b);
		mdio_write(tp, 0x06, 0x88ec);
		mdio_write(tp, 0x06, 0x0019);
		mdio_write(tp, 0x06, 0xa98b);
		mdio_write(tp, 0x06, 0x90f9);
		mdio_write(tp, 0x06, 0xeeff);
		mdio_write(tp, 0x06, 0xf600);
		mdio_write(tp, 0x06, 0xeeff);
		mdio_write(tp, 0x06, 0xf7fc);
		mdio_write(tp, 0x06, 0xd100);
		mdio_write(tp, 0x06, 0xbf82);
		mdio_write(tp, 0x06, 0x3402);
		mdio_write(tp, 0x06, 0x3a21);
		mdio_write(tp, 0x06, 0xd101);
		mdio_write(tp, 0x06, 0xbf82);
		mdio_write(tp, 0x06, 0x3702);
		mdio_write(tp, 0x06, 0x3a21);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8aad);
		mdio_write(tp, 0x06, 0x2014);
		mdio_write(tp, 0x06, 0xee8b);
		mdio_write(tp, 0x06, 0x8a00);
		mdio_write(tp, 0x06, 0x0220);
		mdio_write(tp, 0x06, 0x8be0);
		mdio_write(tp, 0x06, 0xe426);
		mdio_write(tp, 0x06, 0xe1e4);
		mdio_write(tp, 0x06, 0x27ee);
		mdio_write(tp, 0x06, 0xe426);
		mdio_write(tp, 0x06, 0x23e5);
		mdio_write(tp, 0x06, 0xe427);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8e0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad20);
		mdio_write(tp, 0x06, 0x1af6);
		mdio_write(tp, 0x06, 0x20e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0x022b);
		mdio_write(tp, 0x06, 0x1e02);
		mdio_write(tp, 0x06, 0x2c7c);
		mdio_write(tp, 0x06, 0x0203);
		mdio_write(tp, 0x06, 0xc002);
		mdio_write(tp, 0x06, 0x8203);
		mdio_write(tp, 0x06, 0x022e);
		mdio_write(tp, 0x06, 0x6f02);
		mdio_write(tp, 0x06, 0x047b);
		mdio_write(tp, 0x06, 0x022f);
		mdio_write(tp, 0x06, 0x9ae0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad21);
		mdio_write(tp, 0x06, 0x0bf6);
		mdio_write(tp, 0x06, 0x21e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0x0281);
		mdio_write(tp, 0x06, 0x1602);
		mdio_write(tp, 0x06, 0x1cd9);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8ead);
		mdio_write(tp, 0x06, 0x2208);
		mdio_write(tp, 0x06, 0xf622);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x8e02);
		mdio_write(tp, 0x06, 0x35f4);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8ead);
		mdio_write(tp, 0x06, 0x2308);
		mdio_write(tp, 0x06, 0xf623);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x8e02);
		mdio_write(tp, 0x06, 0x31e8);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8ead);
		mdio_write(tp, 0x06, 0x2405);
		mdio_write(tp, 0x06, 0xf624);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x8ee0);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xad25);
		mdio_write(tp, 0x06, 0x05f6);
		mdio_write(tp, 0x06, 0x25e4);
		mdio_write(tp, 0x06, 0x8b8e);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8ead);
		mdio_write(tp, 0x06, 0x2608);
		mdio_write(tp, 0x06, 0xf626);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x8e02);
		mdio_write(tp, 0x06, 0x2d8a);
		mdio_write(tp, 0x06, 0xe08b);
		mdio_write(tp, 0x06, 0x8ead);
		mdio_write(tp, 0x06, 0x2705);
		mdio_write(tp, 0x06, 0xf627);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x8e02);
		mdio_write(tp, 0x06, 0x0386);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8fa);
		mdio_write(tp, 0x06, 0xef69);
		mdio_write(tp, 0x06, 0xe0e0);
		mdio_write(tp, 0x06, 0x00e1);
		mdio_write(tp, 0x06, 0xe001);
		mdio_write(tp, 0x06, 0xad27);
		mdio_write(tp, 0x06, 0x32e0);
		mdio_write(tp, 0x06, 0x8b40);
		mdio_write(tp, 0x06, 0xf720);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x40bf);
		mdio_write(tp, 0x06, 0x32c1);
		mdio_write(tp, 0x06, 0x0239);
		mdio_write(tp, 0x06, 0xf4ad);
		mdio_write(tp, 0x06, 0x2821);
		mdio_write(tp, 0x06, 0xe0e0);
		mdio_write(tp, 0x06, 0x20e1);
		mdio_write(tp, 0x06, 0xe021);
		mdio_write(tp, 0x06, 0xad20);
		mdio_write(tp, 0x06, 0x18e0);
		mdio_write(tp, 0x06, 0x8b40);
		mdio_write(tp, 0x06, 0xf620);
		mdio_write(tp, 0x06, 0xe48b);
		mdio_write(tp, 0x06, 0x40ee);
		mdio_write(tp, 0x06, 0x8b3b);
		mdio_write(tp, 0x06, 0xffe0);
		mdio_write(tp, 0x06, 0x8a8a);
		mdio_write(tp, 0x06, 0xe18a);
		mdio_write(tp, 0x06, 0x8be4);
		mdio_write(tp, 0x06, 0xe000);
		mdio_write(tp, 0x06, 0xe5e0);
		mdio_write(tp, 0x06, 0x01ef);
		mdio_write(tp, 0x06, 0x96fe);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0xf8f9);
		mdio_write(tp, 0x06, 0xface);
		mdio_write(tp, 0x06, 0xfaef);
		mdio_write(tp, 0x06, 0x69fa);
		mdio_write(tp, 0x06, 0xd401);
		mdio_write(tp, 0x06, 0x55b4);
		mdio_write(tp, 0x06, 0xfebf);
		mdio_write(tp, 0x06, 0x1c5e);
		mdio_write(tp, 0x06, 0x0239);
		mdio_write(tp, 0x06, 0xf4ac);
		mdio_write(tp, 0x06, 0x280b);
		mdio_write(tp, 0x06, 0xbf1c);
		mdio_write(tp, 0x06, 0x5b02);
		mdio_write(tp, 0x06, 0x39f4);
		mdio_write(tp, 0x06, 0xac28);
		mdio_write(tp, 0x06, 0x49ae);
		mdio_write(tp, 0x06, 0x64bf);
		mdio_write(tp, 0x06, 0x1c5b);
		mdio_write(tp, 0x06, 0x0239);
		mdio_write(tp, 0x06, 0xf4ac);
		mdio_write(tp, 0x06, 0x285b);
		mdio_write(tp, 0x06, 0xd000);
		mdio_write(tp, 0x06, 0x0281);
		mdio_write(tp, 0x06, 0xe8ac);
		mdio_write(tp, 0x06, 0x2105);
		mdio_write(tp, 0x06, 0xac22);
		mdio_write(tp, 0x06, 0x02ae);
		mdio_write(tp, 0x06, 0x4ebf);
		mdio_write(tp, 0x06, 0xe0c4);
		mdio_write(tp, 0x06, 0xbe85);
		mdio_write(tp, 0x06, 0xecd2);
		mdio_write(tp, 0x06, 0x04d8);
		mdio_write(tp, 0x06, 0x19d9);
		mdio_write(tp, 0x06, 0x1907);
		mdio_write(tp, 0x06, 0xdc19);
		mdio_write(tp, 0x06, 0xdd19);
		mdio_write(tp, 0x06, 0x0789);
		mdio_write(tp, 0x06, 0x89ef);
		mdio_write(tp, 0x06, 0x645e);
		mdio_write(tp, 0x06, 0x07ff);
		mdio_write(tp, 0x06, 0x0d65);
		mdio_write(tp, 0x06, 0x5cf8);
		mdio_write(tp, 0x06, 0x001e);
		mdio_write(tp, 0x06, 0x46dc);
		mdio_write(tp, 0x06, 0x19dd);
		mdio_write(tp, 0x06, 0x19b2);
		mdio_write(tp, 0x06, 0xe2d4);
		mdio_write(tp, 0x06, 0x0001);
		mdio_write(tp, 0x06, 0xbf1c);
		mdio_write(tp, 0x06, 0x5b02);
		mdio_write(tp, 0x06, 0x3a21);
		mdio_write(tp, 0x06, 0xae1d);
		mdio_write(tp, 0x06, 0xbee0);
		mdio_write(tp, 0x06, 0xc4bf);
		mdio_write(tp, 0x06, 0x85ec);
		mdio_write(tp, 0x06, 0xd204);
		mdio_write(tp, 0x06, 0xd819);
		mdio_write(tp, 0x06, 0xd919);
		mdio_write(tp, 0x06, 0x07dc);
		mdio_write(tp, 0x06, 0x19dd);
		mdio_write(tp, 0x06, 0x1907);
		mdio_write(tp, 0x06, 0xb2f4);
		mdio_write(tp, 0x06, 0xd400);
		mdio_write(tp, 0x06, 0x00bf);
		mdio_write(tp, 0x06, 0x1c5b);
		mdio_write(tp, 0x06, 0x023a);
		mdio_write(tp, 0x06, 0x21fe);
		mdio_write(tp, 0x06, 0xef96);
		mdio_write(tp, 0x06, 0xfec6);
		mdio_write(tp, 0x06, 0xfefd);
		mdio_write(tp, 0x06, 0xfc05);
		mdio_write(tp, 0x06, 0xf9e2);
		mdio_write(tp, 0x06, 0xe0ea);
		mdio_write(tp, 0x06, 0xe3e0);
		mdio_write(tp, 0x06, 0xeb5a);
		mdio_write(tp, 0x06, 0x070c);
		mdio_write(tp, 0x06, 0x031e);
		mdio_write(tp, 0x06, 0x20e6);
		mdio_write(tp, 0x06, 0xe0ea);
		mdio_write(tp, 0x06, 0xe7e0);
		mdio_write(tp, 0x06, 0xebe0);
		mdio_write(tp, 0x06, 0xe0fc);
		mdio_write(tp, 0x06, 0xe1e0);
		mdio_write(tp, 0x06, 0xfdfd);
		mdio_write(tp, 0x06, 0x04f8);
		mdio_write(tp, 0x06, 0xfaef);
		mdio_write(tp, 0x06, 0x69e0);
		mdio_write(tp, 0x06, 0x8b80);
		mdio_write(tp, 0x06, 0xad27);
		mdio_write(tp, 0x06, 0x22bf);
		mdio_write(tp, 0x06, 0x47ba);
		mdio_write(tp, 0x06, 0x0239);
		mdio_write(tp, 0x06, 0xf4e0);
		mdio_write(tp, 0x06, 0x8b44);
		mdio_write(tp, 0x06, 0x1f01);
		mdio_write(tp, 0x06, 0x9e15);
		mdio_write(tp, 0x06, 0xe58b);
		mdio_write(tp, 0x06, 0x44ad);
		mdio_write(tp, 0x06, 0x2907);
		mdio_write(tp, 0x06, 0xac28);
		mdio_write(tp, 0x06, 0x04d1);
		mdio_write(tp, 0x06, 0x01ae);
		mdio_write(tp, 0x06, 0x02d1);
		mdio_write(tp, 0x06, 0x00bf);
		mdio_write(tp, 0x06, 0x823a);
		mdio_write(tp, 0x06, 0x023a);
		mdio_write(tp, 0x06, 0x21ef);
		mdio_write(tp, 0x06, 0x96fe);
		mdio_write(tp, 0x06, 0xfc04);
		mdio_write(tp, 0x06, 0x00e1);
		mdio_write(tp, 0x06, 0x4077);
		mdio_write(tp, 0x06, 0xe140);
		mdio_write(tp, 0x06, 0xbbe0);
		mdio_write(tp, 0x06, 0x2a00);
		mdio_write(tp, 0x05, 0xe142);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_0;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x05, 0xe140);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_0;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1f, 0x0005);
		for (i = 0; i < 200; i++) {
			udelay(100);
			gphy_val = mdio_read(tp, 0x00);
			if (gphy_val & BIT_7)
				break;
		}
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x0023);
		gphy_val = mdio_read(tp, 0x17);
		gphy_val |= BIT_1;
		mdio_write(tp, 0x17, gphy_val);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x00, 0x9200);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8b80);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_2 | BIT_1;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1f, 0x0007);
		mdio_write(tp, 0x1e, 0x002D);
		gphy_val = mdio_read(tp, 0x18);
		gphy_val |= BIT_4;
		mdio_write(tp, 0x18, gphy_val);
		mdio_write(tp, 0x1f, 0x0000);
		gphy_val = mdio_read(tp, 0x14);
		gphy_val |= BIT_15;
		mdio_write(tp, 0x14, gphy_val);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8B86);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_0;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8B85);
		gphy_val = mdio_read(tp, 0x06);
		gphy_val |= BIT_14;
		mdio_write(tp, 0x06, gphy_val);
		mdio_write(tp, 0x1f, 0x0000);
		mdio_write(tp, 0x1F, 0x0003);
		mdio_write(tp, 0x09, 0xA20F);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8B55);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x05, 0x8B5E);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x05, 0x8B67);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x05, 0x8B70);
		mdio_write(tp, 0x06, 0x0000);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0007);
		mdio_write(tp, 0x1E, 0x0078);
		mdio_write(tp, 0x17, 0x0000);
		mdio_write(tp, 0x19, 0x00aa);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1F, 0x0005);
		mdio_write(tp, 0x05, 0x8B79);
		mdio_write(tp, 0x06, 0xAA00);
		mdio_write(tp, 0x1F, 0x0000);
		mdio_write(tp, 0x1f, 0x0000);
		gphy_val = mdio_read(tp, 0x15);
		gphy_val |= BIT_12;
		mdio_write(tp, 0x15, gphy_val);
	}
	mdio_write(tp, 0x1F, 0x0000);
}

static u8 rtl8168_efuse_read(struct rtl8168_private *tp, u16 reg)
{
	u8 efuse_data;
	u32 temp;
	int cnt;

	if (tp->efuse == EFUSE_NOT_SUPPORT)
		return EFUSE_READ_FAIL;

	temp = EFUSE_READ | ((reg & EFUSE_Reg_Mask) << EFUSE_Reg_Shift);
	RTL_W32(tp, EFUSEAR, temp);

	do {
		delay(100);
		temp = RTL_R32(tp, EFUSEAR);
		cnt++;
	} while (!(temp & EFUSE_READ_OK) && (temp < EFUSE_Check_Cnt));

	if (temp == EFUSE_Check_Cnt)
		efuse_data = EFUSE_READ_FAIL;
	else
		efuse_data = (u8)(RTL_R32(tp, EFUSEAR) & EFUSE_Data_Mask);

	return efuse_data;
}

static void rtl8168_link_option(int idx, u8 *aut, u16 *spd, u8 *dup)
{
	unsigned char opt_speed;
	unsigned char opt_duplex;
	unsigned char opt_autoneg;

	opt_speed = ((idx < MAX_UNITS) && (idx >= 0)) ? speed[idx] : 0xff;
	opt_duplex = ((idx < MAX_UNITS) && (idx >= 0)) ? duplex[idx] : 0xff;
	opt_autoneg = ((idx < MAX_UNITS) && (idx >= 0)) ? autoneg[idx] : 0xff;

	if ((opt_speed == 0xff) |
	    (opt_duplex == 0xff) |
	    (opt_autoneg == 0xff)) {
		*spd = SPEED_1000;
		*dup = DUPLEX_FULL;
		*aut = AUTONEG_ENABLE;
	} else {
		*spd = speed[idx];
		*dup = duplex[idx];
		*aut = autoneg[idx];	
	}
}

static int rtl8168_set_speed(struct rtl8168_private *tp, u8 autoneg, u16 speed, u8 duplex)
{
	int ret;

	ret = tp->set_speed(tp, autoneg, speed, duplex);

	return ret;
}

static int rtl8168_ether_ioctl(struct ifnet *ifp, unsigned long cmd, caddr_t data)
{
	struct ifaddr *ifa = (struct ifaddr *) data;
	struct rtl8168_private *sc = ifp->if_softc;
	int error = 0;
	int s;

	s = splimp();
	switch (cmd) {
	case SIOCPOLL:
		break;
	case SIOCSIFADDR:
		switch (ifa->ifa_addr->sa_family) {
#ifdef INET
		case AF_INET:
			error = rtl8168_open(sc);
			if(error == -1){
				return(error);
			}	
			ifp->if_flags |= IFF_UP;
			
#ifdef __OpenBSD__
			arp_ifinit(&sc->arpcom, ifa);
			printf("\n");
#else
			arp_ifinit(ifp, ifa);
#endif
			
			break;
#endif

		default:
		       rtl8168_open(sc);
			ifp->if_flags |= IFF_UP;
			break;
		}
		break;
	case SIOCSIFFLAGS:
		/*
		 * If interface is marked up and not running, then start it.
		 * If it is marked down and running, stop it.
		 * XXX If it's up then re-initialize it. This is so flags
		 * such as IFF_PROMISC are handled.
		 */
		if(ifp->if_flags & IFF_UP){
             rtl8168_open(sc);
		}
		break;
	default:
		error = EINVAL;
	}

	splx(s);

	return (error);
}

static void rtl8168_start_xmit(struct ifnet *ifp)
{
	struct rtl8168_private *tp = ifp->if_softc;
	unsigned int entry = tp->cur_tx % NUM_TX_DESC;
	struct TxDesc *txd = tp->TxDescArray + entry;

	if(tp->cur_tx - tp->dirty_tx == NUM_TX_DESC){
		dprintk("Full ????\n"); 
		return;
	}
	
	if (le32_to_cpu(txd->opts1) & DescOwn){
		dprintk("DescOwn Error!Desc NUM = %d", entry); 
		tp->stats.tx_dropped++;
		return;
	}

	if(rtl8168_xmit_frags(tp, ifp) < 0){
		dprintk("TX Error!  %s, %d\n", __FUNCTION__, __LINE__);
		return;
	}//transmit all

	RTL_W8(tp, TxPoll, 0x40);	// set polling bit 


	if (TX_BUFFS_AVAIL(tp) < MAX_FRAGS) { //FIXME
		if (TX_BUFFS_AVAIL(tp) >= MAX_FRAGS){
		    ;//FIXME netif_wake_queue(dev);
		}
	
	}

	return ;
}

static int rtl8168_xmit_frags(struct rtl8168_private *tp,  struct ifnet *ifp)
{
	struct TxDesc* td;
	struct mbuf *mb_head;
	unsigned int entry = 0;

	while(ifp->if_snd.ifq_head != NULL){
		u32 status, len;
		u32 IfRingEnd = 0;
		  
		entry = tp->cur_tx % NUM_TX_DESC;
		if(tp->TxDescArray[entry].opts1 & DescOwn){
				printf("NO available TX buffer ! %d\n", __LINE__);
				break;
		}
		
		td = tp->TxDescArray + entry;

		IF_DEQUEUE(&ifp->if_snd, mb_head);
		m_copydata(mb_head, 0, mb_head->m_pkthdr.len, tp->tx_buffer[entry]);
	             
		len = mb_head->m_pkthdr.len;
		IfRingEnd = ((entry + 1) % NUM_TX_DESC) ? 0 : RingEnd;
	    status = DescOwn |FirstFrag | LastFrag |len | IfRingEnd;//no fragement.
		td->opts1 = cpu_to_le32(status);

#ifdef __mips__
#if !(defined(LS3_HT) || defined(LS2G_HT))
		pci_sync_cache(tp->sc_pc, (vm_offset_t)tp->tx_buffer[entry], 
				len, SYNC_W);
#endif
#endif
		m_freem(mb_head);
		wbflush();  
		tp->cur_tx ++;
		
	}

	return 0;
}

static int rtl8168_open_times = 0; 
static int rtl8168_open(struct rtl8168_private *tp)
{
	int retval = 0;
    u32 status;

	/*
	 * Rx and Tx desscriptors needs 256 bytes alignment.
	 * pci_alloc_consistent provides more.
	 */
    if(!tp->TxDescArray){
		tp->TxDescArray = (struct TxDesc*)malloc(R8168_TX_RING_BYTES + 255, M_DEVBUF, M_DONTWAIT);
		if (!tp->TxDescArray) {
			retval = -ENOMEM;
			goto err_free_tx;
		}
		memset((caddr_t)tp->TxDescArray, 0, R8168_TX_RING_BYTES + 255);
#if !(defined(LS3_HT) || defined(LS2G_HT))
       	pci_sync_cache(tp->sc_pc, (vm_offset_t)(tp->TxDescArray),  R8168_TX_RING_BYTES + 255, SYNC_W);
#endif
       	tp->TxDescArray = (struct TxDesc *)(((unsigned long)(tp->TxDescArray) + 255) & ~255);	
	}
#if !(defined(LS3_HT) || defined(LS2G_HT))
	tp->TxDescArray = (struct TxDesc*)CACHED_TO_UNCACHED((unsigned long)(tp->TxDescArray));
#else
	tp->TxDescArray = (struct TxDesc*)((unsigned long)(tp->TxDescArray));
#endif
	tp->TxPhyAddr = (unsigned long)vtophys((unsigned long)(tp->TxDescArray));

	if(!tp->RxDescArray){
		tp->RxDescArray = (struct RxDesc*)malloc(R8168_RX_RING_BYTES + 255, M_DEVBUF, M_DONTWAIT);
	       if (!tp->RxDescArray) {
		      retval = -ENOMEM;
		      goto err_free_rx;
		   }		
		   memset((caddr_t)(tp->RxDescArray), 0, R8168_RX_RING_BYTES + 255);
#if !(defined(LS3_HT) || defined(LS2G_HT))
		   pci_sync_cache(tp->sc_pc, (vm_offset_t)(tp->RxDescArray),  R8168_RX_RING_BYTES + 255, SYNC_W);
#endif
		   tp->RxDescArray = (struct RxDesc *)(((unsigned long)(tp->RxDescArray) + 255) & ~255);	
	}
#if !(defined(LS3_HT) || defined(LS2G_HT))
	tp->RxDescArray = (struct RxDesc*)CACHED_TO_UNCACHED((unsigned long)(tp->RxDescArray));
#else
	tp->RxDescArray = (struct RxDesc*)((unsigned long)(tp->RxDescArray));
#endif
	tp->RxPhyAddr = (unsigned long)vtophys((unsigned long)(tp->RxDescArray));

	tp->tx_buf_sz = TX_BUF_SIZE;
	tp->rx_buf_sz = RX_BUF_SIZE;
	retval = rtl8168_init_ring(tp);
    
	if (retval < 0){
		goto err_free_rx;
	}
	//rtl8168_hw_reset(tp);//by AdonWang
    
	rtl8168_hw_start(tp);
    
	rtl8168_check_link_status(tp);
    tp->arpcom.ac_if.if_flags |=  IFF_RUNNING;

    status = RTL_R16(tp, IntrStatus);

out:
	return retval;

err_free_rx:
	free(tp->RxDescArray, M_DEVBUF);
err_free_tx:
	free(tp->TxDescArray, M_DEVBUF);

	goto out;
}


static void rtl8168_init_ring_indexes(struct rtl8168_private *tp)
{
	tp->dirty_tx = tp->dirty_rx = tp->cur_tx = tp->cur_rx = 0;
}

static int rtl8168_init_ring(struct rtl8168_private *tp)
{
	int tmpres;

	rtl8168_init_ring_indexes(tp);

	if(rtl8168_open_times== 1){
		memset(tp->tx_buffer, 0x0, sizeof(char*) * NUM_TX_DESC);
		memset(tp->rx_buffer, 0x0, sizeof(char*) * NUM_RX_DESC);
	}

	if ((tmpres = rtl8168_rx_fill(tp, 0, NUM_RX_DESC, 0)) != NUM_RX_DESC){
		goto err_out1;
	}

	if (rtl8168_tx_fill(tp, 0, NUM_TX_DESC, 0) != NUM_TX_DESC){
		goto err_out2;
	}

	rtl8168_mark_as_last_descriptor(tp->RxDescArray + NUM_RX_DESC - 1);
	return 0;

err_out2:
	rtl8168_rx_clear(tp);
err_out1:
	rtl8168_tx_clear(tp);

	return -ENOMEM;
}

static u32 rtl8168_rx_fill(struct rtl8168_private *tp,
			   u32 start, u32 end, int caller)
{
	u32 cur = 0, tmprecord = 0;
	int ret;
	if (caller == 0) {
		for(cur = 0; cur < NUM_RX_DESC; cur++) {
			if (tp->rx_buffer[cur])
				continue;

       		ret = rtl8168_alloc_rx(&(tp->rx_buffer[cur]), tp->RxDescArray + cur, tp->rx_buf_sz, caller);
			if(ret < 0){
				printf("ERROR: %s, %d\n", __FUNCTION__, __LINE__);
				break;
			}
		}
		tp->RxDescArray[cur - 1].opts1 |= RingEnd;
		return cur;
	}

	//caller == 1
	for (cur = start; cur < end; cur++) {
		int  entry = cur % NUM_RX_DESC;

		if(tp->RxDescArray[entry].opts1& DescOwn) {
			break;
		}

		tmprecord = tp->RxDescArray[entry].opts1 & RingEnd;
		tp->RxDescArray[entry].opts1 = 0;
		tp->RxDescArray[entry].opts1 = tmprecord | DescOwn | tp->rx_buf_sz;
	}

	return cur - start;
}

static u32 rtl8168_tx_fill(struct rtl8168_private *tp,
			   u32 start, u32 end, u32 caller)
{
	u32 cur, ret, entry;
	for (cur = start;  cur < end; cur++) {
		entry = cur % NUM_TX_DESC;
		if(caller == 0){
			if(tp->tx_buffer[entry])
				continue;
			tp->TxDescArray[entry].opts1 = 0;
			tp->TxDescArray[entry].opts2 = 0;
			tp->TxDescArray[entry].addr = 0;
			tp->tx_buffer[entry] = NULL;

			ret = rtl8168_alloc_tx(&(tp->tx_buffer[entry]),
						tp->TxDescArray + entry, tp->tx_buf_sz);
			if (ret < 0) {
				printf("ret = %d. %s, %d\n", ret, __FILE__, __LINE__);
				break;
			}
		}

		if(caller == 1) {
			if(tp->TxDescArray[entry].opts1 & DescOwn)
				break;
			tp->TxDescArray[entry].opts1 &= RingEnd;
		}
	}
	return cur - start;
}


static void rtl8168_rx_clear(struct rtl8168_private *tp)
{
	int i;

	for (i = 0; i < NUM_RX_DESC; i++) {
		if (tp->rx_buffer[i]) {
			rtl8168_free_rx(tp, tp->rx_buffer[i],tp->RxDescArray + i);
		}
	}
}

static void rtl8168_tx_clear(struct rtl8168_private *tp)
{
	unsigned int i;

	for (i = tp->dirty_tx; i < tp->dirty_tx + NUM_TX_DESC; i++) {
		unsigned int entry = i % NUM_TX_DESC;
		if(tp->tx_buffer[entry]){
			rtl8168_free_tx( tp, &tp->tx_buffer[entry], tp->TxDescArray+entry);
		}
		tp->stats.tx_dropped++;
	}
	tp->cur_tx = tp->dirty_tx = 0;
}

static void rtl8168_free_tx(struct rtl8168_private * tp, unsigned char ** buf, struct TxDesc * desc)
{
	free(buf, M_DEVBUF);
	*buf = NULL;
	rtl8168_make_unusable_by_asic(desc);
	return;
}

static void rtl8168_free_rx(struct rtl8168_private *tp,
				unsigned char *buf, struct RxDesc *desc)
{
	free(buf, M_DEVBUF);
	buf = NULL;
	rtl8168_make_unusable_by_asic(desc);
}

static inline void rtl8168_make_unusable_by_asic(struct RxDesc *desc)
{
	desc->addr = 0x0badbadbadbadbadull;;
	desc->opts1 &=  ~cpu_to_le32(DescOwn | RsvdMask);
}


static int rtl8168_alloc_tx(unsigned char **tx_buffer,
				struct TxDesc *desc, int tx_buf_sz)
{
	unsigned long mapping;
	int ret = 0;
	unsigned long tmp;


  	tmp = (unsigned long)malloc(tx_buf_sz, M_DEVBUF, M_DONTWAIT);
	if(!tmp){
		goto err_out;
	}

#ifdef __mips__
#if !(defined(LS3_HT) || defined(LS2G_HT))
	pci_sync_cache(NULL, (vm_offset_t)tmp, tx_buf_sz, SYNC_W);
#endif
#endif

#if !(defined(LS3_HT) || defined(LS2G_HT))
	*tx_buffer = (unsigned char *)CACHED_TO_UNCACHED(tmp);
#else
	*tx_buffer = (unsigned char *)(tmp);
#endif

	mapping = (unsigned long)vtophys(*tx_buffer);

	desc->addr = cpu_to_le64(mapping);

	return ret;
err_out:
	ret = -ENOMEM;
	return ret;
}

static int rtl8168_alloc_rx(unsigned char **rx_buffer,
				struct RxDesc *desc, int rx_buf_sz, int caller)
{
	unsigned long mapping;
	int ret = 0;


	*rx_buffer = (char*)malloc(rx_buf_sz + 7, M_DEVBUF, M_DONTWAIT);
	if(!(*rx_buffer)){
		goto err_out;
	}
	*rx_buffer = (unsigned char *)(((unsigned long)(*rx_buffer) + 7) & ~7);
#if !(defined(LS3_HT) || defined(LS2G_HT))
	*rx_buffer = (unsigned char *)CACHED_TO_UNCACHED(*rx_buffer);	
#else
	*rx_buffer = (unsigned char *)(*rx_buffer);	
#endif

	mapping = (unsigned long)vtophys(*rx_buffer);

	rtl8168_map_to_asic(desc, mapping, rx_buf_sz, caller);

out:
	return ret;

err_out:
	ret = -ENOMEM;
	rtl8168_make_unusable_by_asic(desc);
	goto out;
}

static inline void rtl8168_map_to_asic(struct RxDesc *desc, unsigned long mapping, u32 rx_buf_sz, int caller)
{
       desc->addr = cpu_to_le64(mapping);

	rtl8168_mark_to_asic(desc, rx_buf_sz, caller);
}


static inline void rtl8168_mark_to_asic(struct RxDesc *desc, u32 rx_buf_sz, int caller)
{
        unsigned int eor = 0;
	if(caller == 1)
            eor = le32_to_cpu(desc->opts1) & RingEnd;

      	desc->opts1 = cpu_to_le32(DescOwn | eor | (u16)rx_buf_sz);
}

static inline void rtl8168_mark_as_last_descriptor(struct RxDesc *desc)
{
	desc->opts1 |= cpu_to_le32(RingEnd);
}


static void 
rtl8168_hw_reset(struct  rtl8168_private *tp)
{
	/* Disable interrupts */
	rtl8168_irq_mask_and_ack(tp);

	rtl8168_nic_reset(tp);
}

static void 
rtl8168_irq_mask_and_ack(struct rtl8168_private *tp)
{
	RTL_W16(tp, IntrMask, 0x0000);
}

static void 
rtl8168_nic_reset(struct rtl8168_private *tp)
{
	int i;
#if 0
	if ((tp->mcfg != CFG_METHOD_1) && 
	    (tp->mcfg != CFG_METHOD_2) &&
	    (tp->mcfg != CFG_METHOD_2) &&
	    (tp->mcfg != CFG_METHOD_11)) {
		RTL_W8(tp, ChipCmd, StopReq | CmdRxEnb | CmdTxEnb);
		delay(100);
	}

	if (tp->mcfg == CFG_METHOD_11)
		RTL_W32(tp, RxConfig, RTL_R32(tp, RxConfig) & ~(AcceptErr | AcceptRunt | AcceptBroadcast | AcceptMulticast | AcceptMyPhys |  AcceptAllPhys));

	/* Soft reset the chip. */
	RTL_W8(tp, ChipCmd, CmdReset);

	/* Check that the chip has finished the reset. */
	for (i = 1000; i > 0; i--) {
		if ((RTL_R8(tp, ChipCmd) & CmdReset) == 0)
			break;
		delay(100);
	}

	if (tp->mcfg == CFG_METHOD_11) {
		RTL_W32(tp, OCPDR, 0x01);
		RTL_W32(tp, OCPAR, 0x80001030);
		RTL_W32(tp, OCPAR, 0x0000F034);

		for (i = 1000; i > 0; i--) {
			if ((RTL_R32(tp, OCPAR) & 0xFFFF) == 0)
				break;
			delay(100);
		}
	}
#else
	RTL_W32(tp,RxConfig, RTL_R32(tp,RxConfig) &
		~(AcceptErr | AcceptRunt | AcceptBroadcast | AcceptMulticast |
		  AcceptMyPhys |  AcceptAllPhys));

	if (tp->mcfg == CFG_METHOD_11 || tp->mcfg == CFG_METHOD_12 ||
	    tp->mcfg == CFG_METHOD_13) {
		while (RTL_R8(tp,TxPoll) & NPQ)
			delay(20);
	} else if (tp->mcfg == CFG_METHOD_16 || tp->mcfg == CFG_METHOD_17 ||
		   tp->mcfg == CFG_METHOD_18 || tp->mcfg == CFG_METHOD_19 ||
		   tp->mcfg == CFG_METHOD_20) {
		RTL_W8(tp,ChipCmd, StopReq | CmdRxEnb | CmdTxEnb);
		while (!(RTL_R32(tp,TxConfig) & BIT_11)) delay(100);
	} else if ((tp->mcfg != CFG_METHOD_1) &&
		   (tp->mcfg != CFG_METHOD_2) &&
		   (tp->mcfg != CFG_METHOD_3)) {
		RTL_W8(tp,ChipCmd, StopReq | CmdRxEnb | CmdTxEnb);
		delay(100);
	}

	/* Soft reset the chip. */
	RTL_W8(tp,ChipCmd, CmdReset);

	/* Check that the chip has finished the reset. */
	for (i = 100; i > 0; i--) {
		delay(100);
		if ((RTL_R8(tp,ChipCmd) & CmdReset) == 0)
			break;
	}

	if (tp->mcfg == CFG_METHOD_11)
	{
		OOB_mutex_lock(tp);
		OCP_write(tp, 0x3, 0x10, OCP_read(tp, 0xF, 0x010)&~0x00004000);
		OOB_mutex_unlock(tp);

		OOB_notify(tp, OOB_CMD_RESET);

		for (i=0;i<10;i++)
		{
			mdelay(10);
			if (OCP_read(tp, 0xF, 0x010)&0x00004000)
				break;
		}

		for (i=0;i<5;i++)
		{
			if ( (OCP_read(tp, 0xF, 0x034) & 0xFFFF) == 0)
				break;
		}
	}
#endif
}

static void rtl8168_hw_start(struct rtl8168_private *tp)
{
	u8 device_control, options1, options2;
	u16 ephy_data;
	u32 csi_tmp;
	struct pci_attach_args *pa = tp->pa;

	//rtl8168_nic_reset(tp);
	rtl8168_hw_reset(tp);

	RTL_W8(tp, Cfg9346, Cfg9346_Unlock);

	RTL_W8(tp, Reserved1, Reserved1_data);

	tp->cp_cmd |= PktCntrDisable | INTT_1;
	RTL_W16(tp, CPlusCmd, tp->cp_cmd);

	RTL_W16(tp, IntrMitigate, 0x5151);

	//Work around for RxFIFO overflow
	if (tp->mcfg == CFG_METHOD_1) {
		rtl8168_intr_mask |= RxFIFOOver | PCSTimeout;
		rtl8168_intr_mask &= ~RxDescUnavail;
	}

	RTL_W32(tp, TxDescStartAddrLow, ((u64) tp->TxPhyAddr & DMA_32BIT_MASK));
	RTL_W32(tp, TxDescStartAddrHigh, ((u64) tp->TxPhyAddr >> 32));
	RTL_W32(tp, RxDescAddrLow, ((u64) tp->RxPhyAddr & DMA_32BIT_MASK));
	RTL_W32(tp, RxDescAddrHigh, ((u64) tp->RxPhyAddr >> 32));

	/* Set Rx Config register */
	//rtl8168_set_rx_mode(tp);
	RTL_W32(tp, RxConfig, RTL_R32(tp, RxConfig) | (AcceptErr | AcceptRunt | AcceptBroadcast | AcceptMulticast
				| AcceptMyPhys | AcceptAllPhys));

	/* Set DMA burst size and Interframe Gap Time */
	if (tp->mcfg == CFG_METHOD_1) {
		RTL_W32(tp, TxConfig, (TX_DMA_BURST_512 << TxDMAShift) | 
				  (InterFrameGap << TxInterFrameGapShift));
	} else {
		RTL_W32(tp, TxConfig, (TX_DMA_BURST_unlimited << TxDMAShift) | 
				  (InterFrameGap << TxInterFrameGapShift));
	}

	/* Clear the interrupt status register. */
	RTL_W16(tp, IntrStatus, 0xFFFF);

	if (tp->rx_fifo_overflow == 0) {
		/* Enable all known interrupts by setting the interrupt mask. */
		RTL_W16(tp, IntrMask, rtl8168_intr_mask);
		//netif_start_queue(tp);  xiangy
	}

	if (tp->mcfg == CFG_METHOD_4) {
		/*set PCI configuration space offset 0x70F to 0x27*/
		/*When the register offset of PCI configuration space larger than 0xff, use CSI to access it.*/
		csi_tmp = rtl8168_csi_read(tp, 0x70c) & 0x00ffffff;
		rtl8168_csi_write(tp, 0x70c, csi_tmp | 0x27000000);

		RTL_W8(tp, DBG_reg, (0x0E << 4) | Fix_Nak_1 | Fix_Nak_2);

		/*Set EPHY registers	begin*/
		/*Set EPHY register offset 0x02 bit 11 to 0 and bit 12 to 1*/
		ephy_data = rtl8168_ephy_read(tp, 0x02);
		ephy_data &= ~(1 << 11);
		ephy_data |= (1 << 12);
		rtl8168_ephy_write(tp, 0x02, ephy_data);

		/*Set EPHY register offset 0x03 bit 1 to 1*/
		ephy_data = rtl8168_ephy_read(tp, 0x03);
		ephy_data |= (1 << 1);
		rtl8168_ephy_write(tp, 0x03, ephy_data);

		/*Set EPHY register offset 0x06 bit 7 to 0*/
		ephy_data = rtl8168_ephy_read(tp, 0x06);
		ephy_data &= ~(1 << 7);
		rtl8168_ephy_write(tp, 0x06, ephy_data);
		/*Set EPHY registers	end*/

		RTL_W8(tp, Config3, RTL_R8(tp, Config3) & ~Beacon_en);

		//disable clock request.
		_pci_conf_writen(pa->pa_tag, 0x81, 0x00, 1);

		RTL_W16(tp, CPlusCmd, RTL_R16(tp, CPlusCmd) & 
			~(EnableBist | Macdbgo_oe | Force_halfdup | Force_rxflow_en | Force_txflow_en | 
			  Cxpl_dbg_sel | ASF | PktCntrDisable | Macdbgo_sel));

#if 0
		if (tp->mtu > ETH_DATA_LEN) {
			RTL_W8(tp, Reserved1, Reserved1_data);
			RTL_W8(tp, Config3, RTL_R8(tp, Config3) | Jumbo_En0);
			RTL_W8(tp, Config4, RTL_R8(tp, Config4) | Jumbo_En1);

			//Set PCI configuration space offset 0x79 to 0x20
			/*Increase the Tx performance*/
			device_control = _pci_conf_readn(pa->pa_tag, 0x79, 1);
			device_control &= ~0x70;
			device_control |= 0x20;
			_pci_conf_writen(pa->pa_tag, 0x79, device_control, 1);

			//tx checksum offload disable
			tp->features &= ~NETIF_F_IP_CSUM;

			//rx checksum offload disable
			tp->cp_cmd &= ~RxChkSum;
			RTL_W16(tp, CPlusCmd, tp->cp_cmd);
		} else {
			RTL_W8(tp, Reserved1, Reserved1_data);
			RTL_W8(tp, Config3, RTL_R8(tp, Config3) & ~Jumbo_En0);
			RTL_W8(tp, Config4, RTL_R8(tp, Config4) & ~Jumbo_En1);

			//Set PCI configuration space offset 0x79 to 0x50
			/*Increase the Tx performance*/
			device_control = _pci_conf_readn(pa->pa_tag, 0x79, 1);
			device_control &= ~0x70;
			device_control |= 0x50;
			_pci_conf_writen(pa->pa_tag, 0x79, device_control, 1);

			//tx checksum offload enable
			tp->features |= NETIF_F_IP_CSUM;

			//rx checksum offload enable
			tp->cp_cmd |= RxChkSum;
			RTL_W16(tp, CPlusCmd, tp->cp_cmd);
		}
#endif
	} else if (tp->mcfg == CFG_METHOD_5) {
		/*set PCI configuration space offset 0x70F to 0x27*/
		/*When the register offset of PCI configuration space larger than 0xff, use CSI to access it.*/
		csi_tmp = rtl8168_csi_read(tp, 0x70c) & 0x00ffffff;
		rtl8168_csi_write(tp, 0x70c, csi_tmp | 0x27000000);

		/******set EPHY registers for RTL8168CP	begin******/
		//Set EPHY register offset 0x01 bit 0 to 1.
		ephy_data = rtl8168_ephy_read(tp, 0x01);
		ephy_data |= (1 << 0);
		rtl8168_ephy_write(tp, 0x01, ephy_data);

		//Set EPHY register offset 0x03 bit 10 to 0, bit 9 to 1 and bit 5 to 1.
		ephy_data = rtl8168_ephy_read(tp, 0x03);
		ephy_data &= ~(1 << 10);
		ephy_data |= (1 << 9);
		ephy_data |= (1 << 5);
		rtl8168_ephy_write(tp, 0x03, ephy_data);
		/******set EPHY registers for RTL8168CP	end******/

		RTL_W8(tp, Config3, RTL_R8(tp, Config3) & ~Beacon_en);

		//disable clock request.
		_pci_conf_writen(pa->pa_tag, 0x81, 0x00, 1);

		RTL_W16(tp, CPlusCmd, RTL_R16(tp, CPlusCmd) & 
			~(EnableBist | Macdbgo_oe | Force_halfdup | Force_rxflow_en | Force_txflow_en | 
			  Cxpl_dbg_sel | ASF | PktCntrDisable | Macdbgo_sel));
#if 0
		if (dev->mtu > ETH_DATA_LEN) {
			RTL_W8(tp, Reserved1, Reserved1_data);
			RTL_W8(tp, Config3, RTL_R8(tp, Config3) | Jumbo_En0);
			RTL_W8(tp, Config4, RTL_R8(tp, Config4) | Jumbo_En1);

			//Set PCI configuration space offset 0x79 to 0x20
			/*Increase the Tx performance*/
			device_control = pci_conf_readn(tp, 0x79, 1);
			device_control &= ~0x70;
			device_control |= 0x20;
			_pci_conf_writen(pa->pa_tag, 0x79, device_control, 1);

			//tx checksum offload disable
			dev->features &= ~NETIF_F_IP_CSUM;

			//rx checksum offload disable
			tp->cp_cmd &= ~RxChkSum;
			RTL_W16(tp, CPlusCmd, tp->cp_cmd);
		} else {
			RTL_W8(tp, Reserved1, Reserved1_data);
			RTL_W8(tp, Config3, RTL_R8(tp, Config3) & ~Jumbo_En0);
			RTL_W8(tp, Config4, RTL_R8(tp, Config4) & ~Jumbo_En1);

			//Set PCI configuration space offset 0x79 to 0x50
			/*Increase the Tx performance*/
			device_control = _pci_conf_readn(pa->pa_tag, 0x79, 1);
			device_control &= ~0x70;
			device_control |= 0x50;
			pci_conf_writen(tp, 0x79, device_control, 1);

			//tx checksum offload enable
			dev->features |= NETIF_F_IP_CSUM;

			//rx checksum offload enable
			tp->cp_cmd |= RxChkSum;
			RTL_W16(tp, CPlusCmd, tp->cp_cmd);
		}
#endif
	} else if (tp->mcfg == CFG_METHOD_6) {
		/*set PCI configuration space offset 0x70F to 0x27*/
		/*When the register offset of PCI configuration space larger than 0xff, use CSI to access it.*/
		csi_tmp = rtl8168_csi_read(tp, 0x70c) & 0x00ffffff;
		rtl8168_csi_write(tp, 0x70c, csi_tmp | 0x27000000);

		RTL_W8(tp, Config3, RTL_R8(tp, Config3) & ~Beacon_en);

		RTL_W16(tp, CPlusCmd, RTL_R16(tp, CPlusCmd) & 
			~(EnableBist | Macdbgo_oe | Force_halfdup | Force_rxflow_en | Force_txflow_en | 
			  Cxpl_dbg_sel | ASF | PktCntrDisable | Macdbgo_sel));
#if 0
		if (dev->mtu > ETH_DATA_LEN) {
			RTL_W8(tp, Reserved1, Reserved1_data);
			RTL_W8(tp, Config3, RTL_R8(tp, Config3) | Jumbo_En0);
			RTL_W8(tp, Config4, RTL_R8(tp, Config4) | Jumbo_En1);

			//Set PCI configuration space offset 0x79 to 0x20
			/*Increase the Tx performance*/
			device_control = _pci_conf_read(tp, 0x79, 1);
			device_control &= ~0x70;
			device_control |= 0x20;
			_pci_conf_writen(pa->pa_tag, 0x79, device_control, 1);

			//tx checksum offload disable
			dev->features &= ~NETIF_F_IP_CSUM;

			//rx checksum offload disable
			tp->cp_cmd &= ~RxChkSum;
			RTL_W16(tp, CPlusCmd, tp->cp_cmd);
		} else {
			RTL_W8(tp, Reserved1, Reserved1_data);
			RTL_W8(tp, Config3, RTL_R8(tp, Config3) & ~Jumbo_En0);
			RTL_W8(tp, Config4, RTL_R8(tp, Config4) & ~Jumbo_En1);

			//Set PCI configuration space offset 0x79 to 0x50
			/*Increase the Tx performance*/
			_pci_conf_readn(pa->pa_tag, 0x79, &device_control);
			device_control &= ~0x70;
			device_control |= 0x50;
			_pci_conf_writen(pa->pa_tag, 0x79, device_control);

			//tx checksum offload enable
			dev->features |= NETIF_F_IP_CSUM;

			//rx checksum offload enable
			tp->cp_cmd |= RxChkSum;
			RTL_W16(tp, CPlusCmd, tp->cp_cmd);
		}
#endif
	} else if (tp->mcfg == CFG_METHOD_7) {
		/*set PCI configuration space offset 0x70F to 0x27*/
		/*When the register offset of PCI configuration space larger than 0xff, use CSI to access it.*/
		csi_tmp = rtl8168_csi_read(tp, 0x70c) & 0x00ffffff;
		rtl8168_csi_write(tp, 0x70c, csi_tmp | 0x27000000);
		rtl8168_eri_write(tp, 0x1EC, 1, 0x07, ERIAR_ASF);

		RTL_W16(tp, CPlusCmd, RTL_R16(tp, CPlusCmd) & 
			~(EnableBist | Macdbgo_oe | Force_halfdup | Force_rxflow_en | Force_txflow_en | 
			  Cxpl_dbg_sel | ASF | PktCntrDisable | Macdbgo_sel));

		RTL_W8(tp, Config3, RTL_R8(tp, Config3) & ~Beacon_en);
#if 0
		if (dev->mtu > ETH_DATA_LEN) {
			RTL_W8(tp, Reserved1, Reserved1_data);
			RTL_W8(tp, Config3, RTL_R8(tp, Config3) | Jumbo_En0);
			RTL_W8(tp, Config4, RTL_R8(tp, Config4) | Jumbo_En1);

			//Set PCI configuration space offset 0x79 to 0x20
			/*Increase the Tx performance*/
			_pci_conf_readn(pa->pa_tag, 0x79, &device_control);
			device_control &= ~0x70;
			device_control |= 0x20;
			_pci_conf_writen(pa->pa_tag, 0x79, device_control);

			//tx checksum offload disable
			dev->features &= ~NETIF_F_IP_CSUM;

			//rx checksum offload disable
			tp->cp_cmd &= ~RxChkSum;
			RTL_W16(tp, CPlusCmd, tp->cp_cmd);
		} else {
			RTL_W8(tp, Reserved1, Reserved1_data);
			RTL_W8(tp, Config3, RTL_R8(tp, Config3) & ~Jumbo_En0);
			RTL_W8(tp, Config4, RTL_R8(tp, Config4) & ~Jumbo_En1);

			//Set PCI configuration space offset 0x79 to 0x50
			/*Increase the Tx performance*/
			_pci_conf_readn(pa->pa_tag, 0x79, &device_control);
			device_control &= ~0x70;
			device_control |= 0x50;
			_pci_conf_writen(pa->pa_tag, 0x79, device_control);

			//tx checksum offload enable
			dev->features |= NETIF_F_IP_CSUM;

			//rx checksum offload enable
			tp->cp_cmd |= RxChkSum;
			RTL_W16(tp, CPlusCmd, tp->cp_cmd);
		}
#endif
	} else if (tp->mcfg == CFG_METHOD_8) {
		/*set PCI configuration space offset 0x70F to 0x27*/
		/*When the register offset of PCI configuration space larger than 0xff, use CSI to access it.*/
		csi_tmp = rtl8168_csi_read(tp, 0x70c) & 0x00ffffff;
		rtl8168_csi_write(tp, 0x70c, csi_tmp | 0x27000000);
		rtl8168_eri_write(tp, 0x1EC, 1, 0x07, ERIAR_ASF);

		RTL_W16(tp, CPlusCmd, RTL_R16(tp, CPlusCmd) & 
			~(EnableBist | Macdbgo_oe | Force_halfdup | Force_rxflow_en | Force_txflow_en | 
			  Cxpl_dbg_sel | ASF | PktCntrDisable | Macdbgo_sel));

		RTL_W8(tp, Config3, RTL_R8(tp, Config3) & ~Beacon_en);

		RTL_W8(tp, 0xD1, 0x20);
#if 0
		if (dev->mtu > ETH_DATA_LEN) {
			RTL_W8(tp, Reserved1, Reserved1_data);
			RTL_W8(tp, Config3, RTL_R8(tp, Config3) | Jumbo_En0);
			RTL_W8(tp, Config4, RTL_R8(tp, Config4) | Jumbo_En1);

			//Set PCI configuration space offset 0x79 to 0x20
			/*Increase the Tx performance*/
			_pci_conf_readn(pa->pa_tag, 0x79, &device_control);
			device_control &= ~0x70;
			device_control |= 0x20;
			_pci_conf_writen(pa->pa_tag, 0x79, device_control);

			//tx checksum offload disable
			dev->features &= ~NETIF_F_IP_CSUM;

			//rx checksum offload disable
			tp->cp_cmd &= ~RxChkSum;
			RTL_W16(tp, CPlusCmd, tp->cp_cmd);
		} else {
			RTL_W8(tp, Reserved1, Reserved1_data);
			RTL_W8(tp, Config3, RTL_R8(tp, Config3) & ~Jumbo_En0);
			RTL_W8(tp, Config4, RTL_R8(tp, Config4) & ~Jumbo_En1);

			//Set PCI configuration space offset 0x79 to 0x50
			/*Increase the Tx performance*/
			_pci_conf_readn(pa->pa_tag, 0x79, &device_control);
			device_control &= ~0x70;
			device_control |= 0x50;
			_pci_conf_writen(pa->pa_tag, 0x79, device_control);

			//tx checksum offload enable
			dev->features |= NETIF_F_IP_CSUM;

			//rx checksum offload enable
			tp->cp_cmd |= RxChkSum;
			RTL_W16(tp, CPlusCmd, tp->cp_cmd);
		}
#endif
	} else if (tp->mcfg == CFG_METHOD_9) {
		/*set PCI configuration space offset 0x70F to 0x13*/
		/*When the register offset of PCI configuration space larger than 0xff, use CSI to access it.*/
		csi_tmp = rtl8168_csi_read(tp, 0x70c) & 0x00ffffff;
		rtl8168_csi_write(tp, 0x70c, csi_tmp | 0x13000000);

		/* disable clock request. */
		_pci_conf_writen(pa->pa_tag, 0x81, 0x00, 1);

		RTL_W8(tp, Config3, RTL_R8(tp, Config3) & ~BIT_4);
		RTL_W8(tp, DBG_reg, RTL_R8(tp, DBG_reg) | BIT_7 | BIT_1);
#if 0
		if (dev->mtu > ETH_DATA_LEN) {
			RTL_W8(tp, Reserved1, Reserved1_data);
			RTL_W8(tp, Config3, RTL_R8(tp, Config3) | Jumbo_En0);
			RTL_W8(tp, Config4, RTL_R8(tp, Config4) | Jumbo_En1);

			/* Set PCI configuration space offset 0x79 to 0x20 */
			_pci_conf_readn(pa->pa_tag, 0x79, &device_control);
			device_control &= ~0x70;
			device_control |= 0x20;
			_pci_conf_writen(pa->pa_tag, 0x79, device_control);

			/* tx checksum offload disable */
			dev->features &= ~NETIF_F_IP_CSUM;

			/* rx checksum offload disable */
			tp->cp_cmd &= ~RxChkSum;
			RTL_W16(tp, CPlusCmd, tp->cp_cmd);
		} else {
			RTL_W8(tp, Reserved1, Reserved1_data);
			RTL_W8(tp, Config3, RTL_R8(tp, Config3) & ~Jumbo_En0);
			RTL_W8(tp, Config4, RTL_R8(tp, Config4) & ~Jumbo_En1);

			/* Set PCI configuration space offset 0x79 to 0x50 */
			_pci_conf_readn(pa->pa_tag, 0x79, &device_control);
			device_control &= ~0x70;
			device_control |= 0x50;
			_pci_conf_writen(pa->pa_tag, 0x79, device_control);

			/* tx checksum offload enable */
			dev->features |= NETIF_F_IP_CSUM;

			/* rx checksum offload enable */
			tp->cp_cmd |= RxChkSum;
			RTL_W16(tp, CPlusCmd, tp->cp_cmd);
		}
#endif
		/* set EPHY registers */
		rtl8168_ephy_write(tp, 0x01, 0x7C7D);
		rtl8168_ephy_write(tp, 0x02, 0x091F);
		rtl8168_ephy_write(tp, 0x06, 0xB271);
		rtl8168_ephy_write(tp, 0x07, 0xCE00);
	} else if (tp->mcfg == CFG_METHOD_10) {
		/*set PCI configuration space offset 0x70F to 0x13*/
		/*When the register offset of PCI configuration space larger than 0xff, use CSI to access it.*/
		csi_tmp = rtl8168_csi_read(tp, 0x70c) & 0x00ffffff;
		rtl8168_csi_write(tp, 0x70c, csi_tmp | 0x13000000);

		RTL_W8(tp, DBG_reg, RTL_R8(tp, DBG_reg) | BIT_7 | BIT_1);
#if 0
		if (dev->mtu > ETH_DATA_LEN) {
			RTL_W8(tp, Reserved1, Reserved1_data);
			RTL_W8(tp, Config3, RTL_R8(tp, Config3) | Jumbo_En0);
			RTL_W8(tp, Config4, RTL_R8(tp, Config4) | Jumbo_En1);

			/* Set PCI configuration space offset 0x79 to 0x20 */
			_pci_conf_readn(pa->pa_tag, 0x79, &device_control);
			device_control &= ~0x70;
			device_control |= 0x20;
			_pci_conf_writen(pa->pa_tag, 0x79, device_control);

			/* tx checksum offload disable */
			dev->features &= ~NETIF_F_IP_CSUM;

			/* rx checksum offload disable */
			tp->cp_cmd &= ~RxChkSum;
			RTL_W16(tp, CPlusCmd, tp->cp_cmd);
		} else {
			RTL_W8(tp, Reserved1, Reserved1_data);
			RTL_W8(tp, Config3, RTL_R8(tp, Config3) & ~Jumbo_En0);
			RTL_W8(tp, Config4, RTL_R8(tp, Config4) & ~Jumbo_En1);

			/* Set PCI configuration space offset 0x79 to 0x50 */
			_pci_conf_readn(pa->pa_tag, 0x79, &device_control);
			device_control &= ~0x70;
			device_control |= 0x50;
			_pci_conf_writen(pa->pa_tag, 0x79, device_control);

			/* tx checksum offload enable */
			dev->features |= NETIF_F_IP_CSUM;

			/* rx checksum offload enable */
			tp->cp_cmd |= RxChkSum;
			RTL_W16(tp, CPlusCmd, tp->cp_cmd);
		}
#endif
		//RTL_W8(tp, Config1, 0xDF);

		/* set EPHY registers */
		rtl8168_ephy_write(tp, 0x01, 0x6C7F);
		rtl8168_ephy_write(tp, 0x02, 0x011F);
		rtl8168_ephy_write(tp, 0x03, 0xC1B2);
		rtl8168_ephy_write(tp, 0x1A, 0x0546);
		rtl8168_ephy_write(tp, 0x1C, 0x80C4);
		rtl8168_ephy_write(tp, 0x1D, 0x78E4);
		rtl8168_ephy_write(tp, 0x0A, 0x8100);

		/* disable clock request. */
		_pci_conf_writen(pa->pa_tag, 0x81, 0x00, 1);

		RTL_W8(tp, 0xF3, RTL_R8(tp, 0xF3) | (1 << 2));

	} else if (tp->mcfg == CFG_METHOD_11) {
		/*set PCI configuration space offset 0x70F to 0x37*/
		/*When the register offset of PCI configuration space larger than 0xff, use CSI to access it.*/
		csi_tmp = rtl8168_csi_read(tp, 0x70c) & 0x00ffffff;
		rtl8168_csi_write(tp, 0x70c, csi_tmp | 0x37000000);

		/* Set PCI configuration space offset 0x79 to 0x50 */
		device_control = _pci_conf_read8(pa->pa_tag, 0x79);
		device_control &= ~0x70;
		device_control |= 0x50;
		_pci_conf_writen(pa->pa_tag, 0x79, device_control, 1);
#if 0
		if (dev->mtu > ETH_DATA_LEN) {
			RTL_W8(tp, Reserved1, Reserved1_data);
			RTL_W8(tp, Config3, RTL_R8(tp, Config3) | Jumbo_En0);

			/* tx checksum offload disable */
			dev->features &= ~NETIF_F_IP_CSUM;

			/* rx checksum offload disable */
			tp->cp_cmd &= ~RxChkSum;
			RTL_W16(tp, CPlusCmd, tp->cp_cmd);
		} else {
			RTL_W8(tp, Reserved1, Reserved1_data);
			RTL_W8(tp, Config3, RTL_R8(tp, Config3) & ~Jumbo_En0);

			/* tx checksum offload enable */
			dev->features |= NETIF_F_IP_CSUM;

			/* rx checksum offload enable */
			tp->cp_cmd |= RxChkSum;
			RTL_W16(tp, CPlusCmd, tp->cp_cmd);
		}
#endif
		//RTL_W8(tp, Config1, 0xDF);

	} else if (tp->mcfg == CFG_METHOD_1) {
		RTL_W8(tp, Config3, RTL_R8(tp, Config3) & ~Beacon_en);

		RTL_W16(tp, CPlusCmd, RTL_R16(tp, CPlusCmd) & 
			~(EnableBist | Macdbgo_oe | Force_halfdup | Force_rxflow_en | Force_txflow_en | 
			  Cxpl_dbg_sel | ASF | PktCntrDisable | Macdbgo_sel));
#if 0
		if (dev->mtu > ETH_DATA_LEN) {
			_pci_conf_readn(pa->pa_tag, 0x69, &device_control);
			device_control &= ~0x70;
			device_control |= 0x28;
			_pci_conf_writen(pa->pa_tag, 0x69, device_control);
		} else {
			_pci_conf_readn(pa->pa_tag, 0x69, &device_control);
			device_control &= ~0x70;
			device_control |= 0x58;
			_pci_conf_writen(pa->pa_tag, 0x69, device_control);
		}
#endif
	} else if (tp->mcfg == CFG_METHOD_2) {
		RTL_W8(tp, Config3, RTL_R8(tp, Config3) & ~Beacon_en);

		RTL_W16(tp, CPlusCmd, RTL_R16(tp, CPlusCmd) & 
			~(EnableBist | Macdbgo_oe | Force_halfdup | Force_rxflow_en | Force_txflow_en | 
			  Cxpl_dbg_sel | ASF | PktCntrDisable | Macdbgo_sel));
#if 0
		if (dev->mtu > ETH_DATA_LEN) {
			_pci_conf_readn(pa->pa_tag, 0x69, &device_control);
			device_control &= ~0x70;
			device_control |= 0x28;
			_pci_conf_writen(pa->pa_tag, 0x69, device_control);

			RTL_W8(tp, Reserved1, Reserved1_data);
			RTL_W8(tp, Config4, RTL_R8(tp, Config4) | (1 << 0));
		} else {
			_pci_conf_readn(pa->pa_tag, 0x69, &device_control);
			device_control &= ~0x70;
			device_control |= 0x58;
			_pci_conf_writen(pa->pa_tag, 0x69, device_control);

			RTL_W8(tp, Reserved1, Reserved1_data);
			RTL_W8(tp, Config4, RTL_R8(tp, Config4) & ~(1 << 0));
		}
#endif
	} else if (tp->mcfg == CFG_METHOD_3) {
		RTL_W8(tp, Config3, RTL_R8(tp, Config3) & ~Beacon_en);

		RTL_W16(tp, CPlusCmd, RTL_R16(tp, CPlusCmd) & 
			~(EnableBist | Macdbgo_oe | Force_halfdup | Force_rxflow_en | Force_txflow_en | 
			  Cxpl_dbg_sel | ASF | PktCntrDisable | Macdbgo_sel));
#if 0
		if (dev->mtu > ETH_DATA_LEN) {
			_pci_conf_readn(pa->pa_tag, 0x69, &device_control);
			device_control &= ~0x70;
			device_control |= 0x28;
			_pci_conf_writen(pa->pa_tag, 0x69, device_control);

			RTL_W8(tp, Reserved1, Reserved1_data);
			RTL_W8(tp, Config4, RTL_R8(tp, Config4) | (1 << 0));
		} else {
			_pci_conf_readn(pa->pa_tag, 0x69, &device_control);
			device_control &= ~0x70;
			device_control |= 0x58;
			_pci_conf_writen(pa->pa_tag, 0x69, device_control);

			RTL_W8(tp, Reserved1, Reserved1_data);
			RTL_W8(tp, Config4, RTL_R8(tp, Config4) & ~(1 << 0));
		}
#endif
	}
	if ((tp->mcfg == CFG_METHOD_1) || (tp->mcfg == CFG_METHOD_2) || (tp->mcfg == CFG_METHOD_3)) {
		/* csum offload command for RTL8168B/8111B */
		tp->tx_tcp_csum_cmd = TxIPCS | TxTCPCS;
		tp->tx_udp_csum_cmd = TxIPCS | TxUDPCS;
		tp->tx_ip_csum_cmd = TxIPCS;
	} else {
		/* csum offload command for RTL8168C/8111C and RTL8168CP/8111CP */
		tp->tx_tcp_csum_cmd = TxIPCS_C | TxTCPCS_C;
		tp->tx_udp_csum_cmd = TxIPCS_C | TxUDPCS_C;
		tp->tx_ip_csum_cmd = TxIPCS_C;
	}

	RTL_W8(tp, ChipCmd, CmdTxEnb | CmdRxEnb);

	RTL_W8(tp, Cfg9346, Cfg9346_Lock);
#define PCI_CACHE_LINE_SIZE 0x0c    /* 8 bits */
#define PCI_BASE_ADDRESS_0  0x10    /* 32 bits */
#define PCI_BASE_ADDRESS_1  0x14    /* 32 bits [htype 0,1 only] */
#define PCI_BASE_ADDRESS_2  0x18    /* 32 bits [htype 0 only] */
#define PCI_BASE_ADDRESS_3  0x1c    /* 32 bits */
#define PCI_BASE_ADDRESS_4  0x20    /* 32 bits */
#define PCI_BASE_ADDRESS_5  0x24    /* 32 bits */
#define PCI_INTERRUPT_LINE  0x3c    /* 8 bits */

#if 1
	if (!tp->pci_cfg_is_read) {
		tp->pci_cfg_space.cmd = _pci_conf_read8(pa->pa_tag, PCI_COMMAND_STATUS_REG);
		tp->pci_cfg_space.cls = _pci_conf_read8(pa->pa_tag, PCI_CACHE_LINE_SIZE);
		tp->pci_cfg_space.io_base_l = _pci_conf_read16(pa->pa_tag,PCI_BASE_ADDRESS_0);
		tp->pci_cfg_space.io_base_h = _pci_conf_read16(pa->pa_tag, PCI_BASE_ADDRESS_0 + 2);
		tp->pci_cfg_space.mem_base_l = _pci_conf_read16(pa->pa_tag, PCI_BASE_ADDRESS_2);
		tp->pci_cfg_space.mem_base_h = _pci_conf_read16(pa->pa_tag, PCI_BASE_ADDRESS_2 + 2);
		tp->pci_cfg_space.ilr = _pci_conf_read8(pa->pa_tag, PCI_INTERRUPT_LINE);
		tp->pci_cfg_space.resv_0x20_l = _pci_conf_read16(pa->pa_tag, PCI_BASE_ADDRESS_4);
		tp->pci_cfg_space.resv_0x20_h = _pci_conf_read16(pa->pa_tag, PCI_BASE_ADDRESS_4 + 2);
		tp->pci_cfg_space.resv_0x24_l = _pci_conf_read16(pa->pa_tag, PCI_BASE_ADDRESS_5);
		tp->pci_cfg_space.resv_0x24_h = _pci_conf_read16(pa->pa_tag, PCI_BASE_ADDRESS_5 + 2);

		tp->pci_cfg_is_read = 1;
	}
#endif

	rtl8168_dsm(tp, DSM_MAC_INIT);

	options1 = RTL_R8(tp, Config3);
	options2 = RTL_R8(tp, Config5);
	if ((options1 & LinkUp) || (options1 & MagicPacket) || (options2 & UWF) || (options2 & BWF) || (options2 & MWF))
		tp->wol_enabled = WOL_ENABLED;
	else
		tp->wol_enabled = WOL_DISABLED;
	delay(100);
}


static void rtl8168_set_rx_mode(struct rtl8168_private *tp)
{
	u32 mc_filter[2];	/* Multicast hash filter */
	int rx_mode;
	u32 tmp = 0;

	if (tp->flags & IFF_PROMISC) {
		/* Unconditionally log net taps. */
		printf("Promiscuous mode enabled.\n");
		rx_mode =
		    AcceptBroadcast | AcceptMulticast | AcceptMyPhys |
		    AcceptAllPhys;
		mc_filter[1] = mc_filter[0] = 0xffffffff;
	} else if ((tp->mc_count > multicast_filter_limit)
		   || (tp->flags & IFF_ALLMULTI)) {
		/* Too many to filter perfectly -- accept all multicasts. */
		rx_mode = AcceptBroadcast | AcceptMulticast | AcceptMyPhys;
		mc_filter[1] = mc_filter[0] = 0xffffffff;
	} else {
		//struct dev_mc_list *mclist;
		rx_mode = AcceptBroadcast | AcceptMyPhys;
		mc_filter[1] = mc_filter[0] = 0;
	}

	tmp = tp->rtl8168_rx_config | rx_mode | (RTL_R32(tp, RxConfig) & rtl_chip_info[tp->chipset].RxConfigMask);

	RTL_W32(tp, RxConfig, tmp);

	RTL_W32(tp, MAR0 + 0, mc_filter[0]);
	RTL_W32(tp, MAR0 + 4, mc_filter[1]);

}

static void rtl8168_check_link_status(struct rtl8168_private *tp)
{
    int i; 
    int old_value;

	if (tp->link_ok(tp)) {
		printf("r8110: link up :  ");
        printf("PHY status:0x%x\n",RTL_R8(tp,0x6c));
        if(RTL_R8(tp,0x6c) == if_frequency){
            return;
        }
	} else {
		printf("r8110: link down\n");
        //printf("PHY status:0x%x\n",RTL_R8(tp,0x6c));
	}
#if 0
    //by liuqi we try to get the 8110 up
    if (if_in_attach != 1){
 	   for (i = 0; i<10; i++){
    	    printf("Trying the %d time--->\n", i);

	    	old_value = mdio_read(tp, PHY_CTRL_REG);
	        old_value |= 0x0200;       //set to re auto negotiation
    	    mdio_write(tp, PHY_CTRL_REG, old_value);
        	delay(10000000);
	        if(RTL_R8(tp,0x6c) == if_frequency){
    	        printf("Connect done!!\n");
        	    break; 
        	}
    	}
    }
#endif

    rtl8168dp_10mbps_gphy_para(tp);

}


static void rtl8168dp_10mbps_gphy_para(struct rtl8168_private *tp)
{
    u8 status = RTL_R8(tp, PHYstatus);

    if (tp->mcfg != CFG_METHOD_11)
        return;

    if ((status & LinkStatus) && (status & _10bps)) {
        mdio_write(tp, 0x1f, 0x0000);
        mdio_write(tp, 0x10, 0x04EE);
    } else {
        mdio_write(tp, 0x1f, 0x0000);
        mdio_write(tp, 0x10, 0x01EE);
    }
}


static void rtl8168_ephy_write(struct rtl8168_private *tp, int RegAddr, int value)
{
	int i;

	RTL_W32(tp, EPHYAR, 
		EPHYAR_Write | 
		(RegAddr & EPHYAR_Reg_Mask) << EPHYAR_Reg_shift | 
		(value & EPHYAR_Data_Mask));

	for (i = 0; i < 10; i++) {
		udelay(100);

		/* Check if the RTL8168 has completed EPHY write */
		if (!(RTL_R32(tp, EPHYAR) & EPHYAR_Flag)) 
			break;
	}

	udelay(20);
}

static u16
rtl8168_ephy_read(struct rtl8168_private * tp, int RegAddr)
{
	int i;
	u16 value = 0xffff;

	RTL_W32(tp, EPHYAR, 
		EPHYAR_Read | (RegAddr & EPHYAR_Reg_Mask) << EPHYAR_Reg_shift);

	for (i = 0; i < 10; i++) {
		udelay(100);

		/* Check if the RTL8168 has completed EPHY read */
		if (RTL_R32(tp, EPHYAR) & EPHYAR_Flag) {
			value = (u16) (RTL_R32(tp, EPHYAR) & EPHYAR_Data_Mask);
			break;
		}
	}

	udelay(100);

	return value;
}

static void
rtl8168_csi_write(struct rtl8168_private * tp, 
		   int addr,
		   int value)
{
	int i;

	RTL_W32(tp, CSIDR, value);
	RTL_W32(tp, CSIAR, 
		CSIAR_Write |
		CSIAR_ByteEn << CSIAR_ByteEn_shift |
		(addr & CSIAR_Addr_Mask));

	for (i = 0; i < 10; i++) {
		udelay(100);

		/* Check if the RTL8168 has completed CSI write */
		if (!(RTL_R32(tp, CSIAR) & CSIAR_Flag)) 
			break;
	}

	udelay(20);
}

int 
rtl8168_eri_read(struct rtl8168_private * tp, 
		 int addr,
		 int len,
		 int type)
{
	int i, val_shift, shift = 0;
	u32 value1 = 0, value2 = 0, mask;

	if (len > 4 || len <= 0)
		return -1;

	while (len > 0) {
		val_shift = addr % ERIAR_Addr_Align;
		addr = addr & ~0x3;

		RTL_W32(tp, ERIAR, 
			ERIAR_Read |
			type << ERIAR_Type_shift |
			ERIAR_ByteEn << ERIAR_ByteEn_shift |
			addr);

		for (i = 0; i < 10; i++) {
			udelay(100);

			/* Check if the RTL8168 has completed ERI read */
			if (RTL_R32(tp, ERIAR) & ERIAR_Flag) 
				break;
		}

		if (len == 1)		mask = (0xFF << (val_shift * 8)) & 0xFFFFFFFF;
		else if (len == 2)	mask = (0xFFFF << (val_shift * 8)) & 0xFFFFFFFF;
		else if (len == 3)	mask = (0xFFFFFF << (val_shift * 8)) & 0xFFFFFFFF;
		else			mask = (0xFFFFFFFF << (val_shift * 8)) & 0xFFFFFFFF;

		value1 = RTL_R32(tp, ERIDR) & mask;
		value2 |= (value1 >> val_shift * 8) << shift * 8;

		if (len <= 4 - val_shift)
			len = 0;
		else {
			len -= (4 - val_shift);
			shift = 4 - val_shift;
			addr += 4;
		}
	}

	return value2;
}

int
rtl8168_eri_write(struct rtl8168_private * tp, 
		  int addr,
		  int len,
		  int value,
		  int type)
{

	int i, val_shift, shift = 0;
	u32 value1 = 0, mask;

	if (len > 4 || len <= 0)
		return -1;

	while(len > 0) {
		val_shift = addr % ERIAR_Addr_Align;
		addr = addr & ~0x3;

		if (len == 1)		mask = (0xFF << (val_shift * 8)) & 0xFFFFFFFF;
		else if (len == 2)	mask = (0xFFFF << (val_shift * 8)) & 0xFFFFFFFF;
		else if (len == 3)	mask = (0xFFFFFF << (val_shift * 8)) & 0xFFFFFFFF;
		else			mask = (0xFFFFFFFF << (val_shift * 8)) & 0xFFFFFFFF;

		value1 = rtl8168_eri_read(tp, addr, 4, type) & ~mask;
		value1 |= (((value << val_shift * 8) >> shift * 8));

		RTL_W32(tp, ERIDR, value1);
		RTL_W32(tp, ERIAR, 
			ERIAR_Write |
			type << ERIAR_Type_shift |
			ERIAR_ByteEn << ERIAR_ByteEn_shift |
			addr);

		for (i = 0; i < 10; i++) {
			udelay(100);

			/* Check if the RTL8168 has completed ERI write */
			if (!(RTL_R32(tp, ERIAR) & ERIAR_Flag)) 
				break;
		}

		if (len <= 4 - val_shift)
			len = 0;
		else {
			len -= (4 - val_shift);
			shift = 4 - val_shift;
			addr += 4;
		}
	}

	return 0;
}

static int rtl8168_csi_read(struct rtl8168_private * tp,int addr)
{
	int i, value = -1;

	RTL_W32(tp, CSIAR, 
		CSIAR_Read | 
		CSIAR_ByteEn << CSIAR_ByteEn_shift |
		(addr & CSIAR_Addr_Mask));

	for (i = 0; i < 10; i++) {
		udelay(100);

		/* Check if the RTL8168 has completed CSI read */
		if (RTL_R32(tp, CSIAR) & CSIAR_Flag) {
			value = (int)RTL_R32(tp, CSIDR);
			break;
		}
	}

	udelay(20);

	return value;
}

static void
rtl8168_dsm(struct rtl8168_private *tp, int dev_state)
{
	switch (dev_state) {
	case DSM_MAC_INIT:
		if ((tp->mcfg == CFG_METHOD_5) || (tp->mcfg == CFG_METHOD_6)) {
			if (RTL_R8(tp, MACDBG) & 0x80) {
				RTL_W8(tp, GPIO, RTL_R8(tp, GPIO) | GPIO_en);
			} else {
				RTL_W8(tp, GPIO, RTL_R8(tp, GPIO) & ~GPIO_en);
			}
		}

		break;
	case DSM_NIC_GOTO_D3:
	case DSM_IF_DOWN:
		if ((tp->mcfg == CFG_METHOD_5) || (tp->mcfg == CFG_METHOD_6))
			if (RTL_R8(tp, MACDBG) & 0x80)
				RTL_W8(tp, GPIO, RTL_R8(tp, GPIO) & ~GPIO_en);

		break;
	case DSM_NIC_RESUME_D3:
	case DSM_IF_UP:
		if ((tp->mcfg == CFG_METHOD_5) || (tp->mcfg == CFG_METHOD_6))
			if (RTL_R8(tp, MACDBG) & 0x80)
				RTL_W8(tp, GPIO, RTL_R8(tp, GPIO) | GPIO_en);

		break;
	}
}

static int rtl8168_interrupt(void *arg)
{
	struct rtl8168_private *tp = (struct rtl8168_private *)arg; 
	int boguscnt = max_interrupt_work;
	u32 status = 0;
	int handled = 0;
    u16 intr_clean_mask = SYSErr | PCSTimeout | SWInt | LinkChg | RxDescUnavail	| TxErr | TxOK | RxErr | RxOK;

	/* FIXME: We should not get an error interrupt here. */
	if (RTL_R16(tp, IntrStatus) == 0x80)
		return 0;

	do {
		status = RTL_R16(tp, IntrStatus);

		/* hotplug/major error/no more work/shared irq */
		if ((status == 0xFFFF) || !status){
			break;
		}

		handled = 1;

		status &= tp->intr_mask;
		RTL_W16(tp, IntrStatus,	intr_clean_mask);

		if (!(status & rtl8168_intr_mask)){
			break;
		}

		if ((status & SYSErr)) { 
			rtl8168_pcierr_interrupt(tp);
			break;
		}

		/* Rx interrupt */
		if (status & (RxOK | RxDescUnavail | RxFIFOOver)) {
			rtl8168_rx_interrupt(tp);
		}

		/* Tx interrupt */		
		if (status & (TxOK | TxErr)){
			rtl8168_tx_interrupt(tp);
		}

		boguscnt--;
	} while (boguscnt > 0);

	if (boguscnt <= 0) {
		/* Clear all interrupt sources. */
		RTL_W16(tp, IntrStatus, 0xffff);
	}
	return (handled);
}

static void rtl8168_pcierr_interrupt(struct rtl8168_private *tp)
{
	u16 pci_status, pci_cmd;
	u32 tmp;
	struct pci_attach_args *pa = tp->pa;

	tmp = _pci_conf_readn(pa->pa_tag, PCI_COMMAND_STATUS_REG,4);
	pci_cmd = tmp & 0xffff; 
	pci_status = tmp & 0xffff0000;

	/*
	 * The recovery sequence below admits a very elaborated explanation:
	 * - it seems to work;
	 * - I did not see what else could be done.
	 *
	 * Feel free to adjust to your needs.
	 */
	_pci_conf_writen(pa->pa_tag, PCI_COMMAND_STATUS_REG,
			      pci_cmd | PCI_COMMAND_SERR_ENABLE | PCI_COMMAND_PARITY_ENABLE, 2);

	pci_status &= (PCI_STATUS_PARITY_ERROR |
		PCI_STATUS_SPECIAL_ERROR | PCI_STATUS_MASTER_ABORT |
		PCI_STATUS_TARGET_TARGET_ABORT| PCI_STATUS_MASTER_TARGET_ABORT);
	pci_status >>= 16;

	_pci_conf_writen(pa->pa_tag, PCI_COMMAND_STATUS_REG + 2, pci_status, 2);

	rtl8168_hw_reset(tp);
}

static int rtl8168_rx_interrupt(struct rtl8168_private *tp)
{
	unsigned int cur_rx, rx_left;
	unsigned int delta, count;
	struct ether_header *eh;
	struct  ifnet *ifp = &tp->arpcom.ac_if;
	u32 status = 0;
	struct RxDesc *desc;

	cur_rx = tp->cur_rx;
	rx_left = NUM_RX_DESC + tp->dirty_rx - cur_rx;     

	for (; rx_left > 0; rx_left--, cur_rx++) {
		unsigned int entry = cur_rx % NUM_RX_DESC;
		desc = &(tp->RxDescArray[entry]);
		status = 0;

		status = le32_to_cpu(desc->opts1);

		if (status & DescOwn)
			break;

  		if ((status & RxRES)) {
			printf( "Rx ERROR. status = %08x\n", status);
			tp->stats.rx_errors++;
			if (status & (RxRWT | RxRUNT))
				tp->stats.rx_length_errors++;
			if (status & RxCRC)
				tp->stats.rx_crc_errors++;
			rtl8168_mark_to_asic(desc, tp->rx_buf_sz, 1);
		} else {
			struct mbuf *m;
			int pkt_size = (status & 0x00001FFF) - 4;

			/*
			 * The driver does not support incoming fragmented
			 * frames. They are seen as a symptom of over-mtu
			 * sized frames.
			 */
			if ((status & (FirstFrag|LastFrag)) != (FirstFrag|LastFrag)) {
    				tp->stats.rx_dropped++;
				tp->stats.rx_length_errors++;
				rtl8168_mark_to_asic(desc, tp->rx_buf_sz, 1);
				continue;
			}

			//rtl8168_rx_csum(tp, desc); //FIXME

			m = getmbuf(tp);
			if(m == NULL) {
				printf("rx_interrupt getmbuf failed\n");
				return -1;
			}

			bcopy(tp->rx_buffer[entry], mtod(m, caddr_t), pkt_size);

			m->m_pkthdr.rcvif = ifp;
			m->m_pkthdr.len = m->m_len = pkt_size - sizeof(struct ether_header);

			eh = mtod(m, struct ether_header *);

			m->m_data += sizeof(struct ether_header);

			ether_input(ifp, eh, m);

			tp->stats.rx_bytes += pkt_size;
			tp->stats.rx_packets++;

			delta = rtl8168_rx_fill(tp,  entry, entry + 1, 1);//should this work be done here ? lihui.
			tp->dirty_rx += delta; //FIXME
		}
	}

	count = cur_rx - tp->cur_rx;
	tp->cur_rx = cur_rx;

	/*
	 * FIXME: until there is periodic timer to try and refill the ring,
	 * a temporary shortage may definitely kill the Rx process.
	 * - disable the asic to try and avoid an overflow and kick it again
	 *   after refill ?
	 * - how do others driver handle this condition (Uh oh...).
	 */
	if ((tp->dirty_rx + NUM_RX_DESC) == tp->cur_rx)
		printf("Rx buffers exhausted\n");

	return count;
}

#define CHECKSUM_UNNECESSARY 0x01
#define CHECKSUM_NONE        0x02

static int rtl8168_rx_csum(struct rtl8168_private *tp,struct RxDesc *desc)
{
    u32 opts1 = le32_to_cpu(desc->opts1);
    u32 opts2 = le32_to_cpu(desc->opts2);
    u32 status = opts1 & RxProtoMask;

    if ((tp->mcfg == CFG_METHOD_1) ||
        (tp->mcfg == CFG_METHOD_2) ||
        (tp->mcfg == CFG_METHOD_3)) {
        /* rx csum offload for RTL8168B/8111B */
        if (((status == RxProtoTCP) && !(opts1 & RxTCPF)) ||
            ((status == RxProtoUDP) && !(opts1 & RxUDPF)) ||
            ((status == RxProtoIP) && !(opts1 & RxIPF)))
            return  CHECKSUM_UNNECESSARY;
        else
            return CHECKSUM_NONE;
    } else {
        /* rx csum offload for RTL8168C/8111C and RTL8168CP/8111CP */
        if (((status == RxTCPT) && !(opts1 & RxTCPF)) ||
            ((status == RxUDPT) && !(opts1 & RxUDPF)) ||
            ((status == 0) && (opts2 & RxV4F) && !(opts1 & RxIPF)))
            return CHECKSUM_UNNECESSARY;
        else
            return CHECKSUM_NONE;
    }
}


static struct mbuf * getmbuf(struct rtl8168_private *tp)
{
	struct mbuf *m;

	MGETHDR(m, M_DONTWAIT, MT_DATA);
	if(m == NULL){
		printf("getmbuf failed, Out of memory!!!\n");
		return  NULL;
	} else {
		MCLGET(m, M_DONTWAIT);
		if ((m->m_flags & M_EXT) == 0) {
			m_freem(m);
			return NULL;
		}
		if(m->m_data != m->m_ext.ext_buf){
			printf("m_data not equal to ext_buf!!!\n");
		}
	}
	
#if defined(__mips__)
	/*
	 * Sync the buffer so we can access it uncached.
	 */
	if (m->m_ext.ext_buf!=NULL) {
#if !(defined(LS3_HT) || defined(LS2G_HT))
		pci_sync_cache(tp->sc_pc, (vm_offset_t)m->m_ext.ext_buf, MCLBYTES, SYNC_R);
#endif
	}
#define RFA_ALIGNMENT_FUDGE 2
	m->m_data += RFA_ALIGNMENT_FUDGE;
#else
	m->m_data += RFA_ALIGNMENT_FUDGE;
#endif
	return m;
}

/*
 *	type : 0 is tx;
 *		   1 is rx;
 */
void print_rt(struct rtl8168_private *tp, int entry, int type) {
	int j, m = 0, n = 0;
	char s[16] = {0};
	int count = 0;

	if (type == 0)
		count = tp->tx_buf_sz;
	else if (type == 1)
		count = tp->rx_buf_sz;

	printf("0000000: ");
	for (j = 0; j < tp->tx_buf_sz; j += 2) {
		if ((j%16 == 0 && j > 0)) {
			if (m == 0) {
				sprintf(s, "0000%x%x0: ", n, m);
			}
			else if (m == 1)
				sprintf(s, "0000%x%x0: ", n, m);
			else if (m == 2)
				sprintf(s, "0000%x%x0: ", n, m);
			else if (m == 3)
				sprintf(s, "0000%x%x0: ", n, m);
			else if (m == 4)
				sprintf(s, "0000%x%x0: ", n, m);
			else if (m == 5)
				sprintf(s, "0000%x%x0: ", n, m);
			else if (m == 6)
				sprintf(s, "0000%x%x0: ", n, m);
			else if (m == 7)
				sprintf(s, "0000%x%x0: ", n, m);
			else if (m == 8)
				sprintf(s, "0000%x%x0: ", n, m);
			else if (m == 9)
				sprintf(s, "0000%x%x0: ", n, m);
			else if (m == 0xa)
				sprintf(s, "0000%x%x0: ", n, m);
			else if (m == 0xb)
				sprintf(s, "0000%x%x0: ", n, m);
			else if (m == 0xc)
				sprintf(s, "0000%x%x0: ", n, m);
			else if (m == 0xd)
				sprintf(s, "0000%x%x0: ", n, m);
			else if (m == 0xe)
				sprintf(s, "0000%x%x0: ", n, m);
			else if (m == 0xf)
				sprintf(s, "0000%x%x0: ", n, m);

			printf("\n");
			printf("%s", s);

			if (m == 0xf) {
				n++;
				m = 0;
			} else {
				m++;
			}
		}
		if (j == 14)
			m++;
		if (type == 0)
			sprintf(s, "%02x%02x ", tp->tx_buffer[entry][j], tp->tx_buffer[entry][j+1]);
		else if (type == 1)
			sprintf(s, "%02x%02x ", tp->rx_buffer[entry][j], tp->rx_buffer[entry][j+1]);
		printf("%s ", s);
	}
	printf("\n");

}

static void rtl8168_tx_interrupt(struct rtl8168_private *tp)
{
	unsigned int dirty_tx, tx_left;

	dirty_tx = tp->dirty_tx;
	tx_left = tp->cur_tx - dirty_tx;

	while (tx_left > 0) {
		unsigned int entry = dirty_tx % NUM_TX_DESC;
		u32 status = 0;

		status = le32_to_cpu(tp->TxDescArray[entry].opts1);
		if (status & DescOwn){
            printf("TX Interrupt error!  %s, %d\n", __FUNCTION__, __LINE__);
			break;
		}

		tp->stats.tx_packets++;

		if(rtl8168_tx_fill(tp, entry, entry + 1, 1) != 1){//should I do it here ? lihui
			printf("TX fill eror!  %s, %d\n", __FUNCTION__, __LINE__);
			return;
		}
		dirty_tx++;
		tx_left--;


	}
	tp->dirty_tx = dirty_tx;

	return;
}

void rtl8168_config_reg()
{
	int i;
	pcitag_t rtl8168_dev;

	rtl8168_dev = _pci_make_tag(2, 0, 0);

	printf("\n");
	for(i = 0; i < 0x100; i += 16){
		printf("reg 0x%x: %8x %8x %8x %8x\n", i,
					   			_pci_conf_read(rtl8168_dev, i),
								_pci_conf_read(rtl8168_dev, i+4),
								_pci_conf_read(rtl8168_dev, i+8),
								_pci_conf_read(rtl8168_dev, i+12));
	}

}

void rtl8168_io_reg()
{
	int i;
	u32	rtl8168_iobase;
	pcitag_t rtl8168_dev;

	rtl8168_dev = _pci_make_tag(2, 0, 0);
	rtl8168_iobase = 0xb8000000 | (_pci_conf_read(rtl8168_dev, 0x10) & 0xfffffff0);

	for(i = 0; i < 0x100; i += 4){
		printf("io reg 0x%x: %02x %02x %02x %02x\n", i,
						*(volatile unsigned char *)(rtl8168_iobase + i),
						*(volatile unsigned char *)(rtl8168_iobase + i + 1),
						*(volatile unsigned char *)(rtl8168_iobase + i + 2),
						*(volatile unsigned char *)(rtl8168_iobase + i + 3));
	}
}

#define RTL_CLOCK_RATE	3
#define RTL_EEPROM_READ_OPCODE      06
#define RTL_EEPROM_WRITE_OPCODE     05
#define RTL_EEPROM_ERASE_OPCODE     07
#define RTL_EEPROM_EWEN_OPCODE      19
#define RTL_EEPROM_EWDS_OPCODE      16

#define Cfg9346_EESK (1<<2)
#define Cfg9346_EEDI (1<<1)
#define Cfg9346_EEDO (1<<0)
#define Cfg9346_EECS (1<<3)
#define Cfg9346_EEM1 (1<<7)
#define Cfg9346 0x50

void rtl_raise_clock(u8 *x, struct rtl8168_private *tp)
{
	*x = *x | Cfg9346_EESK;
	RTL_W8(tp, Cfg9346, *x);
	udelay(RTL_CLOCK_RATE);
}

void rtl_eeprom_cleanup(struct rtl8168_private *tp)
{
	u8 x;

	x = RTL_R8(tp, Cfg9346);
	x &= ~(Cfg9346_EEDI | Cfg9346_EECS);

	RTL_W8(tp, Cfg9346, x);

	rtl_raise_clock(&x, tp);
	rtl_lower_clock(&x, tp);
}

void rtl_lower_clock(unsigned char *x, struct rtl8168_private *tp)
{
  	*x = *x & ~Cfg9346_EESK;
	RTL_W8(tp, Cfg9346, *x);
	udelay(RTL_CLOCK_RATE);
}

u16 rtl_shift_in_bits(struct rtl8168_private *tp)
{
	u8 x;
	u16 d, i;

	x = RTL_R8(tp, Cfg9346);
	x &= ~(Cfg9346_EEDI | Cfg9346_EEDO);

	d = 0;

	for (i = 0; i < 16; i++) {
		d = d << 1;
		rtl_raise_clock(&x, tp);

		x = RTL_R8(tp, Cfg9346);
		x &= ~Cfg9346_EEDI;

		if (x & Cfg9346_EEDO)
			d |= 1;

		rtl_lower_clock(&x, tp);
	}

	return d;
}

void rtl_shift_out_bits(int data, int count, struct rtl8168_private *tp)
{
	u8 x;
	int  mask;

	mask = 0x01 << (count - 1);
	x = RTL_R8(tp, Cfg9346);
	x &= ~(Cfg9346_EEDI | Cfg9346_EEDO);

	do {
		if (data & mask)
			x |= Cfg9346_EEDI;
		else
			x &= ~Cfg9346_EEDI;

		RTL_W8(tp, Cfg9346, x);
		udelay(RTL_CLOCK_RATE);
		rtl_raise_clock(&x, tp);
		rtl_lower_clock(&x, tp);
		mask = mask >> 1;
	} while(mask);

		x &= ~Cfg9346_EEDI;
		RTL_W8(tp, Cfg9346, x);
}

/* EEPROM CODE */
#define EE_WEN_CMD      ( (4 << 6) | ( 3 << 4 ) )
#define EE_WRITE_CMD    (5 << 6)
#define EE_WDS_CMD      (4 << 6)
#define EE_READ_CMD     (6 << 6)
#define EE_ERASE_CMD    (7 << 6)

/*  EEPROM_Ctrl bits. */
#define EE_CS           0x08    /* EEPROM chip select. */
#define EE_SHIFT_CLK    0x04    /* EEPROM shift clock. */
#define EE_DATA_WRITE   0x02    /* EEPROM chip data in. */
#define EE_WRITE_0      0x00
#define EE_WRITE_1      0x02
#define EE_DATA_READ    0x01    /* EEPROM chip data out. */
#define EE_ENB          (0x80 | EE_CS)

#define myoutb(v, a)  (*(volatile unsigned char *)(a) = (v))
#define myoutw(v, a)  (*(volatile unsigned short*)(a) = (v))
#define myoutl(v, a)  (*(volatile unsigned long *)(a) = (v))
#define eeprom_delay()  inl(ee_addr)

int read_eeprom(struct rtl8168_private *tp, int RegAddr)
{
	int i;
	long ee_addr = tp->ioaddr + Cfg9346;
	int read_cmd = RegAddr | EE_READ_CMD;
	int retval = 0;

	RTL_W8(tp, Cfg9346, EE_ENB & ~EE_CS);
	RTL_W8(tp, Cfg9346, EE_ENB);

#ifndef EPLC46
	/* Shift the read command bits out. */
		for (i = 10; i >= 0; i--) {
			int dataval = (read_cmd & (1 << i)) ? EE_DATA_WRITE : 0;
			RTL_W8(tp, Cfg9346, EE_ENB | dataval);
			delay(100);
			RTL_W8(tp, Cfg9346, EE_ENB | dataval | EE_SHIFT_CLK);
			delay(100);
		}
#else
	for (i = 11; i >= 0; i--) {
		int dataval = (read_cmd & (1 << i)) ? EE_DATA_WRITE : 0;
		RTL_W8(tp, Cfg9346, EE_ENB | dataval);
		delay(100);
		RTL_W8(tp, Cfg9346, EE_ENB | dataval | EE_SHIFT_CLK);
		delay(100);
	}
#endif
	RTL_W8(tp, Cfg9346, EE_ENB);

#ifndef EPLC46
		retval = 0;
	for( i = 16; i > 0; i--) {
		RTL_W8(tp, Cfg9346, EE_ENB | EE_SHIFT_CLK);
		delay(100);
		retval = (retval << 1) | ((RTL_R8(tp, Cfg9346) & EE_DATA_READ) ? 1 : 0);
		RTL_W8(tp, Cfg9346, EE_ENB);
		delay(100);
	}
#else
	for( i = 16; i > 0; i--) {
		RTL_W8(tp, Cfg9346, EE_ENB | EE_SHIFT_CLK);
		delay(100);
		retval = (retval << 1) | ((RTL_R8(tp, Cfg9346) & EE_DATA_READ) ? 1 : 0);
		RTL_W8(tp, Cfg9346, EE_ENB);
		delay(100);
	}
#endif
	return retval;
}

static void write_eeprom_enable(long ioaddr){
	int i;
	long ee_addr = ioaddr + Cfg9346;
	int  cmd = EE_WEN_CMD;

	myoutb(EE_ENB & ~EE_CS, ee_addr);
	myoutb(EE_ENB, ee_addr);

	/* Shift the read command bits out. */
#ifndef EPLC46
	for (i = 8; i >= 0; i--) {
		int dataval = (cmd & (1 << i)) ? EE_DATA_WRITE : 0;
		myoutb(EE_ENB | dataval, ee_addr);
		eeprom_delay();
		myoutb(EE_ENB | dataval | EE_SHIFT_CLK, ee_addr);
		eeprom_delay();
	}
#else
	for (i = 11; i >= 0; i--) {
		int dataval = (cmd & (1 << i)) ? EE_DATA_WRITE : 0;
		myoutb(EE_ENB | dataval, ee_addr);
		eeprom_delay();
		myoutb(EE_ENB | dataval | EE_SHIFT_CLK, ee_addr);
		eeprom_delay();
	}
#endif
	myoutb(~EE_CS, ee_addr);

}

static void write_eeprom_disable(unsigned long ioaddr)
{
	int i;
	long ee_addr = ioaddr + Cfg9346;
	int  cmd = EE_WDS_CMD;

	myoutb(EE_ENB & ~EE_CS, ee_addr);
	myoutb(EE_ENB, ee_addr);

	/* Shift the read command bits out. */

#ifndef EPLC46
	i=10;
#else
	i=11;
#endif
	for (; i >= 0; i--) {
		int dataval = (cmd & (1 << i)) ? EE_DATA_WRITE : 0;
		myoutb(EE_ENB | dataval, ee_addr);
		eeprom_delay();
		myoutb(EE_ENB | dataval | EE_SHIFT_CLK, ee_addr);
		eeprom_delay();
	}
	myoutb(~EE_CS, ee_addr);
}

int rtl8168_write_eeprom(long ioaddr, int location,unsigned short data)
{
	volatile int i;
	long ee_addr = ioaddr + Cfg9346;
	int  cmd = location | EE_WRITE_CMD;

	write_eeprom_enable(ioaddr);

	cmd <<=16;
	cmd |= data;
	myoutb(EE_ENB & ~EE_CS, ee_addr);
	myoutb(EE_ENB, ee_addr);

	/* Shift the read command bits out. */
	for (i = 24; i >= 0; i--) {
		int dataval = (cmd & (1 << i)) ? EE_DATA_WRITE : 0;
		myoutb(EE_ENB | dataval, ee_addr);
		eeprom_delay();
		myoutb(EE_ENB | dataval | EE_SHIFT_CLK, ee_addr);
		eeprom_delay();
	}
	/* Terminate the EEPROM access. */
	myoutb(~EE_CS, ee_addr);
		delay(10);

	myoutb(EE_ENB, ee_addr);
	 
    while( ! (inb(ee_addr) & EE_DATA_READ) ){
		myoutb(EE_ENB, ee_addr);
		delay(10);
	}

	return 0;
}

static int cmd_reprom(int ac, char *av[])
{
	int i;
	int n = 0;
	unsigned short data;	

	printf("dump eprom:\n");

	if (!strcmp(av[1], "rte0")){
		n = 0; 
		printf("Now read rte0 e2prom\n");
	}
	else if (!strcmp(av[1], "rte1")){
		n = 1; 
		printf("Now read rte1 e2prom\n");
	}
	else{
		printf("input device Error \n");
		return 0;
	}

    if(!myRTL[n]){
           printf("r8111/8168 interface not initialized\n");
           return 0;
    }
	printf("myRTL[%d] = %x\n", n, myRTL[n]);

	for(i=0; i< 64; i++){
#ifndef EPLC46
		data = read_eeprom(myRTL[n],  i); //run 
#else
		data = read_eeprom(myRTL[n], 2*i);
		data = data | (read_eeprom(myRTL[n],2*i+1)) << 8;
#endif
		printf("%04x ", data);
		if((i+1)%8 == 0)
			printf("\n");
	}

	return data;
}

static int cmd_wrprom(int ac, char *av[])
{
    int i;
	int n = 0; 
    unsigned short data;

static unsigned char rom[]={
/*00000000:*/0x29,0x81,0xEC,0x10,0x68,0x81,0xEC,0x10,0x68,0x81,0x04,0x01,0x9C,0x62,0x00,0xE0,
/*00000010:*/0x4C,0x68,0x00,0x01,0x05,0xCF,0xC3,0xFF,0x04,0x02,0xC0,0x8C,0x80,0x02,0x00,0x00,
/*00000020:*/0x11,0x3C,0x07,0x00,0x10,0x20,0x76,0x00,0x63,0x01,0x01,0xFF,0x00,0x27,0xAA,0x03,
/*00000030:*/0x02,0x20,0x99,0xA5,0x80,0x02,0x00,0x20,0x04,0x40,0x20,0x00,0x04,0x40,0x20,0x20,
/*00000040:*/0x00,0x00,0x20,0xE1,0x22,0xB5,0x60,0x00,0x0A,0x00,0xE0,0x00,0x68,0x4C,0x00,0x00,
/*00000050:*/0x01,0x00,0x00,0x00,0xB2,0x73,0x75,0xEA,0x87,0x75,0x7A,0x39,0xCA,0x98,0x00,0x00,
/*00000060:*/0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
/*00000070:*/0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};

	if (!strcmp(av[1], "rte0")){
		n = 0; 
		printf("Now write rte0 e2prom\n");
	}
	else if (!strcmp(av[1], "rte1")){
		n = 1; 
		printf("Now write rte1 e2prom\n");
	}
	else{
		printf("Error rte device\n");
		return 0;
	}

    if(!myRTL[n]){
    	printf("rtl8111/8168 interface not initialized\n");
        return 0;
    }

    printf("myRTL[%d]->ioaddr = %x\n", n, myRTL[n]->ioaddr);

    for(i = 0; i < 0x40; i++)
    {
        int j = 5;
        printf("program %02x:%04x\n",2*i,(rom[2*i+1]<<8)|rom[2*i]);
rewrite:
        rtl8168_write_eeprom(myRTL[n]->ioaddr, i, (rom[2*i+1]<<8)|rom[2*i]);
        data = read_eeprom(myRTL[n],  i);
        printf("data = %x\n", data);

        if(data != ((rom[2*i+1]<<8)|rom[2*i])){
            printf("write failed, rewrite now\n");
			j--;
			if (j < 0){
				printf("write e2prom failed\n");

				return -1;
			}
            goto rewrite;
        }

    }
    printf("The whole eeprom have been programmed!\n");

	/* set mac address */
	if (NULL != av[2]){
		int v;
		char *p = av[2];
		unsigned short val = 0;

		printf("Set Macaddress\n");
		for (i = 0; i < 3; i++){
			val = 0;
			gethex(&v, av[2], 2);
			val = v;
			av[2] += 3;

			gethex(&v, av[2], 2);
			val = val | (v << 8);
			av[2] += 3;	

			printf("value[%x] = %4x\n", i, val);

			rtl8168_write_eeprom(myRTL[n]->ioaddr, 0x7 + i, val);
		}
		printf("Set Macaddress : %s End\n", p);

	}
	else
		printf("No Set Macaddress\n");

	return 0;
}

static int cmd_setmac(int ac, char *av[])
{
	int i, n;
	unsigned short val = 0;
	int32_t v;

	if (!strcmp(av[1], "rte0")){
		n = 0; 
		printf("Now set rte0 MAC\n");
	}
	else if (!strcmp(av[1], "rte1")){
		n = 1; 
		printf("Now set rte1 MAC\n");
	}
	else{
		printf("Error rte device\n");
		return 0;
	}

	if(!myRTL[n]){
		printf("r8111/8168 interface not initialized\n");
		return 0;
	}

	if(ac != 3){
		printf("MAC ADDRESS ");
		for(i=0; i<6; i++){
			printf("%02x", myRTL[n]->dev_addr[i]);
			if(i==5)
				printf("\n");
			else
				printf(":");
		}
		printf("Use \"setmac rte0/1 <mac> \" to set mac address\n");
		return 0;
	}
	for (i = 0; i < 3; i++) {
		val = 0;
		gethex(&v, av[2], 2);
		val = v ;
		av[2]+=3;

		gethex(&v, av[2], 2);
		val = val | (v << 8);
		av[2] += 3;

		rtl8168_write_eeprom(myRTL[n]->ioaddr, 0x7 + i, val);

	}

	printf("The machine should be restarted to make the mac change to take effect!!\n");

	return 0;
}

int rtl8168_link_ok(int index)
{
  return rtl8168_xmii_link_ok(myRTL[index]);  
}



static const Optdesc netdmp_opts[] =
{
    {"<interface>", "Interface name"},
    {0}
};

static const Cmd Cmds[] =
{
	{"Realtek 8111dl/8168"},

	{"readrom", "", NULL,
			"dump rtl8111dl/8168 eeprom content and mac address", cmd_reprom, 1, 2,0},
	{"writerom", "", NULL,
			"dump rtl8111dl/8168 eeprom content", cmd_wrprom, 1, 3, 0},
	{"setmac", "", NULL,
		    "Set mac address into rtl8111dl/8168 eeprom", cmd_setmac, 1, 5, 0},
	{0, 0}
};


static void init_cmd __P((void)) __attribute__ ((constructor));

static void
init_cmd()
{
	cmdlist_expand(Cmds, 1);
}
